/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

//#include "menugen.h"
#include <EMimIcon>
#include <EIcon>
#include <DesktopFile>

#include "dialogaddaction.h"
#include "ui_dialogaddaction.h"
#include <QDebug>
#include <QFile>

#include <QFileDialog>


DialogAddAction::DialogAddAction(QStringList list, QIcon icon, QWidget *parent) :
          QDialog(parent),
          ui(new Ui::DialogAddAction)
{
          ui->setupUi(this);
          QString locale = QLocale::system().name().section("_",0,0);

          if(locale=="ar")
              setLayoutDirection(Qt::RightToLeft);

          this->setWindowIcon(EIcon::fromTheme("configure",("document-properties")));
          this->setAttribute(Qt::WA_DeleteOnClose,true);
          ui->buttonBox->addButton(QDialogButtonBox::Ok);
          ui->lineEditName->setText(list.at(0));
          ui->lineEditExec->setText(list.at(1));
          ui->pushButton->setIcon(icon.pixmap(48));

m_type=list.at(2);

        m_iconName=list.at(3);
          ui->treeWidget->setVisible(false);
          //ui->lineEditSearsh->setVisible(false);
          ui->lineEditIcon->setText(m_iconName);
          this->resize(430,150);
}
DialogAddAction:: DialogAddAction(QWidget *parent) :
          QDialog(parent),
          ui(new Ui::DialogAddAction)
{
          ui->setupUi(this);
          QString locale =this->locale().name().section("_",0,0);

          if(locale=="ar")
              setLayoutDirection(Qt::RightToLeft);
          this->setAttribute(Qt::WA_DeleteOnClose,true);
        //  qDebug()<<"this 1";
          ui->treeWidget->setVisible(true);
  //        ui->lineEditSearsh->setVisible(true);
          QIcon icn=EIcon::fromTheme("list-add","add");
          QPushButton *btnAdd=new QPushButton(icn,trUtf8("Add"),this);
          ui->buttonBox->addButton(btnAdd,QDialogButtonBox::ApplyRole);
          connect(btnAdd,SIGNAL(clicked()) ,this,SLOT(applyAdd()));
        chargeXmlDocument();
          this->resize(430,300);
}

DialogAddAction::~DialogAddAction()
{
          qDebug()<<"delet 1";
          delete ui;
}

/*****************************************************************************
 *
 *****************************************************************************/
void DialogAddAction::chargeXmlDocument()
{

    QString xmlPath=Edir::configHomeDir()+"/menus/elokab-applications.menu";
    if(!QFile::exists(xmlPath)){
        QDir appDir(qApp->applicationDirPath());
        appDir.cdUp();
        appDir.cdUp();
        xmlPath=  appDir.absolutePath()+"/etc/xdg/menus/elokab-applications.menu";
        //  xmlPath="/etc/xdg/menus/elokab-applications.menu";
    }

    QFile file(xmlPath);



    if (!file.open(QIODevice::ReadOnly)){
        qDebug()<<file.errorString();
        return;
    }
    if (!m_doc.setContent(&file)) {
        qDebug()<<"error xmlContent";
        file.close();
        return;
    }
    file.close();

    // print out the element names of all elements that are direct children
    // of the outermost element.
    QDomElement docElem = m_doc.documentElement();

    QDomNode n = docElem.firstChild();
    while(!n.isNull()) {
        QDomElement menu = n.toElement(); // try to convert the node to an element.
        if(!menu.isNull()) {
         //   qDebug()<<menu.tagName(); // the node really is an element.
            if(menu.tagName()=="Menu"){
                QString name=   menu.firstChildElement("Name").text();
                QString directory=  menu.firstChildElement("Directory").text();
                QString category;

                QDomNode Include = menu.firstChildElement("Include");
                QDomNode And = Include.firstChildElement("And");
                QDomElement Cat = And.toElement();
                if(!Cat.isNull()) {
                    category=   Cat.firstChildElement("Category").text();

                }


                addMenuDirectory(name,directory,category);
            }
        }
        n = n.nextSibling();
    }
    chargeAppDesktop();
}
 void DialogAddAction::addMenuDirectory(QString name, QString directory,QString category)
 {

     QString directoryPath=Edir::dataHomeDir()+"/desktop-directories/"+directory;
     if(!QFile::exists(directoryPath)){
         QDir appDir(qApp->applicationDirPath());
         appDir.cdUp();
         directoryPath=appDir.absolutePath()+"/share/desktop-directories/"+directory;

     }
     QString lc=this->locale().name().section("_",0,0);

     DesktopFile settings(directoryPath,"Desktop Entry",lc);

     QString textOld=settings.value("Name",name).toString();
     QString text=settings.value("Name["+lc+"]",textOld).toString();
     QString commentOld=settings.value("Comment").toString();
     QString comment=settings.value("Comment["+lc+"]",commentOld).toString();
     QString icon=settings.value("Icon").toString();


     QTreeWidgetItem *item=new QTreeWidgetItem(ui->treeWidget);
     item->setText(0,text);
     item->setToolTip(0,comment);
     item->setData(1,Qt::UserRole,category);
     item->setData(0,Qt::UserRole,name);
        item->setIcon(0,EIcon::fromTheme(icon));

 }

void DialogAddAction::chargeAppDesktop()
{

    QFile file(Edir::configDir()+"/applictions");

          if (!file.open(QFile::ReadOnly)){
                    qDebug()<<file.fileName()+" \n  "+file.errorString();
                    return ;
          }
          QTextStream textStream(&file);
          QString line;
 QString lc=this->locale().name().section("_",0,0);
          line= textStream.readLine();//premier line

          while (!line.isNull()) {

              DesktopFile dsg(line.section("|",0,0),"Desktop Entry",lc);

              QString oldname=dsg.value("Name").toString();
              QString name=dsg.value("Name["+lc+"]",oldname).toString();
              QString exec=dsg.value("Exec").toString();
              QString category=dsg.value("Categories","Other").toString();
              QString icon=dsg.value("Icon").toString();

              QTreeWidgetItem *item=new QTreeWidgetItem;
              item->setData(1,1,exec);
              item->setData(2,2,icon);
              item->setText(0,name);
              item->setIcon(0,EIcon::fromTheme(icon));

              for(int i=0;i<ui->treeWidget->topLevelItemCount();i++){
                  QString cat=ui->treeWidget->topLevelItem(i)->data(1,Qt::UserRole).toString();
                  if(category.contains(cat))
                      ui->treeWidget->topLevelItem(i)->addChild(item);
              }


              line= textStream.readLine();
          }
          file.close();

          for(int i=0;i<ui->treeWidget->topLevelItemCount();i++){

              if(ui->treeWidget->topLevelItem(i)->childCount()==0)
                  ui->treeWidget->takeTopLevelItem(i);
          }

}


/*****************************************************************************
 *
 *****************************************************************************/
void DialogAddAction::on_treeWidget_itemSelectionChanged()
{
          QTreeWidgetItem *item=ui->treeWidget->currentItem();
          if(item){
                    if(item->parent()){
                              ui->lineEditName->setText(item->text(0));
                              ui->lineEditExec->setText(item->data(1,1).toString());
                              QPixmap pix=item->icon(0).pixmap(48,48);
                             ui->pushButton->setIcon(QIcon(pix));
                              m_iconName=item->data(2,2).toString();
                            //  ui->lineEditIcon->setText(m_iconName);
                    }
          }
        //  qDebug()<<ui->lineEditExec->text();
}

void DialogAddAction::on_buttonBox_clicked(QAbstractButton *button)
{

          if(ui->buttonBox->standardButton(button)==QDialogButtonBox::Ok){
                    if(ui->lineEditExec->text().isEmpty()||ui->lineEditName->text().isEmpty())
                              return;

QStringList list;
list<<ui->lineEditName->text()
      <<m_type
        <<ui->lineEditExec->text()
          <<m_iconName;
                    emit accepteChange(list);
                    this->accept();
          }
}
void DialogAddAction::applyAdd()
{

          if(ui->lineEditExec->text().isEmpty()||ui->lineEditName->text().isEmpty())
                    return;
          QString name=ui->lineEditName->text();
          QString exec=ui->lineEditExec->text();

          emit accepteAdd(name+"|"+exec+"|"+m_iconName);
}

void DialogAddAction::on_pushButton_clicked()
{
          QFileDialog dlg;
//".png", ".svg", ".svgz", ".svg.gz", ".xpm"
          QString fn = dlg.getOpenFileName(this, tr("Open Images Files..."),
                                           QDir::homePath(), trUtf8("Image File (*.png *.svg *.svgz *.svg.gz *.xpm);;All Files (*)"));
          if (!fn.isEmpty())
          {
                    ui->pushButton->setIcon(QIcon(fn));
                    ui->lineEditIcon->setText(fn);
                    m_iconName=fn;

          }
}

void DialogAddAction::on_toolButton_clicked()
{
          QFileDialog dlg;

          QString fn = dlg.getOpenFileName(this, tr("Open Files..."),
                                           QDir::homePath(), trUtf8("All Files (*)"));
          if (!fn.isEmpty())
          {
                    ui->lineEditExec->setText(fn);

          }
}

void DialogAddAction::on_lineEditIcon_textChanged(const QString &arg1)
{
        QIcon icon;
        icon=EIcon::fromTheme(arg1).pixmap(48,48);
        if(QFile::exists(arg1)){
            icon=QIcon(arg1);
             ui->pushButton->setIcon(icon);
               m_iconName=arg1;
        }else if(!icon.isNull()){

             ui->pushButton->setIcon(icon);
               m_iconName=arg1;
        }

}
