/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <MenuGen>
#include <ElokabSettings>
#include <QMainWindow>
#include <QTreeWidgetItem>
#include <QDomDocument>
#include<QHash>
namespace Ui {
class MainWindow;
}
/**
 * @brief The MainWindow class
 *النافذة الرئيسية
 */
class MainWindow : public QMainWindow
{
    Q_OBJECT


public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
signals:
    void hasClipboard(bool );
private slots:
    /**
     * @brief chargeAppDesktop
     *تحميل ملفات سطح المكتب من
     *"/.config/elokab/applictions"
     */
    void chargeAppDesktop();
    /**
     * @brief chargeXmlDocument
     *تحميل  القوائم من احد المسارين التاليين
     *"/.config/menus/elokab-applications.menu"
     *"/etc/xdg/menus/elokab-applications.menu"
     */
    void chargeXmlDocument();
    /**
     * @brief addMenuDirectory اضافة معلومات القوائم من المسارين
     *"/.local/share/desktop-directories/"
     *"/share/desktop-directories/"
     * @param name اسم القائمة
     * @param directory الملف
     * @param category المجموعة
     */
    void refrechIcons();
    void addMenuDirectory(QString name, QString directory, QString category);
    void on_treeWidget_itemSelectionChanged();
    void editCurrentItem();
    void editCurrentMenu();
    void addNewMenu();
    void addNewItem();
    void deletItem();
    void removeItem(QTreeWidgetItem *itemSelected);
    bool removeNode(const QString &name);
    void saveDocumment();
    void creatAppDesktop(const QString &fileApp);
    //
    void reloadAppList();
    void removeUnicodeSpace(const QString &fileName);

    QStringList menusList();
    //cliboard
    void copyItem();
    void cutItem();
    void pastItem();

    void on_comboBox_currentIndexChanged(const QString &arg1);

private:
    Ui::MainWindow *ui;
    bool m_hasClipboard;
    int m_clipboardType;
    int m_n;
    QDomDocument m_doc;
    MenuGen *gen ;
    QTreeWidgetItem *m_clipboardItem;
ElokabSettings *msetting;
QHash <QString,QIcon*>mhashIcons;
QCache<QString, QIcon> IconCache;
};

#endif // MAINWINDOW_H
