/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "dialogconfigwidgets.h"
#include <DesktopFile>
#include "ui_dialogconfigwidgets.h"
#include "dialogqmlproperty.h"
#include <EIcon>
#include<EDir>
#include<QSettings>
#include<QDebug>
#include<QTextCodec>

DialogConfigWidgets::DialogConfigWidgets(QWidget *parent) :
        QDialog(parent),
        ui(new Ui::DialogConfigWidgets)
{
  QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8"));

        this->setAttribute(Qt::WA_DeleteOnClose,true);
        ui->setupUi(this);
    //    m_setting.setIniCodec(QTextCodec::codecForName("UTF-8"));
        ui->toolButtonAdd->setIcon(EIcon::fromTheme("list-add",("add")));
     ui->toolButtonRemove->setIcon(EIcon::fromTheme("list-remove",("remove")));
       ui->toolButtonProperty->setIcon(EIcon::fromTheme("configure",("document-edit")));
        QSettings setting("elokab","settings");
     //   setting.setIniCodec(QTextCodec::codecForName("UTF-8"));
                setting.beginGroup("Desktop-Widget");
              m_listWidget=   setting.value("ListWidget").toStringList();
                 setting.endGroup();

            //  qDebug()<<m_listWidget;
                 loadQmlWidgets();
}

DialogConfigWidgets::~DialogConfigWidgets()
{
        delete ui;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void DialogConfigWidgets::loadQmlWidgets()
{


     QString   userPath= Edir::dataAppDir()+"/qmlWidgets";
     QString   HomePath=Edir::dataHomeAppDir()+"/qmlWidgets";
     loadQmlpath(userPath);
     loadQmlpath(HomePath);


}
void DialogConfigWidgets::loadQmlpath(const QString & path)
{
    QTextCodec::setCodecForLocale(QTextCodec::codecForName("UTF-8"));
    QDir dir(path);

    foreach (const QString &dirName, dir.entryList(QDir::AllDirs |
                                                QDir::NoDotAndDotDot | QDir::Readable | QDir::Executable))
    {
        if(QFile::exists(path+"/"+dirName+"/qmlwidget.desktop")){
            QString filename=(path+"/"+dirName+"/qmlwidget.desktop");



            //     hatsh=getEntry(filename);
            QString Lc=this->locale().name().section("_",0,0);
            DesktopFile  hatsh(filename,"Desktop Entry",Lc);

            QString type=hatsh.value("Type").toString();
            if(type!="qmlWidget")continue;

            QListWidgetItem *item=new QListWidgetItem;
            QString oldtxt=hatsh.value("Name").toString();
            QString txt=hatsh.value("Name["+Lc+"]",oldtxt).toString();



            item->setText(txt);
            item->setData(Qt::ToolTipRole,path+"/"+dirName+"/"+hatsh["Exec"].toString());
            item->setData(Qt::UserRole,dirName+"/"+hatsh["Exec"].toString());

            QString iconname=hatsh.value("Icon").toString();
            if(QFile::exists(path+"/"+dirName+"/"+iconname))
                item->setIcon(QIcon(path+"/"+dirName+"/"+iconname));
            else
                item->setIcon(EIcon::fromTheme(iconname));


            ui->listWidgetQml->addItem(item);
            QString exec=hatsh.value("Exec").toString();
            if(m_listWidget.contains(path+"/"+dirName+"/"+exec))
                item->setCheckState(Qt::Checked);
            else
                item->setCheckState(Qt::Unchecked);
        }

    }

}

void DialogConfigWidgets::saveSettings()
{
        m_listWidget.clear();
        int count=ui->listWidgetQml->count();
        for(int i=0;i<count;i++){
                QListWidgetItem *item=ui->listWidgetQml->item(i);
                if(item->checkState()==Qt::Checked)
                        m_listWidget.append(item->data(Qt::ToolTipRole).toString());
        }

        QSettings setting("elokab","settings");
     //   setting.setIniCodec(QTextCodec::codecForName("UTF-8"));
          setting.beginGroup("Desktop-Widget");
                 setting.setValue("ListWidget", m_listWidget);
         setting.endGroup();
}


void DialogConfigWidgets::on_buttonBox_clicked(QAbstractButton *button)
{
        if(ui->buttonBox->standardButton(button)==QDialogButtonBox::Apply)
        {
                saveSettings();
                emit settingChanged();

        }else if(ui->buttonBox->standardButton(button)==QDialogButtonBox::Ok){

                saveSettings();
                emit settingChanged();
                accept();
        }
}

void DialogConfigWidgets::on_toolButtonProperty_clicked()
{
   QListWidgetItem *item=ui->listWidgetQml->currentItem();
   if(!item)
           return;
   QString path=item->data(Qt::ToolTipRole).toString();
   QFileInfo fi(path);
   QString fileName=fi.dir().path()+"/Properties.qml";
QString title=item->text();
   if(!QFile::exists(fileName)){
            qDebug()<<fileName;
           return;
}

   DialogQmlProperty *dlg = this->findChild<DialogQmlProperty*>("DialogQmlProperty");

   if (!dlg)
   {
           dlg = new DialogQmlProperty(fileName,title,this);
            // connect(dlg, SIGNAL(settingChanged()), this, SLOT(loadSettings()));
   }

   dlg->show();

}
