/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "dialogqmlproperty.h"
#include "ui_dialogqmlproperty.h"
#include <QFile>
#include <QDebug>
#include <QSpacerItem>
#include <QTextCodec>

DialogQmlProperty::DialogQmlProperty(const QString &fileName,const QString &title, QWidget *parent) :
        QDialog(parent),
        ui(new Ui::DialogQmlProperty),m_fileName(fileName)
{
        this->setAttribute(Qt::WA_DeleteOnClose,true);
        ui->setupUi(this);
        this->setWindowTitle(title+tr(" Properties"));
        loadQmlProprety(m_fileName);
}

DialogQmlProperty::~DialogQmlProperty()
{
        foreach (LayoutString *lS, listString) {
               delete lS;
        }

        foreach (LayoutInt *lI, listInt) {
                delete lI;
        }

        foreach (LayoutBool *lB, listBool) {
               delete lB;
        }
        foreach (LayoutColor *lC, listColor) {
               delete lC;
        }

        delete ui;
}
void DialogQmlProperty::loadQmlProprety(const QString &fileName)
{
   QTextCodec::setCodecForCStrings(QTextCodec::codecForLocale());
        QFile file(fileName)  ;
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
                return;

        while (!file.atEnd()) {
                QString line = file.readLine();

                if(line.trimmed().startsWith("property")){
                        QString txt=line;

                        txt= txt.remove("property").trimmed();
                        QString value=txt;
                        value=value.remove("string").trimmed();
                        value=value.remove("int").trimmed();
                        value=value.remove("bool").trimmed();
                        value=value.remove("color").trimmed();
                        QString caption=value.section(":",0,0).trimmed();
                        QString val=value.section(":",1,1).trimmed();
                        val=val.remove("\"");

                        //string----------------------
                        if(txt.startsWith("string")){
                                LayoutString *lS=new LayoutString(caption,val.remove("\""));
                                ui->vLayoutWidget->addLayout(lS);
                                listString.append(lS);

                        }
                        //Int -----------------------
                        if(txt.startsWith("int")){
                                LayoutInt *lI=new LayoutInt(caption,val.toInt());
                                ui->vLayoutWidget->addLayout(lI);
                                listInt.append(lI);

                        }
                        //Bool-----------------------
                        if(txt.startsWith("bool")){
                                bool checked=false;
                                if(val=="true")
                                        checked=true;

                                LayoutBool *lB=new LayoutBool(caption,checked);
                                ui->vLayoutWidget->addLayout(lB);
                                listBool.append(lB);
                                //  addBoolProperty(txt);
                        }
                        //Color---------------------
                        if(txt.startsWith("color")){
                                LayoutColor *lC=new LayoutColor(caption,QColor(val));
                                ui->vLayoutWidget->addLayout(lC);
                                listColor.append(lC);

                        }

                }//if "property"

        }//while
}
void DialogQmlProperty::on_buttonBox_accepted()
{
        QFile fileS(m_fileName);

       if (! fileS.open(QIODevice::WriteOnly ))
                return;

        QTextStream out(&fileS);
        out.setCodec(QTextCodec::codecForName("UTF-8"));
         out<<"import QtQuick 1.0 \n QtObject { \n";

        foreach (LayoutString *lS, listString) {
              //  qDebug()<<lS->propertyValue()<<lS->propertyName();
                out<<"\n   property string "+ lS->propertyName()+ " : \""+ lS->propertyValue()+"\"";
        }

        foreach (LayoutInt *lI, listInt) {
              //  qDebug()<<lI->propertyValue()<<lI->propertyName();
                out<<"\n   property int "+ lI->propertyName()+ " : "+ lI->propertyValue();
        }

        foreach (LayoutBool *lB, listBool) {
               // qDebug()<<lB->propertyValue()<<lB->propertyName();
                out<<"\n   property bool "+ lB->propertyName()+ " : "+ lB->propertyValue();
        }
        foreach (LayoutColor *lC, listColor) {
// qDebug()<<lC->propertyValue()<<lC->propertyName();
                out<<"\n   property color "+ lC->propertyName()+ " : \""+ lC->propertyValue()+"\"";
        }
         out<<"\n\n }";
          fileS.close();
        //save
}

//! Class int String bool color -------------------------
//-----------string Property----------------------------------------
LayoutString::LayoutString(const QString &caption,const QString &txt)
{
        label = new QLabel(caption);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
        label->setSizePolicy(sizePolicy);
        addWidget(label);
       QSpacerItem *horizontalSpacer = new QSpacerItem(10, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
       addItem(horizontalSpacer);
        lineEdit = new QLineEdit(txt);
        addWidget(lineEdit);

}

//-----------Int Property----------------------------------------
LayoutInt::LayoutInt(const QString &caption, int value)
{
        label = new QLabel(caption);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
        label->setSizePolicy(sizePolicy);
        addWidget(label);
        QSpacerItem *horizontalSpacer = new QSpacerItem(10, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
        addItem(horizontalSpacer);
        spinBox = new QSpinBox;
        spinBox->setMaximum(16777215);
        spinBox->setValue(value);
        addWidget(spinBox);
        qDebug()<<value;
}

//-----------Bool Property----------------------------------------
LayoutBool::LayoutBool(const QString &caption, bool value)
{
        label = new QLabel(caption);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
        label->setSizePolicy(sizePolicy);
        addWidget(label);
        QSpacerItem *horizontalSpacer = new QSpacerItem(10, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
        addItem(horizontalSpacer);
        checkBox = new QCheckBox;
        checkBox->setChecked(value);
        addWidget(checkBox);

}

//-----------Color Property----------------------------------------
LayoutColor::LayoutColor(const QString &caption, QColor color)
{
        label = new QLabel(caption);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
        label->setSizePolicy(sizePolicy);
        addWidget(label);
        QSpacerItem *horizontalSpacer = new QSpacerItem(10, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
        addItem(horizontalSpacer);
        buttonColor = new ButtonColor(QString(),true,color);
        addWidget(buttonColor);

}
