/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "classname.h"
#include "qmlcontainer.h"
//#include "stickman.h"

#include <QRectF>
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsProxyWidget>
#include <QSettings>
#include <QApplication>
#include <QDesktopWidget>
#include <QPropertyAnimation>
#include <QDebug>

QmlContainer::QmlContainer(const QString &file, QGraphicsItem *parent)
        : QGraphicsObject(parent),m_fileName(file)
{

      //  setFlag(QGraphicsItem::ItemSendsGeometryChanges);
        setFlag(QGraphicsItem::ItemIsMovable);
        setZValue(1);
        QString style="background-color: transparent;border: 0px;";


        // -----------QWidget ----------------
        QWidget *w=new QWidget;
        w-> setStyleSheet(style);
        // -----------QDeclarativeView ----------------
        view=new QDeclarativeView(w);
        view->setSource(QUrl(file));

item=view->rootObject();
         // -----------QGraphicsProxyWidget ----------------
        QGraphicsProxyWidget *toolProxy = new QGraphicsProxyWidget(this);
        toolProxy->setWidget(w);
        ///-----------------------------------------------
        m_rectF=w->rect();

        ///---------------load the last pos--------------
        loadSettings();

}

QmlContainer::~QmlContainer()
{
}

QRectF QmlContainer::boundingRect() const
{
    return QRectF(m_rectF);
}

void QmlContainer::paint(QPainter */*painter*/, const QStyleOptionGraphicsItem *, QWidget *)
{    

}

QVariant QmlContainer::itemChange(GraphicsItemChange change, const QVariant &value)
{
        if (change == ItemPositionHasChanged && scene()) {
                // value is the new position.

            //   positionHasChanged();


        }

    return QGraphicsObject::itemChange(change, value);
}
void QmlContainer::positionHasChanged()
{

        int height=m_rectF.height();
        int width=m_rectF.width();
        QRectF rect=QApplication::desktop()->availableGeometry();

        //leftTop corner
        if(pos().x()<rect.left() && pos().y()<rect.top()){
                animMe(QPointF(rect.left(),rect.top()))   ;
                return;
        }
        //rightTop corner
        if(pos().x()+width>rect.right()&& pos().y()<rect.top()){
                animMe(QPointF(rect.right()-width,rect.top()))   ;
                return;
        }
        //leftBottum corner
        if(pos().x()<rect.left() && pos().y()+height>rect.bottom()){
                animMe(QPointF(rect.left(),rect.bottom()-height))   ;
                return;
        }

        //righttBottum corner
        if(pos().x()+width>rect.right() && pos().y()+height>rect.bottom()){
                animMe(QPointF(rect.right()-width,rect.bottom()-height))   ;
                return;
        }
        //---------------------------------
        //right
        if(pos().x()+width>rect.right())
                animMe(QPointF(rect.right()-width,pos().y()))   ;
        //bottum
        if(pos().y()+height>rect.bottom())
                animMe(QPointF(pos().x(),rect.bottom()-height))   ;
        //top
        if(pos().y()<rect.top())
                animMe(QPointF(pos().x(),rect.top()))   ;
        //left
        if(pos().x()<rect.left())
                animMe(QPointF(rect.left(),pos().y()) )  ;
}
///------------Mouse Event------------------
void QmlContainer::mousePressEvent(QGraphicsSceneMouseEvent *)
{
        setZValue(2);

        m_oldPosition = pos();
}

void QmlContainer::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
        QGraphicsObject::mouseMoveEvent(event);
//    if (m_dragging)
//        setPos(mapToParent(event->pos()));
//       lasP=event->lastPos().x();
//         curP=event->pos().x();

}

void QmlContainer::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
         QGraphicsObject::mouseReleaseEvent(event);
//         if(curP-lasP>4)
//                 animtoBorder();
         setZValue(1);
        positionHasChanged();
         if(pos()!=m_oldPosition)
                 saveSettings();
}
///-----------------Signals to qml -------------
//void QmlContainer::setClassToconnect(int name)
//{
//        if(name==className::CpuValue){
//                QObject::connect(this, SIGNAL(setCpuValue(int)),
//                                     item, SLOT(cpuValue(int)));
//        }
//        if(name==className::MemValue){
//                QObject::connect(this, SIGNAL(setMemValue(int)),
//                                     item, SLOT(memValue(int)));
//        }
//        if(name==className::SwapValue){
//                QObject::connect(this, SIGNAL(setSwapValue(int)),
//                                     item, SLOT(swapValue(int)));
//        }
//        if(name==className::UpTimeValue){
//                QObject::connect(this, SIGNAL(setUptimeValue(QString)),
//                                     item, SLOT(uptimeValue(QString)));

//        }

//}
//------------------load and save pos-------------------
void QmlContainer::loadSettings()
{
        QSettings setting("elokab","QmlWidget");
        setting.beginGroup(m_fileName);
        m_pos=  setting.value("Pos",QPoint(-40,40)).toPoint();
        setting.endGroup();
        setPos(m_pos);
        positionHasChanged();
}
void QmlContainer::saveSettings()
{


        m_pos=this->pos();
        if(m_oldPosition!=m_pos){
                QSettings setting("elokab","QmlWidget");
                setting.beginGroup(m_fileName);
                setting.setValue("Pos",m_pos);
                setting.endGroup();
        }

}

//----------------ANIMMATION------------------------

void QmlContainer::animMe(QPointF p)
{
        QPropertyAnimation  *anim = new QPropertyAnimation(this, "pos");
        connect(anim,SIGNAL(finished()),this,SLOT(positionHasChanged()));
        anim->setEndValue(QPointF(p));

        anim->setDuration(500);

        anim->setEasingCurve(QEasingCurve::OutBack);
        anim->start();
}
void QmlContainer::animtoBorder()
{
           QRect rect=QApplication::desktop()->availableGeometry();
           QPointF p(rect.right()-100, this->pos().y());
        QPropertyAnimation  *anim = new QPropertyAnimation(this, "pos");
        connect(anim,SIGNAL(finished()),this,SLOT(positionHasChanged()));
        anim->setEndValue(QPointF(p));

        anim->setDuration(500);

        anim->setEasingCurve(QEasingCurve::OutBack);
        anim->start();
}
