/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "iconview/iconsview.h"
#include "desktopsceen.h"
#include <EIcon>
#include "DesktopWidget/dialogconfigwidgets.h"
#include<QDesktopServices>
#include<QApplication>
#include<QDesktopWidget>
#include<EDir>
#include<QMenu>
#include<QDebug>
#include<QSettings>
#include<QTextCodec>
#include<QStyleFactory>
#include<QTimer>
#include<QUrl>
#include<QProcess>
#include<QDBusInterface>
#include <X11UTILLS>
/**
 * @brief النافذة الحاوية لسطح المكتب
 * @note كل الاوامر والاعدادات تنتقل عبر النافذة الرئيسية ثم يتلقاها المشهد او الودجات
 * @param iconsView مشخد الايقونات افتراضيا يساوي الصفر
 * @param m_actReload امر التحدسث يساوي الصفر
 * @return
 */
DesktopSceen::DesktopSceen(QWidget *parent):
    QGraphicsView(parent),iconsView(0),m_actReload(0)
{

    setWindowFlags(Qt::CustomizeWindowHint);
    setAttribute(Qt::WA_X11NetWmWindowTypeDesktop);
    //setAttribute(Qt::WA_X11NetWmWindowTypeDock);
    X11UTILLS::moveWindowToDesktop(this->winId(), -1);
    setFrameShape(QFrame::NoFrame);
    //   setWindowTitle(QString("Elokab-Desktop"));
    setAcceptDrops(true);

    QApplication::setAttribute(Qt::AA_DontShowIconsInMenus ,false);

    m_settings=new ElokabSettings(this);


    m_actNextWall=new QAction(EIcon::fromTheme("user-desktop").pixmap(24),trUtf8("Next wallpaper"),this);
    m_actSetting=new QAction(EIcon::fromTheme("configure").pixmap(24),trUtf8("Desktop settings"),this);
    m_actQmlWidget=new QAction(EIcon::fromTheme("configure").pixmap(24),trUtf8("Add/Remove Widget"),this);
    m_actShudown=new QAction(EIcon::fromTheme("system-shutdown",("Shutdown")).pixmap(32),
                             trUtf8("Exit"),this);

    m_actRun=new QAction(EIcon::fromTheme("system-run",("gnome-run")).pixmap(32),
                         trUtf8("Run"),this);

    m_actOpenDesktopFolder=new QAction(EIcon::fromTheme("folder-open","document-open-folder"),trUtf8("Open Desktop folder"),this);

    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    loadIconThemes();

    QMenu *menu=new QMenu;

    menu->addAction(m_actNextWall);
    menu->addAction(m_actOpenDesktopFolder);
    menu->addSeparator();
    menu->addAction(m_actRun);
    menu->addAction(m_actShudown);
    menu->addAction(m_actQmlWidget);
    menu->addAction(m_actSetting);

    m_sceen=new Scene(this);
    m_sceen->setMenuStandard(menu);
    setScene(m_sceen);

    connect(m_actNextWall,SIGNAL(triggered()),m_sceen,SLOT(nextWallpaper()));
    connect(m_sceen,SIGNAL(isFolderImages(bool)),m_actNextWall,SLOT(setVisible(bool)));
    connect(m_actSetting,SIGNAL(triggered()),this,SLOT(showSettingDialog()));
    connect(m_actQmlWidget,SIGNAL(triggered()),this,SLOT(showAddWidgetDialog()));
    connect(m_actOpenDesktopFolder,SIGNAL(triggered()),this,SLOT(openDesktopFolder()));
    connect(m_actShudown,SIGNAL(triggered()),this,SLOT(showDialogPower()));
    connect(m_actRun,SIGNAL(triggered()),this,SLOT(showDialogRun()));

    // m_actShudown->setVisible();
    desktopObjectWidget= new WidgetDesktop(m_sceen);
    //m_sceen->addItem(desktopObjectWidget);
    setDragMode(QGraphicsView::RubberBandDrag);

    setViewportUpdateMode(QGraphicsView::SmartViewportUpdate);

    setCacheMode(QGraphicsView::CacheBackground);
    //  setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this,SIGNAL(customContextMenuRequested(QPoint)) ,
            this,SLOT(showContextMenu(QPoint)));
    //qDebug() << "etap:3";

    connect(QApplication::desktop(), SIGNAL(workAreaResized(int)),
            this, SLOT(resizeDesktop(int)));
    //  qDebug() << "etap:5";

    //*****shortcuts********************

    connect(m_settings,SIGNAL(iconThemeChanged()) ,this,SLOT(loadIconThemes()));
    connect(m_settings,SIGNAL(iconThemeChanged()) ,m_sceen,SLOT(updatIcons()));
    connect(m_settings,SIGNAL(ThemesChanged()) ,m_sceen,SLOT(loadSettings()));
    connect(m_settings,SIGNAL(shortcutsKeysChanged()) ,this,SLOT(updatShortcut()));
    connect(m_settings,SIGNAL(desktopThemeChanged()) ,this,SLOT(loadSettings()));

    updatShortcut();






    //    qDebug() << "etap:7";
    resizeDesktop(-1);

    //*********************
    //  QTimer::singleShot(1000,this,SLOT(loadSettings()));
    loadSettings();

    gen =    new MenuGen(this->locale().name().section("_",0,0));

    delete gen;

}

/**
 * @brief
 * @note
 * @param
 * @param
 * @return
 */
void DesktopSceen::updatShortcut()
{



    QSettings sArray("elokab","shortcut") ;
    //! begin the settings shortcut for Desktop

    sArray.beginGroup("Desktop");
    //----------------------------------------
    sArray.beginGroup("NextWallpaper");
    if(!sArray.contains("Comment")){
        sArray.setValue("Key","Ctrl+Shift+N");
        sArray.setValue("Comment","Next desktop Wallpaper");
    }
    QKeySequence ks( sArray.value("Key","Ctrl+Shift+N").toString());
    m_actNextWall->setShortcut(ks);
    sArray.endGroup();//NextWallpaper
    //----------------------------------------
    sArray.beginGroup("Configure");
    if(!sArray.contains("Comment")){
        sArray.setValue("Key","Ctrl+Shift+D");
        sArray.setValue("Comment","Configure desktop");
    }
    QKeySequence ksP( sArray.value("Key","Ctrl+Shift+D").toString());
    m_actSetting->setShortcut(ksP);
    sArray.endGroup();//Configure
    //----------------------------------------
    sArray.beginGroup("OpenDesktopFolder");
    if(!sArray.contains("Comment")){
        sArray.setValue("Key",QKeySequence(Qt::Key_HomePage).toString());
        sArray.setValue("Comment","Open curent desktop folder");
    }
    QKeySequence ksO( sArray.value("Key",QKeySequence(Qt::Key_HomePage)).toString());
    m_actOpenDesktopFolder->setShortcut(ksO);
    sArray.endGroup();//OpenDesktopFolder
    //----------------------------------------
    sArray.endGroup();//Desktop

    addAction(m_actNextWall);
    addAction(m_actSetting);
    addAction(m_actOpenDesktopFolder);


    //*****shortcuts********************
}

void DesktopSceen::loadIconThemes()
{

    //  m_settings->loadIconThems();
    //        QString icnThem=m_settings->loadIconThems();
    //        QIcon::setThemeName(icnThem);
    m_actNextWall->setIcon(EIcon::fromTheme("user-desktop","folder-desktop").pixmap(24));
    m_actSetting->setIcon(EIcon::fromTheme("configure",("document-properties")).pixmap(24));
    m_actQmlWidget->setIcon(EIcon::fromTheme("configure",("document-properties")).pixmap(24));
    m_actShudown->setIcon(EIcon::fromTheme("system-shutdown",("Shutdown")).pixmap(24));

    m_actRun->setIcon(EIcon::fromTheme("system-run",("gnome-run")).pixmap(24));


    m_actOpenDesktopFolder->setIcon(EIcon::fromTheme("folder-open","document-open-folder").pixmap(24));
    if (iconsView)
    {
        //  qDebug()<<"loadIconsList----------------------------";
        iconsView->updatIcons();
    }

}

/****************************************
 *جلب الاعدادات في حالة تغيرها*************
 ***************************************/
void DesktopSceen::loadSettings()
{
    QSettings setting("elokab","settings");
    setting.beginGroup("Desktop-Wallpaper");
    bool  iconView=setting.value("IconView",false).toBool();
    setting.endGroup();
    // actIconView->setChecked(m_iconView);


    if(iconView!=m_iconView){

        //  qDebug()<<iconView << " : "<<m_iconView;
        m_iconView=iconView;
        showHideFileView(m_iconView);


    }
    if (iconsView)
    {
        iconsView->loadSettings();
    }
    m_sceen->loadSettings();
    //desktopObjectWidget->loadsettings();
}
/*************************************************
*تحجيم سطح المكتب في حالة تغيره*******************
****************************************************/
void DesktopSceen:: resizeDesktop(int /*screen*/)
{
    QRect rect(QApplication::desktop()->geometry());
    setGeometry(rect);
    m_sceen->setSceneRect(rect);
    m_sceen->setBackground();
    if (iconsView)
    {
        //  iconsView->sortBy();
        // fileView->setGeometry(QApplication::desktop()->availableGeometry());
    }

}
/********************************************
 *القائمة المنسدلة الرئيسية******************
 *********************************************/
/**
 * @brief القائمة المنسدلة الرئيسية
 */
void DesktopSceen::showContextMenu(QPoint)
{

    QMenu menu;

    menu.addSeparator();
    menu.addAction(m_actNextWall);

    menu.addAction(m_actOpenDesktopFolder);
    menu.addSeparator();
    menu.addAction(m_actQmlWidget);
    menu.addAction(m_actSetting);
    menu.exec(QCursor::pos());
}
/*******************************************
 *اظهار او اخفاء مشهد الايقونات*****************
 **********************************************/
/**
 * @brief اظهار او اخفاء مشهد الايقونات
 * @param showView
 */
void DesktopSceen::showHideFileView(bool showView)
{
    QMenu menu;


    if(showView==true){
        if (iconsView)
        {
            iconsView->deletAll();
            delete iconsView;
            iconsView = 0;
        }
        iconsView=new IconsView(QPoint(0,0),m_sceen,this->locale().name().section("_",0,0));
        m_sceen->addItem(iconsView);



        menu.addMenu(iconsView->menuTemplates) ;
        menu.addMenu(iconsView->menuRealigne);
        m_sceen->setMenuIcon(iconsView->menuIconFile);
        if(!m_actReload)
            m_actReload=new QAction(tr("Refresh"),this);
        // actReload->setShortcut(QKeySequence("F5"));
        connect(m_actReload,SIGNAL(triggered()),iconsView,SLOT(updateIconList()));
        menu.addAction(m_actReload);
        m_actReload->setShortcut(QKeySequence("F5"));
        addAction(m_actReload);
        m_sceen->setMenu(&menu);


    }else{
        if (iconsView)
        {
            iconsView->deletAll();
            delete iconsView;
            delete m_actReload;
            iconsView = 0;
            m_actReload=0;
            m_sceen->setMenu(&menu);


        }


    }

}
/***************************************
 *نافذة الاعدادات************************
 ***************************************/
/**
 * @brief فتح نافذة الاعدادات
 * @param settingChanged امنظار اشارة تغير الاخدادات
 */
void DesktopSceen::showSettingDialog()
{
    //  mKeyNextWallp->setactivated();

    //        DialogConfigDesktop *dlg=new DialogConfigDesktop;

    //        connect(dlg,SIGNAL(settingChanged()),this,SLOT(loadSettings()));
    //connect(dlg,SIGNAL(settingChanged()),m_sceen,SLOT(setBackground()));
    //        DialogConfigDesktop *dlg = this->findChild<DialogConfigDesktop*>("DialogConfigDesktop");

    //        if (!dlg)
    //        {
    //                dlg = new DialogConfigDesktop(this);
    //                  connect(dlg, SIGNAL(settingChanged()), this, SLOT(loadSettings()));
    //        }
    //dlg->hide();

    //DialogConfigDesktop *dlg = new DialogConfigDesktop;
    //  connect(dlg, SIGNAL(settingChanged()), this, SLOT(loadSettings()));
    //        dlg->show();
    //        dlg->raise();
    //        dlg->activateWindow();
 QString fileName=Edir::progPath("elokab-settings");
    QProcess prosess;
 //   QString p="elokab-settings desktopsetting";
    prosess.startDetached(fileName,QStringList()<<"desktopsetting");
}
/**
 * @brief فتح نافذة اضافة ودجات
 * @param settingChanged امنظار اشارة تغير الاخدادات
 */
void DesktopSceen::showAddWidgetDialog()
{
    //        DialogConfigWidgets *dlg = this->findChild<DialogConfigWidgets*>("DialogConfigWidgets");

    //        if (!dlg)
    //        {
    //                dlg = new DialogConfigWidgets(this);
    //                  connect(dlg, SIGNAL(settingChanged()), desktopObjectWidget, SLOT(loadsettings()));
    //        }
    // dlg->hide();

    DialogConfigWidgets  *dlg = new DialogConfigWidgets;
    connect(dlg, SIGNAL(settingChanged()), desktopObjectWidget, SLOT(loadsettings()));
    dlg->show();
    dlg->raise();
    dlg->activateWindow();


}

/****************************************************************
 *فتح مجلد سطح المكتب بواسطة مدسر الملفات************************
 ***************************************************************/
/**
 * @brief فتح مجلد سطح المكتب
 * @param defaultBrowser النظام
 * @param  costumBrowser المدخلة بواسطة المستخدم
 */
void DesktopSceen::openDesktopFolder()
{

    QSettings settingL("elokab","settings");

    settingL.beginGroup("Desktop-Wallpaper");

    QString localDir=settingL.value("DirLocation",Edir::desktopDir()).toString();
    settingL.endGroup();

    QSettings setting("elokab","elokabsettings");
    setting.beginGroup("DefaultBrowser");

    bool defaultBrowser=(setting.value("defaultBrowser",false).toBool());




    QDir dir(localDir);

    if(!dir.exists(localDir))
        localDir= Edir::desktopDir();

    if(defaultBrowser==true)     {
        //تشغيل مجلدات المستخدم كمجلد المستندات والصور وغيرها


        QDesktopServices::openUrl( QUrl::fromLocalFile(localDir) );

    }else{



        QString costumBrowser=(setting.value("BROWSER","elokab-fm").toString());
        QString prog=costumBrowser.section(" ",0,0);
        QStringList list;
        if(!costumBrowser.section(" ",1,1).trimmed().isEmpty())
            list<<costumBrowser.section(" ",1,1).trimmed();

        list<<localDir;
        QProcess process;
        process.startDetached(prog,list);



    }
    setting.endGroup();

    //iconsView->deletAll();
}
void DesktopSceen::showDialogPower()
{
    QProcess::startDetached("elokab-syspower");
}
void DesktopSceen::showDialogRun()
{
    QDBusInterface dbus("org.elokab.run",
                        "/org/elokab/run",
                        "org.elokab.run.Interface");

    if (!dbus.isValid()) {
        qDebug() << "QDBusInterface is not valid!";
        return ;
    }
    dbus.call("run");

}
