/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "iconsview.h"
#include "icondesktop.h"
#include "filesutils/propertiesdlg.h"
#include "filesutils/openwithdlg.h"

#include <QtGui/QPainter>
#include <QtGui/QPalette>
#include <QtGui/QApplication>
#include <QtGui/QIcon>
#include <QFileInfo>
#include <QtGui/QFileIconProvider>
#include <QtGui/QDesktopServices>
#include <QUrl>
#include <QtGui/QGraphicsSceneMouseEvent>
#include <QtGui/QGraphicsDropShadowEffect>
#include <QtDebug>
#include <QProcess>
#include <QMenu>
#include <QSettings>
#include<QDesktopWidget>
#include<QInputDialog>
#include<QDrag>
#include<QMimeData>
#define SINGLECLICK   0
#define DOUBLECLICK   1

//! file=مسار الملف
//! with=طول وعرض الايقونة
//! lc=لغة التطبيق

IconDesktop::IconDesktop(const QString & file, const int &width, const QString &lc, const int &mode, QGraphicsItem * parent)
     : QGraphicsObject(parent),  m_file(file),m_iconWidth(width),
       m_highlight(false),m_execAppMode(mode)

{
     // not yet setFlag(QGraphicsItem::ItemIsMovable);
     setFlag(QGraphicsItem::ItemIsMovable);
     setFlag(QGraphicsItem::ItemIsSelectable);
     setFlag(QGraphicsItem::ItemIsFocusable);
     setFlag(QGraphicsItem::ItemSendsScenePositionChanges);
     setAcceptDrops(true);
     setAcceptTouchEvents(true);
     setAcceptHoverEvents(true);
     if(m_execAppMode==SINGLECLICK){
          setCursor(Qt::PointingHandCursor);
     }else{
          setCursor(Qt::ArrowCursor);
     }


 setZValue(1);

     setData(0,file);
     setDataFile(file,lc);
}
//! SINGLECLICK=0  DOUBLECLICK=1
void IconDesktop::setExecAppMode(int mode){
     m_execAppMode=mode;
     if(m_execAppMode==SINGLECLICK){
          setCursor(Qt::PointingHandCursor);
     }else{
          setCursor(Qt::ArrowCursor);
     }
}
void IconDesktop::setIconWidth(int width)
{
     m_iconWidth=width;
     if(QFile::exists(m_icon)){

          setIcon(QIcon(m_icon));
     }else{
          QIcon icon =EMimIcon::icon(QFileInfo(m_file));
          setIcon(icon);
     }
     //qDebug()<<"setIconWidth"<<m_icon;
}
void IconDesktop::setIcon(const QIcon & icon)
{
     int w=m_iconWidth ;
     m_pm = icon.pixmap(w, w, QIcon::Active);
     m_pm_hi = icon.pixmap(w, w, QIcon::Selected);
}

void IconDesktop::setText(const QString & text)
{
     m_text = text;
}

QRectF IconDesktop::boundingRect() const
{
     return QRectF(-5,-5, m_iconWidth+50, m_iconWidth+50);
}

void IconDesktop::paint(QPainter * painter, const QStyleOptionGraphicsItem * /*option*/, QWidget * /*widget*/)
{
     painter->setRenderHint(QPainter::SmoothPixmapTransform,true);
     int iconW=m_iconWidth ;
     QRect source(0, 0, m_pm.width(), m_pm.height());
     int w = (m_iconWidth+40)/2 ;
     //   int h = m_iconWidth+40 ;
     int iw = iconW / 2;
     // int ih = iconW / 2;
     //  QRect target(w - iw, h - ih - 10, 32, 32);
     QRect target(w - iw,6, iconW, iconW);

     painter->drawPixmap(target, m_highlight ? m_pm_hi : m_pm, source);

     QRectF textRect(0, iconW+6, m_iconWidth+40,40);
     // qDebug()<<textRect;
     QRectF hRect(0, 0, m_iconWidth+40, m_iconWidth+40);
     // text now - it has to follow potential QSS
     //  QColor borderColor = QColor(255,255,255);


     QColor bgColor = qApp->palette().color(QPalette::Highlight);

     // painter->setPen(QPen(bgColor, 0.3, Qt::SolidLine, Qt::RoundCap));
     painter->setPen(bgColor);
     QColor colortxt2=QColor(255,255,255);

     bgColor.setAlpha(m_highlight ? 80 : 40);

     painter->setBrush(QBrush(bgColor, Qt::SolidPattern));





     if(isSelected() )
          painter->drawRoundedRect( hRect, 6, 6);
     QString txt=m_text;
     //    if(txt.length()>17)
     //            txt=txt.left(17)+"...";
     //    if(txt.count(" ")>1)
     //    txt=txt.section(" ",0,1)+"...";

     textRect.adjust(2, 2, -2, -2);

     painter->setPen(QPen(QColor(0,0,0)));
     painter->drawText(textRect.translated(1, 1),
                       Qt::AlignCenter| Qt::TextWordWrap| Qt::TextIncludeTrailingSpaces,
                       txt);

     //  colortxt2.setAlpha(175);

     painter->setPen(QPen(colortxt2));
     painter->drawText(textRect,
                       Qt::AlignCenter | Qt::TextWordWrap | Qt::TextIncludeTrailingSpaces,
                       txt);

}

//! عتد تغير موضع الفأرة
QVariant IconDesktop::itemChange(GraphicsItemChange change, const QVariant &value)
{

     if (change == ItemPositionHasChanged && scene()) {
          // value is the new position.
          QPointF newPos = pos();
          //   qDebug()<<"icon pos change -------------------"<<newPos;
          QRect rectParent=QApplication::desktop()->availableGeometry();
          if(pos().x()+m_iconWidth>rectParent.width())
               setPos(rectParent.width()-m_iconWidth,newPos.y())   ;
          if(pos().y()+m_iconWidth>rectParent.height())
               setPos(newPos.x(),rectParent.height()-m_iconWidth)   ;
          if(pos().y()<rectParent.top())
               setPos(newPos.x(),rectParent.top())   ;
          if(pos().x()<rectParent.left())
               setPos(rectParent.left(),newPos.y())   ;

     }

     return QGraphicsItem::itemChange(change, value);
}
//! عتد ارخاء مؤشر الفأرة
void IconDesktop::mouseReleaseEvent(QGraphicsSceneMouseEvent * event)
{
     QGraphicsItem::mouseReleaseEvent(event);
     setZValue(1);
     if(pos()!=m_oldPosition)
          saveIttemPos();
     if (m_execAppMode == DOUBLECLICK)
          return;


     if ((pos() != m_oldPosition))
          return;

     EMimIcon::launchApp(m_data,EMimIcon::mimeTyppe(m_data));
     //    launchApp();



}


//! عتد اضغط على مؤشر الفأرة
void IconDesktop::mousePressEvent(QGraphicsSceneMouseEvent * /*event*/)
{
     setActive(true);
     setZValue(2);
     setSelected(true);
     m_oldPosition = pos();


}
void IconDesktop:: mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{

     QGraphicsItem::mouseMoveEvent(event);
/*
   Qt::KeyboardModifiers mods = QApplication::keyboardModifiers();
        if (mods != Qt::ControlModifier)
            return;

//        if (QLineF(event->screenPos(), event->buttonDownScreenPos(Qt::LeftButton))
//                 .length()< QApplication::startDragDistance()) {
//                 return;
//             }


        QDrag *drag = new QDrag(event->widget());
        QMimeData *mimeData = new QMimeData();
        QList<QUrl> list;
        list.append(QUrl(m_file));
        mimeData->setUrls(list);
         drag->setMimeData(mimeData);
drag->setPixmap(this->m_pm);
                    drag->exec(Qt::MoveAction);
*/
}

//! عتد اخراج مؤشر الفأرة
void IconDesktop::hoverLeaveEvent(QGraphicsSceneHoverEvent * event)
{

     m_highlight = false;

     QGraphicsItem::hoverLeaveEvent(event);
}
//! عتد مرور مؤشر الفأرة
void IconDesktop::hoverEnterEvent(QGraphicsSceneHoverEvent * event)
{

     m_highlight = true;



     QGraphicsItem::hoverEnterEvent(event);
}
//! النقر المزدوج
void IconDesktop::mouseDoubleClickEvent(QGraphicsSceneMouseEvent * event)
{

     if (m_execAppMode == SINGLECLICK)
          return;

     if (m_execAppMode == DOUBLECLICK && event->button() == Qt::LeftButton)
     {
          EMimIcon::launchApp(m_data,EMimIcon::mimeTyppe(m_data));
          //  launchApp();
     }
}
//! تحديث بيانات الايقونة
void IconDesktop::setDataFile(QString fileName, const QString &lc)
{

     QString lC=lc;

     //   QFileInfo fi(fileName);
     QFileInfo fi(fileName);
     if(fi.isExecutable())
          m_type =  ActionExec;
     else
          m_type =  ActionFile;

     setText(fi.fileName());
     m_data=fileName;

     m_icon=EMimIcon::icon(fi).name();
     setIconWidth(m_iconWidth);




}
void IconDesktop::updatIcon()
{
     if(QFile::exists(m_icon)){
          //  qDebug()<<"MimeTypeXdg::getIcon"<<m_icon;
          setIcon(QIcon(m_icon));
     }else{
          QIcon icon =EMimIcon::icon(QFileInfo(m_file));
          setIcon(icon);
     }
     // setIcon(MimeTypeXdg::getIcon(m_file,m_icon));
}

//! launch the app or file with asissiated app
void IconDesktop::launchApp()
{

     EMimIcon::launchApplication(m_data);

}
//! save the pos if position changed
void IconDesktop::saveIttemPos()
{
     QSettings settings("elokab","desktopApps");
     //    settings.setIniCodec(QTextCodec::codecForName("UTF-8"));

     settings.beginGroup("DesktopApps");

     QFileInfo fi(m_file);

     settings.beginGroup(fi.fileName());
     settings.setValue("file", m_file);
     settings.setValue("pos", pos().toPoint());

     settings.endGroup();


     settings.endGroup();

}
//! actions events
void IconDesktop::getPropertie()
{
     PropertiesDlg *dlg=new PropertiesDlg(QStringList()<<m_file,0);
     dlg->show();
}

void IconDesktop::getOpenWith()
{

    //if(!openWithDlg)
    OpenWithDlg *openWithDlg =new OpenWithDlg(0);

    if(openWithDlg->exec()==QDialog::Accepted){


         QString proc=openWithDlg->execName();

         if(!proc.isEmpty()){
              EMimIcon::launchAppFile(m_file,proc);

         }

    }
   // openWithDlg=0;
    delete openWithDlg;

}

void IconDesktop::renameFile()
{
     bool ok;
     QString newFile;
     QFileInfo fi(m_file);
     QString text = QInputDialog::getText(0, QObject::tr("File Rename"),
                                          QObject::tr("Enter New name:"), QLineEdit::Normal,
                                          fi.fileName(), &ok);
     if (ok && !text.isEmpty()){
          newFile=(text);

          QString path=fi.dir().absolutePath();

          QProcess process;
          process.startDetached("mv  \""+m_file+"\" \""+path+"/"+newFile+"\"" );
     }

}
