/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "iconsview.h"
#include "iconview/icondesktop.h"
#include "filesutils/dialogxdesktop.h"
#include "filesutils/symlinkdlg.h"

#include <QUrl>
#include <QFileInfo>
#include <QProcess>
#include <QMessageBox>
#include <QRectF>
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsScene>
#include <QDebug>
#include <EDir>
#include <QSettings>
#include<QApplication>
#include<QDesktopWidget>
#include<QPropertyAnimation>
#include<QParallelAnimationGroup>
#include<QSequentialAnimationGroup>
#include<QMenu>
#include<QGraphicsWidget>
#include<QTextCodec>
#include<QFileDialog>
#include<QInputDialog>
#include<QMessageBox>
#include<QDateTime>
#include <QtCore>
#include <QGraphicsProxyWidget>
#include <QDeclarativeView>

#define BY_NAME   1
#define BY_TYPE   2
#define BY_SIZE   3
#define SINGLECLICK   0
#define DOUBLECLICK   1
IconsView::IconsView(const QPointF &pos,QGraphicsScene *scene,const QString & lc, QGraphicsItem *parent)
        : QGraphicsObject(parent),m_fileSystemWatcher(0),m_lC(lc)
{
        setPos(pos);
        m_scene=scene;
        m_sort=1;
        m_iconWidth=80;
        m_alignHorizontale=false;
        creatMenuTemplate();
        creatMenuFile();
        this->setAcceptDrops(true);
        loadSettings();
        // setDirFilesName(QDesktopServices::storageLocation(QDesktopServices::DesktopLocation));
setZValue(1);
}

IconsView::~IconsView()
{
}
void IconsView::paint(QPainter */*painter*/, const QStyleOptionGraphicsItem *, QWidget *)
{
        //    painter->setPen(Qt::white);
        //    painter->drawEllipse(QPointF(0.0, 0.0), 60.0, 60.0);
}
QRectF IconsView::boundingRect() const
{
        QRectF rectParent=QApplication::desktop()->availableGeometry();
        return rectParent;
}
//! ***********************************************************************
void IconsView::dragEnterEvent(QGraphicsSceneDragDropEvent *e)
{
        //   qDebug() << "IconScene::dragEnterEvent" << e->mimeData()->hasUrls();
        // Getting URL from mainmenu...
        if (e->mimeData()->hasUrls())
                e->acceptProposedAction();

}

void IconsView::dragMoveEvent(QGraphicsSceneDragDropEvent */*e*/)
{

    //   dragEnterEvent(e);
}

void IconsView::dropEvent(QGraphicsSceneDragDropEvent *e)
{
        //  qDebug() << "IconScene::dropEvent" << e->mimeData()->urls();
        const QMimeData *mime = e->mimeData();
        // urls from mainmenu

        QList<QUrl> files = mime->urls();
        // Don't do anything if drag and drop in same folder
        if (QFileInfo(files.at(0).path()).canonicalPath() == m_curentdirLocation) {
                qDebug() << "this prarent ::dropEvent" << files.at(0).path();
                return ;
        }

        foreach (QUrl url, files)
        {
                QFileInfo fi(url.toLocalFile());
                QFile f(url.toLocalFile());

                if (!f.copy(m_curentdirLocation + "/" + fi.fileName()))
                {
                        QMessageBox::information(0, tr("Copy File Error"),
                                                 tr("Cannot copy file %1 to %2").arg(url.toLocalFile()).arg(m_curentdirLocation));
                }
        }
}

//! ***************************************************************************************
void IconsView:: deletAll()
{
        foreach (IconDesktop *icn, listItemesFiles) {
                delete icn;
        }
        listItemesFiles.clear();

}
void IconsView::loadSettings()
{

        QSettings setting("elokab","settings");
        setting.beginGroup("Desktop-Wallpaper");

        int  iconSize=(setting.value("iconSize",2).toInt());
        m_alignHorizontale=setting.value("AlignHorizontale",false).toBool();
        m_execMode=setting.value("ExecMode",DOUBLECLICK).toInt();
        QString dirN= setting.value("DirLocation"
                                    ,Edir::desktopDir()).toString();

        setting.endGroup();

        m_iconWidth=(iconSize+40);
        setDirFilesName(dirN);

        foreach (IconDesktop *Ib, listItemesFiles){

                Ib->setExecAppMode(m_execMode);
                Ib-> setIconWidth(iconSize);
        }

}

void IconsView::updatIcons(){
        foreach (IconDesktop *Ib, listItemesFiles){

                Ib-> updatIcon();

        }

}

void IconsView:: setDirFilesName(QString dir)
{

        if (!dir.isEmpty() && m_curentdirLocation == dir)
                return;

        if (m_fileSystemWatcher)
        {
                delete m_fileSystemWatcher;
                m_fileSystemWatcher = 0;
        }



        if (!dir.isEmpty() && QDir(dir).exists())
        {
                //   qDebug() << "Setting dir from user" << dir;
                m_curentdirLocation = dir;
        }
        else if (QDir(Edir::desktopDir()).exists())
        {
                //  qDebug() << "Setting dir" << "DesktopLocation";
                m_curentdirLocation =Edir::desktopDir();
        }
        else if (QDir(QDir::homePath()).exists())
        {
                //   qDebug() << "Setting dir" << "homePath";
                m_curentdirLocation = QDir::homePath();
        }
        else
        {
                //   qDebug() << "Setting dir" << "filesystem root";
                m_curentdirLocation = QDir::rootPath();
        }



        m_fileSystemWatcher = new QFileSystemWatcher(QStringList() << m_curentdirLocation, this);

        connect(m_fileSystemWatcher, SIGNAL(directoryChanged(const QString&)), this, SLOT(updateIconList(const QString&)));
        //    connect(m_fileSystemWatcher, SIGNAL(directoryChanged(QString)), this, SLOT(updateIconFile(QString)));
        QTimer::singleShot(1000,this,SLOT(loadIconsList()));
     //   loadIconsList();

        //    QtConcurrent::run(this, &IconsView::loadIconsList);
}
void IconsView::updateIconList()
{
        updateIconList(QString());
}

void IconsView::updateIconList(const QString&)
{
        if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(true);
        //QDirIterator dirIter(m_fsw->directories().at(0));
        QDir d(m_curentdirLocation);

        QStringList tempList;
        foreach (QFileInfo dirIter, d.entryInfoList(QDir::NoDotAndDotDot|
                                                    QDir::AllEntries|QDir::System, QDir::DirsFirst|
                                                    QDir::Name|QDir::IgnoreCase ))
        {
                QString df(dirIter.filePath());
                tempList.append(df);
        }
        int count=tempList.count();//عدد الملفات في القائمة المؤقتة

        int countData=listItemesFiles.keys().count();//عدد الملفات في القائمة الموجودة
        if(count>0&&countData==0){
                loadIconsList();
                if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(false);

                return;
        }
        if(count>0&&countData>0){

                if (count==countData){// هذا كان العدد متساوي يتم المقارنة بين الملفات ملفا ملفا
                        QString changedFileSource;
                        QString changedFileDest;

                        foreach(QString fileN,tempList){

                                if(! listItemesFiles.contains(fileN)){
                                        changedFileSource=fileN;
                                        //  qDebug()<<"changedFileS : "<<changedFileSource;
                                }

                        }
                        foreach(QString fileN,listItemesFiles.keys()){

                                if(!tempList.contains(fileN)){
                                        changedFileDest=fileN;
                                        // qDebug()<<"changedFileD : "<<changedFileDest;
                                }
                        }
                        if(changedFileSource.isEmpty()){
                                if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(false);
                                return;
                        }
                        IconDesktop *b=  listItemesFiles[changedFileDest];
                        // b->setData(0,changedFileSource);
                        //b->setText(QFileInfo(changedFileSource).fileName());
                        //! delete changed file from sceen and list
                        QPointF pos=b->pos();
                        listItemesFiles.take(changedFileDest);
                        delete   b;
                        //! insert changed file
                        IconDesktop *tb = new IconDesktop(changedFileSource,m_iconWidth-40,m_lC,m_execMode);
                        m_scene-> addItem(tb);
                        listItemesFiles.insert(changedFileSource,tb);
                        tb->setPos(pos);

                        //                        QHash<QString, IconDesktop*>::iterator i;
                        //                        for (i = listItemesFiles.begin(); i != listItemesFiles.end(); ++i)
                        //                                if(i.key()==changedFileDest){
                        //                                        i = listItemesFiles.erase(i);
                        //                                        i=listItemesFiles.insert(changedFileSource,b);
                        //                                }


                        if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(false);
                        return;


                }else if(count<countData){//حالة نقصان الملفات

                        foreach(QString vl,listItemesFiles.keys()){

                                if(!tempList.contains(vl)){

                                        delete   listItemesFiles[vl];
                                        listItemesFiles.take(vl);
                                        if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(false);
                                        return;
                                }

                        }



                }else if(count>countData){//حالة زيادة الملفات
                        for(int i=0;i<count;i++){
                                foreach(QString vl,tempList){

                                        if(!listItemesFiles.contains(vl)){

                                                //      qDebug()<<"listToolB no exist **************: "<<vl;

                                                IconDesktop *tb = new IconDesktop(vl,m_iconWidth-40,m_lC,m_execMode);
                                                m_scene-> addItem(tb);
                                                listItemesFiles.insert(vl,tb);
                                                tb->setPos(QCursor::pos());
                                                animeOpacity(tb);
                                                if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(false);
                                                return;

                                        }


                                }

                        }

                }

        }
        if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(false);
}
/**************************************************************************
 *تحميل ملفات سطح المكتب ومقارنتها مع الاعدادات السابقة للمستخدم************
 **************************************************************************/
void IconsView::loadIconsList()
{
        if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(true);



        //---------delet all icons-----------------

        foreach (IconDesktop *t, listItemesFiles){
                delete t;

        }
        listItemesFiles.clear();
        //*****************************************

        //--------------load positions of icons if exists---------------------
        QHash<QString, QPoint> listDesktop ;
        QString file;
        QPoint point=QPoint(0,0);
        QSettings setting("elokab","desktopApps") ;
        setting.beginGroup("DesktopApps");
        foreach(const QString & group , setting.childGroups())
        {
                //   qDebug()<<"group :"<<group;
                setting.beginGroup(group);
                file = setting.value("file", "").toString();;
                point = setting.value("pos", QPoint(0,0)).toPoint();

                // qDebug()<<"point :"<<point;
                listDesktop[file]=point;
                setting.endGroup();
        }

        setting.endGroup();
        //*****************************************

        //--------initialise x y pos-------------------------------------
        QRect rectParent=QApplication::desktop()->availableGeometry();
        int x =rectParent.left()+ 20;
        int y = rectParent.top()+ 20;
        if(QApplication::isRightToLeft()){
                x=  (rectParent.width()-m_iconWidth)-20;
        }
        //*****************************************


        //! reserved for annimation
        // QHash< IconDesktop*,QPoint> listItemesPoints;
        //---------------------------load the files of dir--------------------------------
        QDir d(m_curentdirLocation);
        foreach (QFileInfo dirIter, d.entryInfoList(QDir::NoDotAndDotDot|QDir::AllEntries, QDir::DirsFirst|QDir::Name|QDir::IgnoreCase))
        {
                QString df(dirIter.filePath());
                IconDesktop *iDs = new IconDesktop(df,m_iconWidth-40,m_lC,m_execMode);

                if (!iDs)
                        continue;

                m_scene-> addItem(iDs);
                listItemesFiles.insert(df,iDs);
iDs->setPos(x,y);


                //*****load the point x y from temp list dektop if exists*****************
                if(listDesktop.contains(df)){

                        x=listDesktop.value(df).x();
                        y=listDesktop.value(df).y();

                }else{ //**********if the point no exist

                        if(y>=rectParent.bottom()-m_iconWidth){
                                y= rectParent.top()+ 20;;

                                if(QApplication::isRightToLeft())
                                        x= x-m_iconWidth-10;
                                else                //leftToRight
                                        x=x+m_iconWidth+10;

                        }
                }


                iDs->setPos(x,y);
                //------the next pos for next icon-----
                if(m_alignHorizontale){
                        if(QApplication::isRightToLeft()){
                                x=x-m_iconWidth-10;

                                if(x<rectParent.left()){
                                        x=  (rectParent.width()-m_iconWidth)-20;
                                        y=y+m_iconWidth+10;
                                }

                        }else{              //leftToRight
                                x=x+m_iconWidth+10;

                                if(x+m_iconWidth>rectParent.width()){
                                        x =rectParent.left()+ 20;
                                        y=y+m_iconWidth+10;
                                }
                        }
                }else{                  //alignVerticall
                        y=y+m_iconWidth+10;
                }




        }  //foreach
        //*****************************************
        listDesktop.clear();

        if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(false);

}



/****************************************************
 *حفظ مواقع الملفات الى اعدادات المستخدم
 *****************************************************/
void IconsView::saveDesktopIttemsPos()
{
        QSettings settings("elokab","desktopApps");
        //    settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
        settings.remove("DesktopApps");
        settings.beginGroup("DesktopApps");
      //  int i = 0;

        foreach (IconDesktop *Ib, listItemesFiles)
        {

                QFileInfo fi(Ib->data(0).toString());

                settings.beginGroup(fi.fileName());
                settings.setValue("file", Ib->data(0));
                settings.setValue("pos", Ib->pos().toPoint());


                settings.endGroup();
        }

        settings.endGroup();


}


/******************************************************
 *ترتيب الايقونات حسب الاسم او النوع او الحجم************
 ******************************************************/
void IconsView::realigneBy(int sort)
{
        if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(true);
        QDir d(m_curentdirLocation);

        QRect rectParent=QApplication::desktop()->availableGeometry();
        //اجداثيات س ع المبدئية
        int x =rectParent.left()+ 20;
        int y = rectParent.top()+ 20;
        if(QApplication::isRightToLeft()){
                x=  (rectParent.right()-m_iconWidth)-20;
        }


        switch (sort) {
        case BY_NAME:
                d.setSorting(QDir::DirsFirst|QDir::Name|QDir::IgnoreCase);
                break;
        case BY_TYPE:
                d.setSorting(QDir::DirsFirst|QDir::Type|QDir::IgnoreCase);
                break;
        case BY_SIZE:
                d.setSorting(QDir::DirsFirst|QDir::Size|QDir::IgnoreCase);
                break;
        default:
                break;
        }


        QHash< IconDesktop*,QPoint> listItemesPoints;

        foreach (QFileInfo dirIter, d.entryInfoList(QDir::NoDotAndDotDot|QDir::AllEntries |QDir::System))
        {

                QString df(dirIter.filePath());

                if  (listItemesFiles.contains(df)){
                        if(y>=rectParent.bottom()-m_iconWidth){
                                y=rectParent.top()+20;

                                if(QApplication::isRightToLeft()){
                                        x=    x-m_iconWidth-10;
                                }else{
                                        x=x+m_iconWidth+10;
                                }
                        }
                        IconDesktop *Ib=listItemesFiles.value(df);

                        if(QPointF(x,y)!=Ib->pos()){


                                listItemesPoints.insert(Ib,QPoint(x,y));
                        }
                        if(m_alignHorizontale){
                                if(QApplication::isRightToLeft()){
                                        x=    x-m_iconWidth-10;
                                        if(x<rectParent.left()){
                                                x=  (rectParent.right()-m_iconWidth)-20;
                                                y=y+m_iconWidth+10;
                                        }

                                }else{
                                        x=x+m_iconWidth+10;
                                        if(x+m_iconWidth>rectParent.right()){

                                                x =rectParent.left()+ 20;
                                                y=y+m_iconWidth+10;
                                        }
                                }
                        }else{
                                y=y+m_iconWidth+10;
                        }

                }
        }

        animeIcons(listItemesPoints);
        if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(false);
}
/******************************************************
 *********************Animation*************************
 ******************************************************/
void IconsView::animeOpacity(IconDesktop *Ib)
{

        Ib->setOpacity(qreal(0.0));

        QPropertyAnimation  *anim = new QPropertyAnimation(Ib, "opacity");


        anim->setEndValue(qreal(1.0));

        anim->setDuration(3000);

        anim->setEasingCurve(QEasingCurve::OutBack);

        anim->start();
}

void IconsView::animeIcons(QHash<IconDesktop *, QPoint> listItemesPoints )
{
        if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(true);
        //مجموعة الحركة المتوازية
        QParallelAnimationGroup *group = new QParallelAnimationGroup;
        //عملية حذف العناصر التي تحمل القوائم بعد الانتهاء من الحركة
         connect(group,SIGNAL(finished()),this,SLOT(saveIttemsPos()));
        QHashIterator<IconDesktop *,QPoint> i(listItemesPoints);
        while (i.hasNext()) {
                i.next();

                IconDesktop *Ib=i.key();
                QPoint p= i.value();


                QPropertyAnimation  *anim = new QPropertyAnimation(Ib, "pos");


                anim->setEndValue(QPointF(p));

                anim->setDuration(500);

                anim->setEasingCurve(QEasingCurve::OutBack);

                group->addAnimation(anim);

        }
        //بداية التنقل
        group->start();
}
void IconsView::saveIttemsPos()
{
        foreach (IconDesktop *Ib, listItemesFiles){

                Ib->saveIttemPos();

        }
}

//void IconsView::removeAllWid()
//{

//        foreach (IconDesktop *Ib,listGraphicsWidget)
//        {
//                //اخراج القائمة من العنصر الحاوي ثم حذف الحاوي
//                if(Ib){
//                         Ib->setParentItem(this);

//                          QGraphicsWidget *wid=    listGraphicsWidget.key(Ib) ;
//                      if(wid){
//                          Ib->setPos(wid->pos());
//                            delete wid ;
//                      }

//                }




//        }

//        listGraphicsWidget.clear();
//saveDesktopIttemsPos();
//   if (m_fileSystemWatcher) m_fileSystemWatcher->blockSignals(false);
//}
/******************************************************
 *********************template*************************
 ******************************************************/
void IconsView::creatMenuFile()
{
        menuIconFile=new QMenu(trUtf8("Create New"));

        QAction   *m_actOpen=new QAction(EIcon::fromTheme("document-open"),trUtf8("Open"),this);
        QAction   *m_actOpenWith=new QAction(EIcon::fromTheme("document-open"),trUtf8("Open With"),this);
        QAction *m_actRename=new QAction(EIcon::fromTheme("edit-rename"),trUtf8("Rename"),this);
        QAction  *m_actremove=new QAction(EIcon::fromTheme("edit-delete"),trUtf8("Remove"),this);
        QAction * m_actProprety=new QAction(EIcon::fromTheme("configure"),trUtf8("Properties"),this);


        menuIconFile->addAction(m_actOpen);
        menuIconFile->addAction(m_actOpenWith);
        menuIconFile->addAction(m_actRename);
        menuIconFile->addAction(m_actremove);

        menuIconFile->addAction(m_actProprety);

        connect(m_actOpen,SIGNAL(triggered()),this,SLOT(openSelectedIcons()));
        connect(m_actOpenWith,SIGNAL(triggered()),this,SLOT(openWithSelectedIcons()));
        connect(m_actremove,SIGNAL(triggered()),this,SLOT(removeSelectedIcons()));
        connect(m_actRename,SIGNAL(triggered()),this,SLOT(renameSelectedIcons()));
        connect(m_actProprety,SIGNAL(triggered()),this,SLOT(getSelectedIttemsProperty()));
}

void IconsView::creatMenuTemplate()
{
        menuTemplates=new QMenu(trUtf8("Create New"));
        menuTemplates->setIcon(EIcon::fromTheme("list-add",("add")));

        QAction *a;
        //***************************************************

        QStringList list;
        list<<Edir::dataHomeDir()+"/templates"
            <<Edir::dataDir()+"/templates"
            <<"/usr/share/templates";

        QStringList listTemp;
        QStringList listPath;
        foreach (QString s, list)
        {
            QDir d(s);
            foreach (QFileInfo fi, d.entryInfoList(QDir::NoDotAndDotDot|QDir::AllEntries ))
            {
                if(fi.fileName().startsWith("link"))
                    continue;

                if(!listTemp.contains(fi.fileName())){
                    listTemp<<fi.fileName();
                    listPath<<fi.filePath();
                }

            }
        }

        foreach (QString fileName, listPath) {

            QString line=getTemplateApp((fileName));
            QAction *act=new QAction(EIcon::fromTheme(line.section("|",1,1)),
                                     line.section("|",0,0),this);
            act->setData(line.section("|",2,2));
            connect(act,SIGNAL(triggered()),this,SLOT(execTemplate()));

            menuTemplates->addAction(act);
       //    qDebug()<<"templat================================"<<fileName<<line;

        }

        //***************************************************
        menuRealigne=new QMenu(trUtf8("Sort Icons By"));
        menuRealigne->setIcon(EIcon::fromTheme("preferences-desktop-icons",("view-list-icons")));


        a=menuRealigne->addAction(trUtf8("BY name"));
        a->setData(BY_NAME);
        connect(a,SIGNAL(triggered()),this,SLOT(sortBy()));


        a=menuRealigne->addAction(trUtf8("BY Type"));
        a->setData(BY_TYPE);
        connect(a,SIGNAL(triggered()),this,SLOT(sortBy()));

        a=menuRealigne->addAction(trUtf8("BY Size"));
        a->setData(BY_SIZE);
        connect(a,SIGNAL(triggered()),this,SLOT(sortBy()));

}
QString  IconsView::getTemplateApp(const QString &fileName)
{
    QFileInfo fi(fileName);
    QString baseName=fi.baseName();

    QSettings settings(fileName,QSettings::IniFormat);
    settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
    settings.beginGroup("Desktop Entry");
    QString nameold=settings.value("Name",baseName).toString();
    QString name=settings.value("Name["+m_lC+"]",nameold).toString();
    QString URL=settings.value("URL","").toString();
    QString icon=settings.value("Icon",baseName).toString();
    settings.endGroup();



    return name+"|"+icon+"|"+URL;
    /*
        QFileInfo fi(fileName);
        QString baseName=fi.baseName();
        //  QString m_lC="ar";//parentObject()->locale().name().section("_",0,0);
        QSettings settings(fileName,QSettings::IniFormat);
        settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
        settings.beginGroup("Desktop Entry");
        QString nameold=settings.value("Name",baseName).toString();
        QString name=settings.value("Name["+m_lC+"]",nameold).toString();
        QString URL=settings.value("URL","").toString();


        QString icon=settings.value("Icon",baseName).toString();
        settings.endGroup();



        return name+"|"+icon+"|"+URL;
        */
}
void IconsView::sortBy()
{
        QAction *action = qobject_cast<QAction *>(sender());
        if (action){
                int data=action->data().toInt();
                m_sort=data;
                realigneBy(data);
        }else{
                realigneBy(m_sort);
        }
}
//********************انشاء اختصار الى ملف
//void IconsView::creatLinkFile()
//{
//        QString fileName = QFileDialog::getOpenFileName(0, tr("Open File"),
//                                                        QDir::homePath(),
//                                                        tr("All file (*)"));
//        if(fileName.isEmpty())return;

//        creatLink( fileName);
//}
//********************انشاء اختصار الى مجلد
//void IconsView::creatLinkDir()
//{
//        QString fileName = QFileDialog::getExistingDirectory(0, tr("Open"),
//                                                             QDir::homePath());

//        if(fileName.isEmpty())return;

//     //   creatLink( fileName);
//}
void IconsView::creatLink()
{

    /**********************************************************************
     *                        *انشاء اختصار
     *********************************************************************/


         SymLinkDlg *dlg=new SymLinkDlg(m_curentdirLocation);
         dlg->exec();



    /*
    QFileInfo fi(fileName);
        bool ok;
        QString newName = QInputDialog::getText(0, tr("Link File"),
                                                tr("Enter name:"), QLineEdit::Normal,
                                                fi.fileName(), &ok);


        if (ok && !newName.isEmpty()){
                QFile file(m_curentdirLocation+"/"+newName);
                if(file.exists()){
                        creatLink( fileName);
                }else{
                        QFile::link(fileName,m_curentdirLocation+"/"+newName);

                }
        }
        */
}

void IconsView::creatAppxFile()
{
        DialogXdesktop *dlg=new DialogXdesktop(m_curentdirLocation);
        dlg->show();

}


void IconsView::execTemplate()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action)
    {
         QString data=action->data().toString();

         if(data.contains("emptydir"))
         {
              creatNewDir();
         }else if(data=="__CREATE_SYMLINK__"){
             creatLink();
         }else if(data=="__CREATE_XDGPROGRAME__"){
             creatAppxFile();
          }else{
              QString fileSource="/usr/share/templates/"+data;
              creatNewFile(fileSource);
         }

    }
    /*
    QAction *action = qobject_cast<QAction *>(sender());
        if (action){
                QString data=action->data().toString();

                if(data.contains("emptydir")){
                        creatNewDir();
                }else{
                        QString fileSource="/usr/share/templates/"+data;
                        creatNewFile(fileSource);
                }

        }
        */
}


//********************انشاء ملف فارغ
void IconsView::creatNewFile(const QString fileSource)
{

        //   QString rootPath=m_curentdirLocation;

        QFileInfo fiS(fileSource);
        bool ok;
        QString newName = QInputDialog::getText(0, tr("New File"),
                                                tr("Enter name:"), QLineEdit::Normal,
                                                fiS.baseName(), &ok);
        if (ok && !newName.isEmpty()){
                QFile file(m_curentdirLocation+"/"+newName+"."+fiS.suffix());
                if(file.exists()){
                        creatNewFile(fileSource);
                }else{
                        //   file.open(QIODevice::WriteOnly | QIODevice::Text);
                        QFile::copy(fileSource,m_curentdirLocation+"/"+newName+"."+fiS.suffix());
                }
                file.close();
        }

}
//********************انشاء مجلد فارغ
void IconsView::creatNewDir()
{


        QDir  path(m_curentdirLocation);

        bool ok;
        QString newName = QInputDialog::getText(0, tr("New Folder"),
                                                tr("Enter name:"), QLineEdit::Normal,
                                                trUtf8("New Folder"), &ok);

        if (ok && !newName.isEmpty()){
                QFile file(m_curentdirLocation+"/"+newName);
                if(file.exists()){
                        creatNewDir();
                }else{
                        path.mkdir(newName);

                }

        }
}

void IconsView::getSelectedIttemsProperty()
{

        foreach (QGraphicsItem *ittem, m_scene->selectedItems()) {
                QString file=ittem->data(0).toString();
                if(listItemesFiles.contains(file)){
                        IconDesktop  *Ib=   listItemesFiles.value(file);

                        if(Ib)
                                Ib->getPropertie();
                }
        }
}
void IconsView::openWithSelectedIcons()
{

        foreach (QGraphicsItem *ittem, m_scene->selectedItems()) {
                QString file=ittem->data(0).toString();
                if(listItemesFiles.contains(file)){
                        IconDesktop  *Ib=   listItemesFiles.value(file);

                        if(Ib)
                                Ib->getOpenWith();
                }
        }
}
void IconsView::renameSelectedIcons()
{
        foreach (QGraphicsItem *ittem, m_scene->selectedItems()) {
                QString file=ittem->data(0).toString();
                if(listItemesFiles.contains(file)){
                        IconDesktop  *Ib=   listItemesFiles.value(file);

                        if(Ib)
                                Ib->renameFile();
                }
        }
}

void IconsView::openSelectedIcons()
{
        foreach (QGraphicsItem *ittem, m_scene->selectedItems()) {
                QString file=ittem->data(0).toString();
                if(listItemesFiles.contains(file)){
                        IconDesktop  *Ib=   listItemesFiles.value(file);

                        if(Ib)
                                Ib->launchApp();
                }
        }
}

void IconsView::removeSelectedIcons()
{

        QString txt;
        QStringList list;

        foreach (QGraphicsItem *ib, m_scene->selectedItems()) {
                QFileInfo fi=ib->data(0).toString();
                txt=txt+fi.fileName()+"\n";
                list<<ib->data(0).toString();
        }
        QMessageBox msgBox;
        //     msgBox.setText("Delete .");
        msgBox.setText(trUtf8("delete files..."));
        msgBox.setInformativeText(trUtf8("Do you want to permanently delete all files? This action cannot be undone."));
        msgBox.setDetailedText(txt);
        msgBox.setIcon(QMessageBox::Warning);

        msgBox.setStandardButtons( QMessageBox::Ok | QMessageBox::Cancel);
        msgBox.setDefaultButton(QMessageBox::Cancel);
        int ret = msgBox.exec();

        if(ret==QMessageBox::Ok){

                QString prog=QApplication::applicationDirPath()+"/elokab-fa";
                QProcess p;
                p.startDetached(prog,QStringList()<<"-rm"<<list.join(","),QApplication::applicationDirPath());


        }

}
