/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "iconview/iconsview.h"
#include "iconview/icondesktop.h"
#include "scene.h"
#include "qmlanimation.h"

#include <QApplication>
#include<QDesktopWidget>
#include<QPropertyAnimation>
#include<QDir>
#include<QDebug>
#include<QSettings>
#include<QDesktopServices>
#include<QGraphicsProxyWidget>
#include<QTimer>
#include<QTime>
#include<QUrl>
#include<QMenu>
#include<QDeclarativeComponent>
#include<QDeclarativeEngine>

#define TYPE_COLOR   0
#define TYPE_PATTERN   1
#define TYPE_IMAGE   2
#define TYPE_IMAGEFOLDER   3
#define TYPE_ANIMATION   4

Scene::Scene( QObject *parent) :
    QGraphicsScene(parent),mPmenu(0),mQmlAnimation(0)
{
    //     qDebug() << "etap_Seen:1";
//    QDir appDir(QApplication::applicationDirPath());
//    appDir.cdUp();
//    m_appPath=  appDir.absolutePath()+"/share/elokab/elokab-desktop";
 m_appPath=Edir::dataAppDir();
    //qDebug() << "etap_Seen:2";
    m_graphicsWidget=new QGraphicsWidget;
    m_itemBackground=new QGraphicsPixmapItem(m_graphicsWidget);
    m_itemBackgroundCache=new QGraphicsPixmapItem();
    addItem(m_itemBackgroundCache);
    addItem(m_graphicsWidget);
    m_color=QColor(60,60,60);
    //**************************

    m_timerSlide=new QTimer;
    connect(m_timerSlide,SIGNAL(timeout()),this,SLOT(nextWallpaper()));
    //**************************
    m_showMenuApp=false;


    loadSettings();
    // setBackgroundimage(m_pixFileName);




}

/************************************
 *حلب اعدادات المستخدم**************
 ***********************************/
void Scene::loadSettings()
{
    //  qDebug() << "etap_Seenloadsetting:1==========================";
    QString   curDirectori;
    if(QFile::exists("/usr/share/wallpapers")){
        curDirectori="/usr/share/wallpapers";
    }else  if(QFile::exists("/usr/share/backgrounds")){
        curDirectori="/usr/share/backgrounds";
    }else{
        curDirectori=Edir::picturesDir();
    }
    QSettings setting("elokab","settings");
    setting.beginGroup("Desktop-Wallpaper");

    m_color=setting.value("wallpaperColor",QColor(60,60,60).name()).toString();
    m_type=(setting.value("wallpaperType",TYPE_IMAGE).toInt());
    m_aspectRatio=(setting.value("aspectRatio",0).toInt());
    m_pixFileName=(setting.value("imagePath",m_appPath+"/images/tawhid.png").toString());
    m_folderDir=(setting.value("folderPath",curDirectori).toString());
    //        m_pixPattern=setting.value("pixPattern").value<QPixmap>();
    m_animationFile=setting.value("animationFile").toString();
    bool showMenuApp=setting.value("showMenuApp",false).toBool();
    QTime time=setting.value("slideShowTaime",QTime::fromString("0:30", "h:m")).toTime();

    setting.endGroup();
    QString fileName=Edir::dataHomeAppDir();
    m_pixPattern=QPixmap(fileName+"/pixPattern.png");

    int m=(time.hour()*60)+time.minute();
    int s=m*60;
    m_slideTime=  s*1000;
    // qDebug() << "m_showMenuApp=========================="<<m_showMenuApp;
    //qDebug() << "showMenuApp=========================="<<showMenuApp;
    if  (m_showMenuApp!=showMenuApp){
        m_showMenuApp=showMenuApp;
        setApplicationsMenu();
    }

    setBackground();
}


void Scene::setBackground()
{
    //      qDebug() << "etap_Seen:3_1";
    //  loadSettings();
    m_timerSlide->stop();
    //     qDebug() << "etap_Seen:3_2";
    if(mQmlAnimation) {

        delete mQmlAnimation;
        mQmlAnimation=0;
    }
    switch (m_type)
    {
    case TYPE_COLOR:

        setBackgroundColor();
        break;

    case TYPE_PATTERN:

        setBackgroundPattern();
        break;

    case TYPE_IMAGE:

        setBackgroundImage();
        break;

    case TYPE_IMAGEFOLDER:

        setBackgroundImageFolder();
        break;

    case TYPE_ANIMATION:

        setBackgroundAnimation();
        break;


    default:

        break;
    }
    if(m_type==TYPE_IMAGEFOLDER){

        emit    isFolderImages(true);
    }else{

        emit    isFolderImages(false);
    }

}
/****************************************************
 *معرفة هل يستخدم مجاد الصور لانشاء زر الصورة التالية*
 ****************************************************/
//bool Scene:: isFoldderType()
//{
//        if(m_type==TYPE_IMAGEFOLDER){
//                return true;
//                emit    isFolderImages(true);
//        }else{
//                emit    isFolderImages(false);
//        }

//        return false;
//}

//! [0]----TYPE_COLOR
void Scene::setBackgroundColor()
{
    m_itemBackgroundCache->setOpacity(0.0);
    m_graphicsWidget->setOpacity(0.0);
    setBackgroundBrush(QBrush(m_color));
}
//! [1]----TYPE_PATTERN
void Scene::setBackgroundPattern()
{
    if(m_pixPattern.isNull()){
        setBackgroundColor();
        return;
    }
    m_itemBackgroundCache->setOpacity(0.0);
    m_graphicsWidget->setOpacity(0.0);
    setBackgroundBrush(QBrush(m_pixPattern));
}
//! [2]----TYPE_IMAGE
void Scene::setBackgroundImage()
{
    m_graphicsWidget->setOpacity((1.0));
    m_itemBackgroundCache->setOpacity((0.0));


    QRect rect(QApplication::desktop()->geometry());
    setBackgroundBrush(QBrush(m_color));

    QPixmap   pix(m_pixFileName);
    if(m_aspectRatio==0){

        int x=rect.width()/2-pix.width()/2;
        int y=rect.height()/2-pix.height()/2;
        m_graphicsWidget->setPos(x,y);
        m_itemBackgroundCache->setPos(x,y);
        m_itemBackground->setPixmap(pix);
    }else{
        m_graphicsWidget->setPos(0,0);
        m_itemBackgroundCache->setPos(0,0);
        m_itemBackground->setPixmap(pix.scaled(rect.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
        //    m_itemBackgroundCache->setPixmap(pix.scaled(rect.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));

    }

    //startAnimmation();
    // setBackgroundAnimation();
}

//! [4]----TYPE_ANIMATION
void Scene::setBackgroundAnimation()
{

    QSettings m_setting(m_animationFile,QSettings::IniFormat);

    m_setting.beginGroup("Desktop Entry");

    QString main1=  m_setting.value("MainScript").toString();

    const QString &main=  m_setting.value("X-Plasma-MainScript",main1).toString();

    m_setting.endGroup();

    if(!main.toLower().endsWith("qml")){
        setBackgroundColor();
        return;
    }


    QFileInfo fi(m_animationFile);
    QString fileName=fi.absolutePath()+"/contents/"+main;

    if(!QFile::exists(fileName)){
        setBackgroundColor();
        return;
    }



    m_graphicsWidget->setOpacity(qreal(0.0));
    m_itemBackgroundCache->setOpacity(qreal(0.0));
    QRect rect(QApplication::desktop()->geometry());

    if(mQmlAnimation) {

        delete mQmlAnimation;
        mQmlAnimation=0;
    }

    // -----------QDeclarativeView ----------------
    // QWidget *w=new QWidget;
    mQmlAnimation=new QmlAnimation(rect);

    if (!mQmlAnimation->setQmlFile(fileName)){
        delete mQmlAnimation;
        mQmlAnimation=0;
        QSettings setting("elokab","settings");
        setting.beginGroup("Desktop-Wallpaper");

        setting.setValue("wallpaperType",TYPE_COLOR);

        setting.endGroup();

        m_type=TYPE_COLOR;

        setBackgroundColor();

        return;
    }

    addItem(mQmlAnimation);
    mQmlAnimation->setZValue(0);
    /*
    if(mQmlAnimation->status()==QDeclarativeView::Error){
        QString msg;
        foreach (QDeclarativeError derror, mQmlAnimation->errors()) {
            msg+=  derror.toString()+"\n";
        }

        delete mQmlAnimation;
        mQmlAnimation=0;

        QSettings setting("elokab","settings");
        setting.beginGroup("Desktop-Wallpaper");

        setting.setValue("wallpaperType",TYPE_COLOR);

        setting.endGroup();

        m_type=TYPE_COLOR;

        setBackgroundColor();
        QMessageBox::information(0,"Error",msg);

        return;
    }


    connect(mQmlAnimation,SIGNAL(statusChanged(QDeclarativeView::Status)),this,
            SLOT(aimationStatusChanged(QDeclarativeView::Status)));

    mQmlAnimation->setResizeMode(QDeclarativeView::SizeRootObjectToView);
    mQmlAnimation->setSceneRect(rect);
    mQmlAnimation->rootObject()->setProperty("width",rect.width());
    mQmlAnimation->rootObject()->setProperty("height",rect.height());
    mQmlAnimation->setFixedSize(rect.size());
    mQmlAnimation->setOptimizationFlags(QGraphicsView::DontSavePainterState);
    mQmlAnimation->setViewportUpdateMode(QGraphicsView::BoundingRectViewportUpdate);
    //mDeclarativeView->setItemIndexMethod(QGraphicsScene::NoIndex);

    //qDebug()<<mDeclarativeView->rootObject()->sceneBoundingRect();;
   // addWidget(mDeclarativeView);
    // -----------QWidget ----------------


    // -----------QDeclarativeView ----------------


     // -----------QGraphicsProxyWidget ----------------
    QGraphicsProxyWidget *toolProxy = new QGraphicsProxyWidget;
    toolProxy->setWidget(w);
    addItem(toolProxy);
*/
}


//! [3]----TYPE_IMAGEFOLDER
void Scene::setBackgroundImageFolder()
{
    //   qDebug() << "etap_setBackgroundImageFolder:1";
    setBackgroundBrush(QBrush(m_color));
    //   qDebug() << "etap_setBackgroundImageFolder:2";
    creatListOfBgr();

    nextWallpaper();



}
//! [3-0]----TYPE_IMAGEFOLDER
void Scene:: creatListOfBgr()
{
    //qDebug() << "creatListOfBgr:1";
    /* انشاء قائمة بالصور */
    m_listBgr.clear();//تفريغ القائمة السابقة

    QDir dirS(m_folderDir);
    QString subfile;

    foreach ( subfile, dirS.entryList(QDir::Files|QDir::NoDotAndDotDot)){
        QFileInfo fi(m_folderDir+"/"+subfile);
        QString ext = fi.suffix();//جلب الامتداد
        if(ext.toLower()=="png"||ext.toLower()=="jpg"||ext.toLower()=="xpm"){//اذا كانت الصورة تحمل هذه الامتدادات
            m_listBgr.append(m_folderDir+"/"+subfile);//اضافة الصورة الى القائمة
        }
    }

    int count=m_listBgr.count()-1;
    if(count==-1)//اذا كانت القائمة لا تحتوي على صور سيتم استخدام الصورة الحالية
        m_listBgr.append(m_pixFileName);
    if(count>-1){//اذا كانت القائمة تحتوي غلى الصور ستم جلب اول صورة
        int i=rand() % ++count;
        //    qDebug()<<"m_curListIndexOf : "<<i;
        m_pixFileName=m_listBgr.at( i);
        m_curListIndexOf= i;//المعرف الحالي للصورة

    }
    //qDebug() << "creatListOfBgr:2";

}
//! [3-1]----TYPE_IMAGEFOLDER
void Scene:: nextWallpaper()
{
    //   qDebug() << "nextWallpaper:1";
    m_timerSlide->stop();
    if (m_type!=TYPE_IMAGEFOLDER)return;
    /* تبديل الصورة الى الصورة التالية عند النقر على الامر الصورة التالية*/
    int count=m_listBgr.count()-1;                          //غدد الصور في القائمة
    if (count<=0)return;

    if (m_curListIndexOf+1>count){
        // qDebug() << "nextWallpaper:2";
        //اذا كان المعرف +1 اكبر من عدد الصور سيجلب الصورة الاولى
        m_curListIndexOf=0;
        m_pixFileName=m_listBgr.at(0);

    }else{
        //   qDebug() << "nextWallpaper:2";
        //الصورة التالية
        m_curListIndexOf++;
        //   qDebug() << "nextWallpaper:3";
        m_pixFileName=m_listBgr.at(m_curListIndexOf);
        // qDebug() << "nextWallpaper:4";
    }
    //جلب الصورة لرسمها على الشاشة
    QPixmap pix(m_pixFileName);

    setBackgroundimage(pix);
    m_timerSlide->start(m_slideTime);

}

//! [3-2]----TYPE_IMAGEFOLDER
void Scene::setBackgroundimage(const QPixmap &pix)
{
    QRect rect(QApplication::desktop()->geometry());
    m_itemBackgroundCache->setOpacity(1.0);
    m_itemBackgroundCache->setPixmap(m_pixPattern);

    m_graphicsWidget->setPos(0,0);
    m_itemBackgroundCache->setPos(0,0);
    m_pixPattern=pix.scaled(rect.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);


    m_graphicsWidget->setOpacity(0.0);
    m_itemBackground->setPixmap(m_pixPattern);

    startAnimmation();
}
/*****************************************************
 *عملية الرسم عند اخفاء واظهار صورة اخرى-*************
 *****************************************************/
void Scene::startAnimmation()
{
    QPropertyAnimation *animation = new QPropertyAnimation(m_graphicsWidget, "opacity");
    animation->setDuration(750);
    animation->setStartValue(qreal(0.1));
    animation->setEndValue(qreal(1.0));
    connect(animation,SIGNAL(finished()),this,SLOT(setBackgroundCache()));
    animation->start();
}

void Scene::setBackgroundCache()
{
    m_itemBackgroundCache->setOpacity(0.0);
}

/***************************************************
******************عملية القوائم*********************
***************************************************/
void Scene:: mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    //    qDebug()<<"QGraphicsItem moved";
    QGraphicsScene::mouseMoveEvent(event);
    Qt::KeyboardModifiers mods = QApplication::keyboardModifiers();
    if (mods != Qt::ControlModifier&&mods != Qt::ShiftModifier) {
        event->ignore();
        return;

    }

    if ( event->buttons()!=Qt::LeftButton){

    }

    if (QLineF(event->screenPos(), event->buttonDownScreenPos(Qt::LeftButton))
            .length()< QApplication::startDragDistance()) {
        return;
    }
    QDrag *drag = new QDrag(event->widget());
    QMimeData *mimeData = new QMimeData();
    QList<QUrl> list;

    foreach (QGraphicsItem *it, selectedItems()) {


        list.append(QUrl(it->data(0).toString()));

    }
    mimeData->setUrls(list);
    drag->setMimeData(mimeData);

    // drag->setPixmap(it->pi);
    drag->exec();



}
void Scene::contextMenuEvent(QGraphicsSceneContextMenuEvent * event)
{

    //     qDebug() << "IconBase::contextMenuEvent" << event;
    if(this->selectedItems().count()>0){


        menuIcon.exec(event->screenPos());
    }else{


        mymenu.exec(event->screenPos());
    }


}
void Scene::setMenuStandard(QMenu *menu)
{
    menuStandard.addActions(menu->actions());
    mymenu.addActions(menu->actions());
}
void Scene::setMenu(QMenu *menu)
{
    mymenu.clear();

    mymenu.addActions(menu->actions());

    mymenu.addActions(menuStandard.actions());
    setApplicationsMenu();
}
void Scene::setApplicationsMenu()
{
    if(mPmenu) {

        delete mPmenu;
        mPmenu=0;
    }
    if(m_showMenuApp==true){


        mPmenu=new MenuProgrammes();
        mPmenu->setIcon(EIcon::fromTheme("applications-other"
                                           ).pixmap(QSize(128, 128)));

        mymenu.insertMenu(mymenu.actions().first(),mPmenu);
        //    connect(mPmenu,SIGNAL(menuRecharged()),this,SLOT(setapplicationsMenu()));
    }
}

void   Scene::setMenuIcon(QMenu *menu)
{
    menuIcon.clear();
    menuIcon.addActions(menu->actions());
}

void Scene::updatIcons()
{
    if(mPmenu) {
        mPmenu->refreshIcons();
    }
}
