/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#include "dockanimmation.h"
#include <QtCore>
#include <QtGui>
#include <QDebug>
#include <QToolButton>
//#define ANIM_NONE 0
//#define ANIM_ZOOM 1
//#define ANIM_ROTAT 2
//#define ANIM_IMAGE 3
DockAnimmation::DockAnimmation(QObject *parent) :
    QObject(parent)

{


          timerAnim=new QTimer;
           connect(timerAnim,SIGNAL(timeout()),this,SLOT(goAnimation()));
}

void DockAnimmation::AnimationStart(QAction *btn,QSize size)
{

    if(m_style==ANIM_NONE)return;

    buttonAct=btn;
    m_iconSize=size.width();//toolButton->iconSize().width();
    m_pix=buttonAct->icon().pixmap(size);

    if(m_style==ANIM_ZOOM){
        //  connect(timerAnim,SIGNAL(timeout()),this,SLOT(buttonZoomIn()));
        m_rot=9.0;
        m_def=1.0;
        timerAnim->start(30);
    }

    if(m_style==ANIM_ROTAT){
        m_rot=0.95;
        m_def=0.05;
        timerAnim->start(10);
    }

    if(m_style==ANIM_IMAGE){
        m_rot=0.0;
        timerAnim->start(20);
    }

    if(m_style==ANIM_BUNSE){
        m_rot=10.0;
        m_def=1.0;
        timerAnim->start(30);
    }

}
void DockAnimmation::goAnimation()
{
     if(m_style==ANIM_ZOOM){buttonZoomIn();}
     if(m_style==ANIM_ROTAT){buttonRotate();}
     if(m_style==ANIM_IMAGE){buttonImage();}
      if(m_style==ANIM_BUNSE){buttonBunse();}
}
/******************************
 ********ANIM_ZOOM*************
 ******************************/
void DockAnimmation::buttonZoomIn()
{
    /* zoomin and zoomout the icon*/
    if(m_rot<=0.0)
        m_def=-1.0;
    if(m_rot>=10.0){
        timerAnim->stop();

        buttonAct->setIcon(m_pix);
        return;
    }

    m_rot=m_rot-m_def;

    QPixmap pix2(m_iconSize,m_iconSize);
    pix2.fill(Qt::transparent);
    QPainter p;
    p.begin(&pix2);

    QRectF rect( m_rot,m_rot,m_pix.width()-(m_rot*2),m_pix.width()-(m_rot*2));
    p.drawPixmap(rect,m_pix,m_pix.rect());
    p.end();
    buttonAct->setIcon(QIcon(pix2));

}

/******************************
 **********ANIM_ROTAT**********
 ******************************/
void DockAnimmation::buttonRotate()
{

    /*rotate the icon*/
    if(m_rot<=0.01)
        m_def=-0.05;

    if(m_rot>=1.00){

        timerAnim->stop();
        buttonAct->setIcon(m_pix);

        return;
    }

    m_rot=m_rot-m_def;

    QPixmap pix2(m_iconSize,m_iconSize);
    pix2.fill(Qt::transparent);
    QPainter p;
    p.begin(&pix2);
    p.translate(m_iconSize/2, m_iconSize/2);
    p.scale(m_rot, 1.0);
    p.translate(-m_iconSize/2, -m_iconSize/2);
    p.drawPixmap(m_pix.rect(),m_pix,m_pix.rect());
    p.end();
    buttonAct->setIcon(QIcon(pix2));
}

/******************************
 **********ANIM_IMAGE**********
 ******************************/
void DockAnimmation::buttonImage()
{
    if(m_rot>=360.00){

        timerAnim->stop();
        buttonAct->setIcon(m_pix);
        return;
    }

    m_rot=m_rot+10;

    QPixmap pix2(m_pix);

    QPixmap pix(m_iconSize,m_iconSize);
    pix.fill(Qt::transparent);
    QPainter p;
    p.begin(&pix);
    p.translate(m_iconSize/2, m_iconSize/2);
    p.rotate(m_rot);
    p.translate(-m_iconSize/2, -m_iconSize/2);
    p.drawPixmap(0,0,m_iconSize,m_iconSize,QPixmap(":/images/progress.png"),0,0,64,64);
    p.end();
    p.begin(&pix2);
    p.drawPixmap(m_pix.rect(),pix,m_pix.rect());
    p.end();

    buttonAct->setIcon(QIcon(pix2));

}
void DockAnimmation::buttonBunse()
{
    /* zoomin and zoomout the icon*/
    if(m_rot<=0.0)
        m_def=-1.0;
    if(m_rot>=21.0){
        timerAnim->stop();

        buttonAct->setIcon(m_pix);
        return;
    }

    m_rot=m_rot-m_def;
qDebug()<<"m_rot"<<m_rot;
    QPixmap pix2(m_iconSize,m_iconSize);
    pix2.fill(Qt::transparent);
    QPainter p;
    p.begin(&pix2);

    QRectF rect( 5,m_rot,m_pix.width()-10,m_pix.height()-20);
    p.drawPixmap(rect,m_pix,m_pix.rect());
    p.end();
    buttonAct->setIcon(QIcon(pix2));
}

void DockAnimmation::AnimationStop()
{
     timerAnim->stop();
     if(!m_pix.isNull())
      buttonAct->setIcon(m_pix);
}

