﻿/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "addactions/dialogaddaction.h"
#include <EMimIcon>
#include <DesktopFile>
#include <EIcon>
#include "dockbutton.h"
#include "dockbarwidget.h"

#include <QtGui/QAction>
#include <QtGui/QMenu>
#include <QtGui/QStylePainter>
#include <QtGui/QStyleOptionToolButton>
#include <QtGui/QMouseEvent>
#include <QtGui/QApplication>
#include <QToolTip>
#include <QDesktopWidget>
#include <QtDebug>
#include <QProcess>
#include <QUrl>
#include <QtGui/QMessageBox>
#include <QtGui/QDesktopServices>

#include <QSettings>
#include <QTextCodec>
#include <QLocale>

#define MIMETYPE "ddock"

//____________________________________________________________________________________
ActionDock::ActionDock(const QString & name,
                       const QString & exec,
                       const QString & icon,
                       QWidget * parent)
    : QAction(parent)
{


    // qDebug()<<"icon----------------------"<<icon;


    connect(this, SIGNAL(triggered()), this, SLOT(execAction()));



    //qDebug()<<"iconF----------------------"<<iconF.name();
    setText(name.section("|",0,0));
    setData(exec);

    m_settingsMap["name"] = name;
    m_settingsMap["exec"] = exec;
    m_settingsMap["icon"] = icon;

    refreshIcon();
}

//____________________________________________________________________________________
ActionDock::ActionDock(const QString & fileName, QWidget * parent)
    : QAction(parent)
{

    DockBarWidget *quick = qobject_cast<DockBarWidget*>(parent);
    QString lC=quick->locale().name();
    lC= lC.section("_",0,0);
    /******************************************************/

     QFileInfo fi(fileName);
      if(fi.suffix()=="desktop"){
DesktopFile xdg(fileName,"Desktop Entry",lC);
          // QHash<QString ,QString> hash=EMimIcon::desktopFile(fileName,lC);
           m_settingsMap["name"] = /*hash ["Name"];*/xdg.value("Name").toString();
           m_settingsMap["exec"] =  /*hash ["Exec"];*/xdg.value("Exec").toString();
           m_settingsMap["icon"] = /*hash ["Icon"];*/xdg.value("Icon").toString();
      }else{
           m_settingsMap["name"] = fi.fileName();
           m_settingsMap["exec"] = fileName;
           m_settingsMap["icon"] = EMimIcon::icon(fi).name();
      }
       setData( m_settingsMap["exec"] );
       setText(m_settingsMap["name"]);
      connect(this, SIGNAL(triggered()), this, SLOT(execAction()));

      refreshIcon();


}

//____________________________________________________________________________________
void ActionDock::execAction()
{

    EMimIcon::launchApplication(data().toString());
}

//____________________________________________________________________________________
void ActionDock::setSettingMap(const QStringList &list)
{

    m_settingsMap["name"] =list.at(0);
    m_settingsMap["exec"] = list.at(2);
    m_settingsMap["icon"] = list.at(3);
    setText(list.at(0));
    setData(list.at(2));
   refreshIcon();

}

//____________________________________________________________________________________
void ActionDock::refreshIcon()
{

    QIcon iconF;


    iconF=EIcon::fromTheme(m_settingsMap["icon"]).pixmap(128);


 myIcon=iconF;
    setIconNew();

}
//____________________________________________________________________________________
void ActionDock::setIconNew()
{
   QIcon icon=myIcon;
   QPixmap pixIcon=QPixmap(icon.pixmap(128)).scaled(128,128);
    QPainter  painter;

    QSize iSize;

    QRect target;

    //draw orig icon
    switch (m_screenEdge) {
    case EDGE::POS_BOT:
        iSize=QSize(128,170);
        target=QRect(0,0, 128, 128);
        break;
    case EDGE::POS_TOP:
        iSize=QSize(128,170);
        target=QRect(0,41, 128, 128);
        break;
    case EDGE::POS_RIGHt:
        iSize=QSize(128,128);
        target=QRect(0,0, 128, 128);
        break;
    case EDGE::POS_LEFT:
        iSize=QSize(128,128);
        target=QRect(0,0, 128, 128);
        break;
    default:
        break;
    }

    QPixmap pix(iSize);
    pix.fill(Qt::transparent);
    painter.begin(&pix);
    painter.drawPixmap(target,icon.pixmap(128) );

    qreal rx=0.80;
    int r;
    switch (m_screenEdge) {
    case EDGE::POS_BOT:
        r=128;
        for (int i =130; i <171; ++i) {

            painter.setOpacity(rx);
            painter.drawPixmap(0,i,128,1,pixIcon,0,r,128,1);
            r--;
            rx=rx-0.02;
            if(rx<0.01)rx=0.00;
        }
        break;
    case EDGE::POS_TOP:
        r=0;
        for (int i =40; i >-1; --i) {
            //        qDebug()<<i;
            painter.setOpacity(rx);
            painter.drawPixmap(0,i,128,1,pixIcon,0,r,128,1);
            r++;
            rx=rx-0.02;
            if(rx<0.02)rx=0.00;
        }
        break;
    case EDGE::POS_RIGHt:

        break;
    case EDGE::POS_LEFT:

        break;
    default:
        break;
    }

     //active window

    painter.end();

    setIcon(QIcon(pix));
}


//____________________________________________________________________________________
DockButton::DockButton(QSize size,int id, ActionDock * act, QWidget * parent)
    : QToolButton(parent),
      m_act(act),
      m_id(id),isTaskbar(0)
{
    m_parent=qobject_cast<DockBarWidget*>(parent);;
    setAcceptDrops(true);
    m_size=size;
    setDefaultAction(m_act);
    m_act->setParent(this);
    //    setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
  //  setIconSize(m_size);
 //this->setCheckable(true);
    m_propertyAct=new QAction(EIcon::fromTheme("document-properties"), trUtf8("properties"), this);
    connect(m_propertyAct, SIGNAL(triggered()), this, SLOT(getProperty()));


    m_deleteAct = new QAction(EIcon::fromTheme("dialog-close",("list-remove")), trUtf8("Remove from quicklaunch"), this);
    connect(m_deleteAct, SIGNAL(triggered()), this, SLOT(parentRemoveThis()));
//    DockBarWidget *quick = qobject_cast<DockBarWidget*>(parent);

    m_menu = new QMenu;
    m_taskMenu= new QMenu;
    m_globalMenu= new QMenu;
    m_menu->addAction(m_act);
    m_menu->addAction(m_propertyAct);

    m_menu->addSeparator();
    m_menu->addAction(m_deleteAct);
    m_menu->addSeparator();
    QMenu *mnuDock=new QMenu(tr("Dock Bar"),this);

    mnuDock->addAction(m_parent->actAddActions);
    mnuDock->addAction(m_parent->actSettings);
    mnuDock->addSeparator();
    mnuDock->addAction(m_parent->actQuit);
    m_menu->addMenu(mnuDock);
    m_parent->actSettings->setIcon(EIcon::fromTheme("configure",("document-properties")));
    m_parent->actAddActions->setIcon(EIcon::fromTheme("list-add"));
    m_parent->actQuit->setIcon(EIcon::fromTheme("application-exit"));
  setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this, SIGNAL(customContextMenuRequested(const QPoint&)),
            this, SLOT(this_customContextMenuRequested(const QPoint&)));
   m_anim=new DockAnimmation(this);
    connect(this, SIGNAL(clicked()), this, SLOT(animmation()));
    m_globalMenu=m_menu;
}

//____________________________________________________________________________________
void DockButton::this_customContextMenuRequested(const QPoint & pos)
{
    if(isTaskbar)
        m_globalMenu=m_parent->getTaskMenu(this);

    m_globalMenu->popup(mapToGlobal(pos));
}
//____________________________________________________________________________________
DockButton::~DockButton()
{
    //m_act->deleteLater();
}

//____________________________________________________________________________________
void DockButton::setSize(QSize size)
{
    m_size=size;
//    setIconSize(m_size);

}

//____________________________________________________________________________________
void DockButton::setAnimation(int animation,int edg)
{
    m_anim->setStyle(animation);
    m_act->setScreenEdge(edg);
    m_act->setIconNew();
}
void DockButton::refreshIcon()
{

    m_act->refreshIcon();
}


//____________________________________________________________________________________
QHash<QString,QString> DockButton::settingsMap()
{
    Q_ASSERT(m_act);
    return m_act->settingsMap();
}



//____________________________________________________________________________________
void DockButton::parentRemoveThis()
{
    emit buttonDeleted(m_id);
}

//____________________________________________________________________________________
void DockButton::mousePressEvent(QMouseEvent *e)
{


    if (e->button() == Qt::LeftButton && e->modifiers() == Qt::ControlModifier)
    {

        m_dragStart = e->pos();
        return;
    }
 QToolButton::mousePressEvent(e);

}
//____________________________________________________________________________________


//____________________________________________________________________________________
void DockButton::mouseMoveEvent(QMouseEvent *e)
{
    //     QToolTip::showText ( QPoint (this->pos().x(),this->pos().y()-10), this->text() );
    if (!(e->buttons() & Qt::LeftButton))
    {
        return;
    }

    if ((e->pos() - m_dragStart).manhattanLength() < QApplication::startDragDistance())
    {
        return;
    }

    if (e->modifiers() != Qt::ControlModifier)
    {
      return;
    }

    QDrag *drag = new QDrag(this);
       QMimeData *mimeData = new QMimeData();
        QByteArray ba;
        ba.setNum(m_id);
        mimeData->setData(MIMETYPE, ba);

     drag->setMimeData(mimeData);
    // drag->setPixmap(this->icon().pixmap(iconSize()));

drag->exec();

    //TODO: fix delete btn from target

    if (!drag->target())
    {
        if(!isTasked()){
         //  drag->setMimeData(new QMimeData());
           //  QTimer::singleShot(10,this,SLOT(parentRemoveThis()));
        //   parentRemoveThis();

        }
    }

}

//____________________________________________________________________________________
void DockButton::dragMoveEvent(QDragMoveEvent * e)
{

    if (e->mimeData()->hasFormat(MIMETYPE))
        e->acceptProposedAction();
    else
        e->ignore();
}
void DockButton::enterEvent ( QEvent * /*e*/ )
{

    emit entered(this);

}

//____________________________________________________________________________________

void DockButton::dragEnterEvent(QDragEnterEvent *e)
{


    if (e->mimeData()->hasFormat(MIMETYPE))
    {
        int other = e->mimeData()->data(MIMETYPE).toInt();
        // do not replace self with self
        if (other == m_id)
            return;

        emit switchButtons(m_id, other);
    }
}

//____________________________________________________________________________________
void DockButton::getProperty()
{


    QRect screen = QApplication::desktop()->screenGeometry(-1);
    QStringList list;
    list<<m_act->text()
       <<m_act->data().toString().section("|",0,0)
      <<m_act->settingsMap()["name"].section("|",1,1)
     <<m_act->settingsMap()["icon"];

    DialogAddAction *dlg=new DialogAddAction(list,m_act->icon());
    connect(dlg,SIGNAL(accepteChange(QStringList)) ,this,SLOT(changeAction(QStringList)));
    dlg->move(screen.width()/2-dlg->width()/2,screen.height()/2-dlg->height()/2);
    dlg->show();

}

//____________________________________________________________________________________
void DockButton::changeAction(const QStringList &list)
{

    m_act->setSettingMap(list);
    emit settingsChanged();
}

//____________________________________________________________________________________
void DockButton::animmation()
{
    m_anim->AnimationStart(m_act,this->iconSize());
}
//____________________________________________________________________________________

void DockButton::resetAction()
{
     m_act->refreshIcon();
    setDefaultAction(m_act);
    m_globalMenu=m_menu;
    m_act->refreshIcon();

}

//____________________________________________________________________________________

void DockButton::setTaskAction(QAction *act)
{
    removeAction(m_act);
    setDefaultAction(act);

}

//____________________________________________________________________________________

