/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef BUTTONDOCK_H
#define BUTTONDOCK_H


#include "dockanimmation.h"
#include "dockbarwidget.h"
#include <QToolButton>
#include <QAction>

/**
 * @brief The ActionDock class
 *الامر الذي سيحويه الزر
 */
class ActionDock : public QAction
{
    Q_OBJECT


public:

    /**
     * @brief ActionDock
     * @param name  اسم الزر
     * @param exec التطبيق
     * @param icon الايثونة
     * @param parent
     */
    ActionDock(const QString &name, const QString &exec, const QString &icon, QWidget * parent);
    /**
     * @brief ActionDock
     * @param fileName مسار ملف
     * @param parent
     */
    ActionDock(const QString & fileName, QWidget * parent);
    /**
     * @brief settingsMap قائمة لاسم الزر و التطبيق و الايقونة لهذا الحدث
     * @return  m_settingsMap["name"] = name;
                m_settingsMap["exec"] = exec;
                m_settingsMap["icon"] = icon;
     */
    QHash<QString, QString> settingsMap() { return m_settingsMap; }
    /**
     * @brief setSettingMap
     * @param list قائمة ب اسم ثم تطبيق ثم ايقونة
     */
    void setSettingMap(const QStringList &list);

public slots:
    //! تنفيذ التطيق
    void execAction();
    //! تحديث الايقونات
    void refreshIcon();
 void setScreenEdge(int edg){m_screenEdge=edg;}
   QIcon icon(){return myIcon;}
   //! تحديث ايقونة
   void setIconNew();

private slots:

private:
    /**
     * @brief The ActionType enum
     *ActionExec =0  ملف تطبيق
     *ActionScript=1 ملف سكريبت
     *ActionFile=2 ملف او مجلد
     */
    int m_screenEdge;

    QString m_data;
 QIcon myIcon;
    QHash<QString, QString> m_settingsMap;
};
//------------------------------------------------------------
/**
 * @brief The ButtonDock class
 * الزر الذي سيضاف الى نافذة الارساء
 */
class DockButton : public QToolButton
{
    Q_OBJECT

public:
    /**
     * @brief ButtonDock الزر
     * @param size الحجم
     * @param id معرف الزر في الاعدادات
     * @param act الامر
     * @param parent الاب
     */
    DockButton(QSize size,int id, ActionDock * act, QWidget* parent = 0);
    ~DockButton();
    /**
     * @brief settingsMap  باسم الزر واسم الايقونةقائمة ثنائية
     * @return
     */
    QHash<QString,QString> settingsMap();
    /*!< جلب المعرف */
    int id() { return m_id; }
    /**
     * @brief data المخزن داخل الامر
     * @return
     */
    QString data()
    {
        return m_act->data().toString().section(" ",0,0);
    }
    /**
     * @brief setAnimation امر بنوع الحركة
     * @param animation النوع
     */
    void setAnimation(int animation, int edg);
QIcon icon(){return m_act->icon();}
void refreshIcon();
signals:
    /**
     * @brief buttonDeleted اشارة الى النافذة الاب بحذف الزر
     * @param index  m_id
     */
    void buttonDeleted(int index);
    /**
     * @brief switchButtons اشارة بتغيير موصع الزر
     * @param first السابق
     * @param second التالي
     */
    void switchButtons(int first, int second);
    /**
     * @brief settingsChanged اشارة بتغير الاعدادات
     */
    void settingsChanged();

    /**
     * @brief entered اشارة بدخول المؤشر
     * @param id m_id
     */
    void entered(DockButton*);
    /**
     * @brief leaved اشارة باخراج الزر
     */
    void leaved();
protected:

    void mousePressEvent(QMouseEvent *e);
    void mouseMoveEvent(QMouseEvent *e);

    void dragEnterEvent(QDragEnterEvent *e);
    void dragMoveEvent(QDragMoveEvent * e);

    void enterEvent ( QEvent * /*e*/ );


    void leaveEvent ( QEvent * /*e*/ )
    {
        leaved();
    }
private:
    ActionDock *m_act;

    QAction *m_deleteAct;

   // QAction *m_moveRightAct;
    QAction *m_propertyAct;
    QMenu *m_menu;
      QMenu *m_taskMenu;
       QMenu *m_globalMenu;
    QPoint m_dragStart;
    int m_id;
    QSize m_size;

DockBarWidget *m_parent;

    DockAnimmation *m_anim;

    bool isTaskbar;
private slots:

    /**
     * @brief this_customContextMenuRequested
     * @param pos
     */
    void this_customContextMenuRequested(const QPoint & pos);
    /**
     * @brief parentRemoveThis
     */
    void parentRemoveThis();
    /**
     * @brief getProperty
     */
    void getProperty();
    /**
     * @brief changeAction
     * @param list
     */
    void changeAction(const QStringList &list);
    /**
     * @brief animmation
     */
    void animmation();
public slots:

    /**
     * @brief setSize
     * @param size
     */
 void setSize(QSize size);
void resetAction();
void setTaskAction(QAction *act);
void setTaskbar(bool arg){isTaskbar=arg;}
bool isTasked(){return isTaskbar;}
};

#endif
