/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "dialogaddplugins.h"
#include "ui_dialogaddplugins.h"
#include <QSettings>
#include <QTextCodec>
#include <EIcon>
#include <EDir>
#include <QPluginLoader>
#define ACT_SPACER   "Spacer"
DialogAddPlugins::DialogAddPlugins(QWidget *parent) :
          QDialog(parent),
          ui(new Ui::DialogAddPlugins)
{
          ui->setupUi(this);
       //    QIcon::setThemeName(theme);
          this->setAttribute(Qt::WA_DeleteOnClose,true);

          this->setWindowIcon(EIcon::fromTheme("configure"));
          if(this->isRightToLeft()){
                    ui->toolButtonAdd->setArrowType(Qt::LeftArrow);
                    ui->toolButtonRemove->setArrowType(Qt::RightArrow);
          }
          loadSetting();
          loadPlugins();

}

DialogAddPlugins::~DialogAddPlugins()
{     
          delete ui;
}
//! [1-0] ------------------------------------------------تحميل معلومات الاضافات ---------------------------------------------- */
void DialogAddPlugins::loadPlugins()
{

    //! مسار مجلد الاضافات
    QDir pluginsDir(Edir::libDir());
//    pluginsDir.cdUp();
//    if(QFile::exists(pluginsDir.path()+"/lib/elokab/elokab-panel")){
//        pluginsDir.cd(pluginsDir.path()+"/lib/elokab/elokab-panel");
//    }else if(QFile::exists(pluginsDir.path()+"/lib64/elokab/elokab-panel")){
//        pluginsDir.cd(pluginsDir.path()+"/lib64/elokab/elokab-panel");
//    }
    //---------------------------
    //! مسار مجلد services
    QDir servicePath(Edir::dataAppDir()+"/services");
//       servicePath.cdUp();
//    servicePath.cd(  servicePath.path()+"/share/elokab/elokab-panel/services");
//Edir::dataAppDir()
    //--------------------------------------------------------------------------
    QListWidgetItem *itemL=new QListWidgetItem(ui->listWidgetAvailable);
    itemL->setText(trUtf8("<    Spacer   >") );
    itemL->setData(Qt::UserRole,ACT_SPACER);
    itemL->setIcon(EIcon::fromTheme("measure"));

    //--------------------------------------------


    foreach (QString fileName, servicePath.entryList(QDir::Files)) {
        QFileInfo fi(servicePath.absoluteFilePath(fileName));
        //اذا لم يكن الملف بامتداد ديسكتوب
        if(fi.completeSuffix()!="desktop")
            continue;

        QStringList list=openEntry(servicePath.absoluteFilePath(fileName));
        QString txtName=list.at(0);
        QString plugName=list.at(1);
        QString icon=list.at(2);
        QString comment=list.at(3);
        QString type=list.at(4);
        //اذا كانت الاضافة غير موجودة في المسار
        if(!QFile::exists(pluginsDir.absoluteFilePath(plugName)))
            continue;

        //اذا كان النوع عبارة عن اضافة
        if(type=="panel/Plugin"){

            QListWidgetItem *lItem=new QListWidgetItem(ui->listWidgetAvailable);
            lItem->setText(txtName+"\n"+comment);
            lItem->setToolTip(comment);
            lItem->setData(Qt::UserRole,plugName);
            lItem->setIcon(EIcon::fromTheme(icon));
        }

    }


    loadAvailablePlugins();
}
//! [1-1] ------------------------------------------------فرز الاضافات ---------------------------------------------- */
void DialogAddPlugins::loadAvailablePlugins()
{

          foreach (QString txt,m_listPluginsFiles)
          {
                   // QTreeWidgetItem *item=new QTreeWidgetItem;
                 //   QString txt=m_listPluginsFiles.at(i);


                    for (int i = 0; i < ui->listWidgetAvailable->count(); ++i) {
                        QListWidgetItem *itemL=    ui->listWidgetAvailable->item(i);
                        if(itemL->data(Qt::UserRole).toString().contains(txt)){
                            QListWidgetItem *itemNew=new QListWidgetItem;
                            itemNew=itemL->clone();
                            ui->listWidgetPlugins->addItem(itemNew);

                            ui->listWidgetAvailable->takeItem(i);
                        }
                    }


          }
}


//! [0-1] ------------------------------------------------تحميل اعدادات المستخدم ---------------------------------------------- */
void DialogAddPlugins::loadSetting()
{

          QSettings setting("elokab","settings");
          setting.beginGroup("Panel-Style");
          m_listPluginsFiles=setting.value("ListPluginsFiles",m_listPluginsFiles
                                           <<"libdmainmenu.so"
                                           <<"libdquicklaunch.so"
                                           <<"libdtaskbar.so"
                                           <<"libdswichdesktop.so"
                                           <<"libdvolume.so"
                                           <<"libdsystray.so"
                                           <<"libdtimer.so").toStringList();
          setting.endGroup();

}
//! [0-2] ------------------------------------------------حفظ  اعدادات المستخدم ---------------------------------------------- */
void DialogAddPlugins::saveSetting()
{
          m_listPluginsFiles.clear();
          int count=ui->listWidgetPlugins->count();
          for(int i=0;i<count;i++){
                    QString txt=   ui->listWidgetPlugins->item(i)->data(Qt::UserRole).toString();
                    m_listPluginsFiles.append(txt);
          }
          QSettings setting("elokab","settings");
          setting.beginGroup("Panel-Style");
          setting.setValue("ListPluginsFiles",m_listPluginsFiles);

          setting.endGroup();
}
//! [0] ------------------------------------------------النقر على زر تطبيق او موافق ---------------------------------------------- */
void DialogAddPlugins::on_buttonBox_clicked(QAbstractButton *button)
{        

          if(ui->buttonBox->standardButton(button)==QDialogButtonBox::Apply){
                    saveSetting();
                    emit   accepteChange();
          }else  if(ui->buttonBox->standardButton(button)==QDialogButtonBox::Ok){
                    saveSetting();
                    emit   accepteChange();
                    this->accept();
          }

}
//! [0-1] ------------------------------------------------حذف الاضافة من الاختيار ---------------------------------------------- */
void DialogAddPlugins::on_toolButtonRemove_clicked()
{
          QListWidgetItem *curItem=ui->listWidgetPlugins->currentItem();
          if(!curItem)return;
//          if(curItem->data(Qt::UserRole).toString()==ACT_SPACER)
//              return;

          QListWidgetItem *item=new QListWidgetItem;
          item=curItem->clone();
          ui->listWidgetAvailable->addItem(item);

          int index= ui->listWidgetPlugins->row(curItem);
          ui->listWidgetPlugins->takeItem(index);
          ui->listWidgetAvailable->setCurrentItem(item);
}
//! [0-2] ------------------------------------------------اضافة الاضافة الى الاختيار ---------------------------------------------- */
void DialogAddPlugins::on_toolButtonAdd_clicked()
{
          QListWidgetItem *curItem=ui->listWidgetAvailable->currentItem();
          QListWidgetItem *curItemDest=ui->listWidgetPlugins->currentItem();
          if(!curItem)return;

//          QList<QTreeWidgetItem *> found = ui->treeWidgetPlugins->findItems(curItem->text(1),Qt::MatchExactly | Qt::MatchRecursive,1);
//          if(found.count()>0)return;


          QListWidgetItem *item=new QListWidgetItem;
          item=curItem->clone();
          if(curItemDest){
                    int ind=ui->listWidgetPlugins->row(curItemDest)+1;
                    ui->listWidgetPlugins->insertItem( ind,item);
          }else{
                    ui->listWidgetPlugins->addItem(item);
          }
          int ind=ui->listWidgetAvailable->row(curItem);
          ui->listWidgetAvailable->takeItem(ind);
          ui->listWidgetPlugins->setCurrentItem(item);


}
//! [0-3] ------------------------------------------------نقل الاضافة الى الاعلى ---------------------------------------------- */
void DialogAddPlugins::on_toolButtonUp_clicked()
{
          QListWidgetItem *curItem=ui->listWidgetPlugins->currentItem();
          if(!curItem)return;

          int index= ui->listWidgetPlugins->row(curItem);
          if(index==0)return;
          QListWidgetItem *item=new QListWidgetItem;
          item=curItem->clone();

          ui->listWidgetPlugins->takeItem(index);
          ui->listWidgetPlugins->insertItem(index-1,item);
          ui->listWidgetPlugins->setCurrentItem(item);
          //----------------------
          //    QListWidgetItem  *curItem= ui->listWidget->currentItem();
          //      if(!curItem)return;
          //      int index= ui->listWidget->currentRow();
          //      if(index==0)return;
          //      QListWidgetItem *item=new QListWidgetItem;
          //     item=curItem->clone();
          //     ui->listWidget->takeItem(index);
          //     ui->listWidget->insertItem(index-1,item);
          //     ui->listWidget->setCurrentItem(item);

}
//! [0-4] ------------------------------------------------نقل الاضافة الى الاسفل ---------------------------------------------- */
void DialogAddPlugins::on_toolButtonDown_clicked()
{
          QListWidgetItem *curItem=ui->listWidgetPlugins->currentItem();
          if(!curItem)return;

          int index= ui->listWidgetPlugins->row(curItem);
          if(index== ui->listWidgetPlugins->count()-1)return;
          QListWidgetItem *item=new QListWidgetItem;
          item=curItem->clone();

          ui->listWidgetPlugins->takeItem(index);
          ui->listWidgetPlugins->insertItem(index+1,item);
          ui->listWidgetPlugins->setCurrentItem(item);
}
//! ---------------------------------------------------------------------
QStringList DialogAddPlugins::openEntry(const QString &fileName)
{

    QString  m_locale=this->locale().name().section("_",0,0);
    QSettings settings(fileName,QSettings::IniFormat);

   settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
    settings.beginGroup("Desktop Entry");

    QString nameold=settings.value("Name").toString();
    QString name=settings.value("Name["+m_locale+"]",nameold).toString();

    QString exec=settings.value("Exec").toString();

    QString icon=settings.value("Icon").toString();

    QString commentOld=settings.value("Comment").toString();
     QString comment=settings.value("Comment["+m_locale+"]",commentOld).toString();

    QString serviceTypes=settings.value("ServiceTypes").toString();

    settings.endGroup();
  //  QStringList list;
 return    QStringList()<<name<<exec<<icon<<comment<<serviceTypes;

}
