/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QtGui>
#include "widgetpanel.h"
#include "ui_widgetpanel.h"
#include <EIcon>
#include "dialogaddplugins.h"
#include "X11/Xlib.h"
#include <EDir>
#include <X11/Xatom.h>

#include <QDesktopWidget>
#include <QDebug>
#include <QtGui/QX11Info>
#include <QtGui/QDesktopWidget>

#define ACT_SPACER   "Spacer"
#define TOP_SCREEN "top"
#define BOT_SCREEN "bottom"
WidgetPanel::WidgetPanel(QWidget *parent) :
    QFrame(parent),
    ui(new Ui::WidgetPanel)
{

    setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
    setAttribute(Qt::WA_X11NetWmWindowTypeDock);

    setAttribute(Qt::WA_AlwaysShowToolTips);
    setWindowTitle(tr("Panel"));
    setObjectName("FormPanel");

    ui->setupUi(this);
    this->setPalette(Qt::transparent);
    this->setAttribute(Qt::WA_TranslucentBackground,true);

    this->setContentsMargins(0,0,0,0);
    ui->widgetBgr->setContentsMargins(0,0,0,0);
    connect(QApplication::desktop(), SIGNAL(workAreaResized(int)),
            this, SLOT(   resizePanel()));
    m_settings=new ElokabSettings(this);

     connect(m_settings,SIGNAL(iconThemeChanged()) ,this,SLOT(loadIconThemes()));
     connect(m_settings,SIGNAL(panelThemesChanged()) ,this,SLOT(appliqueChange()));

pluginsDir.setPath(Edir::libDir());

//    pluginsDir.setPath(qApp->applicationDirPath());
//    pluginsDir.cdUp();
//    if(QFile::exists(pluginsDir.path()+"/lib/elokab/elokab-panel")){
//        pluginsDir.cd(pluginsDir.path()+"/lib/elokab/elokab-panel");
//    }else if(QFile::exists(pluginsDir.path()+"/lib64/elokab/elokab-panel")){
//        pluginsDir.cd(pluginsDir.path()+"/lib64/elokab/elokab-panel");
//    }
    //*******************************
    //! [1]----تحميل اعدادات المستخدم

    loadSetting();

    //! [2]----اعدادات لا تتم الا بعد تحميل اعدادات المستخدم
    setupActions();
   loadIconThemes();

    horizontalSpacer = new QSpacerItem(5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum);
    ui->hLayoutTool->addItem(horizontalSpacer);
    //! [3]----تحميل الاضاقات

//
//loadPlugins();
    //! [4]---- تطبيق التغييرات

    connect(m_settings,SIGNAL(ThemesChanged()) ,this,SLOT(appliqueChange()));
    connect(m_settings,SIGNAL(shortcutsKeysChanged()) ,this,SLOT(refreshKeyShortkut()));

    appliqueChange();
     QTimer::singleShot(50,this,SLOT(loadPlugins()));
     QTimer::singleShot(300,this,SLOT(moveToAllDesktop()));

}
void WidgetPanel::paintEvent(QPaintEvent *)
{


}
WidgetPanel::~WidgetPanel()
{
    delete ui;
}

//! [1] -تحميل اعدادات المستخدم ---------------------------------------------- */
void WidgetPanel::loadSetting()
{
    QSettings setting("elokab","settings");
    setting.beginGroup("Panel-Style");

    QString position=setting.value("Position",TOP_SCREEN).toString();
    if(position==BOT_SCREEN)m_Position=1;
    else m_Position=0;

    m_panelHeight=setting.value("panelSize",24).toInt();
    m_listPluginsFiles=setting.value("ListPluginsFiles",m_listPluginsFiles
                                     << "libdmainmenu.so"
                                     <<"libdquicklaunch.so"
                                     <<"libdtaskbar.so"
                                     <<"libdswichdesktop.so"
                                     <<"libdvolume.so"
                                     <<"libdsystray.so"
                                     <<"libdtimer.so").toStringList();
    m_styleDefault=setting.value("styleDefault",false).toBool();
    setting.endGroup();

}

void WidgetPanel::loadIconThemes()
{

    qDebug()<<"themename"<<QIcon::themeName();
 EIcon::seTemeName(QIcon::themeName());
        QIcon  iconF=EIcon::fromTheme("configure","document-properties");
        QIcon  iconF2=EIcon::fromTheme("list-add",("edit-add"));
        QIcon  iconF3=EIcon::fromTheme("document-edit",("edit"));



        m_actCostumiz->setIcon(iconF);
        m_actAddPlugins->setIcon(iconF2);
        m_actEditApps->setIcon(iconF3);

//        foreach (PluginsInterface *pluginsInterface, m_hatchPligins){
//            if(pluginsInterface)
//                pluginsInterface->themeChanged();
//        }
////   }
}


//! [2]انشاء بعض الادوات---------------------------------------------- */
void WidgetPanel::setupActions()
{



    // QIcon::setThemeName (m_iconsTheme ) ;
    this->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this,SIGNAL(customContextMenuRequested(const QPoint & ))
            ,this,SLOT(customContextMenu(const QPoint & )));

    m_actCostumiz=new QAction(trUtf8("Panel Settings"),this);

    m_actAddPlugins =new QAction(trUtf8("Add Remove Plugins"),this);

    m_actEditApps =new QAction(trUtf8("Edit Applications"),this);

    setMenuTiTle(tr("Elokab Panel"));

    connect(m_actCostumiz, SIGNAL(triggered()), this, SLOT(costomizPanale()));
    connect(m_actAddPlugins, SIGNAL(triggered()), this, SLOT(addPluginsConfig()));
    connect(m_actEditApps, SIGNAL(triggered()), this, SLOT(showEditApplications()));


    QSettings sArray("elokab","shortcut") ;

    sArray.beginGroup("Panel");

    sArray.beginGroup("Configure");
    if(!sArray.contains("Destination")){
        sArray.setValue("Key",QKeySequence("Ctrl+Shift+P").toString());
        sArray.setValue("Destination","panel");
        sArray.setValue("Comment","Configure the panel");
    }
    sArray.endGroup();//Configure

    sArray.beginGroup("AddPlugins");
    if(!sArray.contains("Destination")){
        sArray.setValue("Key",QKeySequence("Ctrl+Shift+L").toString());
        sArray.setValue("Destination","panel");
        sArray.setValue("Comment","Add or remove plugins for panel");
    }
    sArray.endGroup();//AddPlugins

    sArray.endGroup();//Panel
}


//!-القائمة المنسدلة للوحة ---------------------------------------------- */
void WidgetPanel::customContextMenu(const QPoint & pos)
{

    QMenu menu(tr("Panel"));

    menu.addAction(m_actTitle);
    menu.addAction(m_actEditApps);
    menu.addAction(m_actCostumiz);
    menu.addSeparator();
    menu.addAction(m_actAddPlugins);
    menu.exec(mapToGlobal(pos));
    //  menu.popup(mapToGlobal(pos));
}


//! تحجيم اللوحة وارساؤها  ---------------------------------------------- */
void WidgetPanel::resizeEvent ( QResizeEvent * /*event*/ )
{
     moveToAllDesktop();
}

//______________________________________________________________________________
void WidgetPanel::resizePanel()
{

 QRect screen(QApplication::desktop()->geometry());
    //qDebug()<<"screen resized >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"<<screen;
    QRect rect;

    this->setMinimumHeight(m_panelHeight+10);
    this->setMaximumHeight(m_panelHeight+10);

    rect.moveLeft(screen.left());
    //  rect.moveTop(screen.top());
    if (m_Position == 0){
        this->setGeometry(screen.left(),screen.top(),screen.width(),m_panelHeight+10);
        rect = QRect(screen.left(),screen.top(),screen.width(),m_panelHeight+10);
        rect.moveTop(screen.top());
    } else {

        this->setGeometry(screen.left(),screen.bottom(),screen.width(),m_panelHeight+10);
        rect = QRect(screen.left(),screen.bottom(),screen.right(),m_panelHeight+10);
        rect.moveBottom(screen.bottom());
    }
     // qDebug()<<"screen resized >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"<<screen;
    // qDebug()<<screen.left()<<screen.bottom()<<screen.width()<<m_panelHeight+10;
    this->setGeometry(rect);
    int mScreenNum= QApplication::desktop()->primaryScreen();
    QRect desktop = QApplication::desktop()->screen(mScreenNum)->geometry();

    if (m_Position == 0){
        setStrut(  rect.bottom(), 0,
                   rect.left(), rect.right(),
                   0, 0
                   );
    }else{
        setStrut(   0, desktop.height() - rect.y(),
                    0, 0,
                    rect.left(), rect.right()
                    );

    }

    this->setGeometry(rect);
    this->resize(rect.width(),rect.height());
    qDebug()<<"widget panel resized================================="<<rect;

    setMaximumWidth(rect.width());
    m_rootWindow = QX11Info::appRootWindow();
}
//!تحميل اللوحة ------------------------------------------------------- */
void WidgetPanel::show()
{
    QWidget::show();

    moveToAllDesktop();
   // resizePanel();
}
//! -x11-اشارات مكتبة  -------------------------------------------------- */
void  WidgetPanel::x11EventFilter(XEvent* event)
{
  if(event->type==87){
      if(mIsCompositing!=QX11Info::isCompositingManagerRunning()){

     appliqueThemes();
 //qDebug()<<"event->type"<<event->type;

      }

//        if (event->type == PropertyNotify)
//        {

////            moveToAllDesktop();
////            resizePanel();
//        }
   }
    foreach (PluginsInterface *pluginsInterface, m_hatchPligins){
        if(pluginsInterface)
            pluginsInterface->x11EventFilter(event);
    }
}

//!تنفي1 محرر البرامج ---------------------------------------------- */
void WidgetPanel::showEditApplications()
{
    QProcess::startDetached("elokab-applications");
}

//! [1-0]تحميل نافذة تعديل اللوحة ---------------------------------- */
void WidgetPanel::costomizPanale()
{

     QString fileName=Edir::progPath("elokab-settings");
    QProcess prosess;
  //  QString p="elokab-settings PanelSetting";
    prosess.startDetached(fileName,QStringList()<<"PanelSetting");

}


//! [1-2]-تطبيق التعديلات على اللوحة -------------------------------------------- */
void WidgetPanel::appliqueChange()
{

    loadSetting();
  //  loadIconThemes();
    appliqueThemes();
    resizePanel();
    foreach (PluginsInterface *pluginsInterface, m_hatchPligins){
        if(pluginsInterface){
            pluginsInterface->setSize(QSize(m_panelHeight,m_panelHeight));

        }

    }

}
void WidgetPanel::appliqueThemes()
{
    QString radius;

    QString itemView=( QString("QAbstractItemView {activate-on-singleclick : %1; }")
                      .arg(false));

    mIsCompositing=QX11Info::isCompositingManagerRunning();
    if(mIsCompositing)
        radius="5";
    else
        radius="0";
    if(m_styleDefault==true){

          m_cssPath= ":/icons/defaultTheme.css";

          QFile file(m_cssPath);

      file.open( QFile::ReadOnly);
          QTextStream textStream(&file);
          textStream.setCodec(QTextCodec::codecForLocale());
          QString line;
          line = textStream.readAll();
          line.replace("border-radius:10","border-radius:"+radius);

          QString styleMenuBar= QString("\n QMenuBar {"
                                        "\n  min-height:%1 ;}" ).arg(m_panelHeight);


          ui->widgetBgr->setStyleSheet(line+styleMenuBar+itemView);
          file.close();


    }else{
          m_cssPath=Edir::configDir()+"/panelTheme.css";
        if(!QFile::exists(m_cssPath))
            m_cssPath=Edir::dataDir()+ "/themes/default/panelTheme.css";

        if(!QFile::exists(m_cssPath))
            m_cssPath= ":/icons/panelTheme.css";

        QFile file;
        file.setFileName(m_cssPath);
        if(file.open( QFile::ReadOnly)){
            QTextStream textStream(&file);
            textStream.setCodec(QTextCodec::codecForLocale());
            QString line;
            line = textStream.readAll();//premier line
            line.replace("border-radius:10","border-radius:"+radius);


            ui->widgetBgr->setStyleSheet(line+itemView);
            file.close();
        }

    }
}

//! [0] تحميل الاضافات عند البدأ ---------------------------------------------- */
void WidgetPanel::loadPlugins()
{
    ui->hLayoutTool->removeItem(horizontalSpacer);
    /************************************************
     تحميل منطقة الاعلام اولا اذا كانت موجوجدة على القائمة
     حتى يتم تحكيل البرامج التي تستخد منطقة الاعلام
     ************************************************/
    if(m_listPluginsFiles.contains("libdsystray.so")){
        addPlugin("libdsystray.so");
    }

    for(int i=0;i<m_listPluginsFiles.count();i++) {
        QString  fileName=  m_listPluginsFiles.at(i);

        if (fileName==ACT_SPACER) {  /*اضافة مباعد*/
            ui->hLayoutTool->addItem(horizontalSpacer);
        } else
            addPlugin(fileName);
    }
  //  resizePanel();
    refreshKeyShortkut();
    resizePanel();
}

void WidgetPanel::setShortcut(const QString &shortcut)
{
    qDebug()<<shortcut;
    if(shortcut==m_keyCostumiz){
        costomizPanale();
    }else if(shortcut==m_keyAddPlugins){
        addPluginsConfig();
    }else{

            pluginsSetKey(shortcut);
    }

}

void WidgetPanel::pluginsSetKey(QKeySequence sequence)
{

    foreach (PluginsInterface *pluginsInterface, m_hatchPligins){
        //                  qDebug()<<sequence;
        if(pluginsInterface)
            pluginsInterface->setKeySequence(sequence);
    }

}

void WidgetPanel::refreshKeyShortkut()
{
    //clear all shortcuts
//    m_listKey.clear();

    QSettings sArray("elokab","shortcut") ;

    //! begin the settings shortcut for panel

    sArray.beginGroup("Panel");

    sArray.beginGroup("Configure");

    m_keyCostumiz =( sArray.value("Key").toString());

    sArray.endGroup();//panel/Config

    sArray.beginGroup("AddPlugins");

    m_keyAddPlugins =( sArray.value("Key").toString());

    sArray.endGroup();//PanelAddPlugins

    sArray.endGroup();//Panel

    foreach (PluginsInterface *pluginsInterface, m_hatchPligins){
        //                  qDebug()<<sequence;
        if(pluginsInterface)
            pluginsInterface->keySequence();
    }

    //! begin the settings shortcut for Plugins panel

}

//! [1-0]تحميل نافذة تعديل الاضافات ---------------------------------------------- */



void WidgetPanel::addPluginsConfig()
{
    DialogAddPlugins *dlg  =new DialogAddPlugins(this);

    connect(dlg, SIGNAL(accepteChange()), this, SLOT(pluginsAppliqueChange()));
    dlg->show();
}
//! [1-1]-تطبيق تغييرات الاضافات ---------------------------------------------- */
void WidgetPanel::pluginsAppliqueChange()
{
    QStringList listTemp;
    QSettings setting("elokab","settings");
    setting.beginGroup("Panel-Style");
    //          listTemp=setting.value("listPluginsFiles",listTemp<< "libdmainmenu.so"<<"libdquicklaunch.so"  <<"libdtaskbar.so"
    //                                 <<ACT_SPACER<<"libdswichdesktop.so"<<"libdsystray.so"<<"libdtimer.so").toStringList();
    listTemp=setting.value("listPluginsFiles",listTemp<< "libdmainmenu.so"<<"libdquicklaunch.so"  <<"libdtaskbar.so"
                           <<"libdswichdesktop.so"<<"libdsystray.so"<<"libdtimer.so").toStringList();

    setting.endGroup();
    ui->hLayoutTool->removeItem(horizontalSpacer);

    foreach(QString file,m_listPluginsFiles){     /*للازالة*/

        if(!listTemp.contains(file)){

            removePlugin(file);
            int d=m_listPluginsFiles.indexOf(file);
            m_listPluginsFiles.takeAt(d);

        }
    }

    foreach(QString file,listTemp){                 /*للاضافة*/
        if(file==ACT_SPACER){
            ui->hLayoutTool->addItem(horizontalSpacer);
        }else  if(!m_listPluginsFiles.contains(file)){
            addPlugin(file);
            m_listPluginsFiles.append(file);
        }else{

            pluginsInterface=     m_hatchPligins.value(file);
            if(pluginsInterface)
                pluginsInterface  ->setLayout(ui->hLayoutTool);
        }
    }

}
//! [1-2] -حذف مكون من اللوحة ---------------------------------------------- */
void WidgetPanel::removePlugin(const QString &pfile)
{
    if(pfile==ACT_SPACER){
        ui->hLayoutTool->removeItem(horizontalSpacer);
        return;
    }
    /* معرفة مسار المكتبات*/

    QPluginLoader pluginLoader(pluginsDir.absoluteFilePath(pfile));
    pluginLoader.load();
    if( pluginLoader.isLoaded()){
        QObject *plugin = pluginLoader.instance();
        if (plugin) {
            m_hatchPligins.remove(pfile);
            delete plugin;//مسحها من الذاكرة
        }
    }

    pluginLoader.unload();//انهاء التحميل
}
//! [1-3]-اضافة مكون الى اللوحة---------------------------------------------- */
void WidgetPanel::addPlugin(QString pfile)
{
    QList<QAction*>list;

    QPluginLoader pluginLoader(pluginsDir.absoluteFilePath(pfile));

    pluginLoader.load();
    if( pluginLoader.isLoaded()){
        QObject *plugin = pluginLoader.instance();
        if (plugin) {
            pluginsInterface    = qobject_cast<PluginsInterface *>(plugin);

            if (pluginsInterface){

                m_hatchPligins.insert(pfile,pluginsInterface);
                pluginsInterface->setSize(QSize(m_panelHeight,m_panelHeight));
                pluginsInterface->setLayout(ui->hLayoutTool);//اضافتها الى اللوحة
               pluginsInterface->setActions(list<<m_actTitle<<m_actEditApps<<m_actCostumiz<<m_actAddPlugins);
                pluginsInterface->themeChanged();
                       QString t(60-pfile.length(),'-');

                qDebug()<<pfile+t<<"Ok";


            }//pluginsInterface

        }else{
             QString t(60-pfile.length(),'-');
              qDebug()<<pfile<<t<<"Failed";
            }// pluginLoader
    }else{
        QString t(60-pfile.length(),'-');
         qDebug()<<pfile<<t<<"pluginFailed";
       }// pluginLoader

}

void WidgetPanel::setMenuTiTle(const QString &text)
{
     //---------toolbox style---------------------------------

     QString colorHighlighName=    this->palette().color(QPalette::Highlight).name();
     //    " background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 %4, stop:0.172222 %4, stop:0.177778 rgba(255, %255, 255, 0), stop:0.65 rgba(255, 255, 255, 0));"

     QString stylbox=QString("QLabel {"
                             " background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 %1, stop:0.172222 %1, stop:0.177778 transparent, stop:0.65 transparent);"
                             "color:black;"
                             "border-radius: 4px;}"
                             ).arg(colorHighlighName)  ;

     m_actTitle = new QWidgetAction(this);
     QFont font = m_actTitle->font();
     titleButton = new QLabel(this);
     titleButton->setAlignment(Qt::AlignCenter);
     //   titleButton->setDefaultAction(m_actTitle);

     //    titleButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     m_actTitle->setEnabled(false);
     titleButton->setStyleSheet(stylbox);
     titleButton->setText(text);
     m_actTitle->setText(text);
     font.setBold(true);
     titleButton->setFont(font);

     m_actTitle->setDefaultWidget(titleButton);
}

/*-----------------------------------
    this code is part of the razor-qt
     http://razor-qt.org
  -----------------------------------*/

void WidgetPanel::moveToAllDesktop()
{

      XClientMessageEvent msg;
    msg.window = this->winId();
    msg.type = ClientMessage;
    msg.message_type = atom("_NET_WM_DESKTOP");
    msg.send_event = true;
    msg.display = QX11Info::display();
    msg.format = 32;
    msg.data.l[0] = -1;

    XSendEvent(QX11Info::display(), QX11Info::appRootWindow(), FALSE, (SubstructureRedirectMask | SubstructureNotifyMask) , (XEvent *) &msg);

}
/*-----------------------------------
    this code is part of the razor-qt
     http://razor-qt.org
  -----------------------------------*/
Atom WidgetPanel::atom(const char* atomName)
{
    static QHash<QString, Atom> hash;

    if (hash.contains(atomName))
        return hash.value(atomName);

    Atom atom = XInternAtom(QX11Info::display(), atomName, false);
    hash[atomName] = atom;
    return atom;
}
/*-----------------------------------
    this code is part of the razor-qt
     http://razor-qt.org
  -----------------------------------*/
void WidgetPanel::setStrut(int top,  int bottom,
                           int topStartX,    int topEndX,
                           int bottomStartX, int bottomEndX )

{

    unsigned long desstrut[12];
    memset(desstrut,0,sizeof(desstrut));

    desstrut[0] = 0;
    desstrut[1] = 0;
    desstrut[2] = top;
    desstrut[3] = bottom;

    desstrut[4] = 0;
    desstrut[5] = 0;
    desstrut[6] = 0;
    desstrut[7] = 0;
    desstrut[8] = topStartX;
    desstrut[9] = topEndX;
    desstrut[10] = bottomStartX;
    desstrut[11] = bottomEndX;


    XChangeProperty(QX11Info::display(), this->winId() , atom("_NET_WM_STRUT_PARTIAL"),
                    XA_CARDINAL, 32, PropModeReplace,  (unsigned char *) desstrut, 12  );


    XChangeProperty(QX11Info::display(), this->winId(), atom("_NET_WM_STRUT"),
                    XA_CARDINAL, 32, PropModeReplace, (unsigned char*) desstrut, 4);
}

//----------------------------------------------------------------
