/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef WIDGETPANEL_H
#define WIDGETPANEL_H
#include <MenuGen>

#include "../dpluginsinterface.h"

#include <ElokabSettings>
#include <X11/Xlib.h>
#include <X11/X.h>
#include <QWidget>
#include <QTimer>
#include <QLabel>
#include <QMenu>

namespace Ui {
class WidgetPanel;
}

class WidgetPanel : public QFrame
{
        Q_OBJECT

public:
        explicit WidgetPanel(QWidget *parent = 0);
        ~WidgetPanel();
        void  x11EventFilter(XEvent* event);
        void show();
protected :

    void paintEvent(QPaintEvent *);
    void resizeEvent (QResizeEvent *);
private:
        Ui::WidgetPanel *ui;

        Window m_rootWindow;
        PluginsInterface *pluginsInterface;
        MenuGen *gen ;

        QHash<QString, PluginsInterface*>m_hatchPligins ;/// قائمة للاضافات المستعملة
        QStringList m_listPluginsFiles;                   /// قائمة بمسارات الاضافات المستعملة
        ElokabSettings *m_settings;                       /// اعدادات السمات
     //   elokabSettings *m_shortcutsSettings;              /// اعدادات الاختصارات
       QAction *m_actEditApps;
        QAction *m_actCostumiz;                           /// زر التحرير الاعدادات
        QMenu *m_actShudown;                              /// زر اغلاق الجهاز
        QMenu *m_mnuShudown;                              /// قائمة ازرار الاغلاق
        QAction *m_actLogout;                             /// زر الخروج
        QAction *m_actLokScreen;                          /// زر لقفل الشاشة
        QAction *m_actAddPlugins ;                         /// زر لاضافة و حذف الاضافات
QWidgetAction *m_actTitle;
QLabel *titleButton;
        QString  m_keyAddPlugins ;             /// اختصار اضافة ازالة الاضافات
        QString  m_keyCostumiz ;               /// اختصار زر الاعدادات
        //QxtGlobalShortcut  *m_keyMainMenu ;               /// اختصار زر القائمة الرئيسية
        QList<QString >m_listKey;
        int m_panelHeight;                    /// حجم البنال
        QString m_pathApp;                  /// مسار البرنامج
        QDir pluginsDir;                    /// مسار الاضافات
        QString  m_iconsTheme;              /// سمة الايقونات
        bool    m_styleDefault;             /// الشكل الافتراضي للادوات
        int m_Position;                    /// موضع البنال 0=فوق غيرها=تحت
        QSpacerItem  *horizontalSpacer ;    /// المباعد بين الاضافات
QString m_cssPath;

//WidgetPanelConf   *widgetPanelConf;
bool mIsCompositing;
private slots:
        //! تحميل الاعدادات
        void loadSetting();
        //! انشاء سمة الادوات
      //  void crateStyle();
        //!  تحميل سمة الايقونات
        void loadIconThemes();
        //! القئمة المنسدلة
        void customContextMenu(const QPoint &pos);
        //! تحجيم البنال الى موضعه من الشاشة
        void resizePanel();
        //! تحميل كل الادوات
        void setupActions();
        //! اشارة لكل اضافة بالاختصار
        void pluginsSetKey(QKeySequence sequence);
        //! نافذة اضافة ازالة الاضافات
        void addPluginsConfig();
        //! حذف اضافة من البنال
        void removePlugin(const QString &pfile);
        //! اضافة اضافة
        void addPlugin(QString pfile);
        //!  تحميل الاضافات من المسارات
        void loadPlugins();
        //! تطبيق التغيرات
        void pluginsAppliqueChange();
        //! اعادة تكوين الاختصارات
        void refreshKeyShortkut();
       void setShortcut(const QString &shortcut);
        //! نافذة الاعدادات
        void costomizPanale();

        //!  تطبيق التغييرات
        void appliqueChange();
        void appliqueThemes();
 //!  نافذة تحرير النطبيقات
        void showEditApplications();
//! عنوان على القائمة المنسدلة
        void setMenuTiTle(const QString &text);

        //! Atom
        Atom atom(const char* atomName);
        //! نقل النافذة الى جميع اسطح المكتب
        void moveToAllDesktop() ;
        /**
         * @brief setStrut اعدادات الموضع لكل جهة من النافذة
         * تم تجاهل اليمين واليسار لانها تأخذ دائما قيمة الصفر
         * لاعتمادنا على وضع اللوحة فوق وتحت فقط
         */
        void setStrut(int top,  int bottom,
                      int topStartX,    int topEndX,
                      int bottomStartX, int bottomEndX
                      );

};
class PopupMenu: public QMenu
{
         Q_OBJECT
public:
    explicit PopupMenu(QWidget *parent = 0): QMenu(parent) {}
    explicit PopupMenu(const QString &title, QWidget *parent = 0): QMenu(title, parent) {}

    QAction* addTitle(const QIcon &icon, const QString &text);
    QAction* addTitle(const QString &text);




protected:

};
#endif // WIDGETPANEL_H
