/* Compal Laptop Control
 * Author: Krzemin <pkrzemin@o2.pl>
 * WWW: http://krzemin.iglu.cz
 * This software is distributed under the GNU General Public License.
 */

#include "compal.h"
#include <QDir>
#include <QDebug>
bool CompalLaptop::init()
{
	// Checking modules' list

	QFile modules("/proc/modules");
	if(!modules.open(QIODevice::ReadOnly | QIODevice::Text))
	{
        return false;
	}

	// Detecting files
	if(QFile::exists("/sys/devices/platform/lcd_level"))
	{
		lcdCurrentFile = "/sys/devices/platform/lcd_level";
		lcdSetFile = "/sys/devices/platform/lcd_level";
	}
	else
	// Since compal-laptop 0.2.6 we're using backlight class...
	if(QFile::exists("/sys/class/backlight/compal-laptop/brightness"))
	{
		lcdSetFile = "/sys/class/backlight/compal-laptop/brightness";
		lcdCurrentFile = "/sys/class/backlight/compal-laptop/brightness";

		if(QFile::exists("/sys/class/backlight/compal-laptop/actual_brightness"))
		{
			lcdCurrentFile = "/sys/class/backlight/compal-laptop/actual_brightness";
		}

		if(QFile::exists("/sys/class/backlight/compal-laptop/max_brightness"))
		{
			lcdMaxFile = "/sys/class/backlight/compal-laptop/max_brightness";
		}
	}
	else
    // /*since kernel 2.6.28*/
	if(QFile::exists("/sys/class/backlight/acpi_video0/brightness"))
	{
		lcdSetFile = "/sys/class/backlight/acpi_video0/brightness";
		lcdCurrentFile = "/sys/class/backlight/acpi_video0/brightness";

		if(QFile::exists("/sys/class/backlight/acpi_video0/actual_brightness"))
		{
			lcdCurrentFile = "/sys/class/backlight/acpi_video0/actual_brightness";
		}

		if(QFile::exists("/sys/class/backlight/acpi_video0/max_brightness"))
		{
			lcdMaxFile = "/sys/class/backlight/acpi_video0/max_brightness";
		}
    }
    // /*since kernel > 2.6.28*/
    else
    {
        QDir dirs("/sys/class/backlight");
        qDebug()<<dirs.absolutePath();
        QStringList list=dirs.entryList(QDir::AllDirs|QDir::NoDotAndDotDot);
        foreach (QString s, list) {
            if(QFile::exists(dirs.absoluteFilePath(s)+"/brightness")){
                lcdSetFile = dirs.absoluteFilePath(s)+"/brightness";
                lcdCurrentFile = dirs.absoluteFilePath(s)+"/brightness";

                if(QFile::exists(dirs.absoluteFilePath(s)+"/actual_brightness"))
                {
                    lcdCurrentFile = dirs.absoluteFilePath(s)+"/actual_brightness";
                }

                if(QFile::exists(dirs.absoluteFilePath(s)+"/max_brightness"))
                {
                    lcdMaxFile = dirs.absoluteFilePath(s)+"/max_brightness";
                }

            }
            qDebug()<<"lcdCurrentFile;"<<lcdCurrentFile;
            qDebug()<<"lcdCurrentFile:"<<lcdMaxFile;

        }
    }

	return true;
}


bool CompalLaptop::isLcdAvailable()
{
    return true;//QFileInfo(lcdSetFile).isWritable();
}


int CompalLaptop::lcdStatus()
{
	return readFromFile(lcdCurrentFile);
}

int CompalLaptop::lcdMax()
{
	return readFromFile(lcdMaxFile);
}

void CompalLaptop::setLcdStatus(int status)
{
	writeToFile(lcdSetFile, status);
}


int CompalLaptop::readFromFile(const QString & file)
{
	if(file.isEmpty())
	{
		return -1;
	}

	QFile f(file);

	if(!f.open(QIODevice::ReadOnly | QIODevice::Text))
	{
		return -1;
	}

	int val = ((QString)(f.readLine())).toInt();

	f.close();
	return val;
}

void CompalLaptop::writeToFile(const QString & file, int value)
{
	if(file.isEmpty())
	{
		return;
	}

	QFile f(file);

	if(!f.open(QIODevice::WriteOnly | QIODevice::Text))
		return;

	QTextStream out(&f);

	out << value;
	f.close();
}
