/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>
#include <QButtonGroup>
#include <QToolButton>
#include <QWheelEvent>
#include <QtDebug>
#include <QSignalMapper>

#include "x11utils/xdesktoputils.h"
#include "ddswitch.h"

//_________________________________________________________________________________
DdSwitch::DdSwitch( QWidget* parent)
          : QWidget( parent),
            m_pSignalMapper(new QSignalMapper(this)),
            m_DeskCount(1)
{
    this->setObjectName("DesktopSwitch");
    this->setWindowTitle(trUtf8("Desktop Switch"));
    this->setWindowIcon(QIcon( ":/icons/panel-workspace-switcher"));
    this->setWhatsThis(tr("Desktop Switch... by AbouZakaria"));//شرح الاضافة

    m_GroupBtns = new QButtonGroup(this);

    connect ( m_pSignalMapper, SIGNAL(mapped(int)), this, SLOT(setDesktop(int)));

    setSizePolicy(QSizePolicy::Maximum,QSizePolicy::Maximum);
    m_gridLayout = new QGridLayout(this);
    m_gridLayout->setSpacing(1);
    m_gridLayout->setContentsMargins(2, 0, 2, 0);
    m_gridLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
    setContentsMargins(2, 0, 2, 0);

    m_size=QSize(16,16);
    setupBtns();

}

//__________________________________________________________________________________
void DdSwitch::setupBtns()
{

    foreach (QAbstractButton * b, m_GroupBtns->buttons())
    {

        m_pSignalMapper->removeMappings(b);
        m_GroupBtns->removeButton(b);
        delete b;

    }

    if(parentH>26&&m_DeskCount>1){
        int x=parentH/2;
        m_size=QSize(x,x);

    }else{
        m_size= QSize(parentH,parentH);;
    }

    int row=0;
    int colum=0;
    int curcolum=0;
    qDebug()<<"m_DeskCount"<<m_DeskCount;
    for (int i = 0; i < m_DeskCount; ++i)
    {

        ToolButton * btn = new ToolButton;
        btn->setText(QString::number(i+1));
        btn->setCheckable(true);
        btn->setToolTip( tr("Desktop %1").arg(i+1));
        m_pSignalMapper->setMapping(btn, i);
        connect(btn, SIGNAL(activated()), m_pSignalMapper, SLOT(map())) ;
        btn->setMaximumSize(m_size);
        m_gridLayout->addWidget(btn,row,colum);

        curcolum++;
        if(parentH >26&&m_DeskCount>1){
            if(row==0)row=1; else row=0;

            if(curcolum==2){
                curcolum=0;
                colum++;

            }
        }else{
            colum++;
        }

        m_GroupBtns->addButton(btn, i);
    }

    int activeDesk = qMax(XDesktop::active(), 0);
    QAbstractButton * button = m_GroupBtns->button(activeDesk);
    if (button)
        button->setChecked(true);

    connect(m_GroupBtns, SIGNAL(buttonClicked(int)),
            this, SLOT(setDesktop(int)));

    realign();

}

DdSwitch::~DdSwitch()
{

}

//_________________________________________________________________________________
void DdSwitch::setDesktop(int desktop)
{

     XDesktop::setCurrent(desktop);
      int activeDesk = qMax(XDesktop::active(), 0);
       m_GroupBtns->button(activeDesk)->setChecked(true);

}

//__________________________________________________________________________________
void DdSwitch::wheelEvent(QWheelEvent* e)
{
          int max = XDesktop::count()- 1;
          int delta = e->delta() < 0 ? 1 : -1;
          int current = XDesktop::active()+ delta;

          if (current > max)
                    current = 0;
          else if (current < 0)
                    current = max;

        XDesktop::setCurrent(current);
}

void DdSwitch::realign()
{


          foreach (QAbstractButton *btn, m_GroupBtns->buttons())
          {

                    btn->setMaximumWidth(m_size.width());
                    btn->setMinimumWidth(m_size.width());
                    btn->setMinimumHeight(m_size.width());

          }


}

/*************************Virtual**********************************/
void DdSwitch::setKeySequence(QKeySequence sequence)
{
    QKeySequence ksNext(mNextShort);
    if(ksNext==sequence){
           goDesktop(+1);
    }
    QKeySequence ksPrev(mPrevShort);
    if(ksPrev==sequence){
            goDesktop(-1);
    }
}
void DdSwitch::goDesktop(int arg)
{
          int max = XDesktop::count()- 1;

          int current = XDesktop::active()+arg;

          if (current > max)
                    current = 0;
          else if (current < 0)
                    current =max;

//          qDebug()<<current<<max;
        XDesktop::setCurrent(current);
}

//______________________________________________________________________________
QString DdSwitch::  keySequence()
{


              QSettings settings("elokab","shortcut");

              settings.beginGroup("Panel-Plugins");

              settings.beginGroup("NextDesktop");
              if(!settings.contains("Destination")){
                  settings.setValue("Key",QKeySequence("Ctrl+Meta+Left").toString());
                  settings.setValue("Destination","panel");
                  settings.setValue("Comment","Switch next desktop...");
              }
              mNextShort= settings.value("Key","Ctrl+Meta+Left").toString();
              settings.endGroup();//NextDesktop

              settings.beginGroup("PrevDesktop");
              if(!settings.contains("Destination")){
                  settings.setValue("Key",QKeySequence("Ctrl+Meta+Right").toString());
                  settings.setValue("Destination","panel");
                  settings.setValue("Comment","Switch previous desktop...");
              }
              mPrevShort= settings.value("Key","Ctrl+Meta+Right").toString();
              settings.endGroup();//PrevDesktop

              settings.endGroup();//Panel-Plugins

              return "";
}

//_____________________________________________________________________
void DdSwitch::x11EventFilter(XEvent* _event)
{
    if (_event->type == PropertyNotify)
    {
        int count = qMax(XDesktop::count(), 1);

        if (m_DeskCount != count)
        {
            //  qDebug() << "Desktop count changed from" << m_DeskCount << "to" << count;
            m_DeskCount = count;
            m_desktopNames = XDesktop::names();

            setupBtns();
        }

        if (m_desktopNames != XDesktop::names())
        {
            m_desktopNames = XDesktop::names();

            setupBtns();
        }

        int activeDesk = qMax(XDesktop::active(), 0);
        m_GroupBtns->button(activeDesk)->setChecked(true);
    }

}

//______________________________________________________________________
void DdSwitch::setLayout(QHBoxLayout *p)
{
          p->addWidget(this);
}

void DdSwitch::setSize(QSize size)
{
    qDebug()<<"m_DeskCount"<<m_DeskCount;
        parentH=size.width();

          m_size= size;

          setupBtns();

}

Q_EXPORT_PLUGIN2(dpanel_switch, DdSwitch)
