/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "btndesktop.h"
#include <EDir>
#include <QMimeData>
#include <QDrag>
#include <QMouseEvent>
#include <QUrl>
#include <QAction>
#include <QMenu>
#include <QFileInfo>

#define NO_FAVO 0
#define AD_FAVO 1
#define RM_FAVO 2
BtnDesktop::BtnDesktop(int favo, QWidget *parent) :
    QToolButton(parent),m_isfavo(favo)
{
   setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
   setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Fixed);
 setIconSize(QSize(24,24));
   setAcceptDrops(true);
   setObjectName("btnMenu");
    setContextMenuPolicy(Qt::CustomContextMenu);
   connect(this, SIGNAL(customContextMenuRequested(const QPoint&)),
           this, SLOT(this_customContextMenuRequested(const QPoint&)));

}


void BtnDesktop::mousePressEvent(QMouseEvent *e)
{
          //

//          if (e->button() == Qt::LeftButton && e->modifiers() == Qt::ControlModifier)
//          {
//                    m_dragStart = e->pos();
//                    return;
//          }

         QToolButton::mousePressEvent(e);
}

void BtnDesktop::mouseMoveEvent(QMouseEvent *e)
{
          if (!(e->buttons() & Qt::LeftButton))
          {
                    return;
          }

//          if ((e->pos() - m_dragStart).manhattanLength() < QApplication::startDragDistance())
//          {
//                    return;
//          }

//          if (e->modifiers() != Qt::ControlModifier)
//          {
//                    return;
//          }

          QDrag *drag = new QDrag(this);
          QMimeData *mimeData = new QMimeData();
          QList<QUrl> list;
          list.append(QUrl(m_mimeType));
          mimeData->setUrls(list);
         drag->setMimeData(mimeData);
          drag->exec(Qt::MoveAction);

          // Icon was droped outside the panel, remove button

}

void BtnDesktop::dragMoveEvent(QDragMoveEvent * /*e*/)
{
//          if (e->mimeData()->hasFormat(MIMETYPE))
//                    e->acceptProposedAction();
//          else
//                    e->ignore();
}

void BtnDesktop::dragEnterEvent(QDragEnterEvent */*e*/)
{
//          if (e->mimeData()->hasFormat(MIMETYPE))
//          {
//                    int other = e->mimeData()->data(MIMETYPE).toInt();
//                    // do not replace self with self
//                    if (other == m_id)
//                              return;

//                    emit switchButtons(m_id, other);
//          }
}
void BtnDesktop::this_customContextMenuRequested(const QPoint & /*pos*/)
{


        if(m_isfavo==NO_FAVO)return;
        QMenu menu;

        QAction *actOpen=new QAction(this);
        QAction *actFavo=new QAction(this);
        QAction *actDesktop=new QAction(this);
        //QAction *actQuikL=new QAction(this);
        actDesktop->setText(trUtf8("add to Desktop"));
        actOpen->setText(this->text().section("\n",0,0));
        if(m_isfavo==AD_FAVO){
                actFavo->setText(trUtf8("add to favorite"));

                //  actQuikL->setText(trUt	f8("add to Quick Launche"));
                connect(actFavo,SIGNAL(triggered()),this,SLOT(addThisToFavo()));

        }else if(m_isfavo==RM_FAVO){
                actFavo->setText(trUtf8("remove from favorite"));
                connect(actFavo,SIGNAL(triggered()),this,SLOT(removeThisfromFavo()));
        }
        connect(actDesktop,SIGNAL(triggered()),this,SLOT(addThisToDesktop()));
        connect(actOpen,SIGNAL(triggered()),this,SIGNAL(clicked()));

        menu.addAction(actOpen);
        menu.setDefaultAction(actOpen);

        menu.addSeparator();
        menu.addAction(actFavo);

        menu.addAction(actDesktop);
        //menu.addAction(actQuikL);
        menu.exec(QCursor::pos());

}
void BtnDesktop::addThisToFavo()
{


     emit addBtnToFavo( m_mimeType);

}

void BtnDesktop::removeThisfromFavo()
{
      emit removeBtnFromFavo(this);
}

void BtnDesktop::addThisToDesktop()
{
        QFileInfo fi=m_mimeType;
        QFile::copy(m_mimeType,Edir::desktopDir()+"/"+fi.fileName());
}

 void BtnDesktop::setIconMode(bool mode)
 {
     QString txt=defaultAction()->text();
     QString tip=defaultAction()->toolTip();
     if(mode){
        setText(txt);
        setToolTip(tip);

         setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
         setIconSize(QSize(64,64));
         setMaximumWidth(64+30);
         setMinimumWidth(64+20);

     }else{
         if(txt==tip){
             setText(txt);
             setToolTip(tip);
         }else{
             setText(txt+"\n"+tip);
             setToolTip(txt);
         }
         setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
         setMaximumWidth(360);
         setIconSize(QSize(24,24));
     }
 }
