/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef BTNDESKTOP_H
#define BTNDESKTOP_H

#include <QToolButton>
/**
 * @brief The BtnDesktop class خاصة بزر القائمة الرئيسية
 */
class BtnDesktop : public QToolButton
{
    Q_OBJECT
public:
    /**
     * @brief BtnDesktop زر التطبيق
     * @param favo  التعامل مع المفضلة
     * NO_FAVO 0 لايضاف
     * AD_FAVO 1 يصاف
     * RM_FAVO 2 يحذف
     * @param parent
     */
    explicit BtnDesktop(int favo,QWidget *parent = 0);



signals:
    /**
     * @brief addBtnToFavo
     * @param url
     */
    void addBtnToFavo(QString url);
    /**
     * @brief removeBtnFromFavo
     * @param w
     */
    void removeBtnFromFavo(BtnDesktop *w);
protected:

    /**
     * @brief mousePressEvent
     * @param e
     */
    void mousePressEvent(QMouseEvent *e);
    /**
     * @brief mouseMoveEvent
     * @param e
     */
    void mouseMoveEvent(QMouseEvent *e);
    /**
     * @brief dragEnterEvent
     * @param e
     */
    void dragEnterEvent(QDragEnterEvent *);
    /**
     * @brief dragMoveEvent
     * @param e
     */
    void dragMoveEvent(QDragMoveEvent *);

public slots:
    /**
     * @brief setData
     * @param Data
     */
    void setData(QString Data)
    {
        m_data=Data;
    }
    /**
     * @brief mydata
     * @return
     */
    QString mydata()
    {
        return m_data;
    }
    /**
     * @brief iconName
     * @return
     */
    QString iconName()
    {
        return m_icon;
    }
    /**
     * @brief mimeType
     * @return
     */
    QString mimeType()
    {
        return m_mimeType;
    }
    /**
     * @brief setMimType
     * @param mime
     */
    void setMimType(QString mime)
    {
        m_mimeType=mime;
    }
    /**
     * @brief setFavo
     * @param favo
     */
    void setFavo(int favo)
    {
        m_isfavo=favo;
    }
    /**
     * @brief setIconName
     * @param icon
     */
    void setIconName(const QString &icon)
    {
        m_icon=icon;

    }
    /**
     * @brief this_customContextMenuRequested
     */
    void this_customContextMenuRequested(const QPoint &);
    /**
     * @brief addThisToFavo
     */
    void addThisToFavo();
    /**
     * @brief removeThisfromFavo
     */
    void removeThisfromFavo();
    /**
     * @brief addThisToDesktop
     */
    void addThisToDesktop();

    void setIconMode(bool mode);
private:
    QString m_data;
    QString m_icon;
    int m_isfavo;
    QString m_mimeType;


private slots:

};

#endif // BTNDESKTOP_H
