/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "dialogconfigmainmenu.h"
#include <EIcon>
#include "ui_dialogconfigmainmenu.h"
#include <QFileDialog>
#include <QSettings>
#include <QDebug>


DialogConfigMainMenu::DialogConfigMainMenu(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DialogConfigMainMenu)
{
    ui->setupUi(this);
          this->setAttribute(Qt::WA_DeleteOnClose,true);
    ui->toolButtonDefault->setIcon(EIcon::fromTheme("document-revert"));
    ui->toolButtonSystemIcon->setIcon(EIcon::fromTheme("document-revert"));
    loadSetting();
}

DialogConfigMainMenu::~DialogConfigMainMenu()
{
        qDebug()<<"DialogConfigMainMenu() menu settings Ui deleted";
    delete ui;
}
void DialogConfigMainMenu::loadSetting()
{

          QSettings setting("elokab","settings");
          setting.beginGroup("MainMenu");

          ui->comboBoxButtonStyle->setCurrentIndex(setting.value("ButtonStyle",0).toInt());
          ui->lineEditButtonName->setText(setting.value("ButtonText",trUtf8("Start Here")).toString().trimmed());
          QString iconName=setting.value("ButtonIcon",":/icons/start-here.png").toString();

          if(QFile::exists(iconName)){
                    ui->toolButtonIcon->setIcon(QIcon(iconName));
          }else{
                    ui->toolButtonIcon->setIcon(EIcon::fromTheme(iconName,"start-here"));
          }
          ui->lineEditIcon->setText(iconName);
          m_iconName=iconName;
          ui->checkBoxImage->setChecked(setting.value("IsCostumImage",false).toBool());
         ui->lineEditNormalImage->setText(setting.value("NormalImage").toString());
          ui->lineEditHoverImage->setText(setting.value("HoverImage").toString());
          setting.endGroup();

}
void DialogConfigMainMenu::saveSetting()
{

          QSettings setting("elokab","settings");
          setting.beginGroup("MainMenu");

          setting.setValue("ButtonStyle", ui->comboBoxButtonStyle->currentIndex());
          setting.setValue("ButtonText",ui->lineEditButtonName->text()+"  ");
          setting.setValue("ButtonIcon",m_iconName);
          setting.setValue("NormalImage",ui->lineEditNormalImage->text());
           setting.setValue("HoverImage",ui->lineEditHoverImage->text());
           setting.setValue("IsCostumImage",ui->checkBoxImage->isChecked());
          setting.endGroup();
}

void DialogConfigMainMenu::on_toolButtonIcon_clicked()
{
    QFileDialog dlg;

    QString fn = dlg.getOpenFileName(this, tr("Open Images Files..."),
                                     QDir::homePath(), trUtf8("Image File (*.png );;All Files (*)"));
    if (!fn.isEmpty())
    {
              ui->toolButtonIcon->setIcon(QIcon(fn));
              m_iconName=fn;
               ui->lineEditIcon->setText(fn);

    }
}

void DialogConfigMainMenu::on_toolButtonDefault_clicked()
{
    ui->toolButtonIcon->setIcon(QIcon(":/icons/start-here.png"));
    ui->lineEditIcon->setText(":/icons/start-here.png");
    m_iconName=":/icons/start-here.png";
}

void DialogConfigMainMenu::on_lineEditIcon_textChanged(const QString &arg1)
{
    QIcon icon;
    icon=EIcon::fromTheme(arg1);
    if(QFile::exists(arg1)){
        icon=QIcon(arg1);
         ui->toolButtonIcon->setIcon(icon);
           m_iconName=arg1;
    }else if(!icon.isNull()){

         ui->toolButtonIcon->setIcon(icon);
           m_iconName=arg1;
    }
}

void DialogConfigMainMenu::on_buttonBox_accepted()
{
    saveSetting();
    emit settingChanged();
}

void DialogConfigMainMenu::on_toolButtonSystemIcon_clicked()
{
    ui->toolButtonIcon->setIcon(EIcon::fromTheme("start-here"));
    ui->lineEditIcon->setText("start-here");
    m_iconName="start-here";
}

void DialogConfigMainMenu::on_toolButtonImage_clicked()
{
    QFileDialog dlg;

    QString fn = dlg.getOpenFileName(this, tr("Open Images Files..."),
                                     QDir::homePath(), trUtf8("Image File (*.png );;All Files (*)"));
    if (!fn.isEmpty())
    {

               ui->lineEditNormalImage->setText(fn);

    }
}

void DialogConfigMainMenu::on_toolButtonHoverImage_clicked()
{
    QFileDialog dlg;

    QString fn = dlg.getOpenFileName(this, tr("Open Images Files..."),
                                     QDir::homePath(), trUtf8("Image File (*.png );;All Files (*)"));
    if (!fn.isEmpty())
    {

               ui->lineEditHoverImage->setText(fn);

    }
}
