/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "formmainmenu.h"
#include "btndesktop.h"
#include "page.h"
#include "ui_formmainmenu.h"
#include <QToolButton>
#include <QListWidgetItem>

#include <QFont>
#include <EIcon>
#include <EMimIcon>
#include <DesktopFile>
#include <QUrl>
#include <QFile>
#include <QDir>
#include <QDebug>
#include <QProcess>
#include <QSettings>
#include "itemdelegate.h"
#include <QIconEngine>

#define NO_FAVO 0
#define AD_FAVO 1
#define RM_FAVO 2

FormMainMenu::FormMainMenu(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::FormMainMenu)
{
    ui->setupUi(this);

    myModel = new MenuModel(this);
    myListView=new QListView;
    myListView->setModel(myModel);
    myListView->setObjectName("listWidgetMenu");
    myListView-> setFrameShape(QFrame::NoFrame);
     myListView->setDragDropMode(QAbstractItemView::DragDrop);
      myListView->setDefaultDropAction(Qt::MoveAction);
     myListView->setDropIndicatorShown(true);
     myListView->setResizeMode(QListView::Adjust);
    ui->stackedWidget->addWidget(myListView);
    myListView->setItemDelegate(new ItemDelegate(this));
    myListView->setContextMenuPolicy(Qt::CustomContextMenu);
    listSelectionModel=myListView->selectionModel();
    connect(ui->lineEdit,SIGNAL(textChanged(QString)),this,SLOT(goFindApp(QString)));
    connect(ui->tBnIconsView,SIGNAL(toggled(bool)),this,SLOT(switchMode(bool)));
    connect(ui->listWidgetMenu,SIGNAL(currentRowChanged(int)),this,SLOT(setCurrentRow(int)));
    connect(myListView,SIGNAL(activated(QModelIndex)),this,SLOT(indexClicked(QModelIndex)));
    connect(myListView,SIGNAL(customContextMenuRequested(QPoint)),this,SLOT(showContextMenu(QPoint)));
    connect(myListView,SIGNAL(clicked(QModelIndex)),this,SLOT(indexClicked(QModelIndex)));

    mParent=parent;

    mMSystem=new MenuSystem;
    mMProgrammes=new MenuProgrammes(this);
    mMFolders=new MenuFolders;
     mMSettingsApps=new MenuSettingsApps;
    //--------------By Order-------------
    connect(mMProgrammes,SIGNAL(menuRecharged()),this,SLOT(setupMenusAppDesktop()));

    QSettings setting("elokab","settings");
    setting.beginGroup("MainMenu");
    iconMode=  setting.value("IconsView",false).toBool();
    setting.endGroup();
    ui->tBnIconsView->setChecked(iconMode);
    switchMode(!iconMode);

    setMinimumSize(600,500);

    setupMenuPages();  //0

    setupFavorites();//1

    setupFolders();//2

    setupActionsSystem();  //3

     setupActionsSettings();  //4

    setupMenusAppDesktop();//5

    switchMode(!iconMode);

}

FormMainMenu::~FormMainMenu()
{

    delete ui;
}

//! [0]---------------------------------------------------
void FormMainMenu::setupMenuPages()
{

    itemMenuFavorite=new QListWidgetItem(trUtf8("Favorites"),ui->listWidgetMenu); //0

    itemMenuFolder=new QListWidgetItem(mMFolders->title(),ui->listWidgetMenu); //1

    itemMenuFolder->setIcon(EIcon::fromTheme("folder"));

    itemMenuPower=new QListWidgetItem(mMSystem->title(),ui->listWidgetMenu); //2

    itemMenuSettings=new QListWidgetItem(mMSettingsApps->title(),ui->listWidgetMenu); //2

//    QListWidgetItem  *itemTitleApps=new QListWidgetItem(mMProgrammes->title(),ui->listWidgetMenu); //3

//    QFont font;
//    font.setBold(true);
//    itemTitleApps->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter|Qt::AlignCenter);
//    itemTitleApps->setFont(font);
//    itemTitleApps->setFlags(Qt::NoItemFlags);
}

//! [1]---------------------------------------------------
void FormMainMenu::setupFavorites()
{

    listBtnFavo.clear();
    QSettings s ;

    int count = s.beginReadArray("FavoriteMenu");

    for (int i = 0; i < count; ++i)
    {
        s.setArrayIndex(i);
        QString    url = s.value("Desktop", "").toString();
        addNewApp(url);
        listBtnFavo.append(url);

    }
    s.endArray();
}

//! [2]---------------------------------------------------
void FormMainMenu::setupFolders()
{

    foreach (QAction *act, mMFolders->actions()) {

        DesktopAppData *th=new DesktopAppData;
        th->setType(APPEXEC);
        th->setTitle(act->text());
        th->setDescription(act->data().toString());
        th->setPath(act->data().toString());
        th->setIcon(act->icon());
        myModel->addFile(th);

    }


}

//! [3]---------------------------------------------------
void FormMainMenu::setupActionsSystem()
{

    int count=mMSystem->actions().count();
    for(int i=0;i<count;i++)
    {
        QAction *act=mMSystem->actions().at(i);
        if(act->isEnabled()&&!act->isSeparator()){
            DesktopAppData *th=new DesktopAppData;
            th->setType(APPSYS);
            th->setTitle(act->text());
            th->setDescription(act->toolTip());
            th->setPath(act->data().toString());
            th->setIcon(act->icon());
            myModel->addFile(th);
            listActSys.insert(act->data().toString(),act);
        }
    }
}
//! [4]---------------------------------------------------
void FormMainMenu::setupActionsSettings()
{
    foreach (QAction *act, mMSettingsApps->actions()) {

        DesktopAppData *th=new DesktopAppData;
        th->setType(APPEXEC);
        th->setTitle(act->text());
        th->setDescription(act->toolTip());
        th->setPath(act->data().toString());
        th->setIcon(act->icon());
        myModel->addFile(th);

    }
}

//! [5]---------------------------------------------------
void FormMainMenu::setupMenusAppDesktop()
{

    ui->listWidgetMenu->setCurrentRow(0);
    // حذف كل الازرار
    foreach(QListWidgetItem *ittem,listMenuAppsItems ){
        //qDebug()<<" ittem-----------------------"<<ittem->text();
        ui->listWidgetMenu->removeItemWidget(ittem);
        delete ittem;
    }

    listMenuAppsItems.clear();

    //وقف الاشارات من كلاس القوائم حتى يتم تحميل الازرار
    mMProgrammes->blockSignals(true);

    foreach (QMenu *menu, mMProgrammes->menus) {

        QListWidgetItem  *menuItem=new QListWidgetItem(menu->icon(),menu->title(),ui->listWidgetMenu); //Network;WebBrowser;
        menuItem->setData(Qt::UserRole,menu->objectName());
        menuItem->setToolTip(menu->toolTip());
        listMenuAppsItems.append(menuItem);

    }

    mMProgrammes->blockSignals(false);

}

//_____________________________________________________________
void FormMainMenu::showContextMenu(QPoint /*point*/)
{
    int row= ui->listWidgetMenu->currentRow();

    if(listSelectionModel->selectedIndexes().count()<1)return;
    QModelIndex index=listSelectionModel->currentIndex();
    if(!index.isValid())return;

    DesktopAppData *app=myModel->desktopApplication(index);
    QMenu menu;
    QAction* a;

    a=menu.addAction(EIcon::fromTheme("folder-open"),tr("Open ")+app->title(),this,SLOT(openApp()));
    a->setData(app->path());
    QFont boldfont = this->font();
    boldfont.setBold(true);
    a->setFont(boldfont);
    if(row==PagePower||row==PageFolder){
    menu.exec(QCursor::pos());
    return;
    }
    menu.addSeparator();

    if(row==PageFavo){
        a=menu.addAction(EIcon::fromTheme("list-remove"),tr("remove from favorite"),this,SLOT(removeBtnFromFavorite()));
        a->setData(app->path());

    }else {
        a=menu.addAction(EIcon::fromTheme("bookmark_add"),tr("Add to favorite"),this,SLOT(addNewBtnToVavorite()));
        a->setData(app->path());
    }


    a=menu.addAction(EIcon::fromTheme("user-desktop"),tr("Add to desktop"),this,SLOT(addBtnToDesktop()));
    a->setData(app->path());


    menu.exec(QCursor::pos());
    listSelectionModel->clearSelection();
}

//_____________________________________________________________
void FormMainMenu::setCurrentRow(int currentRow)
{
    myModel->removeAll();
    //listSelectionModel->clearSelection();
    switch (currentRow) {
    case PageFavo:
        setupFavorites();
        return;
    case PageFolder:
        setupFolders();
        return;
    case PagePower:
        setupActionsSystem();
        return;
    case PageSettings:
        setupActionsSettings();
        return;
    case -1:
        return;
    default:
        break;
    }

    if(currentRow>3)
        setupAppsDesktop(currentRow-4);

}

void FormMainMenu::setupAppsDesktop(int row)
{
    mMProgrammes->blockSignals(true);
    if(mMProgrammes->menus.count()==0)return;
    if(row>=mMProgrammes->menus.count())return;

    mMProgrammes->blockSignals(true);
    QMenu *menu= mMProgrammes->menus.at(row);

    foreach (QAction *actM, menu->actions()) {
        // QAction *actM=actM;
        QString url=actM->data().toStringList().at(2);
        addNewApp(url);
    }

    mMProgrammes->blockSignals(false);
}

//_____________________________________________________________
void FormMainMenu::refreshIcons()
{
    mMProgrammes->refreshIcons();
    mMFolders->refreshIcons();
    mMSystem->refreshIcons();

    itemMenuFavorite->setIcon(EIcon::fromTheme("favorites",("user-bookmarks")).pixmap(32));
    itemMenuFolder->setIcon(EIcon::fromTheme("folder"));
    itemMenuPower->setIcon(mMSystem->icon());
    itemMenuSettings->setIcon(mMSettingsApps->icon());
    ui->labelFindPixmap->setPixmap(EIcon::fromTheme("find",("gtk-find")).pixmap(24));
    ui->tBnLIstView->setIcon(EIcon::fromTheme("view-list-tree","view-list-details"));
    ui->tBnIconsView->setIcon(EIcon::fromTheme("view-list-icons","view-group"));

}

//_____________________________________________________________
void FormMainMenu::addNewApp(const QString &url)
{
    QString lc=this->locale().name().section("_",0,0);

    DesktopFile dsf(url,"Desktop Entry",lc);

    QString oldname=dsf["Name"].toString();
    QString name=dsf.value("Name["+lc+"]",oldname).toString();

    QString icon=dsf["Icon"].toString();
    QString oldGenericName=dsf.value("GenericName",name).toString();
    QString GenericName=dsf.value("GenericName["+lc+"]",oldGenericName).toString();

    QString exec=dsf.value("Exec").toString().trimmed();

    DesktopAppData *th=new DesktopAppData;
    th->setType(APPEXEC);
    th->setTitle(name);
    th->setDescription(GenericName);
    th->setPath(exec);
    th->setIcon(EIcon::fromTheme(icon));
    myModel->addFile(th);
}

//_____________________________________________________________
void  FormMainMenu::indexClicked(QModelIndex index)
{

    if(!index.isValid())return;

    DesktopAppData *app=myModel->desktopApplication(index);
    listSelectionModel->clearSelection();
    mParent->hide();
    QString appPath=app->path();
    if(app->type()==APPEXEC){
       EMimIcon::launchApplication(appPath);

    }else if(app->type()==APPSYS){
        QAction *act=listActSys.value(appPath);
        if(act)
            act->trigger();
    }

    qDebug()<<app->path();
}

//_____________________________________________________________PopupMenu
void FormMainMenu::openApp()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (!action)return;

    QString url=action->data().toString();

    if(ui->listWidgetMenu->currentRow()==PagePower){
        QAction *act=listActSys.value(url);
        if(act)
            act->trigger();
    }else{
        EMimIcon::launchApplication(url);
    }

    mParent->hide();

}

//_____________________________________________________________PopupMenu
void FormMainMenu::addBtnToDesktop()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action){
        QString url=action->data().toString();
        qDebug()<<url;
        QFileInfo fi=url;
        QFile::copy(url,Edir::desktopDir()+"/"+fi.fileName());

    }

}

//_____________________________________________________________PopupMenu
void FormMainMenu::addNewBtnToVavorite()
{
    QAction *action = qobject_cast<QAction *>(sender());
    qDebug()<<action->text();
    if (action){
        QString url=action->data().toString();
        qDebug()<<url;
        listBtnFavo<<url;
        savFavoSetting();
    }

}

//_____________________________________________________________PopupMenu
void FormMainMenu::removeBtnFromFavorite()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action){
        QModelIndex index=myListView->currentIndex();
        if(!index.isValid())return;
        QString url=action->data().toString();
        listBtnFavo.removeOne(url);
        myModel->removeFile(index);
        savFavoSetting();

    }

}

//_____________________________________________________________PopupMenu
void FormMainMenu::savFavoSetting()
{
    QSettings settings;

    settings.remove("FavoriteMenu");
    settings.beginWriteArray("FavoriteMenu");

    for(int i=0;i<listBtnFavo.count();i++){
        settings.setArrayIndex(i);
        QString mime=listBtnFavo.at(i);
        settings.setValue("Desktop",mime);
    }

    settings.endArray();
}

//_____________________________________________________________
void FormMainMenu::goFindApp(const QString &arg1)
{

    if(arg1.length()<3)return;

    myModel->removeAll();

    ui->listWidgetMenu->setCurrentRow(-1);
    mMProgrammes->blockSignals(true);

    foreach (QMenu *menu, mMProgrammes->menus) {
        foreach (QAction *actM, menu->actions()) {
            QStringList list=actM->data().toStringList();
            QString mdata;
            if(list.count()>0)
                mdata=list.at(0);
            if (actM->text().toLower().contains(arg1.toLower())
                    ||mdata.toLower().contains(arg1.toLower()))
            {
                addNewApp(actM->data().toStringList().at(2));
            }
        }
    }

    mMProgrammes->blockSignals(false);

}

//_____________________________________________________________
void FormMainMenu::switchMode(bool checked)
{
    if(iconMode==checked)
        return;

    iconMode=checked;

    QSettings setting("elokab","settings");
    setting.beginGroup("MainMenu");
    setting.setValue("IconsView",iconMode);
    setting.endGroup();

    if(iconMode){
        myListView->setIconSize(QSize(64,64));
        myListView->setGridSize(QSize(96,96));
        myListView->setViewMode(QListView::IconMode);
    }else{
        myListView->setIconSize(QSize(24,24));
        myListView->setGridSize(QSize());
        myListView->setViewMode(QListView::ListMode);
    }
    myListView-> setDragDropMode(QAbstractItemView::DragDrop);

}




