/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef FORMMAINMENU_H
#define FORMMAINMENU_H
#include "page.h"
#include "menuprogrammes.h"
#include "menufolders.h"
#include "menusettingsapps.h"
#include <MenuSystem>
#include "menumodel.h"
#include "desktopappdata.h"

#include <QWidget>
#include <QListWidgetItem>

namespace Ui {
class FormMainMenu;
}
/**
 * @brief The FormMainMenu class
 */
class FormMainMenu : public QWidget
{
    Q_OBJECT

public:
    /**
     * @brief FormMainMenu
     * @param parent
     */
    explicit FormMainMenu(QWidget *parent = 0);
    ~FormMainMenu();
    enum AppType{
        APPEXEC,
        APPSYS
    };
    enum PageType{
        PageFavo,
        PageFolder,
        PagePower,
        PageSettings,
        PageApp
    };

public slots:

    /**
     * @brief refreshIcons
     */
    void refreshIcons();
private slots:

    /**
     * @brief setupMenuPages
     */
    void setupMenuPages();
    /**
     * @brief setupFolders
     */
    void setupFolders();

    void setupActionsSettings();
    /**
     * @brief chargeAppDesktop
     */
    void setupMenusAppDesktop();


    /**
     * @brief chargeActionsSystem
     */
    void setupActionsSystem();

    //!
    void  indexClicked(QModelIndex index);
//!
    void showContextMenu(QPoint );
    /**
     * @brief addNewBtnToVavorite
     * @param url
     */
    void addNewBtnToVavorite();
    /**
     * @brief addBtnToVavorite
     * @param url
     */
    void addNewApp(const QString &url);
    /**
     * @brief removeBtnFromFavorite
     * @param w
     */

    void openApp();
    void removeBtnFromFavorite();

    /**
     * @brief addBtnToDesktop
     */
    void addBtnToDesktop();
    /**
     * @brief savFavoSetting
     */
    void savFavoSetting();
    /**
     * @brief loadFavoSetting
     */
    void setupFavorites();
    /**
     * @brief on_lineEdit_textChanged
     * @param arg1
     */
    void goFindApp(const QString &arg1);


    /**
     * @brief on_tBnIconsView_toggled
     * @param checked
     */
    void switchMode(bool checked);
    //  void realign();
    void setupAppsDesktop(int row);

    void setCurrentRow(int currentRow);

private:
    MenuProgrammes *mMProgrammes;
    MenuFolders *mMFolders;
    MenuSystem *mMSystem;
    MenuSettingsApps *mMSettingsApps;
    QWidget *mParent;
    Ui::FormMainMenu *ui;
    /*!< cmment */
    QStringList listBtnFavo;
QHash<QString,QAction*>listActSys;
    /*!< cmment */
    QList<QListWidgetItem *>listMenuAppsItems;

    /*!< cmment */
    QListWidgetItem *itemMenuFavorite;
    /*!< cmment */
    QListWidgetItem *itemMenuFolder;
    /*!< cmment */
    QListWidgetItem *itemMenuPower;
    /*!< cmment */
    QListWidgetItem *itemMenuSettings;
    //!
    QItemSelectionModel *listSelectionModel;

    bool iconMode;

    //--------------
    MenuModel *myModel;
    QListView *myListView;
};

#endif // FORMMAINMENU_H
