﻿/* Copyright © 2006-2007 Fredrik Höglund <fredrik@kde.org>
 * (c)GPL2 (c)GPL3
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License version 2 or at your option version 3 as published 
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */
/*
 * additional code: Ketmar // Vampire Avalon (psyc://ketmar.no-ip.org/~Ketmar)
 */
#include "itemdelegate.h"


#include <QtGui/QApplication>
#include <QModelIndex>
#include <QDebug>
#include <QtGui/QPainter>
#include "desktopappdata.h"

namespace {
    const int decorationMargin = 8;
}


///////////////////////////////////////////////////////////////////////////////
ItemDelegate::ItemDelegate(QObject *parent) : QAbstractItemDelegate(parent)
{

}


ItemDelegate::~ItemDelegate()
{
}

QString ItemDelegate::firstLine(const QModelIndex &index) const
{
    if (index.isValid()) return index.model()->data(index, Qt::DisplayRole).toString();
    return QString();
}

QString ItemDelegate::secondLine(const QModelIndex &index) const
{
    if (index.isValid()) return index.model()->data(index, DesktopAppData::GenericRole).toString();
    return QString();
}

QIcon ItemDelegate::decoration(const QModelIndex &index) const
{
    if (index.isValid()) return qvariant_cast<QIcon>(index.model()->data(index, Qt::DecorationRole));
    return QIcon();
}

QSize ItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    if (!index.isValid()) return QSize();
    bool iconMode=false;
    if(option.decorationPosition == QStyleOptionViewItem::Top ||
            option.decorationPosition == QStyleOptionViewItem::Bottom ){
        iconMode=true;
    }
    QFont normalfont = option.font;
    QFont boldfont = normalfont;
    boldfont.setBold(true);
    // Extract the items we want to measure
    QString firstRow = firstLine(index);
    QString secondRow = secondLine(index);
    // Compute the height
    QFontMetrics fm1(boldfont);
    QFontMetrics fm2(normalfont);
    int height = fm1.lineSpacing() + fm2.lineSpacing();
    height = qMax(height, option.decorationSize.height());
    // Compute the text width
    int width = fm1.width(firstRow);
   width = qMax(width, fm2.width(secondRow));
    // Add decoration width + margin
    width += option.decorationSize.width()+decorationMargin;
    if(iconMode)
    return QSize(width, height+20);
    else
    return QSize(width, height+10);
}

QPalette::ColorRole ItemDelegate::foregroundRole(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_UNUSED(index)
    if (option.state & QStyle::State_Selected) return QPalette::HighlightedText;
    return QPalette::Text;
}

void ItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{

    if (!index.isValid()) return;
    painter->save();
bool iconMode=false;
if(option.decorationPosition == QStyleOptionViewItem::Top ||
        option.decorationPosition == QStyleOptionViewItem::Bottom ){
    iconMode=true;
}
    QFont normalfont = option.font;
    QFont boldfont = normalfont;
    boldfont.setBold(true);

    QString firstRow = firstLine(index);
    QString secondRow = secondLine(index);
    QPixmap pixmap = decoration(index).pixmap(option.decorationSize);

    QColor textcol = option.palette.color(foregroundRole(option, index));

    // Draw the background
    QStyleOptionViewItemV4 opt = option;
    QStyle *style = opt.widget ? opt.widget->style() : QApplication::style();
    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);
 int x,y;int rWidth=option.rect.width();
    // Draw the icon
    if(iconMode){
        x = option.rect.left()+rWidth/2-(pixmap.width())/2;
//        x = option.rect.left()+(option.decorationSize.width()-pixmap.width()+decorationMargin)/2;

        y = option.rect.top()+4;
      }else{
        x = option.rect.left()+(option.decorationSize.width()-pixmap.width()+decorationMargin)/2;
        y = option.rect.top()+(option.rect.height()-pixmap.height())/2;
    }

    QRect pixmapRect = QStyle::visualRect(option.direction, option.rect, QRect(x, y, pixmap.width(), pixmap.height()));
    painter->drawPixmap(pixmapRect.x(), pixmapRect.y(), pixmap);

    // Draw the text
    QFontMetrics fm1(boldfont);
    QFontMetrics fm2(normalfont);

    int textAreaHeight = fm1.lineSpacing()+fm2.lineSpacing();

    int y1,y2;
     painter->setPen(textcol);
    if(iconMode){
        if(fm1.width(firstRow)>=rWidth-decorationMargin){

            firstRow=fm1.elidedText(firstRow,Qt::ElideRight,rWidth-decorationMargin);

        }

        x = option.rect.left()+(rWidth)/2-(fm1.width(firstRow)/2);
        y1 = option.rect.top()+option.decorationSize.height()+4;

    }else{
        x = option.rect.left()+option.decorationSize.width()+decorationMargin;
        y1 = option.rect.top()+(option.rect.height()-textAreaHeight)/2;
        y2 = y1+fm1.lineSpacing();
           painter->setFont(boldfont);

    }

    QRect firstRowRect = QStyle::visualRect(option.direction, option.rect, QRect(x, y1, fm1.width(firstRow), fm1.lineSpacing()));
           // First line

        painter->drawText(firstRowRect, Qt::AlignCenter, firstRow);
        if(!iconMode){
            // Second line
            QRect secondRowRect = QStyle::visualRect(option.direction, option.rect, QRect(x, y2, fm2.width(secondRow), fm2.lineSpacing()));
            painter->setFont(normalfont);
            painter->drawText(secondRowRect, Qt::AlignCenter, secondRow);
        }
    painter->restore();

}
