/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "mainmenu.h"
#include "dialogconfigmainmenu.h"
#include "../widgetpopup/widgetpopup.h"
#include <QtGui/QX11Info>
#include <QDebug>
#include <X11/Xlib.h>

MainMenu::MainMenu(QWidget *parent) :
    QToolButton(parent),m_fmain(0)
{

    this->setObjectName("Main_Menu");     //اسم الاضافة
    this->setWindowTitle(tr("Main Menu"));   //النص الذي سيتم عرضه لتعريف الاضافة
    this->setWhatsThis(tr("Main menu launcher favorite,applications,folders by AbouZakaria"));//شرح الاضافة
    this->setWindowIcon(QIcon::fromTheme("start-here",QIcon(":/icons/start-here.png")));//ايقونة الاضافة
    mWidgetPopup=new WidgetPopup(this);


    connect(this,SIGNAL(clicked()),this,SLOT(showFormMain()));
    this->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this,SIGNAL(customContextMenuRequested(const QPoint & )) ,this,SLOT(customContextMenu(const QPoint & )));
    loadSetting();
//   keySequence();
   QTimer::singleShot(3000, this, SLOT(initFormMain()));
  //  initFormMain();
}

//************************************
void MainMenu::coztumizMainMenu()
{
    DialogConfigMainMenu *dlg=new DialogConfigMainMenu;
    connect(dlg,SIGNAL(settingChanged()),this,SLOT(loadSetting()));
    dlg->show();
}

void MainMenu::loadSetting()
{



    QSettings setting("elokab","settings");
    setting.sync();
    setting.beginGroup("MainMenu");

    int bstyle=(setting.value("ButtonStyle",0).toInt());
    QString mytext=(setting.value("ButtonText",trUtf8("Start Here")).toString());

    QString icon=(setting.value("ButtonIcon",":/icons/start-here.png").toString());
    bool isImage=setting.value("IsCostumImage",false).toBool();
    QString imgNormal=setting.value("NormalImage",":/icons/startNormal.png").toString();
    QString imgHover=setting.value("HoverImage",":/icons/startHover.png").toString();
    setting.endGroup();
    if(!QFile::exists(imgNormal)||!QFileInfo(imgNormal).isFile())
        imgNormal=":/icons/startNormal.png";
    if(!QFile::exists(imgHover)||!QFileInfo(imgHover).isFile())
        imgHover=":/icons/startHover.png";


    if(isImage)   {
        QString styleButton= QString(" QToolButton#Main_Menu{\n"
                                     "\n "

                                     "\n      border:0px outset rgba(0; 0, 0, 0);"
                                     "border-image: url(%1);}"
                                     ).arg(imgNormal);

        QString styleButtonHover= QString("\n   QToolButton#Main_Menu:hover{\n"
                                          "border-image: url(%1);}"
                                          ).arg(imgHover);



        QPixmap pix(imgNormal);
        this->setMinimumWidth(pix.width());
        this->setToolButtonStyle(Qt::ToolButtonIconOnly);
        this->setIcon(QIcon());
        this->setText(QString());
        setStyleSheet(
                     styleButton+styleButtonHover);
        setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

    }  else{
        setStyleSheet("");
        this->setText(mytext);
        if(bstyle==0){
            this->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        }else{
            this->setToolButtonStyle(Qt::ToolButtonIconOnly);
        }


        this->setIcon(QIcon::fromTheme(icon,QIcon(":/icons/start-here.png")).pixmap(QSize(128, 128)));


        setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);

    }

}
void MainMenu::customContextMenu(const QPoint & /*pos*/)
{
    QMenu menu;
    menu.setTitle(this->text());
    QAction *act=new QAction(QIcon::fromTheme("configure",QIcon::fromTheme("document-properties"))
                             ,trUtf8("Menu Settings"),this);
    connect(act,SIGNAL(triggered()),this,SLOT(coztumizMainMenu()));
    menu.addAction(act);
    menu.addActions(m_actList);
    menu.exec(QCursor::pos());
}

//*************************
void MainMenu::showFormMain()
{
    if(!m_fmain){
        initFormMain();

    }
/*
    QPoint mPos;
    Qt::Corner mAnchor;
    int mPosbtn=mapToGlobal(QPoint(geometry().width()/2,geometry().bottom()/2)).x();
    if (isLeftToRight())
    {
        mPos=mapToGlobal(QPoint(0,geometry().bottom()));
        mAnchor= Qt::TopLeftCorner;
    }
    else
    {

        mPos=mapToGlobal(QPoint(geometry().width(),geometry().bottom()));
        mAnchor=Qt::TopRightCorner;
    }

    QRect rect;
    rect.setSize(m_fmain->size());
    switch (mAnchor)
    {
    case Qt::TopLeftCorner:
        rect.moveTopLeft(mPos);
        break;

    case Qt::TopRightCorner:
        rect.moveTopRight(mPos);
        break;
    case Qt::BottomLeftCorner:
        rect.moveBottomLeft(mPos);
        break;

    case Qt::BottomRightCorner:
        rect.moveBottomRight(mPos);
        break;
    }

    QRect screen = QApplication::desktop()->availableGeometry(mPos);

    if (rect.right() > screen.right())
        rect.moveRight(screen.right());

    if (rect.left()< screen.left())
        rect.moveLeft(screen.left());

    if (rect.bottom() > screen.bottom()){
        rect.moveBottom(screen.bottom());
        m_fmain->setScreenBotum(true,mPosbtn-rect.left()-22);
    }else{
        m_fmain->setScreenBotum(false,mPosbtn-rect.left()-22);
    }


    m_fmain-> move(rect.topLeft());
   */
    int mPosbtn=mapToGlobal(QPoint(geometry().width()/2,geometry().bottom()/2)).x();

    int wW=(this->width()/2);

    mWidgetPopup->open(mapToGlobal(QPoint(geometry().width()-wW,geometry().bottom()))
                  , mPosbtn);

 //  mWidgetPopup->hide();
//    mWidgetPopup->show();
    mWidgetPopup->activateWindow();
    m_fmain->setFocus();
    mWidgetPopup->open(mapToGlobal(QPoint(geometry().width()-wW,geometry().bottom()))
                  , mPosbtn);

}

//******************virtual***************
void MainMenu::setSize(QSize size)
{
    setIconSize(size);
    loadSetting();
}

void MainMenu::setLayout(QHBoxLayout *p)
{
    p->addWidget(this);
}
void  MainMenu::x11EventFilter(XEvent*)
{
    //qDebug()<<event->type;
}

QString  MainMenu:: keySequence()
{

    QSettings settings("elokab","shortcut");
    settings.beginGroup("Panel-Plugins");

    settings.beginGroup("MenuBar");
    if(!settings.contains("Destination")){
        settings.setValue("Key",QKeySequence("Alt+F1").toString());
        settings.setValue("Destination","panel");
        settings.setValue("Comment","show the Main Menu panal");
    }
    m_ks= settings.value("Key","Alt+F1").toString();
    settings.endGroup();//MenuBar

    settings.endGroup();//Panel-Plugins
    return m_ks;
}
void  MainMenu::setActions(QList<QAction *>list)
{
    m_actList=list;
    //          setMultiMehnu(m_multi);
}
void MainMenu::setKeySequence(QKeySequence sequence)
{

    QKeySequence ks(m_ks);
    if(ks==sequence){
        if(!m_fmain){
            initFormMain();
        }
        mWidgetPopup->show();
        mWidgetPopup->activateWindow();
        QTimer::singleShot(10, this, SLOT(showFormMain()));
        // showFormMain();
    }
}
void MainMenu::initFormMain()
{
    if(!m_fmain){
        m_fmain=new FormMainMenu(mWidgetPopup);
        mWidgetPopup->setWidget(m_fmain);
        m_fmain->refreshIcons();
    }
}

void MainMenu::themeChanged()
{
    if(m_fmain)
        m_fmain->refreshIcons();

    loadSetting();
}

Q_EXPORT_PLUGIN2(dpanel_Mainmenu, MainMenu)
