﻿/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "formmainmenu.h"
#include "btndesktop.h"
#include "page.h"
#include "ui_formmainmenu.h"
#include <QToolButton>
#include <QListWidgetItem>

#include <EIcon>
#include <DesktopFile>
#include <QUrl>
#include <QFile>
#include <QDir>
#include <QDebug>
#include <QProcess>
#include <QSettings>

#include <QIconEngine>

#define NO_FAVO 0
#define AD_FAVO 1
#define RM_FAVO 2

FormMainMenu::FormMainMenu(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::FormMainMenu)
{
    ui->setupUi(this);
    connect(ui->lineEdit,SIGNAL(textChanged(QString)),this,SLOT(goFindApp(QString)));
    connect(ui->tBnIconsView,SIGNAL(toggled(bool)),this,SLOT(switchMode(bool)));
    connect(ui->listWidgetMenu,SIGNAL(currentRowChanged(int)),this,SLOT(setCurrentRow(int)));

    mParent=parent;
    mSMenu=new MenuSystem;
    mPMenu=new MenuProgrammes(this);
    mFMenu=new MenuFolders;
    //--------------By Order-------------
    connect(mPMenu,SIGNAL(menuRecharged()),this,SLOT(setupAppDesktop()));

    QSettings setting("elokab","settings");
    setting.beginGroup("MainMenu");
    iconMode=  setting.value("IconsView",false).toBool();
    setting.endGroup();




    setMinimumSize(600,500);

  //  QTimer::singleShot(1000, this, SLOT(  init()));
    setupMenuPages();  //0

    setupFavorites();//1

    setupFolders();//2

    setupActionsSystem();  //3

    setupAppDesktop();//4


    ui->tBnIconsView->setChecked(iconMode);
}

FormMainMenu::~FormMainMenu()
{

    delete ui;
}

//! [0]---------------------------------------------------
void FormMainMenu::setupMenuPages()
{

    pageFavorite = new Page();ui->stackedWidget->addWidget(pageFavorite);//0

    pageFolders = new Page();ui->stackedWidget->addWidget(pageFolders);//1

    pagePower = new Page();ui->stackedWidget->addWidget(pagePower);//2

    pageFind= new Page();ui->stackedWidget->addWidget(pageFind);//3

    pageApps= new Page();ui->stackedWidget->addWidget(pageApps);//4


    itemMenuFavorite=new QListWidgetItem(trUtf8("Favorites"),ui->listWidgetMenu); //0

    itemMenuFolder=new QListWidgetItem(mFMenu->title(),ui->listWidgetMenu); //1
    itemMenuFolder->setIcon(EIcon::fromTheme("folder"));

    itemMenuPower=new QListWidgetItem(mSMenu->title(),ui->listWidgetMenu); //2

    QListWidgetItem  *itemTitleApps=new QListWidgetItem(mPMenu->title(),ui->listWidgetMenu); //3

    QFont font;
    font.setBold(true);
    itemTitleApps->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter|Qt::AlignCenter);
    itemTitleApps->setFont(font);
    itemTitleApps->setFlags(Qt::NoItemFlags);
}

//! [1]---------------------------------------------------
void FormMainMenu::setupFavorites()
{


    QSettings s ;

    int count = s.beginReadArray("FavoriteMenu");

    for (int i = 0; i < count; ++i)
    {
        s.setArrayIndex(i);
        QString    url = s.value("Desktop", "").toString();
        addBtnToVavorite(url);

    }
    s.endArray();
}

//! [2]---------------------------------------------------
void FormMainMenu::setupFolders()
{

    foreach (QAction *act, mFMenu->actions()) {
        BtnDesktop *btnF=new BtnDesktop(NO_FAVO);
        btnF->setDefaultAction(act);
        btnF->setMimType(act->data().toString());


        listBtnFolder.append(btnF);
        connect(btnF, SIGNAL(clicked()), mParent, SLOT(hide()));

        btnF->setIconMode(iconMode);
         pageFolders->addWidget(btnF,iconMode);
    }


}

//! [3]---------------------------------------------------
void FormMainMenu::setupActionsSystem()
{

    int count=mSMenu->actions().count();
    for(int i=0;i<count;i++)
    {
        QAction *act=mSMenu->actions().at(i);
        if(act->isSeparator()){

            QFrame  *line = new QFrame;
            line->setObjectName(QString::fromUtf8("line"));
            line->setFrameShape(QFrame::HLine);
            line->setFrameShadow(QFrame::Sunken);
            pagePower->addWidget(line,0);

        }else if(act->isEnabled()){
            BtnDesktop *btn=new BtnDesktop(NO_FAVO,this);
            // act->addAction(menusysApp->actions().at(i));

            btn-> setDefaultAction(act);
            act->setParent(btn);

            pagePower->addWidget(btn,0);
            connect(btn, SIGNAL(clicked()), mParent, SLOT(hide()));
        }
    }
}

//! [4]---------------------------------------------------
void FormMainMenu::setupAppDesktop()
{
    qDebug()<<"FormMainMenu << signal menuRecharged-----------------------Ok";

ui->listWidgetMenu->setCurrentRow(0);

    // حذف كل الازرار
    foreach(BtnDesktop *btn,listBtnActApps ){
        delete btn;
    }
  //  qDebug()<<" delete btn-----------------------Ok";

    foreach(QListWidgetItem *ittem,listMenuAppsItems ){
         //qDebug()<<" ittem-----------------------"<<ittem->text();
        ui->listWidgetMenu->removeItemWidget(ittem);
        delete ittem;
    }

   // qDebug()<<" delete ittems-----------------------Ok";

    listBtnActApps.clear();
    listMenuAppsItems.clear();

    //وقف الاشارات من كلاس القوائم حتى يتم تحميل الازرار
    mPMenu->blockSignals(true);

    foreach (QMenu *menu, mPMenu->menus) {

        QListWidgetItem  *menuItem=new QListWidgetItem(menu->icon(),menu->title(),ui->listWidgetMenu); //Network;WebBrowser;
        menuItem->setData(Qt::UserRole,menu->objectName());
        menuItem->setToolTip(menu->toolTip());

        listMenuAppsItems.append(menuItem);

    }

    mPMenu->blockSignals(false);

}

//_____________________________________________________________
void FormMainMenu::setCurrentRow(int currentRow)
{

    if(currentRow<4)return;

   // ui->stackedWidget->setCurrentIndex(4);
    ui->stackedWidget->setCurrentWidget(pageApps);
    foreach(BtnDesktop *btn,listBtnActApps ){
        pageApps->removeWidget(btn);
        delete btn;

    }

    listBtnActApps.clear();

    pageApps->removeAll();

    if(mPMenu->menus.count()==0)return;
    if(currentRow-4>=mPMenu->menus.count())return;

    mPMenu->blockSignals(true);
    QMenu *menu= mPMenu->menus.at(currentRow-4);

    foreach (QAction *actM, menu->actions()) {
        // QAction *actM=actM;

        BtnDesktop *btn=newBtnDesktop(actM);
        pageApps->addWidget(btn,iconMode);
        listBtnActApps.append(btn);
    }

    mPMenu->blockSignals(false);

}

//_____________________________________________________________
BtnDesktop *FormMainMenu::newBtnDesktop(QAction *actM)
{
    BtnDesktop *btn=new BtnDesktop(AD_FAVO,this);
    btn->setDefaultAction(actM);
    QStringList list=actM->data().toStringList();
    if(list.count()>2){

        btn->setData(list.at(0));
        btn->setIconName(list.at(1));
        btn->setMimType(list.at(2));
    }

    btn->setIconMode(iconMode);
    connect(btn, SIGNAL(clicked()), mParent, SLOT(hide()));
    connect(btn, SIGNAL(addBtnToFavo(QString)), this, SLOT(addNewBtnToVavorite(QString)));
    return btn;
}

//_____________________________________________________________
void FormMainMenu::refreshIcons()
{
    qDebug()<<"FormMainMenu::refreshIcons()----------------------------BEGIN";
    mPMenu->refreshIcons();

    mFMenu->refreshIcons();

    mSMenu->refreshIcons();

    foreach(BtnDesktop *btn,listBtnFavo ){
        QString icon=btn->iconName();
        //    qDebug()<<icon;

             QIcon iconF=EIcon::fromTheme(icon);
              btn->setIcon(iconF);


    }

    foreach(QListWidgetItem *ittem,listMenuAppsItems ){

        QString icon=ittem->data(Qt::UserRole).toString();
            QIcon iconF=EIcon::fromTheme(icon);

            ittem->setIcon(iconF);

    }

    itemMenuFavorite->setIcon(EIcon::fromTheme("favorites",("user-bookmarks")).pixmap(32));

    itemMenuFolder->setIcon(EIcon::fromTheme("folder"));


    itemMenuPower->setIcon(mSMenu->icon());

    ui->labelFindPixmap->setPixmap(EIcon::fromTheme("find",("gtk-find")).pixmap(24));
    ui->tBnLIstView->setIcon(EIcon::fromTheme("view-list-tree","view-list-details"));
    ui->tBnIconsView->setIcon(EIcon::fromTheme("view-list-icons","view-group"));
    qDebug()<<"FormMainMenu::refreshIcons()------------------------------END";
}

//_____________________________________________________________
void FormMainMenu::execApplication()
{
    BtnDesktop *action = qobject_cast<BtnDesktop *>(sender());
    if (action){

        QString data=action->mydata();
        QString exec=data.section("|",0,0);
         exec=exec.trimmed().section(" ",0,0);
          qDebug()<<"FormMainMenu::execApplication()===================="<<exec;
        if(exec.contains("kdesudo")||exec.contains("gksudo")){
            exec.remove("kdesudo")   ;

            exec.remove("gksudo")   ;
            exec=QString("ekbsudo %1 -i %2").arg(exec).arg(data.at(1));

        }
        QProcess process;
        process.setWorkingDirectory(QDir::homePath());

        process.startDetached(exec);

       mParent->hide();
    }



}

//_____________________________________________________________
void FormMainMenu::addNewBtnToVavorite(const QString &url)
{

    addBtnToVavorite(url);
    savFavoSetting();
}

//_____________________________________________________________
void FormMainMenu::addBtnToVavorite(const QString &url)
{
    QString lc=this->locale().name().section("_",0,0);

   DesktopFile dsf(url,"Desktop Entry",lc);

    QString oldname=dsf["Name"].toString();
    QString name=dsf.value("Name["+lc+"]",oldname).toString();
    QString exec=dsf["Exec"].toString();
    QString icon=dsf["Icon"].toString();
    QString oldGenericName=dsf.value("GenericName").toString();
    QString GenericName=dsf.value("GenericName["+lc+"]",oldGenericName).toString();


    BtnDesktop *btn=new BtnDesktop(RM_FAVO,this);
    QAction *act=new QAction(name,this);



        QIcon icnF=EIcon::fromTheme(icon);
        act->setIcon(icnF);


    act->setToolTip(GenericName);
    btn->setIconName(icon);
    btn->setDefaultAction(act);
    btn->setData(exec);
    btn->setMimType(url);

    btn->setIconMode(iconMode);
    pageFavorite->addWidget(btn,iconMode);
    listBtnFavo.append(btn);
    connect(btn, SIGNAL(clicked()), this, SLOT(execApplication()));
    connect(btn, SIGNAL(removeBtnFromFavo(BtnDesktop*)), this, SLOT(removeBtnFromFavorite(BtnDesktop*)));

}

//_____________________________________________________________
void FormMainMenu::removeBtnFromFavorite(BtnDesktop*w)
{

    listBtnFavo.removeOne(w);
    pageFavorite->removeWidget(w);
    delete w;
    savFavoSetting();
    pageFavorite->realign(iconMode);

}

//_____________________________________________________________
void FormMainMenu::savFavoSetting()
{
    QSettings settings;
    //    settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
    settings.remove("FavoriteMenu");
    settings.beginWriteArray("FavoriteMenu");

    for(int i=0;i<listBtnFavo.count();i++){
        settings.setArrayIndex(i);

        QString mime=listBtnFavo.at(i)->mimeType();
        settings.setValue("Desktop",mime);

    }

    settings.endArray();
}

//_____________________________________________________________
void FormMainMenu::goFindApp(const QString &arg1)
{
    removeAllFind();
    if(arg1.length()<3)return;

    ui->stackedWidget->setCurrentWidget(pageFind);

    ui->listWidgetMenu->setCurrentRow(-1);
    mPMenu->blockSignals(true);

    foreach (QMenu *menu, mPMenu->menus) {
        foreach (QAction *actM, menu->actions()) {
            QStringList list=actM->data().toStringList();
            QString mdata;
            if(list.count()>0)
              mdata=list.at(0);


            if (actM->text().toLower().contains(arg1.toLower())
                    ||mdata.toLower().contains(arg1.toLower()))
            {
                addBtnFind(actM);

            }
        }
    }
      mPMenu->blockSignals(false);

}

//_____________________________________________________________
void FormMainMenu::addBtnFind(QAction *actM)
{

    BtnDesktop *btn=newBtnDesktop(actM);

    pageFind->addWidget(btn,iconMode);
  //  connect(act, SIGNAL(clicked()), this, SLOT(execApplication()));
    listBtnFind<<btn;
}

//_____________________________________________________________
void FormMainMenu::removeAllFind()
{
    foreach (BtnDesktop *btn, listBtnFind) {
        listBtnFind.removeOne(btn);
        pageFind->removeWidget(btn);
        delete btn;
    }

    listBtnFind.clear();

}

//_____________________________________________________________
void FormMainMenu::switchMode(bool checked)
{
    if(iconMode==checked)
        return;

    iconMode=checked;

    QSettings setting("elokab","settings");
    setting.beginGroup("MainMenu");
    setting.setValue("IconsView",iconMode);
    setting.endGroup();

    foreach(BtnDesktop *btn,listBtnFavo ){
       if(btn)
       btn->setIconMode(iconMode);
    }
     pageFavorite->realign(iconMode);

    //تجديث شكل ايقونات التطبيقات
    foreach(BtnDesktop *btn,listBtnActApps ){
        if(btn)
        btn->setIconMode(iconMode);
    }
    //تخديث الشبكة للتطبيقات
       pageApps->realign(iconMode);

    //تحديث ازرار المجلدات
    foreach(BtnDesktop *btn,listBtnFolder ){
       if(btn)
        btn->setIconMode(iconMode);
    }
    //تخديث شبكة المجلدات
    pageFolders->realign(iconMode);
    //  realign();
}




