/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef FORMMAINMENU_H
#define FORMMAINMENU_H
#include "page.h"
#include "menuprogrammes.h"
#include "menufolders.h"
#include <MenuSystem>
#include "btndesktop.h"


#include <QWidget>
#include <QListWidgetItem>

namespace Ui {
class FormMainMenu;
}
/**
 * @brief The FormMainMenu class
 */
class FormMainMenu : public QWidget
{
          Q_OBJECT

     public:
          /**
     * @brief FormMainMenu
     * @param parent
     */
          explicit FormMainMenu(QWidget *parent = 0);
          ~FormMainMenu();
     public slots:

          /**
     * @brief refreshIcons
     */
          void refreshIcons();
     private slots:

          /**
     * @brief setupMenuPages
     */
          void setupMenuPages();
          /**
     * @brief setupFolders
     */
          void setupFolders();
          /**
     * @brief chargeAppDesktop
     */
          void setupAppDesktop();
          /**
     * @brief chargeActionsSystem
     */
          void setupActionsSystem();
          /**
     * @brief execApplication
     */
          void execApplication();
          /**
     * @brief addNewBtnToVavorite
     * @param url
     */
          void addNewBtnToVavorite(const QString &url);
          /**
     * @brief addBtnToVavorite
     * @param url
     */
          void addBtnToVavorite(const QString &url);
          /**
     * @brief removeBtnFromFavorite
     * @param w
     */
          void removeBtnFromFavorite(BtnDesktop *w);
          /**
     * @brief savFavoSetting
     */
          void savFavoSetting();
          /**
     * @brief loadFavoSetting
     */
          void setupFavorites();
          /**
     * @brief on_lineEdit_textChanged
     * @param arg1
     */
          void goFindApp(const QString &arg1);
          /**
     * @brief addBtnFind
     * @param btn
     */
          void addBtnFind(QAction *actM);
          /**
     * @brief removeAllFind
     */
          void removeAllFind();
          /**
     * @brief on_tBnIconsView_toggled
     * @param checked
     */
          void switchMode(bool checked);
        //  void realign();


          void setCurrentRow(int currentRow);
BtnDesktop *newBtnDesktop(QAction *actM);
private:
          MenuProgrammes *mPMenu;
          MenuFolders *mFMenu;
          MenuSystem *mSMenu;
QWidget *mParent;
          Ui::FormMainMenu *ui;
          /*!< cmment */
          QList<BtnDesktop *>listBtnFavo;
          /*!< cmment */
          QList<BtnDesktop *>listBtnFind;
          /*!< cmment */
          QList<BtnDesktop *>listBtnActApps;
          /*!< cmment */
          QList<BtnDesktop *>listBtnFolder;
          /*!< cmment */
          QList<QListWidgetItem *>listMenuAppsItems;
          /*!< cmment */
          QList<Page *>listPages;


          // QFileSystemWatcher *m_sWatcher;

          /*!< cmment */
          QListWidgetItem *itemMenuFavorite;
          /*!< cmment */
          QListWidgetItem *itemMenuFolder;
          /*!< cmment */
          QListWidgetItem *itemMenuPower;
Page  *pageFavorite ;
          Page  *pagePower ;
          Page  *pageFind;
          Page  *pageFolders;
           Page  *pageApps;
          bool iconMode;
};

#endif // FORMMAINMENU_H
