/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QtDebug>

#include <QUrl>
#include <QtGui/QToolTip>

#include <QtGui/QLabel>
#include <EIcon>
#include "dialogmountconfig.h"
#include "devicemount.h"
#include <RazorMount>
#include <QX11Info>

#define ACT_SHOW_MENU "showMenu"
#define ACT_SHOW_INFO "showInfo"
#define ACT_NOTHING   "nothing"


Popup::Popup(RazorMountManager *manager, QWidget* parent):
           QWidget(parent,  Qt::Popup ),
          mManager(manager),
          mPos(0,0),
                   mDisplayCount(0)
{

          setObjectName("PopupInfo");
          setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);


          //! انشاء نافذة المعلومات

          setAttribute(Qt::WA_AlwaysShowToolTips);



         layout = new QVBoxLayout(this);
         layout->setSizeConstraint(QLayout::SetMinAndMaxSize);
          // layout->setSizeConstraint(QLayout::SetFixedSize);
          layout->setSpacing(0);
          layout->setMargin(3);
          mPlaceholder = new QLabel(tr("No devices are available"), this);
         layout->addWidget(mPlaceholder);
          mPlaceholder->hide();


          connect(mManager, SIGNAL(deviceAdded(RazorMountDevice*)),
                  this, SLOT(addItem(RazorMountDevice*)));
          connect(mManager, SIGNAL(deviceRemoved(RazorMountDevice*)),
                  this, SLOT(removeItem(RazorMountDevice*)));




          foreach(RazorMountDevice *device, mManager->devices())
          {
                    addItem(device);
          }
}


MenuDiskItem *Popup::addItem(RazorMountDevice *device)
{
          if (MenuDiskItem::isUsableDevice(device))
          {
                    MenuDiskItem  *item   = new MenuDiskItem(device, this);
                    listItemes[device]=item;

                    layout->addWidget(item);
                    item->setVisible(true);
                    mDisplayCount++;
                    if (mDisplayCount != 0)
                              mPlaceholder->hide();
                    return item;
          }
          else
          {
                    return 0;
          }
}
void Popup::refreshItems()
{
    //    qDebug()<<listItemes.count();

        foreach(MenuDiskItem *wid,listItemes)
        {


             delete wid;
        }
       mDisplayCount=0;
       listItemes.clear();

        foreach(RazorMountDevice *device, mManager->devices())
        {
               addItem(device);
        }

}

void Popup::removeItem(RazorMountDevice *device)
{
          if (MenuDiskItem::isUsableDevice(device))
          {
                    mDisplayCount--;
                    listItemes.remove(device);
                    if (mDisplayCount == 0)
                              mPlaceholder->show();
          }
          //qDebug()<<listItemes.count();
}


void Popup::resizeEvent(QResizeEvent *event)
{
          QWidget::resizeEvent(event);

}


void Popup::showEvent(QShowEvent *event)
{
          //   qDebug() << "DEV" << mDisplayCount <<  mManager->devices().count() << mManager->devices();
          if (mDisplayCount == 0)
                    mPlaceholder->show();

          QWidget::showEvent(event);
          emit visibilityChanged(true);
}


void Popup::hideEvent(QHideEvent *event)
{
          mPlaceholder->hide();

          QWidget::hideEvent(event);
          emit visibilityChanged(false);
}



DeviceMount::DeviceMount(QWidget * parent) :
          QToolButton(parent),
          mPopup(0),
          mDevAction(DevActionInfo),
          mPopupHideDelay(5000)
{
          this->setObjectName("deviceMount");
          this->setWindowTitle(tr("Device Mount"));
          this->setWhatsThis(tr("Mount or unmount a removable device... by AbouZakaria"));//شرح الاضافة
          this->setWindowIcon(EIcon::fromTheme( "device-notifier",( "drive-removable-media")));
          setIcon(EIcon::fromTheme( "device-notifier", ("drive-removable-media" )));

          setToolTip(tr("Removable media/devices manager"));
        //  setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

          mPopup = new Popup(&mManager,this);
          mWidgetPopup=new WidgetPopup(this,mPopup);

       //   connect(mWidgetPopup, SIGNAL(visibilityChanged(bool)), this, SLOT(setDown(bool)));



          connect(this, SIGNAL(clicked()), this, SLOT(showHidePopup()));

          connect(&mManager, SIGNAL(deviceAdded(RazorMountDevice*)),
                  this, SLOT(onDeviceAdded(RazorMountDevice*)));

          connect(&mManager, SIGNAL(deviceRemoved(RazorMountDevice*)),
                  this, SLOT(onDeviceRemoved(RazorMountDevice*)));

          this->setContextMenuPolicy(Qt::CustomContextMenu);
          connect(this,SIGNAL(customContextMenuRequested(const QPoint & )) ,
                  this,SLOT(customContextMenu(const QPoint & )));

          mManager.update();
}


DeviceMount::~DeviceMount()
{
}

void DeviceMount::customContextMenu(const QPoint & pos)
{
          QMenu menu;

          QAction *act=new QAction(EIcon::fromTheme("configure",("document-properties"))
                                   ,trUtf8("Mount Settings"),this);
          connect(act,SIGNAL(triggered()) ,
                  this,SLOT(showSettings()));

          menu.addAction(act);
          menu.exec(mapToGlobal(pos));
}
void DeviceMount::showSettings()
{
          DialogMountConfig *confWindow = this->findChild<DialogMountConfig*>("mountConfig");

          if (!confWindow)
          {
                    confWindow = new DialogMountConfig;

                    connect(confWindow,SIGNAL(settingChanged()),this,SLOT(settingsChanged()));
          }

          confWindow->show();
          confWindow->raise();
          confWindow->activateWindow();
}
void DeviceMount::settingsChanged()
{
          QSettings settings;
          settings.beginGroup("Mount Device");
          QString s = settings.value("deviceAction", DeviceMount::DevActionInfo).toString();
          settings.endGroup();
          if (s == ACT_SHOW_MENU)
                    this->setDevAction(DeviceMount::DevActionMenu);
          else if(s == ACT_NOTHING)
                    this->setDevAction(DeviceMount::DevActionNothing);
          else
                    this->setDevAction(DeviceMount::DevActionInfo);
mPopup->refreshItems();

}
void DeviceMount::showMessage(const QString &text)
{
          QToolTip::showText(mapToGlobal(QPoint(0, 0)), QString("<nobr>%1</nobr>").arg(text));

}


void DeviceMount::onDeviceAdded(RazorMountDevice *device)
{
          switch (mDevAction)
          {
          case DevActionInfo:
                    if (MenuDiskItem::isUsableDevice(device))
                              showMessage(tr("The device <b><nobr>\"%1\"</nobr></b> is connected.").arg(device->label()));
                    break;

          case DevActionMenu:
                    showPopup();
                    mPopupHideTimer.singleShot(mPopupHideDelay, mWidgetPopup, SLOT(hide()));
                    break;

          case DevActionNothing:
                    break;
          }
}

void DeviceMount::onDeviceRemoved(RazorMountDevice *device)
{
          switch (mDevAction)
          {
          case DevActionInfo:
                    if (MenuDiskItem::isUsableDevice(device))
                              showMessage(tr("The device <b><nobr>\"%1\"</nobr></b> is removed.").arg(device->label()));
                    break;

          case DevActionMenu:
                    if (mManager.devices().isEmpty())
                              hidePopup();
                    break;

          case DevActionNothing:
                    break;
          }
}

void DeviceMount::setDown(bool down)
{
          QToolButton::setDown(down);
}


void DeviceMount::showHidePopup()
{
          if (mWidgetPopup->isVisible())
                    mWidgetPopup->hide();
          else
          {
                    mPopupHideTimer.stop();
                    showPopup();
          }
}


void DeviceMount::showPopup()
{

          if (mWidgetPopup->isVisible())
                    return;

          if (mManager.devices().isEmpty())
                    return;
      mPopup->updateGeometry();
          int mPosbtn=mapToGlobal(QPoint(geometry().width()/2,geometry().bottom()/2)).x();

          int wW=(this->width()/2);

          mWidgetPopup->open(mapToGlobal(QPoint(geometry().width()-wW,geometry().bottom()))
                        , mPosbtn);

          /*

          mPopup->updateGeometry();
          int mPosbtn=mapToGlobal(QPoint(geometry().width()/2,geometry().bottom()/2)).x();

          QPoint p;
          if (isLeftToRight())
          {
            mPopup->openPop(mapToGlobal(QPoint(0,geometry().bottom())),mPosbtn, Qt::TopLeftCorner);


          }
          else
          {
            mPopup->openPop(mapToGlobal(QPoint(geometry().width(),geometry().bottom())),mPosbtn, Qt::TopRightCorner);


          }
          */
}


void DeviceMount::hidePopup()
{
          mWidgetPopup->hide();
}
//******************Virtuel*****************************




void   DeviceMount::themeChanged()
{

        foreach(MenuDiskItem *wid, mPopup->listItemes)
        {
              wid->update();
        }
}



Q_EXPORT_PLUGIN2(dpanel_mountDevice, DeviceMount)
