/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef DEVICEMOUNT_H
#define DEVICEMOUNT_H
#include  "../widgetpopup/widgetpopup.h"

#include <QtGui/QToolButton>
#include <QtDBus/QtDBus>
#include <QtGui/QMenu>
#include "../../dpluginsinterface.h"
#include <RazorMount>

#include "menudiskitem.h"


class QLabel;
class UdisksManager;

class Popup: public QWidget
{
          Q_OBJECT

public:
          explicit Popup(RazorMountManager *manager, QWidget* parent = 0);



 QHash<RazorMountDevice *,MenuDiskItem*> listItemes;
       // QList<MenuDiskItem *>listItemes;
   int mDisplayCount;
signals:
          void visibilityChanged(bool visible);

protected:
          void resizeEvent(QResizeEvent *event);
          void showEvent(QShowEvent *event);
          void hideEvent(QHideEvent *event);
public slots:
       void refreshItems();
    // void openPop(QPoint pos,int p, Qt::Corner anchor=Qt::TopLeftCorner);
private slots:
              MenuDiskItem *addItem(RazorMountDevice *device);

            void removeItem(RazorMountDevice *device);
private:

          RazorMountManager *mManager;
          QPoint mPos;
          int mpBtn;

          QLabel *mPlaceholder;
         QVBoxLayout  *verticalLayoutPop ;
        //   QGridLayout *verticalLayoutPop;

  QVBoxLayout *layout;
};



class DeviceMount : public QToolButton,PluginsInterface
{
          Q_OBJECT
          Q_INTERFACES(PluginsInterface)
public:
          DeviceMount(QWidget * parent=0);
          ~DeviceMount();

          enum DevAction {
                    DevActionNothing,
                    DevActionInfo,
                    DevActionMenu
          };
          DevAction devAction() const { return mDevAction; }
          void setDevAction(DevAction devAction) { mDevAction = devAction; }
public slots:
          /*virtual*/ void  x11EventFilter(XEvent* /*event*/){}
          /*virtual*/ void setKeySequence(QKeySequence /*sequence*/){}
          /*virtual*/ QString   keySequence(){return QString();}
          /*virtual*/ void setLayout(QHBoxLayout *p) { p->addWidget(this);}
          /*virtual*/ void setSize(QSize size){ setIconSize(size); }
          /*virtual*/       void setActions(QList<QAction *>/*list*/){}
          void themeChanged();

private slots:
          void onDeviceAdded(RazorMountDevice *device);
          void onDeviceRemoved(RazorMountDevice *device);

          void showHidePopup();
          void showPopup();
          void hidePopup();
          void setDown(bool down);
          void customContextMenu(const QPoint & pos);
          void showSettings();
          void settingsChanged();
private:
          void showMessage(const QString &text);

          Popup *mPopup;
WidgetPopup *mWidgetPopup;
          RazorMountManager mManager;

          //RazorPanel *mPanel;
          DevAction mDevAction;
          QTimer  mPopupHideTimer;
          int mPopupHideDelay;
};

#endif
