/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QtGui/QDesktopServices>
#include "menudiskitem.h"
#include <RazorMount>
#include <EIcon>
#include <QDebug>
#include <QUrl>
#include <QSettings>
#include <QProcess>

/************************************************

 ************************************************/
MenuDiskItem::MenuDiskItem(RazorMountDevice *device, QWidget *parent)
          : QWidget(parent),
            mDevice(device)
{
          setupUi(this);

          eject->setIcon(EIcon::fromTheme("media-eject"));

          connect(device, SIGNAL(destroyed()),
                  this, SLOT(free()));

          connect(device, SIGNAL(changed()),
                  this, SLOT(update()));

//          connect(device, SIGNAL(mounted()),
//                  this, SLOT(mounted()));

          connect(device, SIGNAL(unmounted()),
                  this, SLOT(unmounted()));

          update();
}


/************************************************

 ************************************************/
void MenuDiskItem::free()
{
          this->deleteLater();
}


/************************************************

 ************************************************/
void MenuDiskItem::update()
{
    qDebug()<<"mDevice->iconName()==="<<mDevice->iconName();
    diskButton->setIcon(EIcon::fromTheme( mDevice->iconName(),( "drive-removable-media-usb" ) ).pixmap(32));
    eject->setIcon(EIcon::fromTheme("media-eject").pixmap(32));
          QString label = mDevice->label();
          diskButton->setText(label);

          setMountStatus(mDevice->isMounted());
}


/************************************************

 ************************************************/
bool MenuDiskItem::isUsableDevice(const RazorMountDevice *device)
{
        QSettings settings;
        settings.beginGroup("Mount Device");
           bool showPartition=settings.value("showPartition", false).toBool();
        settings.endGroup();
          switch (device->mediaType())
          {
          case RazorMountDevice::MediaTypeFdd:
                    return  true;

          case RazorMountDevice::MediaTypeOptical:
                                    return true;
                    break;

          case RazorMountDevice::MediaTypePartition:
                    if(showPartition==true)
                  return true;

          case RazorMountDevice::MediaTypeDrive:
                    return device->isExternal();
                    break;

          default:
                    return false;
          }

          return false;
}


/************************************************

 ************************************************/
void MenuDiskItem::changeEvent(QEvent *e)
{
          QWidget::changeEvent(e);
          switch (e->type()) {
          case QEvent::LanguageChange:
                    retranslateUi(this);
                    break;
          default:
                    break;
          }
}


/************************************************

 ************************************************/
void MenuDiskItem::setMountStatus(bool is_mount)
{

          eject->setChecked(is_mount);
}


/************************************************

 ************************************************/
void MenuDiskItem::on_diskButton_clicked()
{
          if (!mDevice->isMounted()){
                    mDevice->mount();
                    mounted();
          }else
                    mounted();

          qobject_cast<QWidget*>(parent()->parent()->parent())->hide();
}


/************************************************

 ************************************************/
void MenuDiskItem::mounted()
{
        QSettings setting("elokab","elokabsettings");
        setting.beginGroup("DefaultBrowser");
        bool defaultBrowser=(setting.value("defaultBrowser",false).toBool());

        if(defaultBrowser==true)     {

                QDesktopServices::openUrl(QUrl(mDevice->mountPath()));

        }else{


                  QString costumBrowser=(setting.value("BROWSER","elokab-fm").toString());
                  QString prog=costumBrowser.section(" ",0,0);
                  QStringList list;
                  if(!costumBrowser.section(" ",1,1).trimmed().isEmpty())
                  list<<costumBrowser.section(" ",1,1).trimmed();

                      list<<mDevice->mountPath();
                   QProcess process;
                   process.startDetached(prog,list);

                      // qDebug()<<list;




        }
        setting.endGroup();
//*************************************************
  setMountStatus(mDevice->isMounted());
}


/************************************************

 ************************************************/
void MenuDiskItem::on_eject_clicked()
{
    if (!mDevice->isMounted()){
              mDevice->mount();
    }else{
          mDevice->unmount();
    }
          setMountStatus(mDevice->isMounted());

          qobject_cast<QWidget*>(parent()->parent()->parent())->hide();
}


/************************************************

 ************************************************/
void MenuDiskItem::unmounted()
{
          if (mDevice->isEjectable())
                    mDevice->eject();

               setMountStatus(mDevice->isMounted());
}


void MenuDiskItem::on_eject_toggled(bool checked)
{
    if(checked)
        eject->setToolTip(tr("Click to eject this disc."));
    else
         eject->setToolTip(tr("Click to mount this disc."));
}
