

/********************************************************************
 *(c)GPL3+

original code  http://razor-qt.org
 modified by abou zakaria
*********************************************************************/
#include "dquicklaunch.h"
#include "quicklaunchbutton.h"
#include "quicklaunchlayout.h"
//#include "quicklaunchaction.h"

#include "../../common/addactions/dialogaddaction.h"
#include <EMimIcon>
#include <QtDebug>
#include <QSettings>
#include <QUrl>
#include <QFileInfo>
#include <QtGui/QDragEnterEvent>
#include <QtGui/QToolButton>
#include <QtGui/QMessageBox>

#include <QtGui/QFileIconProvider>
#include <QFileSystemModel>
#include <QSettings>



DQuickLaunch::DQuickLaunch(QWidget* parent)
          : QWidget( parent)
{
          setObjectName("QuickLaunch");

          this->setWindowTitle(tr("Quick Launch"));
          this->setWhatsThis(tr("by AbouZakaria"));//شرح الاضافة
          this->setWindowIcon(EIcon::fromTheme("application-x-executable"));

          setAcceptDrops(true);

          m_layout = new QuickLaunchLayout(this);
           actAddActions=new QAction(QIcon::fromTheme("list-add"),trUtf8("Add Lancher"),this);
          connect(actAddActions,SIGNAL(triggered()) ,this,SLOT(addLancher()));
          this->setContextMenuPolicy(Qt::CustomContextMenu);
          connect(this,SIGNAL(customContextMenuRequested(QPoint)) ,this,SLOT(customContextMenu(QPoint)));
          this->setMinimumWidth(30);

          QSettings setting("elokab","panelQuickLanche");


           int count = setting.beginReadArray("QuickApps");
          if(count==0){
             addButton(new QuickAction("/usr/share/applications/elokab-settings.desktop", this));
              addButton(new QuickAction("Elokab-terminal", "elokab-terminal","terminal", this));
              addButton(new QuickAction( QDir::homePath(), this));
              return;
          }
          QString desktop;
          QString file;
          QString execname;
          QString exec;
          QString icon;
          for (int i = 0; i < count; ++i){

              setting.setArrayIndex(i);
              desktop = setting.value("desktop", "").toString();
              file = setting.value("file", "").toString();

              if (! file.isEmpty()){
                  addButton(new QuickAction(file, this));
              }else{
                  execname = setting.value("name", "").toString();
                  exec = setting.value("exec", "").toString();
                  icon = setting.value("icon", "").toString();
                  if (icon.isEmpty()) addButton(new QuickAction( exec, this));
                  else addButton(new QuickAction(execname, exec,icon, this));
              }

          } // for

          setting.endArray();
}

DQuickLaunch::~DQuickLaunch()
{
}


void DQuickLaunch::customContextMenu(QPoint)
{
          QMenu menu;

             menu.addAction(actAddActions);
             menu.exec(this->cursor().pos());
}

int DQuickLaunch::indexOfButton(QuickButton* button) const
{
          return m_layout->indexOf(button);
}

int DQuickLaunch::countOfButtons() const
{
          return m_layout->count();
}

void DQuickLaunch::addButton(QuickAction* action)
{

          int newIndex = 0;
          QList<int> keys = m_buttons.uniqueKeys();
          qSort(keys);
          foreach (int index, keys)
          {
                    if (newIndex != index)
                              break;
                    ++newIndex;
          }

          QuickButton* btn = new QuickButton(m_size,newIndex, action, this);

          m_layout->addWidget(btn);
          m_buttons[newIndex] = btn;

          connect(btn, SIGNAL(switchButtons(int,int)), this, SLOT(switchButtons(int,int)));
          connect(btn, SIGNAL(buttonDeleted(int)), this, SLOT(buttonDeleted(int)));
          connect(btn, SIGNAL(settingsChanged()), this, SLOT(saveSettings()));
}

void DQuickLaunch::dragEnterEvent(QDragEnterEvent *e)
{
          // Getting URL from mainmenu...
          if (e->mimeData()->hasUrls())
          {
                    e->acceptProposedAction();
                    return;
          }

          if (e->source() && e->source()->parent() == this)
          {
                    e->acceptProposedAction();
          }
}

void DQuickLaunch::dropEvent(QDropEvent *e)
{
          const QMimeData *mime = e->mimeData();

          QList<QUrl> duplicates;

          foreach (QUrl url, mime->urls())
          {
                    if (duplicates.contains(url))
                              continue;
                    else
                              duplicates << url;

                    QString fileName(url.toLocalFile());
                    QFileInfo fi(fileName);
                    qDebug() << fileName << fi.exists() << fi.isExecutable();




                    if (fi.exists() )
                    {

                              addButton(new QuickAction(fileName, this));

                    }


          }
          saveSettings();
}

void DQuickLaunch::switchButtons(int first, int second)
{
          QuickButton *w1 = m_buttons[first];
          QuickButton *w2 = m_buttons[second];
          m_layout->swapButtons(w1, w2);
          saveSettings();
}

void DQuickLaunch::buttonDeleted(int id)
{
          QuickButton *b = m_buttons[id];
          m_layout->removeWidget(b);
          m_buttons.remove(id);
          b->deleteLater();
          saveSettings();
}




void DQuickLaunch::saveSettings()
{

    QSettings settings("elokab","panelQuickLanche");
          //    settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
          settings.remove("QuickApps");
          settings.beginWriteArray("QuickApps");
          int i = 0;

          foreach(QuickButton *b, m_layout->buttons())
          {
                    settings.setArrayIndex(i);

                    QHashIterator<QString,QString> it(b->settingsMap());
                    while (it.hasNext())
                    {
                              it.next();
                              settings.setValue(it.key(), it.value());
                    }

                    ++i;
          }

          settings.endArray();
}

void DQuickLaunch::addLancher()
{

          DialogAddAction *dlg=new DialogAddAction;
          connect(dlg,SIGNAL(accepteAdd(QString)) ,this,SLOT(addButton(QString)));
          dlg->show();
          //  delete dlg;
}
void DQuickLaunch::addButton(QString txt)
{

          addButton(new QuickAction(txt.section("|",0,0), txt.section("|",1,1), txt.section("|",2,2), this));
          saveSettings();
}

/********************virtual******************************/



void DQuickLaunch::setSize(QSize size)
{

          m_size=size;
          foreach(QuickButton *b, m_layout->buttons())
          {
                    b->setSize(size);

          }
}

 void DQuickLaunch::themeChanged()
 {
     foreach(QuickButton *b, m_layout->buttons())
     {
               b->setSize(m_size);

     }
 }






Q_EXPORT_PLUGIN2(dpanel_dquicklaunch, DQuickLaunch)
