

/********************************************************************
 *(c)GPL3+

original code  http://razor-qt.org
 modified by abou zakaria
*********************************************************************/
#include "quicklaunchaction.h"
#include "dquicklaunch.h"
#include "../../libraries/mimetypexdg.h"
#include <QtDebug>
#include <QtCore/QProcess>
#include <QtCore/QUrl>
#include <QtGui/QMessageBox>
#include <QtGui/QDesktopServices>
#include <QtGui/QFileIconProvider>
#include <QSettings>
#include <QTextCodec>
#include <QApplication>
#include <QLocale>


QuickAction::QuickLaunchAction(const QString & name,
                                     const QString & exec,
                                     QWidget * parent)
    : QAction(name, parent)
{

    m_settingsHash["name"] = name;
    m_settingsHash["exec"] = exec;

    connect(this, SIGNAL(triggered()), this, SLOT(execAction()));
    QString AppS=MimeTypeXdg::getInfoFromUnknowSufix(exec);
    if(AppS!="unknown"){

        QString icon= MimeTypeXdg::getMimeTypeByFile(exec);
        setData(AppS +" \""+exec+"\"");

        QIcon iconF=(QIcon::fromTheme(icon,QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png"))));
        QPixmap pixmap = iconF.pixmap(QSize(128, 128),QIcon::Active,QIcon::Off);
        setIcon(QIcon(pixmap));
        m_type =   ActionScript;

        return;
    }

    setData(exec);
    m_type = ActionLegacy;
    QString App=MimeTypeXdg::getInfoFromDesktopApp(exec);
    if(App!=""){
        setText(App.section("|",0,0));
        QIcon iconF=(QIcon::fromTheme(App.section("|",1,1),QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png"))));
        QPixmap pixmap = iconF.pixmap(QSize(128, 128),QIcon::Active,QIcon::Off);
        setIcon(QIcon(pixmap));

    }else{
        setIcon(QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")));

    }

}

QuickAction::QuickLaunchAction(const QString & name,
                                     const QString & exec,
                                     const QString & icon,
                                     QWidget * parent)
    : QAction(parent)
{

    m_settingsHash["name"] = name;
    m_settingsHash["exec"] = exec;
    m_settingsHash["icon"] = icon;
    int i=name.section("|",1,1).toInt();
    if(i==0){ m_type =  ActionLegacy;}
    if(i==1){ m_type =  ActionScript;}
    if(i==2){ m_type =  ActionFile;}


    QIcon iconF;
    if(QFile::exists(icon)){
        iconF=QIcon(icon);
    }else{
        iconF=QIcon::fromTheme(icon,QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")));
    }


    connect(this, SIGNAL(triggered()), this, SLOT(execAction()));




    setText(name.section("|",0,0));
    setData(exec);
    QPixmap pixmap = iconF.pixmap(QSize(128, 128),QIcon::Active,QIcon::Off);
    setIcon(QIcon(pixmap));
    // setIcon(iconF);
}

QuickAction::QuickLaunchAction(const QString & fileName, QWidget * parent)
    : QAction(parent)
{

    DQuickLaunch *quick = qobject_cast<DQuickLaunch*>(parent);
    QString lC=quick->locale().name();
    lC= lC.section("_",0,0);
    /******************************************************/
  //  QFileInfo fi(fileName);
    QString iconXdg= MimeTypeXdg::getMimeTypeByFile(fileName).trimmed();
    QString exec=fileName;
    QStringList list=MimeTypeXdg::getInTextIconExec(fileName,lC);
    setText(list.at(0));
    setToolTip(list.at(4));
    exec=(list.at(1));
    setData(exec);
    iconXdg=list.at(2);

    int i=list.at(3).toInt();
    if(i==0){ m_type =  ActionLegacy;}
    if(i==1){ m_type =  ActionScript;}
    if(i==2){ m_type =  ActionFile;}
    m_settingsHash["name"] = text()+"|"+QString::number(m_type);
    m_settingsHash["exec"] = exec;
    m_settingsHash["icon"] = iconXdg;
    setIcon(MimeTypeXdg::getIcon(fileName,iconXdg));
    connect(this, SIGNAL(triggered()), this, SLOT(execAction()));

 }

void QuickAction::execAction()
{
        MimeTypeXdg::launchApp(data().toString().section("|",0,0),m_type,m_settingsHash["icon"]);
 }
void QuickAction::setSettingsHash(const QString &txt)
{
 //   qDebug()<<"setSettingMap"<<txt;
    QString name=txt.section("|",0,0);
    name.replace("#","|");
    m_settingsHash["name"] =name;
    m_settingsHash["exec"] = txt.section("|",1,1);
    m_settingsHash["icon"] = txt.section("|",2,2);
    setText(name.section("|",0,0));
    setData(txt.section("|",1,1));
    QIcon iconF=MimeTypeXdg::getIconsFromeTheme( txt.section("|",2,2),"application-x-executable");


            setIcon(iconF);


}
void QuickAction::upgatIcon()
{
  QString icon=  m_settingsHash["icon"] ;
   QIcon iconF=
  MimeTypeXdg::getIconsFromeTheme(icon,"application-x-executable");

            setIcon(iconF.pixmap(QSize(128, 128)));
}
