

/********************************************************************
 *(c)GPL3+

original code  http://razor-qt.org
 modified by abou zakaria
*********************************************************************/
#include "dquicklaunch.h"
#include "../../common/addactions/dialogaddaction.h"
#include "quicklaunchbutton.h"
#include <EMimIcon>
#include <EIcon>

#include <DesktopFile>

#include <QMouseEvent>

#define MIMETYPE "quicklaunch-button"


/**************************************************
 *                  QuickLaunchAction
 **************************************************/


QuickAction::QuickAction(const QString & name,  const QString & exec,  const QString & icon,  QWidget * parent)
    : QAction(parent)
{

    m_settingsHash["name"] = name;
    m_settingsHash["exec"] = exec;
    m_settingsHash["icon"] = icon;
    int i=name.section("|",1,1).toInt();
    if(i==0){ m_type =  ActionLegacy;}
    if(i==1){ m_type =  ActionScript;}
    if(i==2){ m_type =  ActionFile;}


    QIcon iconF;

        iconF=EIcon::fromTheme(icon,"application-x-executable");


    connect(this, SIGNAL(triggered()), this, SLOT(execAction()));

    setText(name.section("|",0,0));
    setData(exec);

     setIcon(iconF);
}

QuickAction::QuickAction(const QString & fileName, QWidget * parent)
    : QAction(parent)
{

    DQuickLaunch *quick = qobject_cast<DQuickLaunch*>(parent);
    QString lC=quick->locale().name();
    lC= lC.section("_",0,0);

    /******************************************************/
    QFileInfo fi(fileName);
     if(fi.suffix()=="desktop"){
DesktopFile xdg(fileName,"Desktop Entry",lC);
       //   QHash<QString ,QString> hash=EMimIcon::desktopFile(fileName,lC);
m_settingsHash["name"] = /*hash ["Name"];*/xdg.value("Name").toString();
m_settingsHash["exec"] =  /*hash ["Exec"];*/xdg.value("Exec").toString();
m_settingsHash["icon"] = /*hash ["Icon"];*/xdg.value("Icon").toString();

//          m_settingsHash["name"] = hash ["Name"];
//          m_settingsHash["exec"] =  hash ["Exec"];
//          m_settingsHash["icon"] = hash ["Icon"];
     }else{
          m_settingsHash["name"] = fi.fileName();
          m_settingsHash["exec"] = fileName;
          m_settingsHash["icon"] = EMimIcon::icon(fi).name();
     }
     setIcon(EIcon::fromTheme(m_settingsHash["icon"],"application-x-executable"));
      setData( m_settingsHash["exec"] );
      setText(m_settingsHash["name"]);
     connect(this, SIGNAL(triggered()), this, SLOT(execAction()));

 }

void QuickAction::execAction()
{
        EMimIcon::launchApplication(data().toString().section("|",0,0));
 }
void QuickAction::setSettingsHash(const QStringList &list)
{
 //   qDebug()<<"setSettingMap"<<txt;
   //m_type=list.at(1);
    m_settingsHash["name"] =list.at(0);
    m_settingsHash["exec"] = list.at(2);
    m_settingsHash["icon"] = list.at(3);
    setText(list.at(0));
    setData(list.at(2));
    QIcon iconF=EIcon::fromTheme( list.at(3),"application-x-executable");
            setIcon(iconF);


}
void QuickAction::upgatIcon()
{

  QString icon=  m_settingsHash["icon"] ;
  QIcon iconF= EIcon::fromTheme(icon,"application-x-executable");


            setIcon(iconF);
}

/**************************************************
 *                  QuickLaunchButton
 **************************************************/
QuickButton::QuickButton(QSize size,int id, QuickAction * act, QWidget * parent)
          : QToolButton(parent),
            m_act(act),
            m_id(id),
            m_size(size)
{
          setAcceptDrops(true);
          if(m_size==QSize(-1,-1))m_size=QSize(16,16);

          setDefaultAction(m_act);
          m_act->setParent(this);
            setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
          setIconSize(m_size);
           setMinimumSize(m_size);


          m_propertyAct=new QAction(EIcon::fromTheme("configure","document-properties"), trUtf8("properties"), this);
          connect(m_propertyAct, SIGNAL(triggered()), this, SLOT(getProperty()));


          m_deleteAct = new QAction(EIcon::fromTheme("dialog-close",("list-remove")), trUtf8("Remove from quicklaunch"), this);
          connect(m_deleteAct, SIGNAL(triggered()), this, SLOT(parentRemoveMe()));
          DQuickLaunch *quick = qobject_cast<DQuickLaunch*>(parent);

          //addAction(m_deleteAct);
          m_menu = new QMenu;

          m_menu->addAction(m_act);
          m_menu->addAction(m_propertyAct);


          m_menu->addSeparator();
          m_menu->addAction(quick->actAddActions);

          m_menu->addSeparator();
          m_menu->addAction(m_deleteAct);

          setContextMenuPolicy(Qt::CustomContextMenu);
          connect(this, SIGNAL(customContextMenuRequested(const QPoint&)),
                  this, SLOT(this_customContextMenuRequested(const QPoint&)));
          QString styleButton= QString( "\n QToolButton{"
                                             "\n      border-radius:3px;"
                                             "\n      border:1px transparent;"
                                             " }") ;

  setStyleSheet(styleButton);
}

QuickButton::~QuickButton()
{
          //m_act->deleteLater();
}

void QuickButton::setSize(QSize size)
{
          m_size=size;
          setIconSize(m_size);
          setMinimumSize(m_size);
          m_act->upgatIcon();

}

QHash<QString,QString> QuickButton::settingsMap()
{
          Q_ASSERT(m_act);
          return m_act->settingsHash();
}


void QuickButton::this_customContextMenuRequested(const QPoint & pos)
{

          m_menu->popup(mapToGlobal(pos));
}

void QuickButton::parentRemoveMe()
{
          emit buttonDeleted(m_id);
}



void QuickButton::mousePressEvent(QMouseEvent *e)
{
          //

          if (e->button() == Qt::LeftButton && e->modifiers() == Qt::ControlModifier)
          {
                    m_dragStart = e->pos();
                    return;
          }

          QToolButton::mousePressEvent(e);
}

void QuickButton::mouseMoveEvent(QMouseEvent *e)
{
          if (!(e->buttons() & Qt::LeftButton))
          {
                    return;
          }

          if ((e->pos() - m_dragStart).manhattanLength() < QApplication::startDragDistance())
          {
                    return;
          }

          if (e->modifiers() != Qt::ControlModifier)
          {
                    return;
          }

          QDrag *drag = new QDrag(this);
          QMimeData *mimeData = new QMimeData();
          QByteArray ba;
          ba.setNum(m_id);
          mimeData->setData(MIMETYPE, ba);
          drag->setMimeData(mimeData);

          drag->exec(Qt::MoveAction);

          // اذا تم سحب الزر خارج اللوحة سيتم حذفه
          if (!drag->target())
          {
                    parentRemoveMe();
          }
}

void QuickButton::dragMoveEvent(QDragMoveEvent * e)
{
          if (e->mimeData()->hasFormat(MIMETYPE))
                    e->acceptProposedAction();
          else
                    e->ignore();
}

void QuickButton::dragEnterEvent(QDragEnterEvent *e)
{
          if (e->mimeData()->hasFormat(MIMETYPE))
          {
                    int other = e->mimeData()->data(MIMETYPE).toInt();
                    // do not replace self with self
                    if (other == m_id)
                              return;

                    emit switchButtons(m_id, other);
          }
}

void QuickButton::getProperty()
{
        QStringList list;
        list<<m_act->text()
           <<m_act->data().toString().section("|",0,0)
           <<m_act->settingsHash()["name"].section("|",1,1)
           <<m_act->settingsHash()["icon"];

          DialogAddAction *dlg=new DialogAddAction(list,m_act->icon());
          connect(dlg,SIGNAL(accepteChange(QStringList)) ,this,SLOT(changeAction(QStringList)));
          dlg->show();
          //delete dlg;
}

void QuickButton::changeAction(const QStringList &list)
{
       //   qDebug()<<"QuickLaunchButton"<<txt;
          m_act->setSettingsHash(list);

         emit settingsChanged();
}

