#include "sensorwidget.h"
#include "widgetsensorconf.h"
#include <QSettings>

//_____________________________________________________________________
SensorWidget::SensorWidget(QWidget *parent) :
    QLabel(parent)
{
    setText(QChar(0x00B0)+QString::number(100));
  //  sensorPopup=new SensorPopup(this);
    mLabelInfo=new QLabel;
    mWidgetPopup=new WidgetPopup(this,mLabelInfo);

    mDetectedChips = mSensors.getDetectedChips();

    this->setContextMenuPolicy(Qt::CustomContextMenu);

    connect(this,SIGNAL(customContextMenuRequested(QPoint)) ,this,SLOT(customContextMenu(QPoint)));

    timer=new QTimer;
    connect(timer,SIGNAL(timeout()),this,SLOT(updateSensor()));

    loadSettings();
}

//_____________________________________________________________________
void SensorWidget::loadSettings()
{
    timer->stop();
    QSettings setting;
    setting.sync();
    setting.beginGroup("sonsors");
    mTimeUpdat=setting.value("TimeUpdat",3).toInt()*1000;
    mFahrenheit=setting.value("Fahrenheit",false).toBool();
    mChipIndex=setting.value("ChipIndex",0).toInt();

    setting.endGroup();
    updateSensor();
    timer->start(mTimeUpdat);

}

//_____________________________________________________________________
void SensorWidget::customContextMenu(QPoint)
{
    QMenu menu;
    QAction *actFormat=new QAction(QIcon::fromTheme("configure",QIcon::fromTheme("document-properties")),
                                   trUtf8("Sensors settings"),this);

    connect(actFormat,SIGNAL(triggered()) ,this,SLOT(configSettings()));

    menu.addAction(actFormat);


    menu.exec(this->cursor().pos());
}

//_____________________________________________________________________
void SensorWidget::configSettings()
{

    QStringList list;
    for (unsigned int i = 0; i < mDetectedChips.size(); ++i)
    {

        const std::vector<Feature>& features = mDetectedChips[i].getFeatures();

        for (unsigned int j = 0; j < features.size(); ++j)
        {

            if (features[j].getType() == SENSORS_FEATURE_TEMP)
            {


                QString name= QString::fromStdString(features[j].getLabel());
                list.append(name);

            }
        }
    }
    WidgetSensorConf *dlg=new WidgetSensorConf(list);


    connect(dlg,SIGNAL(settingSaved()),this,SLOT(loadSettings()));
    dlg->show();
}

//_____________________________________________________________________
void SensorWidget::setSize(QSize size)
{

    QFont font;
    font.setPointSize(size.width()-8);

    setFont(font);

}

//_____________________________________________________________________
void SensorWidget::updateSensor()
{
    QString tooltip;
    int index=-1;

    double curTemp = 0;
    double iconTemp = 0;


    for (unsigned int i = 0; i < mDetectedChips.size(); ++i)
    {

        const std::vector<Feature>& features = mDetectedChips[i].getFeatures();

        for (unsigned int j = 0; j < features.size(); ++j)
        {

            if (features[j].getType() == SENSORS_FEATURE_TEMP)
            {
                index++;

                QString name= QString::fromStdString(features[j].getLabel());

                tooltip +=name.leftJustified(25,'.') ;

                if (mFahrenheit)
                {
                              curTemp = celsiusToFahrenheit(
                                features[j].getValue(SENSORS_SUBFEATURE_TEMP_INPUT));

                    tooltip +=QString::number(int(curTemp)) + QChar(0x00B0)+"F";
                }
                else
                {
                         curTemp = features[j].getValue(SENSORS_SUBFEATURE_TEMP_INPUT);

                    tooltip += QString::number(int(curTemp)) +" C"+ QChar(0x00B0);
                }
                if(mChipIndex==index){
                    iconTemp=curTemp;
                    setToolTip(name);
                }
                         tooltip += "\n";
            }
        }
    }

    if(mWidgetPopup->isVisible()){
        mLabelInfo->setText(tooltip);
     }

    if(mFahrenheit)
        setText(QString::number(int(iconTemp)) +"F"+ QChar(0x00B0));
    else
        setText(QString::number(int(iconTemp)) +"C"+ QChar(0x00B0));


}

//_____________________________________________________________________
double SensorWidget::celsiusToFahrenheit(double celsius)
{

    return 32 + 1.8 * celsius;
}

//_____________________________________________________________________
void SensorWidget::mouseReleaseEvent(QMouseEvent* event)
{
    if (event->button() != Qt::LeftButton)
        return;

    int mPosbtn=mapToGlobal(QPoint(geometry().width()/2,geometry().bottom()/2)).x();

    int wW=(this->width()/2);

    mWidgetPopup->open(mapToGlobal(QPoint(geometry().width()-wW,geometry().bottom()))
                  , mPosbtn);

    updateSensor();

}

Q_EXPORT_PLUGIN2(dsensor, SensorWidget)
