/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "dshowdesktop.h"

#include <QIcon>
#include <X11/Xlib.h>
#include <QX11Info>
#include <X11/Xatom.h>

DshowDesktop::DshowDesktop(QWidget *parent) :
          QToolButton(parent)
{
          setObjectName("showDesktop");
          this->setWindowTitle("Show Desktop");
          this->setWindowIcon(QIcon::fromTheme("user-desktop",QIcon::fromTheme("desktop")));
          this->setWhatsThis(tr("show the desktop minimize all open window... by AbouZakaria"));//شرح الاضافة
          setIcon(QIcon::fromTheme("user-desktop",QIcon::fromTheme("desktop")));
          connect(this,SIGNAL(clicked()),this,SLOT(showDesktopNew()));
          setToolTip(trUtf8("Show/Hide Desktop"));

}
void DshowDesktop:: themeChanged()
{
      setIcon(QIcon::fromTheme("user-desktop",QIcon::fromTheme("desktop")));
      this->setWindowIcon(QIcon::fromTheme("user-desktop",QIcon::fromTheme("desktop")));

}

void DshowDesktop::loadSetting()
{



}

void DshowDesktop::setKeySequence(QKeySequence sequence)
{
        QKeySequence ks(m_ks);
        if(ks==sequence){
                showDesktopNew();
        }
}

QString  DshowDesktop::keySequence()
{

    QSettings settings("elokab","shortcut");

    settings.beginGroup("Panel-Plugins");
    settings.beginGroup("ShowDesktop");
    if(!settings.contains("Destination")){

        settings.setValue("Key",QKeySequence("Ctrl+Alt+D").toString());
        settings.setValue("Destination","panel");
        settings.setValue("Comment","show the desktop minimize all open window...");
    }
    m_ks= settings.value("Key","Ctrl+Alt+D").toString();
    settings.endGroup();//MediaNext

    settings.endGroup();//Panel-Plugins
    return m_ks;
}



void DshowDesktop::setSize(QSize size)
{
          this->setIconSize(size);
    setIcon(QIcon::fromTheme("user-desktop",QIcon::fromTheme("desktop")));

}




void DshowDesktop::showDesktopNew()
{
          bool bShown=false;
          Atom actual_type;
          int actual_format, error;
          Atom _NET_SHOWING_DESKTOP = atom("_NET_SHOWING_DESKTOP" ) ;
          unsigned char * data ;
          unsigned long nitems, after;
          error = XGetWindowProperty(QX11Info::display(), QX11Info::appRootWindow(), _NET_SHOWING_DESKTOP, 0, 1, false, XA_CARDINAL,
                                     &actual_type, &actual_format, &nitems, &after, &data);
          if ( error == Success && NULL != data)
          {
                    bShown = static_cast<bool>(data[0]);
                    XFree(data);
          }

           XClientMessageEvent msg;
          msg.window = QX11Info::appRootWindow();
          msg.type = ClientMessage;
          msg.message_type = atom("_NET_SHOWING_DESKTOP");
          msg.send_event = true;
          msg.display = QX11Info::display();
          msg.format = 32;
          msg.data.l[0] = (unsigned long) !bShown;

          XSendEvent(QX11Info::display(), QX11Info::appRootWindow(), FALSE, (SubstructureRedirectMask | SubstructureNotifyMask) , (XEvent *) &msg);


}
Atom DshowDesktop::atom(const char* atomName)
{
    static QHash<QString, Atom> hash;

    if (hash.contains(atomName))
        return hash.value(atomName);

    Atom atom = XInternAtom(QX11Info::display(), atomName, false);
    hash[atomName] = atom;
    return atom;
}
Q_EXPORT_PLUGIN2(dpanel_showDesktop, DshowDesktop)
