
/********************************************************************
 *(c)GPL3+
  Inspired by freedesktops tint2
original code  http://razor-qt.org
 modified by abou zakaria
*********************************************************************/
//

#include "systray.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/extensions/Xrender.h>
#include <X11/extensions/Xdamage.h>

#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xcomposite.h>


#include <QtGui>
#include <QApplication>
#include <QtDebug>
#include <QX11Info>
#include <QTimer>
#include <QIcon>
#include <QDebug>
#include <QResizeEvent>
#include <QPainter>
#include <QBitmap>
static bool xError;

/************************************************

************************************************/
int windowErrorHandler(Display *d, XErrorEvent *e)
{
          xError = true;
          if (e->error_code != BadWindow) {
                    char str[1024];
                    XGetErrorText(d, e->error_code,  str, 1024);
                    qWarning() << "Error handler" << e->error_code
                               << str;
          }
          return 0;
}



/************************************************

 ************************************************/
SysTray::SysTray(QWidget* parent):
          QWidget( parent),
          m_valid(false),
          m_trayId(0),
          m_damageEvent(0),
          m_damageError(0),
          m_iconSize(22, 22),
          m_row(0),m_col(0),m_panelH(24)
{
          setObjectName("sysTray");
          this->setWindowTitle(trUtf8("System Tray"));
          this->setWhatsThis(tr(" by AbouZakaria"));//شرح الاضافة
          this->setWindowIcon(QIcon::fromTheme("gnome-panel-notification-area",QIcon(":/icons/panel-notification-area")));

          //    layout()->setAlignment(Qt::AlignCenter);
          m_gridLayout = new QGridLayout(this);
          m_gridLayout->setSpacing(1);
          m_gridLayout->setContentsMargins(1, 1, 1, 1);
          m_gridLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
          // Init the selection later just to ensure that no signals are sent until
          // after construction is done and the creating object has a chance to connect.
          QTimer::singleShot(0, this, SLOT(startTray()));
}


/************************************************

 ************************************************/
SysTray::~SysTray()
{
          stopTray();
}


/************************************************
           اشارات مكتبة اكس 11
 ************************************************/
void SysTray::x11EventFilter(XEvent* event)
{
          TrayIcon* icon;

          switch (event->type)
          {
          case ClientMessage:
                    clientMessageEvent(&(event->xclient));
                    break;

                    //        case ConfigureNotify:
                    //            icon = findIcon(event->xconfigure.window);
                    //            if (icon)
                    //                icon->con(&(event->xconfigure));
                    //            break;

          case DestroyNotify:
                    icon = findIcon(event->xany.window);
                    if (icon)
                    {
                              m_iconsList.removeAll(icon);
                              delete icon;
                              realign();
                    }
                    break;

          default:
                    if (event->type == m_damageEvent + XDamageNotify)
                    {
                              XDamageNotifyEvent* dmg = reinterpret_cast<XDamageNotifyEvent*>(event);
                              icon = findIcon(dmg->drawable);
                              if (icon)
                                        icon->update();
                    }
                    break;
          }
}


/************************************************

 ************************************************/
void SysTray::clientMessageEvent(XClientMessageEvent* e)
{
          unsigned long opcode;
          opcode = e->data.l[1];
          Window id;

          switch (opcode)
          {
          case 0:
                    id = e->data.l[2];
                    if (id)
                              addIcon(id);
                    break;


          case 1:
          case 2:
                    qDebug() << "we don't show baloons messages.";
                    break;


          default:
                    if (opcode == Xatom::atom("_NET_SYSTEM_TRAY_MESSAGE_DATA"))
                              qDebug() << "message from dockapp:" << e->data.b;
                    //            else
                    //                qDebug() << "SYSTEM_TRAY : unknown message type" << opcode;
                    break;
          }
}



/************************************************

 ************************************************/
TrayIcon* SysTray::findIcon(Window id)
{
          foreach(TrayIcon* icon, m_iconsList)
          {
                    if (icon->iconId() == id || icon->windowId() == id)
                              return icon;
          }
          return 0;
}


/************************************************

************************************************/
void SysTray::setSize(QSize iconSize)
{
          m_panelH = iconSize.height();
          if(m_panelH<20||m_panelH>30)
              m_iconSize=QSize(16,16);
          else
               m_iconSize=QSize(22,22);

            foreach(TrayIcon* icon, m_iconsList)
                icon->setIconSize(m_iconSize);

          realign();
}
void SysTray::realign()
{
    m_row=0;
    m_col=0;

    foreach(TrayIcon* icon, m_iconsList)
                   m_gridLayout->removeWidget(icon);

    foreach(TrayIcon* icon, m_iconsList){

        m_gridLayout->addWidget(icon,m_row,m_col);


        if(m_panelH >30){
            if(m_row==0){
                m_row=1;
            }else{
                m_row=0;
                m_col++;
            }
        }else{
            m_col++;
        }
    }


}



/************************************************

************************************************/
VisualID SysTray::getVisual()
{
    VisualID visualId = 0;
    Display* dsp = QX11Info::display();

    XVisualInfo templ;
    templ.screen=QX11Info::appScreen();
    templ.depth=32;
    templ.c_class=TrueColor;

    int nvi;
    XVisualInfo* xvi = XGetVisualInfo(dsp, VisualScreenMask|VisualDepthMask|VisualClassMask, &templ, &nvi);

    if (xvi)
    {
        int i;
        XRenderPictFormat* format;
        for (i = 0; i < nvi; i++)
        {
            format = XRenderFindVisualFormat(dsp, xvi[i].visual);
            if (format &&
                    format->type == PictTypeDirect &&
                    format->direct.alphaMask)
            {
                visualId = xvi[i].visualid;
                break;
            }
        }
        XFree(xvi);
    }

    return visualId;
}


/************************************************
   freedesktop systray specification
 ************************************************/
void SysTray::startTray()
{
    Display* dsp = QX11Info::display();
    Window root = QX11Info::appRootWindow();

    QString s = QString("_NET_SYSTEM_TRAY_S%1").arg(DefaultScreen(dsp));
    Atom _NET_SYSTEM_TRAY_S =Xatom::atom(s.toAscii());

    if (XGetSelectionOwner(dsp, _NET_SYSTEM_TRAY_S) != None)
    {
        qWarning() << "Another systray is running";
        m_valid = false;
        return;
    }

    // init systray protocol
    m_trayId = XCreateSimpleWindow(dsp, root, -1, -1, 1, 1, 0, 0, 0);

    XSetSelectionOwner(dsp, _NET_SYSTEM_TRAY_S, m_trayId, CurrentTime);
    if (XGetSelectionOwner(dsp, _NET_SYSTEM_TRAY_S) != m_trayId) {
        qWarning() << "Can't get systray manager";
        stopTray();
        m_valid = false;
        return;
    }

    int orientation =0;
    XChangeProperty(dsp,m_trayId,  Xatom::atom("_NET_SYSTEM_TRAY_ORIENTATION"),  XA_CARDINAL,  32,
                    PropModeReplace, (unsigned char *) &orientation,  1);

    // ** Visual ********************************
    VisualID visualId = getVisual();
    if (visualId){
        XChangeProperty(QX11Info::display(),m_trayId,Xatom::atom("_NET_SYSTEM_TRAY_VISUAL"),XA_VISUALID,
                        32,PropModeReplace, (unsigned char*)&visualId,1);
    }
    // ******************************************

    XClientMessageEvent event;
    event.type = ClientMessage;
    event.window = root;
    event.message_type = Xatom::atom("MANAGER");
    event.format = 32;
    event.data.l[0] = CurrentTime;
    event.data.l[1] = _NET_SYSTEM_TRAY_S;
    event.data.l[2] = m_trayId;
    event.data.l[3] = 0;
    event.data.l[4] = 0;

    XSendEvent(dsp, root, False, StructureNotifyMask, (XEvent*)&event);
    XDamageQueryExtension(QX11Info::display(), &m_damageEvent, &m_damageError);

    qDebug() << "Systray started----------------------------------------------"<<"Ok";
    m_valid = true;
}


/************************************************
              توقيف منطقة الاعلام
 ************************************************/
void SysTray::stopTray()
{
          qDeleteAll(m_iconsList);
          if (m_trayId) {
                    XDestroyWindow(QX11Info::display(), m_trayId);
                    m_trayId = 0;
          }
          m_valid = false;
}


/************************************************
                اصافة الايقونة الى القائمة
 ************************************************/
void SysTray::addIcon(Window winId)
{
          TrayIcon* icon = new TrayIcon(winId, this);
          if (!icon->isValid())
          {
                    delete icon;
                    return;
          }


          m_iconsList.append(icon);

 icon->setIconSize(m_iconSize);
   realign();

}








//!   ***********************************************
//!               CLASS TRAYICON
//!   ***********************************************
TrayIcon::TrayIcon(Window iconId, QWidget* parent):
          QWidget(parent),
          m_iconId(iconId),
          m_windowId(0),
          m_iconSize(22, 22)/*,
                        m_damage(0)*/
{
          setObjectName("TrayIcon");
          setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

          m_iconValid = init();

}


/************************************************
                    التحقق من استخدامها
 ************************************************/
bool TrayIcon::init()
{
    Display* dsp = QX11Info::display();

    XWindowAttributes attr;
    if (! XGetWindowAttributes(dsp, m_iconId, &attr))
        return false;


    unsigned long mask = 0;
    XSetWindowAttributes set_attr;

    Visual* visual = attr.visual;
    set_attr.colormap = attr.colormap;
    set_attr.background_pixel = 0;
    set_attr.border_pixel = 0;
    mask = CWColormap|CWBackPixel|CWBorderPixel;

    m_windowId = XCreateWindow(dsp, this->winId(), 0, 0, m_iconSize.width(), m_iconSize.height(),
                              0, attr.depth, InputOutput, visual, mask, &set_attr);


    xError = false;
    XErrorHandler old;
    old = XSetErrorHandler(windowErrorHandler);
    XReparentWindow(dsp, m_iconId, m_windowId, 0, 0);
    XSync(dsp, false);
    XSetErrorHandler(old);

    if (xError)
    {

        qWarning() << "* Not icon_swallow *";
        XDestroyWindow(dsp, m_windowId);
        return false;
    }


    {
        Atom acttype;
        int actfmt;
        unsigned long nbitem, bytes;
        unsigned char *data = 0;
        int ret;

        ret = XGetWindowProperty(dsp, m_iconId, Xatom::atom("_XEMBED_INFO"),
                                 0, 2, false, Xatom::atom("_XEMBED_INFO"),
                                 &acttype, &actfmt, &nbitem, &bytes, &data);
        if (ret == Success)
        {
            if (data)
                XFree(data);
        }
        else
        {
            qWarning() << "TrayIcon: xembed error";
            XDestroyWindow(dsp, m_windowId);
            return false;
        }
    }

    {
        XEvent e;
        e.xclient.type = ClientMessage;
        e.xclient.serial = 0;
        e.xclient.send_event = True;
        e.xclient.message_type = Xatom::atom("_XEMBED");
        e.xclient.window = m_iconId;
        e.xclient.format = 32;
        e.xclient.data.l[0] = CurrentTime;
        e.xclient.data.l[1] = 0;
        e.xclient.data.l[2] = 0;
        e.xclient.data.l[3] = m_windowId;
        e.xclient.data.l[4] = 0;
        XSendEvent(dsp, m_iconId, false, 0xFFFFFF, &e);
    }

    XSelectInput(dsp, m_iconId, StructureNotifyMask);
     m_damage = XDamageCreate(dsp, m_iconId, XDamageReportRawRectangles);
   XCompositeRedirectWindow(dsp, m_windowId, CompositeRedirectManual);

    XMapWindow(dsp, m_iconId);
    XMapRaised(dsp, m_windowId);

   XResizeWindow(QX11Info::display(),m_windowId, m_iconSize.width(), m_iconSize.height());
   XResizeWindow(QX11Info::display(),m_iconId,   m_iconSize.width(), m_iconSize.height());

    return true;
}


/************************************************
           الخروج
 ************************************************/
TrayIcon::~TrayIcon()
{

          Display* dsp = QX11Info::display();
          XSelectInput(dsp, m_iconId, NoEventMask);
          //***************mDamage**************************************************************************
          if (m_damage)
                   XDamageDestroy(dsp, m_damage);

                    // reparent to root
                    xError = false;
          XErrorHandler old = XSetErrorHandler(windowErrorHandler);

          XUnmapWindow(dsp, m_iconId);
          XReparentWindow(dsp, m_iconId, QX11Info::appRootWindow(), 0, 0);

          XDestroyWindow(dsp, m_windowId);
          XSync(dsp, False);
          XSetErrorHandler(old);
}


/************************************************
                  ارجاع حجم الايقونة
 ************************************************/
QSize TrayIcon::sizeHint() const
{
          QMargins margins = contentsMargins();
          return QSize(margins.left() + m_iconSize.width() + margins.right(),
                       margins.top() + m_iconSize.height() + margins.bottom()
                       );
}


/************************************************

 ************************************************/
void TrayIcon::setIconSize(const QSize &iconSize)
{
    m_iconSize = iconSize;

    if (m_windowId)
        XResizeWindow(QX11Info::display(),m_windowId,m_iconSize.width(), m_iconSize.height());

    if (m_iconId)
        XResizeWindow(QX11Info::display(), m_iconId,  m_iconSize.width(), m_iconSize.height());
}


/************************************************
                       الاحداث
 ************************************************/
bool TrayIcon::event(QEvent *event)
{
          switch (event->type())
          {
          case QEvent::Paint:

                    draw(static_cast<QPaintEvent*>(event));
                    break;

          case QEvent::Resize:
          {
                    QRect rect = iconGeometry();

                     XMoveWindow(QX11Info::display(), m_windowId, rect.left(),  rect.top());

          }
                    break;

          case QEvent::MouseButtonPress:

                    event->accept();

                    break;

          default:
                    break;
          }


          return QWidget::event(event);
}


/************************************************
                   الموضع والاحداث
 ************************************************/
QRect TrayIcon::iconGeometry()
{
          QRect res = QRect(QPoint(0, 0), m_iconSize);

          res.moveCenter(QRect(0, 0, width(), height()).center());
          return res;
}


/************************************************
                     رسم الايقونة
 ************************************************/
void TrayIcon::draw(QPaintEvent* /*event*/)
{
          Display* dsp = QX11Info::display();

          XWindowAttributes attr;
          if (!XGetWindowAttributes(dsp, m_iconId, &attr))
          {
                    qWarning() << "Paint error";
                    return;
          }

          XImage* ximage = XGetImage(dsp, m_iconId, 0, 0, attr.width, attr.height, AllPlanes, ZPixmap);
          if (!ximage)
          {
                    qWarning() << "    * Error image is NULL";
                    return;
          }
         //const uchar* d =(uchar*) ximage->data;
          QImage image = QImage((const uchar*) ximage->data, ximage->width, ximage->height, ximage->bytes_per_line,  QImage::Format_ARGB32_Premultiplied);


          // Draw QImage ...........................
          QPainter painter(this);

          QRect iconRect = iconGeometry();

          if (image.size() != iconRect.size())
          {
                    image = image.scaled(iconRect.size(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
                    QRect r = image.rect();
                    r.moveCenter(iconRect.center());
                    iconRect = r;
          }
      //   qDebug() << " Draw rect:" << iconRect;

          painter.drawImage(iconRect, image);


     XDestroyImage(ximage);
         // qDebug() << "End paint icon **********************************";
}


Atom Xatom::atom(const char* atomName)
{
    static QHash<QString, Atom> hash;

    if (hash.contains(atomName))
        return hash.value(atomName);

    Atom atom = XInternAtom(QX11Info::display(), atomName, false);
    hash[atomName] = atom;
    return atom;
}


Q_EXPORT_PLUGIN2(dpanel_systry, SysTray)
