/********************************************************************
 *(c)GPL3+

original code  http://razor-qt.org
 modified by abou zakaria
*********************************************************************/
#include "dtaskbarwidget.h"
#include "dactiontaskbar.h"
#include "taskbarsettingdialog.h"

#include <QtGui>
#include <QX11Info>
#include <QDesktopWidget>
#include <X11UTILLS>
#define TEXTBICON 0
#define ICONONLY 1

DtaskbarWidget::DtaskbarWidget(QWidget *parent):
    QWidget(parent)
{

    this->setObjectName("dtaskbar");
    this->setWindowTitle(trUtf8("Taskbar"));
    this->setWhatsThis(tr("by AbouZakaria"));//شرح الاضافة
    this->setWindowIcon(QIcon::fromTheme("gnome-panel-window-list",QIcon(":/icons/panel-window-list")));
  //  setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Preferred);
    this->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this,SIGNAL(customContextMenuRequested(QPoint)) ,this,SLOT(customContextMenu(QPoint)));
    m_rootWindow = QX11Info::appRootWindow();
    m_horizontalLayout = new QHBoxLayout(this);
    m_horizontalLayout->setSpacing(1);
    m_horizontalLayout->setContentsMargins(0,0,0,0);
    m_horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
    m_horizontalLayout->addSpacing(10);

    horizontalSpacer = new QSpacerItem(5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum);

    m_actSetting=new QAction(QIcon::fromTheme("configure",QIcon::fromTheme("document-properties")),
                             trUtf8("Taskbar Settings"),this);

    connect(m_actSetting,SIGNAL(triggered()) ,this,SLOT(getStyleIcons()));
    loadSetting();

    //   this->setStyleSheet("QWidget{background-color: red  ;}");

}




void DtaskbarWidget::customContextMenu(QPoint)
{
    QMenu menu;


    menu.addAction(m_actSetting);

    menu.addActions(m_actList);
    menu.exec(this->cursor().pos());
}
void DtaskbarWidget::getStyleIcons()
{
    taskbarSettingDialog *dlg=new taskbarSettingDialog;
    connect(dlg,SIGNAL(settingChanged()),this,SLOT(setIconStyle()));
    dlg->show();


}
void DtaskbarWidget::setIconStyle()
{
    loadSetting();

    QList<DActionTaskbar *> allButtons = this->findChildren<DActionTaskbar *>();
    foreach (DActionTaskbar *btn, allButtons)
    {
        btn->setToolButtonStyle(mButtonStyle);
        btn->setIconSize(m_size);
        if(m_iconStyle==TEXTBICON){

            btn->setMaximumWidth(QWIDGETSIZE_MAX);
        }else{

            btn->setMaximumWidth(m_size.width());
        }
    }
}

void DtaskbarWidget::loadSetting()
{
    QSettings setting;

    setting.beginGroup("Taskbar");
    m_iconStyle=setting.value("iconStyle",0).toInt();
    setting.endGroup();
    if(m_iconStyle==TEXTBICON){
        mButtonStyle=(Qt::ToolButtonTextBesideIcon);
    }else{
        mButtonStyle=(Qt::ToolButtonIconOnly);
    }

}

void DtaskbarWidget::saveSetting()
{
    QSettings setting;
    setting.beginGroup("Taskbar");
    setting.setValue("iconStyle",m_iconStyle);

    setting.endGroup();
}

//*******************taskbar**************************//

void DtaskbarWidget::refreshTaskList()
{


          //              قائمة مؤقة بجميع معرفات النوافذ الحاضرة
          QList<unsigned long> listWindow = X11UTILLS::getClientList();

          //                  حذف جميع الازرار ومعرفاتها السابقة
          if(mButtonsHash.count()>0){
                    //              قائمة بجميع معرفات النولفذ والازرار
                    QMutableHashIterator<unsigned long, DActionTaskbar*> i(mButtonsHash);
                    while (i.hasNext())
                    {

                              i.next();

                              int     r = listWindow.removeAll(i.key());

                              if (!r)
                              {
                                        delete i.value();
                                        i.remove();
                              }

                    }
          }

          m_horizontalLayout->removeItem(horizontalSpacer);

          foreach (unsigned long wnd, listWindow)
          {


                    //            انشاء زر جديد للنوافذ الحاضرة
                    DActionTaskbar *btn=new DActionTaskbar(wnd,this);
                    //           نوع الزر هل هو ارمز والتص ام الرمز فقط
                    btn->setToolButtonStyle(mButtonStyle);
                    btn->setParentAction(m_actSetting);
                    //            اضافة ازر للقائمة
                    mButtonsHash.insert(wnd, btn);

                    //        اضافة الزر للبنال
                    m_horizontalLayout->addWidget(btn);
                    btn->setIconSize(m_size);

          }

        m_horizontalLayout->addItem(  horizontalSpacer);

          int bwidth=300;
          if(mButtonsHash.count()>1){
                    bwidth=this->width()/mButtonsHash.count();
                  if(bwidth>300)     bwidth=300;

          }


          foreach (DActionTaskbar *btn,mButtonsHash){

                    if(m_iconStyle==TEXTBICON){
                             btn->setMaximumWidth(bwidth);
                              btn->resize(bwidth,m_size.height());

                    }else{

                              btn->setMaximumWidth(btn->sizeHint().width());
                              btn->setMinimumWidth(btn->sizeHint().width());

                    }
          }

          activeWindowChanged();

}

// النافذة المفعلة
void DtaskbarWidget::activeWindowChanged()
{


    foreach (DActionTaskbar *btn,mButtonsHash){

        btn->setText(X11UTILLS::getWindowTitle(mButtonsHash.key(btn)));
        btn->setChecked(false);
        btn->setIconSize(m_size);

    }

    //        البحث عن النافذة المفعلة
    unsigned long window =X11UTILLS::getActiveAppWindow();
    //         التاكد من وجودها مع الازرا
    DActionTaskbar* toolbtn =0;
     if (mButtonsHash.contains(window))
           toolbtn=mButtonsHash.value(window);
    //       تفيل زر النافذة المفعلة اذا كان موجودا
    if (toolbtn){
        toolbtn->setChecked(true);
    }


}



//***********************mose wheel*************
void DtaskbarWidget::wheelEvent(QWheelEvent* event)
{

    QList<unsigned long> winList = X11UTILLS::getClientList();
    int current = winList.indexOf(X11UTILLS::getActiveAppWindow());
    int delta = event->delta() < 0 ? 1 : -1;

    for (int ix = current + delta; 0 <= ix && ix < winList.size(); ix += delta)
    {
        unsigned long window = winList.at(ix);

            X11UTILLS::raiseWindow(window);
            break;

    }
}


//******************Virtual************

void  DtaskbarWidget::x11EventFilter(XEvent* event)
{

    if(event->type==PropertyNotify) {


        if (event->xproperty.window == m_rootWindow)
        {

            if (event->xproperty.atom == X11UTILLS::atom("_NET_CLIENT_LIST"))
            {

                refreshTaskList();
                return;
            }

            // Activate window ....................................
            if (event->xproperty.atom == X11UTILLS::atom("_NET_ACTIVE_WINDOW"))
            {
               // qDebug()<<"activeWindowChanged : ";
                activeWindowChanged();
                return;
            }

        }

        unsigned long w=   X11UTILLS::getActiveAppWindow();;
        if(mButtonsHash.contains(w)){
            mButtonsHash[w]->windowPropertyChanged(event->xproperty.atom );
        }

    }

}



void DtaskbarWidget::setKeySequence(QKeySequence /*sequence*/)
{

}

QString   DtaskbarWidget::keySequence(){
    return ("");
}


void DtaskbarWidget::setSize(QSize size)
{
    m_size=size;

    this->setMinimumHeight(m_size.height());
    refreshTaskList();
}




Q_EXPORT_PLUGIN2(dpanel_taskbar, DtaskbarWidget)
