/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "dialogconfigtimer.h"
#include "ui_dialogconfigtimer.h"
#include <QSettings>
#include <QDebug>
DialogConfigTimer::DialogConfigTimer(QWidget *parent) :
        QDialog(parent),
        ui(new Ui::DialogConfigTimer)
{


        ui->setupUi(this);
        ui->labelsystemNumber->setVisible(false);
        this->setWindowIcon(QIcon::fromTheme("configure",QIcon::fromTheme("document-properties")));
        this->setAttribute(Qt::WA_DeleteOnClose,true);
        int curLocaleIndex = -1;
        int index = 0;
        for (int _lang = QLocale::C; _lang <= QLocale::LastLanguage; ++_lang) {
                QLocale::Language lang = static_cast<QLocale::Language>(_lang);
                QList<QLocale::Country> countries = QLocale::countriesForLanguage(lang);
                for (int i = 0; i < countries.count(); ++i) {
                        QLocale::Country country = countries.at(i);
                        QString label = QLocale::languageToString(lang);
                        label += QLatin1Char('-');
                        label += QLocale::countryToString(country);
                        QLocale locale(lang, country);
                        if (this->locale().language() == lang && this->locale().country() == country)
                                curLocaleIndex = index;
                        ui->comboBoxLocale->addItem(label, locale);
                        ++index;
                }
        }
        if (curLocaleIndex != -1)
                ui->comboBoxLocale->setCurrentIndex(curLocaleIndex);

        loadSetting();

}

DialogConfigTimer::~DialogConfigTimer()
{
        //  qDebug()<<"dialog deleted";
        delete ui;

}
void DialogConfigTimer::loadSetting()
{
        QSettings setting;
        setting.beginGroup("TimeDate");
        ui->checkBoxShowSecond->setChecked(setting.value("showSeconds",false).toBool());
        ui->checkBoxHours->setChecked(setting.value("24Hours",true).toBool());
        QString fName=setting.value("FontName",this->font().family()).toString();
        ui->comboBoxNumber->setCurrentIndex(setting.value("NumberStyle",0).toInt());
        QString lc=setting.value("CalendarLocale",this->locale().name()).toString();
        int firstDay=setting.value("CalendarFirstDay",7).toInt();
        setting.endGroup();

        //   on_comboBoxFirstDay_currentIndexChanged(firstDay);

        int indx=  ui->comboBoxLocale->findData(QLocale(lc));
        if (indx != -1){
                ui->comboBoxLocale->setCurrentIndex(indx);

                chargeDays(indx);
                int indxF=ui->comboBoxFirstDay->findData(firstDay);
                ui->comboBoxFirstDay->setCurrentIndex(indxF);
        }
        ui->fontComboBox->setEditText(fName);
        ui->fontComboBox->setCurrentIndex(ui->fontComboBox->findText(fName));


}
void DialogConfigTimer::chargeDays(int indx)
{
        if(indx==-1)
                return;
        int curIndex    = ui->comboBoxFirstDay->currentIndex();
        QString countryName=     ui->comboBoxLocale->itemData(indx).toLocale().name();
        QLocale lc=QLocale(countryName);

        ui->comboBoxFirstDay->clear();
        ui->comboBoxFirstDay->addItem(lc.dayName(Qt::Friday), Qt::Friday);
        ui->comboBoxFirstDay->addItem(lc.dayName(Qt::Saturday), Qt::Saturday);
        ui->comboBoxFirstDay->addItem(lc.dayName(Qt::Sunday), Qt::Sunday);
        ui->comboBoxFirstDay->addItem(lc.dayName(Qt::Monday), Qt::Monday);
        ui->comboBoxFirstDay->addItem(lc.dayName(Qt::Tuesday), Qt::Tuesday);
        ui->comboBoxFirstDay->addItem(lc.dayName(Qt::Wednesday), Qt::Wednesday);
        ui->comboBoxFirstDay->addItem(lc.dayName(Qt::Thursday), Qt::Thursday);
        ui->comboBoxFirstDay->setCurrentIndex(curIndex);
}

void DialogConfigTimer::saveSetting()
{
        QString lc=ui->comboBoxLocale->itemData
                        (ui->comboBoxLocale->currentIndex()).toLocale().name();
        int firstDay=ui->comboBoxFirstDay->itemData
                        (ui->comboBoxFirstDay->currentIndex()).toInt() ;

        QSettings setting;
        setting.beginGroup("TimeDate");
        setting.setValue("showSeconds", ui->checkBoxShowSecond->isChecked());
        setting.setValue("24Hours",ui->checkBoxHours->isChecked());
        setting.setValue("FontName",ui->fontComboBox->currentText());
        setting.setValue("NumberStyle",ui->comboBoxNumber->currentIndex());
        setting.setValue("CalendarLocale",lc);
        setting.setValue("CalendarFirstDay",firstDay);
        setting.endGroup();
}
void DialogConfigTimer::on_buttonBox_accepted()
{
        saveSetting();
        emit settingsChanged();
}

void DialogConfigTimer::on_buttonBox_clicked(QAbstractButton *button)
{
        if(ui->buttonBox->standardButton(button)==QDialogButtonBox::Apply){
                saveSetting();
                emit settingsChanged();
        }
}

void DialogConfigTimer::on_comboBoxNumber_activated(int index)
{
        if(index==0)
                ui->labelsystemNumber->setVisible(true);
        else
                ui->labelsystemNumber->setVisible(false);
}

void DialogConfigTimer::on_comboBoxLocale_currentIndexChanged(int index)
{
        //  calendar->setLocale(localeCombo->itemData(index).toLocale());
        qDebug()<<ui->comboBoxLocale->itemData(index).toLocale().name();
        if(index!=-1)
                chargeDays(index);

}

void DialogConfigTimer::on_comboBoxFirstDay_currentIndexChanged(int index)
{
        //  calendar->setFirstDayOfWeek(Qt::DayOfWeek( firstDayCombo->itemData(index).toInt()));
        qDebug()<<ui->comboBoxFirstDay->itemData(index).toInt()     ;


}
