/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "labeltime.h"
#include "dialogconfigtimer.h"

#include <QTime>
#include <QDateTime>
#include <QSettings>


#include <QMenu>

#include <X11/Xlib.h>


#define NUMBER_SYSTEM 0
#define NUMBER_ARABIC 1
#define NUMBER_INDIC 2
LabelTime::LabelTime(QWidget *parent) :
          QLabel(parent)
{
          this->setObjectName("timeDate");
          this->setWindowTitle(trUtf8("Clock"));
          this->setWhatsThis(tr("by AbouZakaria"));//شرح الاضافة
          this->setWindowIcon(QIcon::fromTheme("clock",QIcon::fromTheme("gnome-panel-clock")));

     mCalendarWidget=new QCalendarWidget;
     mCalendarWidget->setStyleSheet("color: palette(text);");
     mWidgetPopup=new WidgetPopup(this,mCalendarWidget);
          loadSetting();
                 setFrameShape(QFrame::NoFrame);
          //setSegmentStyle(QLCDNumber::Flat);
                 this->setContextMenuPolicy(Qt::CustomContextMenu);

          connect(this,SIGNAL(customContextMenuRequested(QPoint)) ,this,SLOT(customContextMenu(QPoint)));
          timer=new QTimer;
          connect(timer,SIGNAL(timeout()) ,this,SLOT(updatTime()));
          timer->start(1000);

}


void LabelTime::updatTime()
{


   localCalandar.setNumberOptions(QLocale::OmitGroupSeparator);


   this->setText( localTimerNum.toString( QTime::currentTime(),m_timeFormat));
         //  display(text);
          setToolTip(localCalandar.toString(QDate::currentDate(),"dddd dd MMMM  yyyy"));
          //qDebug()<<m_timeFormat;
}


void LabelTime::loadSetting()
{
        QSettings setting;
        setting.beginGroup("TimeDate");
        bool showS=(setting.value("showSeconds",false).toBool());
        bool Hours=(setting.value("24Hours",true).toBool());
        QString fName=setting.value("FontName",this->font().family()).toString();
        m_numberStyle=setting.value("NumberStyle",NUMBER_SYSTEM).toInt();
        QString   m_lc=setting.value("CalendarLocale",this->locale().name()).toString();
        m_firstDay=setting.value("CalendarFirstDay",7).toInt();
        setting.endGroup();
        localCalandar=QLocale(m_lc);
        if(m_numberStyle==NUMBER_SYSTEM)
                localTimerNum=(this->locale());
        else if(m_numberStyle==NUMBER_ARABIC)
                localTimerNum=(QLocale::English);
        else if(m_numberStyle==NUMBER_INDIC)
                localTimerNum=(QLocale::Arabic);

        QFont font;
        font.setFamily(fName);
        font.setPointSize(mSize);
        setFont(font);
        if(showS==true&&Hours==true){
                m_timeFormat="hh:mm:ss";
                //setDigitCount(8);
        }
        if(showS==false&&Hours==false){
                m_timeFormat="hh:mm ap";
                //setDigitCount(8);
        }
        if(showS==true&&Hours==false){
                m_timeFormat="hh:mm:ss ap";
                //setDigitCount(10);
        }
        if(showS==false&&Hours==true){
                m_timeFormat="HH:mm";
                //setDigitCount(5);
        }
}

//void LabelTime::saveSetting()
//{
//          QSettings setting;
//          setting.beginGroup("TimeDate");
//          setting.setValue("timeFormat",m_timeFormat);
//          setting.endGroup();
//}
void LabelTime::customContextMenu(QPoint)
{
          QMenu menu;
         menu.addAction(QIcon::fromTheme("configure",
                                           QIcon::fromTheme("document-properties")),
                                           tr("Clock settings") ,this,SLOT(getFormat()));


     menu.addActions(mlistAction);


          menu.exec(this->cursor().pos());
}

void LabelTime::getFormat()
{

          DialogConfigTimer *dlg=new DialogConfigTimer;
          connect(dlg,SIGNAL(settingsChanged()),this,SLOT(loadSetting()));
          dlg->show();

}

//***********************************************
void LabelTime::mouseReleaseEvent(QMouseEvent* event)
{

    if (event->button() != Qt::LeftButton)
        return;

    mCalendarWidget->setLocale(localCalandar);
    mCalendarWidget->setFirstDayOfWeek(Qt::DayOfWeek(m_firstDay));
//mCalendarWidget->setCurrentPage(QDate::currentDate().day(),QDate::currentDate().month());
mCalendarWidget->setSelectedDate(QDate::currentDate());
    int mPosbtn=mapToGlobal(QPoint(geometry().width()/2,geometry().bottom()/2)).x();

    int wW=(this->width()/2);

    mWidgetPopup->open(mapToGlobal(QPoint(geometry().width()-wW,geometry().bottom()))
                  , mPosbtn);
}

 void LabelTime::themeChanged()
 {
         this->setWindowIcon(QIcon::fromTheme("clock",QIcon::fromTheme("gnome-panel-clock")));

 }
void LabelTime::setSize(QSize size)
{
mSize=size.width()-8;
        QFont font;
        font.setPointSize(size.width()-8);
      font.setFamily(this->font().family());
        setFont(font);
}
Q_EXPORT_PLUGIN2(dpanel_time, LabelTime)
