/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Razor - a lightweight, Qt based, desktop toolset
 * http://razor-qt.org
 *
 * Copyright: 2012 Razor team
 * Authors:
 *   Johannes Zellner <webmaster@nebulon.de>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "elokabvolume.h"

#include "volumebutton.h"
//#include "volumepopup.h"

#include "audiodevice.h"
#include "alsaengine.h"




ElokabVolume::ElokabVolume(QWidget* parent):
        QLabel(parent),
        mEngine(0),
        mSinkIndex(0),
        mSink(0),
        mSize(22,22)
{
// setObjectName("Volume");

setContextMenuPolicy(Qt::CustomContextMenu);
connect(this,SIGNAL(customContextMenuRequested(QPoint)),this,SLOT(showMenuContext()));
     QIcon icon=QIcon::fromTheme("audio-volume-high",QIcon::fromTheme("audio-volume-high-panel",QIcon(":/icons/audio-volume-high")));
     setPixmap(QPixmap(icon.pixmap(mSize)));
keySequence();
setupProgressSlider();
setScaledContents(true);
        //********************************************


        mWidgetVolume=new WidgetVolume();

      mwidgetpopup=new WidgetPopup(this,mWidgetVolume);
        connect(mWidgetVolume, SIGNAL(launchMixer()), this, SLOT(launchMixer()));
        connect(mWidgetVolume, SIGNAL(stockIconChanged(int)), this, SLOT(iconChanged(int)));


        if (!mEngine)
                setAudioEngine(new AlsaEngine(this));

        //**********************************
        settingsChanged();



}
/*virtual*/ QString ElokabVolume::keySequence()

{


        QSettings settings("elokab","shortcut");

//--------------------save
        settings.beginGroup("Panel-Plugins");
        if(!settings.contains(QKeySequence(Qt::Key_VolumeUp).toString())){
                settings.beginGroup("VolumeUp");
                 settings.setValue("Key",QKeySequence(Qt::Key_VolumeUp).toString());
                settings.endGroup();//VolumeUp
        }
        if(!settings.contains(QKeySequence(Qt::Key_VolumeDown).toString())){
                settings.beginGroup("VolumeDown");
                 settings.setValue("Key",QKeySequence(Qt::Key_VolumeDown).toString());
                settings.endGroup();//VolumeDown
        }
        if(!settings.contains(QKeySequence(Qt::Key_VolumeMute).toString())){
                settings.beginGroup("VolumeMute");
                 settings.setValue("Key",QKeySequence(Qt::Key_VolumeMute).toString());
                settings.endGroup();//VolumeMute
        }
        //-------------------load
        settings.beginGroup("VolumeUp");
        m_ksUp= settings.value("Key",QKeySequence(Qt::Key_VolumeUp).toString()).toString();
        settings.endGroup();//VolumeUp

        settings.beginGroup("VolumeDown");
        m_ksDown= settings.value("Key",QKeySequence(Qt::Key_VolumeDown).toString()).toString();
        settings.endGroup();//VolumeDown

        settings.beginGroup("VolumeMute");
        m_ksMute= settings.value("Key",QKeySequence(Qt::Key_VolumeMute).toString()).toString();
        settings.endGroup();//VolumeMute


        settings.endGroup();//Panel-Plugins

        return "";


}

void ElokabVolume::setKeySequence(QKeySequence sequence)
{

//qDebug()<<sequence;
    if(sequence==QKeySequence(m_ksUp)){
       shortcutVolumeUp();
    }else if(sequence==QKeySequence(m_ksDown)){
        shortcutVolumeDown();
    }else if(sequence==QKeySequence(m_ksMute)){
        shortcutVolumeMute();
    }
}
void ElokabVolume::mouseReleaseEvent(QMouseEvent* event)
{
 if (event->button() == Qt::LeftButton){
    togglePopupVolume();
 }
}
void ElokabVolume::showDialogDevice()
{
    QStringList items;

    if (mEngine){
        foreach (const AudioDevice *dev, mEngine->sinks()) {
            items.append(dev->description());

        }
        qDebug()<<mSinkIndex;


        QInputDialog *dlg=new QInputDialog;
        dlg->setWindowTitle( tr("Device:"));
        dlg->setLabelText( tr("select device:"));
        dlg->setComboBoxItems(items);
        dlg->setComboBoxEditable(false);//
        connect(dlg,SIGNAL(textValueSelected(QString)),this,SLOT( sinkSelectionChanged(QString)));
        dlg->show();

    }

}
void ElokabVolume::showDialogMexer()
{

        QInputDialog *dlg=new QInputDialog;
        dlg->setWindowTitle( tr("Mixer:"));
        dlg->setLabelText( tr("Enter Mixer name:"));
       dlg->setTextValue(mMixerCommand);
       dlg->setTextEchoMode(QLineEdit::Normal);
        connect(dlg,SIGNAL(textValueSelected(QString)),this,SLOT( mixerLineEditChanged(QString)));
        dlg->show();
}

void ElokabVolume::showMenuContext()
{

  //اعدادات القائمة المنسدلة
  QMenu menu;
  QAction *a;
  Q_UNUSED(a);
  a=menu.addAction(QIcon::fromTheme(""),tr("Select Device"),this,SLOT(showDialogDevice()));
  a=menu.addAction(QIcon::fromTheme(""),tr("Select Mixer"),this,SLOT(showDialogMexer()));
menu.exec(QCursor::pos());


}
void ElokabVolume::setupProgressSlider()
{


    QString style=(QString::fromUtf8("QProgressBar {\n"
                                     "	background-color: rgba(188, 188, 188, 100);\n"
                                     "     border: 2px solid rgba(255, 255, 255, 195);\n"
                                     "	color: rgb(0, 0, 0);\n"
                                     "     border-radius: 5px;\n"
                                     "text-align: center;\n"
                                     " }\n"
                                     "\n"
                                     "QProgressBar::chunk {\n"
                                     "   \n"
                                     "	background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(255, 255, 255, 110), stop:0.488636 rgba(255, 255, 255, 237), stop:1 rgba(255, 255, 255, 110));\n"
                                     "\n"
                                     "     width: 10px;\n"
                                     "     margin: 1.5px;\n"
                                     " }"));
    progressBar = new QProgressBar;
    progressBar->resize(272, 41);
    progressBar->setObjectName(QString::fromUtf8("progressBar"));
    QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(progressBar->sizePolicy().hasHeightForWidth());
    progressBar->setSizePolicy(sizePolicy);
    QFont font;
    font.setPointSize(14);
    font.setBold(true);
    font.setWeight(75);
    progressBar->setFont(font);
    progressBar->setValue(50);
    progressBar->setStyleSheet(style);
    progressBar->setWindowFlags ( Qt::Popup | Qt::WindowStaysOnTopHint | Qt::CustomizeWindowHint | Qt::X11BypassWindowManagerHint);
    progressBar->setAttribute(Qt::WA_TranslucentBackground,true);
    QRect screen = QApplication::desktop()->availableGeometry(this);
    int x=screen.width()/2-progressBar->width()/2;
    int y=screen.height()/2-progressBar->height()/2;
    progressBar->move(x,y);
    timeSlide = new QTimer(this);
    connect(timeSlide, SIGNAL(timeout()), this, SLOT(progressHide()));

}


void ElokabVolume::mixerLineEditChanged(const QString &command)
{
    qDebug()<<"command :"<<command;
        QSettings settings("elokab","volume");
        settings.beginGroup("Main");
        settings.setValue(SETTINGS_MIXER_COMMAND, command);
        settings.endGroup();
        settingsChanged();
}
void ElokabVolume::sinkSelectionChanged(const QString &item)
{
    int index=0;
    qDebug()<<"item device selected :"<<item;
     if (mEngine)
        foreach (const AudioDevice *dev, mEngine->sinks()) {
            if(item==dev->description())
               index= dev->index();

     }

        QSettings settings("elokab","volume");
        settings.beginGroup("Main");
        settings.setValue(SETTINGS_DEVICE, index);
        settings.endGroup();
        settingsChanged();
}

//! تشغيل مخلط خارجي
void ElokabVolume::launchMixer()
{
        //  QProcess::startDetached(m_mixerCommand);
        QProcess::startDetached(mMixerCommand);
}
//! تغير الايقونات
void ElokabVolume::iconChanged(int value)
{
//    QIcon icon=QIcon::fromTheme(iconName,QIcon::fromTheme(iconName+"-panel",QIcon(":/icons/"+iconName)));
//    setPixmap(QPixmap(icon.pixmap(mSize)));
   // mTrayIcon->setIcon(QIcon::fromTheme(iconName,QIcon::fromTheme(iconName+"-panel",QIcon(":/icons/"+iconName))));

    QPixmap pix(48,48);
    pix.fill(Qt::transparent);

    QPainter painter;
    QColor color=this->palette().text().color();

    painter.begin(&pix);
    painter.setPen(color);


    //
        painter.fillRect(2, 19, 4, 11,color);
        QPolygonF polygon;
        polygon << QPointF(6.0, 18.0) << QPointF(16.0, 8.0)
                << QPointF(16.0, 40.0)<< QPointF(5.0, 29.0)
                   << QPointF(6.0, 18.0) ;



        QPainterPath myPath;

        myPath.addPolygon(polygon);

        painter.setBrush(color);

        painter.drawPath(myPath);
        QRect rectMut(20,13,25,25);
        QRect rect1(20,16,4,14);
        QRect rect2(28,9,4,28);
        QRect rect3(37,4,4,39);
        color.setAlpha(50);
        painter.fillRect(rect1,color);
        painter.fillRect(rect2,color);
        painter.fillRect(rect3,color);
         color.setAlpha(200);
        if (value <= 0){
            QFont font;
            font.setBold(true);
            font.setPointSize(20);
            painter.setFont(font);
             painter.drawText(rectMut, Qt::AlignCenter, ("X"));
         //   iconName = "audio-volume-muted";
        }else if (value <= 18){
             color.setAlpha(100);
            painter.fillRect(rect1,color);
        }else if (value <= 33){
            painter.fillRect(rect1,color);
          }else if (value <= 60){
            painter.fillRect(rect1,color);
             painter.fillRect(rect2,color);
            // iconName = "audio-volume-medium";
       } else{
           //  iconName = "audio-volume-high";
           painter.fillRect(rect1,color);
           painter.fillRect(rect2,color);
           painter.fillRect(rect3,color);
        }
        setPixmap(pix);

}

/*************************************
 **********نافذة تغيير الصوت************
 *************************************/
//! اظهار اخفاء نافذة تغيير الصوت
void ElokabVolume::togglePopupVolume()
{
        if (mwidgetpopup->isVisible()) {
                mwidgetpopup->hide();
        } else {
                realignePopup(false);
        }
}
//!  تموضع نافذة تغيير الصوت
void ElokabVolume::realignePopup(bool progress)
{
        if (mwidgetpopup->isVisible())
                return;

//هنا جلب اعدادت العقاب بانال


mWidgetVolume->updateGeometry();


mWidgetVolume->setProgress(progress);
int mPosbtn=mapToGlobal(QPoint(geometry().width()/2,geometry().bottom()/2)).x();

int wW=(this->width()/2);

mwidgetpopup->open(mapToGlobal(QPoint(geometry().width()-wW,geometry().bottom()))
              , mPosbtn);


}
//  نحديد معامل الصوت المستخدم
void ElokabVolume::setAudioEngine(AudioEngine *engine)
{
        if (mEngine) {
                if (mEngine->backendName() == engine->backendName())
                        return;

                mWidgetVolume->setDevice(0);

                disconnect(mEngine, 0, 0, 0);
                delete mEngine;
                mEngine = 0;
        }

        mEngine = engine;

}

//! تغير الاعدادات
void ElokabVolume::settingsChanged()
{



        QSettings settings("elokab","volume");

        settings.beginGroup("Main");
        mMixerCommand=(settings.value(SETTINGS_MIXER_COMMAND, SETTINGS_DEFAULT_MIXER_COMMAND).toString());


        mSinkIndex = settings.value(SETTINGS_DEVICE, SETTINGS_DEFAULT_DEVICE).toInt();


settings.endGroup();

        if (mEngine && mEngine->sinks().count() > 0) {
                mSinkIndex = qBound(0, mSinkIndex, mEngine->sinks().count()-1);

                mSink = mEngine->sinks().at(mSinkIndex);
                mWidgetVolume->setDevice(mSink);
              mWidgetVolume->setSliderMax(mEngine->volumeMax(mSink));
            //    progressBar->setMaximum(mEngine->volumeMax(mSink));
               mEngine->setIgnoreMaxVolume(true);
        }
}

/*********************************************************
 *************************لوحة المفاتيح*******************
 *********************************************************/
//! عند النقرعلى لوحة المفاتيح للزيادة الصوت
void ElokabVolume::shortcutVolumeUp()
{

        if (mSink)
        {
                mSink->setVolume(mSink->volume() + 3);
           //    QString txt=(tr("Volume: %1").arg(QString::number(mSink->volume())));
             //   mTrayIcon->showMessage(tr("Volume"),txt,QSystemTrayIcon::Information,1000);
progressShow();
        }
}
//! عمد النقرعلى لوحة المفاتيح لنقصان الصوت
void ElokabVolume::shortcutVolumeDown()
{

        if (mSink)
        {

                mSink->setVolume(mSink->volume() - 3);

                progressShow();
        }
}
//! عمد النقرعلى لوحة المفاتيح لكتم الصوت
void ElokabVolume::shortcutVolumeMute()
{
        if (mSink){

                mSink->toggleMute();
                progressShow();
        }
}
//! عرض مؤشر تغير حجم الصوت على الشاشة
void ElokabVolume::progressShow()
{

    timeSlide->stop();

    realignePopup(true);

    timeSlide->start(2000);


}
//! اخفاء مؤشر تغير حجم الصوت
void ElokabVolume::progressHide()
{

     mwidgetpopup->hide();
       timeSlide->stop();
}
/*********************************
 *********************************/
//! تغيير حجم الصوت بواسطة عجلة الفأرة
void ElokabVolume::wheelEvent(QWheelEvent* e)
{
//    int max = 100;
//    if(mEngine)
//        max=  mEngine->volumeMax(mSink);

    int delta = e->delta() < 0 ? 3 : -3;

    int volume =mSink->volume() - delta;

    if (!mwidgetpopup->isVisible())
        progressShow();

    mSink->setVolume( volume);

}


Q_EXPORT_PLUGIN2(dpanel_volume, ElokabVolume)
