/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "elokabsettingsDlg.h"
#include "ui_elokabsettingsDlg.h"
#include "itemdelegate.h"
#include <EDir>
#include <EIcon>
#include <QSettings>
#include <QTextCodec>
#include <QDebug>
#include <QPushButton>
#include <QDirIterator>
#include <QDialog>


ElokabSettingsDlg::ElokabSettingsDlg(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ElokabSettingsDlg),widgetModule(0),pluginsInterface(0)
{
 msetting=new ElokabSettings;

  ui->setupUi(this);
    //! تكوين قائمة بمسارات الايقونات------------------

pluginsDir.setPath(Edir::libDir());
ui->listWidget->setItemDelegate(new ItemDelegate);

if(isRightToLeft())
    ui->toolButtonAll->setIcon(EIcon::fromTheme("go-next"));
else
    ui->toolButtonAll->setIcon(EIcon::fromTheme("go-previous"));

ui->toolButtonAll->setToolTip(ui->toolButtonAll->shortcut().toString());
connect(ui->stackedWidgetAll,SIGNAL(currentChanged(int)),this,SLOT(stackindexChanged(int)));
ui->stackedWidgetAll->setCurrentIndex(0);
stackindexChanged(0);
}

//_________________________________________________________
ElokabSettingsDlg::~ElokabSettingsDlg()
{
    delete ui;
}

//[1]_________________________________________________________ALL Service
void ElokabSettingsDlg::loadAllServices()
{
    QString serviceDir=Edir::dataAppDir()+"/servicesittems";
    QStringList listItem;
    QDirIterator it(serviceDir, QStringList("*.desktop"),
                    QDir::Files | QDir::NoDotAndDotDot);

    while (it.hasNext())
    {
         it.next();
         QString file=it.filePath();
         listItem.append(file);

    }
    listItem.sort();
    foreach (QString file, listItem) {
        QHash<QString,QString>hash=loadModuleInfo(file);
        if(hash["Type"].toLower()!="service"||hash["X-EKB-type"].toLower()!="item")
            continue;

        if(hash["Name"].isEmpty()||hash["X-EKB-name"].isEmpty())
            continue;

        QListWidgetItem *item=new QListWidgetItem(ui->listWidget);
        item->setText(hash["Name"]);
        item->setToolTip(hash["Comment"]);
        QIcon icon=EIcon::fromTheme(hash["Icon"],"preferences-system");
        qDebug()<<hash["Icon"]<<icon.actualSize(QSize(64,64));
        item->setIcon(icon/*.pixmap(64)*/);
        item->setData(Qt::UserRole,hash["X-EKB-name"]);
       // QWidget *w=new QWidget(this);

        addNewPageWidget(hash["X-EKB-name"]);
        QStringList list=hash["X-EKB-plugins"].split(",");
        //!---------------New-------------------

        loadModules(hash["X-EKB-name"],list);
    }
    if(ui->listWidget->count()>0)
        ui->listWidget->setCurrentRow(0);
}

//[1-2]_________________________________________________________ALL Service
void ElokabSettingsDlg::addNewPageWidget(const QString &data)
{
    QWidget *tab=new QWidget(this);
     ui->stackedWidget->addWidget(tab);
  //   mHashItems.insert(data,tab);

     //!---------------New-------------------

     QVBoxLayout *layout = new QVBoxLayout;
     layout->setContentsMargins(0,0,0,0);
     tab->setLayout(layout);
     QListWidget *listw=new QListWidget;
     layout->addWidget(listw);
mHashListWidget.insert(data,listw);
listw->setViewMode(QListView::IconMode);
listw->setIconSize(QSize(64,64));
listw->setGridSize(QSize(150,100));
listw-> setResizeMode(QListView::Adjust);
listw->setMovement(QListView::Static);

connect(listw,SIGNAL(itemActivated(QListWidgetItem*)),this,SLOT(itemActivated(QListWidgetItem*)));
}

void ElokabSettingsDlg::itemActivated(QListWidgetItem *item)
{
    QString modul=item->data(Qt::UserRole).toString();
    qDebug()<<modul;
    loadServicemodule(modul);
}

//[1-3]_________________________________________________________ALL Service
void ElokabSettingsDlg::loadModules(const QString &parent,const QStringList &list)
{

  QDir serviceDir(Edir::dataAppDir()+"/serviceslibs");
    foreach (QString s, list) {
        QString file=serviceDir.absoluteFilePath(s+".desktop");
       // qDebug()<<file;
        QHash<QString,QString>hash=loadModuleInfo(file);

        if(hash["Type"].toLower()!="service"||hash["X-EKB-type"].toLower()!="lib")
            continue;


        if(hash["Name"].isEmpty())
            continue;



      //!new
        addNewIcons(parent,
                hash["Icon"],
                hash["Name"],
                hash["X-EKB-name"]);
    }




}
//!new
void ElokabSettingsDlg::addNewIcons(const QString &parent,
                                     const QString &icon,
                                     const QString &name,
                                     const QString &libName)
{

    QListWidget *listW=mHashListWidget.value(parent);



    if(!listW)return;

    QString fileName="lib"+libName+".so";
    //if(!loadPlugin(fileName))return;
    if(!QFile::exists(pluginsDir.absoluteFilePath(fileName)))return;

    QListWidgetItem *item=new QListWidgetItem(EIcon::fromTheme(icon,"preferences-system"),name,listW);
item->setData(Qt::UserRole,libName);


}
//[3-1]_________________________________________________________ALL Service
/*
void ElokabSettingsDlg::addNewTabWidget(const QString &parent,
                                     const QString &icon,
                                     const QString &name,
                                     const QString &libName)
{

    QTabWidget *tab=mHashItems.value(parent);

    if(!tab)return;

    QString fileName="lib"+libName+".so";
    if(!loadPlugin(fileName))return;

            PluginsInterface *pInterface=m_hatchPligins.value(fileName);
    if(pInterface){
        QWidget *wtab=new QWidget(this);
        tab->addTab(wtab,EIcon::fromTheme(icon,"preferences-system"),name);

        QVBoxLayout *layout = new QVBoxLayout;
        wtab->setLayout(layout);

        QWidget *w=pInterface->widget();
        layout->addWidget(w);

    }



}
*/
//_________________________________________________________ARG
QStringList ElokabSettingsDlg::listModules()
{
    QStringList list;
//    QDir serviceDir(qApp->applicationDirPath());
//    serviceDir.cdUp();
//    serviceDir.cd( serviceDir.absolutePath()+"/share/elokab/elokab-settings/serviceslibs") ;

     QString serviceDir(Edir::dataAppDir()+"/serviceslibs");
    QDirIterator it(serviceDir, QStringList("*.desktop"),
                    QDir::Files | QDir::NoDotAndDotDot);
    while (it.hasNext())
    {
        it.next();
        QString file=it.fileInfo().baseName();

        list.append(file);
    }
    list.sort();
    return list;
}
//_________________________________________________________ARG
bool ElokabSettingsDlg::loadServicemodule(const QString &pfile,bool panel)
{

        QString serviceDir=Edir::dataAppDir()+"/serviceslibs/"+pfile.toLower()+".desktop";
qDebug()<<serviceDir;
QHash<QString,QString>hash=loadModuleInfo(serviceDir);

    if(hash["Type"].toLower()!="service"||hash["X-EKB-type"].toLower()!="lib")
        return false;


    if(hash["Name"].isEmpty())
        return false;

 QString fileName="lib"+hash["X-EKB-name"]+".so";

 if(widgetModule)
     delete widgetModule;

 if(!loadPlugin(fileName))
        return false;

   // PluginsInterface *pInterface=m_hatchPligins.value(fileName);

    if(pluginsInterface){




        ui->widgetPanel->setVisible(panel);
        widgetModule=pluginsInterface->widget();

        ui->stackedWidgetAll->setCurrentIndex(1);
        ui->vLayoutModule->addWidget(widgetModule);
        if(!panel){
            this->adjustSize();
            this->setWindowTitle(hash["Name"]);
            this->setWindowIcon(EIcon::fromTheme(hash["Icon"],"preferences-system"));

        }else{
            ui->labelTitle->setText(hash["Name"]);
        }
        return true;
    }
    return false;
}

//_________________________________________________________
QHash<QString,QString> ElokabSettingsDlg::loadModuleInfo(const QString &file)
{
    QString lc=this->locale().name().section("_",0,0);

    QHash<QString,QString>hash;
    QSettings gs(file,QSettings::IniFormat);
    gs.setIniCodec(QTextCodec::codecForName("UTF-8"));
    gs.beginGroup("Desktop Entry");

    QString type=gs.value("Type").toString();
    QString nameold=gs.value("Name").toString();
    QString name=gs.value("Name["+lc+"]",nameold).toString();
    QString icon=gs.value("Icon").toString();
    QString settingType=gs.value("X-EKB-type").toString();
    QString data=gs.value("X-EKB-name").toString();
    QString libName=gs.value("X-EKB-lib").toString();
    QString commentold=gs.value("Comment").toString();
    QString comment=gs.value("Comment["+lc+"]",commentold).toString();
   //X-EKB-plugins
    if(gs.contains("X-EKB-plugins")){
        QStringList list=gs.value("X-EKB-plugins").toStringList();
        if(list.count()>0){
            QString s=list.join(",").trimmed();
           hash.insert("X-EKB-plugins",s);
           qDebug()<<"list<<<<<<<<<<"<<s;
        }

    }

    hash.insert("X-EKB-type",settingType.toLower());
    hash.insert("X-EKB-name",data.toLower());
    hash.insert("X-EKB-lib",libName.toLower());
    hash.insert("Type",type.toLower());
    hash.insert("Icon",icon);
    hash.insert("Name",name);
    hash.insert("Comment",comment);

    gs.endGroup();
    return hash;
}



//_________________________________________________________
bool ElokabSettingsDlg::loadPlugin(const QString &pfile)
{


    QPluginLoader pluginLoader(pluginsDir.absoluteFilePath(pfile));

    pluginLoader.load();
    if( pluginLoader.isLoaded()){
        QObject *plugin = pluginLoader.instance();
        if (plugin) {
            pluginsInterface    = qobject_cast<PluginsInterface *>(plugin);

            if (pluginsInterface){

                //m_hatchPligins.insert(pfile,pluginsInterface);
                QString t(60-pfile.length(),'-');
                 qDebug()<<pfile+t<<"Ok";
            return true;

            }//pluginsInterface

        }else{
             QString t(60-pfile.length(),'-');
              qDebug()<<pfile+t<<"Failed";
            }// pluginLoader
    }

    return false;

}

//_________________________________________________________
void ElokabSettingsDlg::on_buttonBox_clicked(QAbstractButton *button)
{
    if(ui->buttonBox->standardButton(button)==QDialogButtonBox::Apply||
            ui->buttonBox->standardButton(button)==QDialogButtonBox::Ok){
        //!  ------------عمليات الحفظ-----------------------
//        foreach (PluginsInterface *pInterface, m_hatchPligins) {
//            pInterface->saveSettings();
//        }
        if(!pluginsInterface)return;
        pluginsInterface->saveSettings();
        applyChanges();
    }else if(ui->buttonBox->standardButton(button)==QDialogButtonBox::Help){


        QDialog *dlg=new QDialog(this);
        WidgetAbout *widgetabout=new WidgetAbout(dlg);
        widgetabout->setPixmap(EIcon::fromTheme("preferences-system").pixmap(64));
        widgetabout->setDescription(tr("Central configuration center for elokab"));
        widgetabout->setApplicationText(tr("Elokab settings"));
        QDialogButtonBox *buttonBox=new QDialogButtonBox(QDialogButtonBox::Ok,Qt::Horizontal,dlg);
        connect(buttonBox, SIGNAL(accepted()), dlg, SLOT(accept()));
        QVBoxLayout *lay=new QVBoxLayout(dlg);
        dlg->setLayout(lay);
        lay->addWidget(widgetabout);
        lay->addWidget(buttonBox);
        dlg->resize(widgetabout->size());
        dlg->exec();
        delete widgetabout;
        delete dlg;
    }
}

//_________________________________________________________


#ifdef Q_WS_X11
extern void qt_x11_apply_settings_in_all_apps();
#endif // Q_WS_X11
void ElokabSettingsDlg::applyChanges()
{
#if defined(Q_WS_X11)
    qt_x11_apply_settings_in_all_apps();
#endif // Q_WS_X11
}

void ElokabSettingsDlg::on_toolButtonAll_clicked()
{
    ui->stackedWidgetAll->setCurrentIndex(0);
    ui->labelTitle->setText("");
}

void ElokabSettingsDlg::stackindexChanged(int arg)
{
    ui->toolButtonAll->setEnabled(arg);
    ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(arg);
    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(arg);


}
