﻿/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "dialogconfigdesktop.h"
#include "ui_dialogconfigdesktop.h"
#include <EIcon>
#include <EDir>
#include <QSettings>
#include <QColorDialog>
#include <QFileDialog>
#include <QPainter>
#include <QDebug>
#include <QTime>
#include <QTextCodec>

DialogConfigDesktop::DialogConfigDesktop(QWidget *parent) :
        QWidget(parent),
        ui(new Ui::DialogConfigDesktop)
{
        ui->setupUi(this);
        this->setAttribute(Qt::WA_DeleteOnClose,true);
        btnGlobalColor=new ButtonColor(tr("C&olor :"));
        btnPatternColor1=new ButtonColor(tr("&First color :"));
        btnPatternColor2=new ButtonColor(tr("&Second Color:"));

        connect(btnGlobalColor,SIGNAL(colorChanged()),SLOT( brushcolorChanged()));
         connect(btnPatternColor1,SIGNAL(colorChanged()),SLOT( brushPatternChanged()));
         connect(btnPatternColor2,SIGNAL(colorChanged()),SLOT( brushPatternChanged()));

        ui->hLayoutGlobalColor->insertWidget(0,btnGlobalColor);
        ui->hLayoutPatern->insertWidget(0,btnPatternColor1);
        ui->hLayoutPatern->insertWidget(2,btnPatternColor2);


        ui->toolButtonFolderPath->setIcon(EIcon::fromTheme("document-open-folder",("folder")));
        ui->toolButtonChooseImage->setIcon(EIcon::fromTheme("document-open-folder",("folder")));

//        QDir appDir(QApplication::applicationDirPath());
//        appDir.cdUp();
        m_appPath=Edir::dataDir()+"/elokab-desktop";

        QString imgPatern=m_appPath+"/pattern";
        QString subfile;
        QDir dirS(imgPatern);
        foreach ( subfile, dirS.entryList(QDir::AllEntries|  QDir::NoDotAndDotDot )){
                QFileInfo fi(imgPatern+"/"+subfile);
                if(fi.completeSuffix().toLower()=="png"){
                        QIcon icon1;
                        icon1.addFile(imgPatern+"/"+subfile, QSize(), QIcon::Normal, QIcon::Off);
                        ui->comboBoxPattern->addItem(icon1,fi.baseName());

                }
        }
        connect(ui->comboBoxPattern, SIGNAL(activated(int)),
                this, SLOT(brushPatternChanged()));

        ui->comboBoxType->setCurrentIndex(-1);
        QFileInfo Desktop(Edir::desktopDir());
        QFileInfo HomeLocation(QDir::homePath());
        QFileInfo DocumentsLocation(Edir::documentsDir());
        QFileInfo PicturesLocation(Edir::picturesDir());
        QFileInfo MoviesLocation(Edir::moviesDir());
        QFileInfo MusicLocation(Edir::musicDir());


        ui->comboBoxFolder->addItem(Desktop.fileName() ,Desktop.filePath());
        ui->comboBoxFolder->addItem( trUtf8("Home") ,HomeLocation.filePath());
        ui->comboBoxFolder->addItem(DocumentsLocation.fileName(),DocumentsLocation.filePath());
        ui->comboBoxFolder->addItem( PicturesLocation.fileName(),PicturesLocation.filePath());
        ui->comboBoxFolder->addItem( MoviesLocation.fileName(),MoviesLocation.filePath());
        ui->comboBoxFolder->addItem( MusicLocation.fileName(),MusicLocation.filePath());

        loadQmlAnimation();
        loadSettings();

}

DialogConfigDesktop::~DialogConfigDesktop()
{
      //  qDebug()<<"desktop settings UI deleted";
        delete ui;
}

void DialogConfigDesktop::loadQmlAnimation()
{
    QStringList listPath;

       QString Lc=this->locale().name().section("_",0,0);
    listPath <<Edir::dataHomeDir()+"/elokab/elokab-desktop/qmlAnimations"
    << m_appPath+"/qmlAnimations";
    foreach (QString s, listPath) {
        QDir dir(s);

        foreach (const QString &name, dir.entryList(QDir::AllDirs |
                                                    QDir::NoDotAndDotDot | QDir::Readable | QDir::Executable))
        {
             QString fileName=dir.filePath(name+"/metadata.desktop");
             if(QFile::exists(fileName)){

                 QSettings m_setting(fileName,QSettings::IniFormat);
                 m_setting.setIniCodec(QTextCodec::codecForName("UTF-8"));

                 m_setting.beginGroup("Desktop Entry");
                 QString str=  m_setting.value("Name").toString();
                 QString strLc=  m_setting.value("Name["+Lc+"]",str).toString();
                 QString Type=m_setting.value("ServiceTypes").toString();

                 m_setting.endGroup();
                 if(Type.contains("DeclarativeWallpaper"))
                     ui->comboBoxQml->addItem(strLc,dir.filePath(name+"/metadata.desktop"));


             }

        }
    }
}



/*****************************************
 *****التبديل بين انواع الخلفيات***********
 ******************************************/
void DialogConfigDesktop::on_comboBoxType_currentIndexChanged(int index)
{


        switch (index)
        {
        case 0:         // لون احادي

                ui->labelView->setPixmap(QPixmap(""));
                brushcolorChanged();
                break;
        case 1:         // زخرفة

                brushPatternChanged();
                break;
        case 2:         // صورة

                imagefileChanged();
                break;

        case 3 :       // مجلد الصور

                imagePathChanged();
                break;

        case 4 :       // مجلد الصور

                animationFileChanged();
                break;


        }
}
/*****************************************
 ********تحميل اعدادات المستخدم ***********
 ******************************************/
QColor listToColor(QList<QVariant>list)
{
    int red=0,green=0, bleu=0,alpha=255;
    int count=list.count();
    if(count>0)red=list.at(0).toInt();
    if(count>1)green=list.at(1).toInt();
    if(count>2)bleu=list.at(2).toInt();
    if(count>3)alpha=list.at(3).toInt();
    return QColor(red,green,bleu,alpha);
}
QList<QVariant> colorToList(QColor color)
{
    int red=(color.red());
    int green=(color.green());
    int bleu=(color.blue());
    int alpha=(color.alpha());
    return QList<QVariant>()<<red<<green<<bleu<<alpha;
}
void DialogConfigDesktop::loadSettings()
{

    QString   curDirectori;
    if(QFile::exists("/usr/share/wallpapers")){
        curDirectori="/usr/share/wallpapers";
    }else  if(QFile::exists("/usr/share/backgrounds")){
        curDirectori="/usr/share/backgrounds";
    }else{
        curDirectori=Edir::picturesDir();
    }


    QSettings setting("elokab","settings");
    setting.setIniCodec(QTextCodec::codecForName("UTF-8"));
    setting.beginGroup("Desktop-Wallpaper");

    btnGlobalColor->setColor(setting.value("wallpaperColor",QColor(60,60,60).name()).toString());
    btnPatternColor1->setColor(setting.value("colorPattern1",QColor(148,159,255).name()).toString());
    btnPatternColor2->setColor(setting.value("colorPattern2",QColor(195,222,255).name()).toString());

    int icn=setting.value("iconSize",48).toInt();


    ui->hSliderIconSize->setValue(icn/8);
    QString desktopFolder=setting.value("DirLocation",
                                        Edir::desktopDir()).toString();
    int indxFolder=ui->comboBoxFolder->findData(desktopFolder);
    ui->comboBoxFolder->setCurrentIndex(indxFolder);

    ui->checkBoxIconsView->setChecked(setting.value("IconView",false).toBool());
    ui->comboBoxExecMode->setCurrentIndex(setting.value("ExecMode",0).toInt());


    ui->rdBHorizontalAlignment->setChecked(setting.value("AlignHorizontale",false).toBool());
    ui->comboBoxPattern->setCurrentIndex(setting.value("curentPattern",0).toInt());
    ui->comboBoxAspectRatio->setCurrentIndex(setting.value("aspectRatio",0).toInt());
    ui->lineEditImagePath->setText(setting.value("imagePath",m_appPath+"/images/tawhid.png").toString());
    ui->lineEditFolderPath->setText(setting.value("folderPath",curDirectori).toString());
    QTime time=setting.value("slideShowTaime",QTime::fromString("0:30", "h:m")).toTime();
    ui->checkBoxAppMenu->setChecked(setting.value("showMenuApp",false).toBool());
    ui->spinBoxHoure->setValue(time.hour());
    ui->spinBoxMinute->setValue(time.minute());
    QString animationFile=setting.value("animationFile").toString();
    int idx=ui->comboBoxQml->findData(animationFile);
    ui->comboBoxQml->setCurrentIndex(idx);
    ui->comboBoxType->setCurrentIndex(setting.value("wallpaperType",2).toInt());

    setting.endGroup();


}
/*****************************************
 ********حفظ اعدادات المستخدم ***********
 ******************************************/
void DialogConfigDesktop::saveSettings()
{

        QSettings setting("elokab","settings");
        setting.beginGroup("Desktop-Wallpaper");

        setting.setValue("wallpaperColor",btnGlobalColor->color().name());
        setting.setValue("colorPattern1",btnPatternColor1->color().name());
       setting.setValue("colorPattern2",btnPatternColor2->color().name());

        setting.setValue("iconSize",ui->hSliderIconSize->value()*8);
        setting.setValue("IconView",ui->checkBoxIconsView->isChecked());
 setting.setValue("ExecMode",ui->comboBoxExecMode->currentIndex());
 QString dFolder=ui->comboBoxFolder->itemData(ui->comboBoxFolder->currentIndex()).toString();
setting.setValue("DirLocation",dFolder);

        setting.setValue("wallpaperType",ui->comboBoxType->currentIndex());
        setting.setValue("curentPattern",ui->comboBoxPattern->currentIndex());
        setting.setValue("aspectRatio",ui->comboBoxAspectRatio->currentIndex());
        setting.setValue("imagePath", ui->lineEditImagePath->text());
        setting.setValue("folderPath", ui->lineEditFolderPath->text());
     //   setting.setValue("pixPattern", m_pixPattern);
        setting.setValue("slideShowTaime", QTime(ui->spinBoxHoure->value(),ui->spinBoxMinute->value()));
        setting.setValue("AlignHorizontale", ui->rdBHorizontalAlignment->isChecked());
        setting.setValue("showMenuApp", ui->checkBoxAppMenu->isChecked());
        setting.setValue("animationFile", ui->comboBoxQml->itemData(ui->comboBoxQml->currentIndex()));


        setting.endGroup();
        QString fileName=Edir::dataHomeDir()+"/elokab/elokab-desktop";
        QDir dir;
        dir.mkpath(fileName);
        m_pixPattern.save(fileName+"/pixPattern.png");

        QSettings s("elokab","elokabsettings");

        s.beginGroup("AppChangeSetting");
        QString name=s.value("DesktopTheme").toString();
        QString newName="changed_0";
        int i=0;
        while (name==newName) {
            i++;
            newName="changed_"+QString::number(i);
        }
        s.setValue("DesktopTheme",newName);
        s.endGroup();

}

/*************************************
*********فتح الملفات******************
**************************************/
/// فتح ملف الصورة--------------------------------------------------------
void DialogConfigDesktop::on_toolButtonChooseImage_clicked()
{
        QString curDirectori=ui->lineEditImagePath->text();
        if(curDirectori.isEmpty())
                curDirectori=Edir::picturesDir();


        QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                        curDirectori,
                                                        tr("Images (*.png *.xpm *.jpg)"));

        if(!fileName.isEmpty()){
                ui->lineEditImagePath->setText(fileName);
                imagefileChanged();
        }
}
/// فتح ملف مجلد الصور----------------------------------------------
void DialogConfigDesktop::on_toolButtonFolderPath_clicked()
{
        QString curDirectori=ui->lineEditFolderPath->text();
        if(curDirectori.isEmpty())
                curDirectori=Edir::picturesDir();
        QString dir = QFileDialog::getExistingDirectory(this, tr("Open Directory"),
                                                        curDirectori,QFileDialog::ShowDirsOnly
                                                        | QFileDialog::DontResolveSymlinks);

        if(!dir.isEmpty())
                ui->lineEditFolderPath->setText(dir);
        imagePathChanged();
}
/// جلب الزخرفة المحددة ورسمها بالالوان المنتقاة--------------------------
QPixmap DialogConfigDesktop::getBrushPattern()
{
        QString imgName=ui->comboBoxPattern->currentText()+".png";


        //QString imgPatern=appPath+"/pattern";

        QString pixpattern=m_appPath+"/pattern/"+imgName;
        QPainter::CompositionMode mode = QPainter::CompositionMode_DestinationIn;
        QImage sourceImage(pixpattern);
        QImage resultImage(sourceImage.width(),sourceImage.height(),QImage::Format_ARGB32_Premultiplied);
        QImage destinationImage(sourceImage.width(),sourceImage.height(),QImage::Format_ARGB32_Premultiplied);
        QPainter p(&destinationImage);
        p.fillRect(destinationImage.rect(), btnPatternColor1->color());
        p.end();
        QPainter painter(&resultImage);
        painter.setCompositionMode(QPainter::CompositionMode_Source);
        painter.fillRect(resultImage.rect(), Qt::transparent);
        painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
        painter.drawImage(0, 0, destinationImage);
        painter.setCompositionMode(mode);
        painter.drawImage(0, 0, sourceImage);
        painter.setCompositionMode(QPainter::CompositionMode_DestinationOver);
        painter.fillRect(resultImage.rect(), btnPatternColor2->color());
        painter.end();
        m_pixPattern= QPixmap::fromImage(resultImage);
        // ui->label->setPixmap(QPixmap::fromImage(resultImage));
        return QPixmap::fromImage(resultImage);
}
/// تطبيق التغييرات على المعاينة---------------------------------
void DialogConfigDesktop::brushPatternChanged()
{

        QBrush brush(getBrushPattern());
        QPixmap pixColor( 128, 128);
        pixColor.fill(Qt::transparent);
        QPainter p;
        p.begin(&pixColor);

        p.setBrush(brush);
        p.fillRect(pixColor.rect(),brush);

        p.end();

        ui->labelView->setPixmap(pixColor);
}
/// تطبيق اللون الاحادي على خلفية المعاينة---------------------
void DialogConfigDesktop::brushcolorChanged()
{
    QColor col=btnGlobalColor->color();
        ui->frameBake->setStyleSheet(QString("background-color: rgb(%1, %2, %3);").
                                     arg(col.red()).arg(col.green()).arg(col.blue()));
}
/// تطبيق تغير الصورة على المعاينة--------------------------
void DialogConfigDesktop::imagefileChanged()
{

        QPixmap pixColor( 128, 128);
        QPixmap pix(ui->lineEditImagePath->text());
        pixColor.fill(Qt::transparent);
        QPainter p;
        p.begin(&pixColor);
        p.drawPixmap(0,0,pix.scaled(128,128));
        p.end();
        ui->labelView->setPixmap(pixColor);
              brushcolorChanged();
}
/// تطبيق صور المجلد على المعاينتة------------------------------
void DialogConfigDesktop::imagePathChanged()
{
        QPixmap pixColor( 128, 128);
        pixColor.fill(Qt::transparent);
        QPixmap pix1(98,98);
        QPixmap pix2(98,98);
        QPixmap pix3(98,98);
        QString folderDir=ui->lineEditFolderPath->text();
        QDir dirS(folderDir);
        QString subfile;
        QStringList listBgr;
        foreach ( subfile, dirS.entryList(QDir::Files|QDir::NoDotAndDotDot)){
                QFileInfo fi(folderDir+"/"+subfile);
                QString ext = fi.suffix();//جلب الامتداد
                if(ext.toLower()=="png"||ext.toLower()=="jpg"||ext.toLower()=="xpm"){//اذا كانت الصورة تحمل هذه الامتدادات
                        listBgr.append(folderDir+"/"+subfile);//اضافة الصورة الى القائمة
                }
        }
        int count=listBgr.count()-1;
        if (count<0)return;
        int i=rand() % ++count;
        pix1.load(listBgr.first());
        pix3.load(listBgr.last());
        pix2.load(listBgr.at(i));
        // qDebug()<<"i--------------"<<i;

        QPainter p;
        p.begin(&pixColor);


        p.drawPixmap(5,5,pix1.scaled(98,98));
        p.drawPixmap(15,15,pix2.scaled(98,98));
        p.drawPixmap(25,25,pix3.scaled(98,98));
        p.end();

        ui->labelView->setPixmap(pixColor);
         brushcolorChanged();

}

/// تطبيق صور المجلد على المعاينتة------------------------------
void DialogConfigDesktop::animationFileChanged()
{
    int indx=ui->comboBoxQml->currentIndex();
    QString fileName=ui->comboBoxQml->itemData(indx).toString();
    QFileInfo fi(fileName);
    QString previw=fi.absolutePath()+"/screenshot.png";
   QPixmap pix(128,128);
    if(QFile::exists(previw))
      pix.load(previw);
    else{
        QDir mCurrentDir(fi.absolutePath());

      QStringList  files = mCurrentDir.entryList(QStringList()<<"*.png"<<"*.jpg"<<"*.gif",
                                                 QDir::AllEntries | QDir::NoDotAndDotDot);
   //  qDebug()<<files.at(0);
      if(files.count()>0)
          pix.load(fi.absolutePath()+"/"+files.at(0));
      else
          pix=EIcon::fromTheme("preferences-desktop-wallpaper").pixmap(128);

    }

    ui->labelView->setPixmap(pix);
    brushcolorChanged();

}

void DialogConfigDesktop::on_lineEditImagePath_textEdited(const QString &arg1)
{

        if(QFile::exists(arg1))
                imagefileChanged();
}

void DialogConfigDesktop::on_lineEditFolderPath_textEdited(const QString &arg1)
{
        if(QFile::exists(arg1))
                imagePathChanged();
}

void DialogConfigDesktop::on_hSliderIconSize_valueChanged(int value)
{
   //     qDebug()<<"iconsize "<<value;
        ui->hSliderIconSize->setToolTip(QString::number(value*8));
}

void DialogConfigDesktop::on_comboBoxQml_activated(const QString &/*arg1*/)
{
     animationFileChanged();
}

Q_EXPORT_PLUGIN2(setting_DesktopSetting, DialogConfigDesktop)
