#include "docksetting.h"
#include "ui_docksetting.h"
#include <EIcon>
#include <EDir>
#include <QSettings>
#include <QColorDialog>
#include <QFileDialog>
#include <QDebug>
#include <QTextCodec>
#include <QPainter>
#include <QApplication>
#define STYLE_GRAD  0
#define STYLE_IMAGE  1
#define STYLE_3D 2

#define ANIM_ZOOM 1



//____________________________________________________________________________________
DockSetting::DockSetting(QWidget *parent) :
     QWidget(parent),
      ui(new Ui::DockSetting)
{
     ui->setupUi(this);
     this->setAttribute(Qt::WA_DeleteOnClose,true);
     btn_colorTop   =new ButtonColor(QString(),true);
     btn_colorRight=new ButtonColor(QString(),true);
     btn_colorBot=new ButtonColor(QString(),true);
     btn_colorLeft=new ButtonColor(QString(),true);
     btn_colorBorder=new ButtonColor(tr("B&order Color :"),true);
     btn_colorSelect=new ButtonColor(tr("&Selection Color :"),true);
     btn_colorTop->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
     btn_colorBot->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
     btn_colorRight->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
     btn_colorLeft->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
     connect(btn_colorTop,SIGNAL(colorChanged()),this,SLOT(getStyleGradien()));
     connect(btn_colorRight,SIGNAL(colorChanged()),this,SLOT(getStyleGradien()));
     connect(btn_colorBot,SIGNAL(colorChanged()),this,SLOT(getStyleGradien()));
     connect(btn_colorLeft,SIGNAL(colorChanged()),this,SLOT(getStyleGradien()));
     connect(btn_colorBorder,SIGNAL(colorChanged()),this,SLOT(getStyleGradien()));
     connect(btn_colorSelect,SIGNAL(colorChanged()),this,SLOT(getStyleGradien()));

     ui->horizontalLayout_select->addWidget(btn_colorSelect);
     ui->horizontalLayout_border->addWidget(btn_colorBorder);


     this->setWindowIcon(EIcon::fromTheme("configure",("document-properties")));
     this->setObjectName("dockConfig");
     this->setAttribute(Qt::WA_DeleteOnClose,true);
     /* اضافة صور الشكل الثلاثي الى القائمة المنسدلة*/
     ui->comboBox3D->addItem(QIcon(":/images/bgrGlass"), tr("White"),":/images/bgrGlass");
     ui-> comboBox3D->addItem(QIcon(":/images/bgrBlack"), tr("Black"),":/images/bgrBlack");
 setupThemes();


     qDebug()<<"loqd settings";
     loadSetting();
     /*تطبيق الشكل هلى المعاينة*/
     getStyleGradien();

}
DockSetting::~DockSetting()
{
    delete ui;
}
void DockSetting::setupThemes()
{
    /// مسار مجلد البرنامج



 QString  userThemePath = Edir::dataDir()+"/themes";
  QString  localThemePath=Edir::dataHomeDir()+"/elokab/themes";
  QStringList listPath;
  listPath<<localThemePath<<userThemePath;
  QStringList listThemes;

  foreach (QString dirPath, listPath) {
      QDir dirS(dirPath);
      foreach(QString dir,dirS.entryList(QDir::AllDirs| QDir::NoSymLinks| QDir::NoDotAndDotDot )) {
          QString path=dirS.absoluteFilePath(dir);
          // qDebug()<<"path"<<path;
          if (QFile::exists(path+"/settings.conf")){
                if(!listThemes.contains(dir)){
                    listThemes.append(dir);
                     ui->comboBoxTheme->addItem(dir,path);
                }
          }
      }
  }

   listThemes.clear();
}

QColor listToColor(QList<QVariant>list)
{
    int red=0,green=0, bleu=0,alpha=255;
    int count=list.count();
    if(count>0)red=list.at(0).toInt();
    if(count>1)green=list.at(1).toInt();
    if(count>2)bleu=list.at(2).toInt();
    if(count>3)alpha=list.at(3).toInt();
    return QColor(red,green,bleu,alpha);
}

QList<QVariant> colorToList(QColor color)
{
    int red=(color.red());
    int green=(color.green());
    int bleu=(color.blue());
    int alpha=(color.alpha());
    return QList<QVariant>()<<red<<green<<bleu<<alpha;
}
//____________________________________________________________________________________
void DockSetting::loadSetting(const QString &path)
{


    if(QFile::exists(path)&&!path.isEmpty()){
 qDebug()<<"loadSetting"<<path;
     //   setting.setPath(QSettings::IniFormat,QSettings::UserScope,path);
        msetting=new QSettings(path,QSettings::IniFormat);
    }else{
        msetting=new QSettings("elokab","settings");
    }

     /*تحميل كل اعدادات الممستخدم من الوان واشكال وموضع الى الخ
     كل هذه المعلومات موجودة افتراضيا في مجلد المستخدم في هذا المسار
     QDir::homePath()+"/.config/elokab/applictions*/
     msetting->beginGroup("Dock-Style");

     QString arrangement=msetting->value("Arrangement","auto-hide").toString();
       ui->comboBoxArrangement->setCurrentIndex(EDGE::arrangementToInt(arrangement));

     QString positon=msetting->value("Position","bottom").toString();
     int edge=EDGE::posToInt(positon);
      ui->comboBoxPosition->setCurrentIndex(edge);


     ui->spinBoxIconSize->setValue(msetting->value("iconSize",48).toInt());
     ui->comboBoxAnim->setCurrentIndex(msetting->value("animmationStyle",ANIM_ZOOM).toInt());
     ui->checkBoxExpanding->setChecked(msetting->value("Expanding",false).toBool());
     ui->cBoxCustumToolTip->setChecked(msetting->value("CustomToolTip",true).toBool());

     QList<QVariant>   colorTop=msetting->value("color1",QList<QVariant>()<<56<< 56<< 56<< 255).toList();
     QList<QVariant>   colorRight=msetting->value("color2",QList<QVariant>()<<76<< 76<< 76<<255).toList();
     QList<QVariant>   colorBot=msetting->value("color3",QList<QVariant>()<<0<< 0<< 0<< 255).toList();
     QList<QVariant>   colorLeft=msetting->value("color4",QList<QVariant>()<<43<< 43<< 43<< 255).toList();
     QList<QVariant>   colorBorder=msetting->value("colorBorder",QList<QVariant>()<<34<< 34<<34<<255).toList();
     QList<QVariant>   colorSelect=msetting->value("colorSelect",QList<QVariant>()<<255<< 50<< 20<<255).toList();

     m_stop=msetting->value("stop","0.02").toString();
     m_stop2=msetting->value("stop2","0.96").toString();
     QString image=msetting->value("pixFileName","").toString();
     if(!QFile::exists(image)){
       QString img=  QFileInfo(image).fileName();
QString curpath=QFileInfo(path).absolutePath();

       qDebug()<<curpath+"/images/"+img;
         ui->lineEditImageFile->setText(curpath+"/images/"+img);

     }else{
          ui->lineEditImageFile->setText(image);
     }
   //  ui->lineEditImageFile->setText(msetting->value("pixFileName","").toString());

     ui->checkBoxRepeat->setChecked(msetting->value("imageRepeat",false).toBool());
     ui->spinBoxBleft->setValue(msetting->value("borderLeft",5).toInt());
     ui->spinBoxBright->setValue(msetting->value("borderRight",5).toInt());
     ui->spinBoxBtop->setValue(msetting->value("borderTop",3).toInt());
     ui->spinBoxBbottom->setValue(msetting->value("borderBot",1).toInt());
     ui->comboBoxBackStyle->setCurrentIndex(msetting->value("backgroundStyle",STYLE_3D).toInt());
     ui->comboBox3D->setCurrentIndex(msetting->value("image3D",0).toInt());
     ui->checkBoxSelectionColor->setChecked(msetting->value("showSelection",false).toBool());

     msetting->endGroup();

     btn_colorTop->setColor(listToColor(colorTop));
     btn_colorRight->setColor(listToColor(colorRight));
     btn_colorBot->setColor(listToColor(colorBot));
     btn_colorLeft->setColor(listToColor(colorLeft));
     btn_colorBorder->setColor(listToColor(colorBorder));
     btn_colorSelect->setColor(listToColor(colorSelect));

     //add buttons colors to layouts
     if(edge<2)
          setupGradientTools(true);
     else
          setupGradientTools(false);


     //applique gui---------------------------------------
     QString dd=m_stop;
     dd.remove("0.");
     ui->verticalSlider->setValue(dd.toInt());
     QString dd2=m_stop2;
     dd2.remove("0.");
     ui->verticalSlider2->setValue(dd2.toInt());
}

//____________________________________________________________________________________
void DockSetting::setupGradientTools(bool horizontal)
{
     QLayoutItem *child;
     while ((child = ui->gridLayoutGradient->takeAt(0)) != 0)
     {
          //  qDebug()<<"geometry---------------"<<child->geometry();
          delete child;
     }
     if(horizontal==true){
          ui->verticalSlider2->setOrientation(Qt::Vertical);
          ui->verticalSlider->setOrientation(Qt::Vertical);
          ui->verticalSlider2->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Expanding);
          ui->verticalSlider->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Expanding);

          ui->gridLayoutGradient->addWidget(btn_colorTop, 0, 2,Qt::AlignHCenter);
          ui->gridLayoutGradient->addWidget(btn_colorLeft, 1, 0 );
          ui->gridLayoutGradient->addWidget(ui->verticalSlider2, 1, 1);
          ui->gridLayoutGradient->addWidget(ui->widgetBack, 1, 2);
          ui->gridLayoutGradient->addWidget(ui->verticalSlider, 1, 3);
          ui->gridLayoutGradient->addWidget(btn_colorRight, 1, 4);
          ui->gridLayoutGradient->addWidget(btn_colorBot, 2, 2,Qt::AlignHCenter);

     }else{
          ui->verticalSlider2->setOrientation(Qt::Horizontal);
          ui->verticalSlider->setOrientation(Qt::Horizontal);
          ui->verticalSlider2->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
          ui->verticalSlider->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);

          ui->gridLayoutGradient->addWidget(btn_colorLeft, 0, 1,Qt::AlignHCenter);
          ui->gridLayoutGradient->addWidget( ui->verticalSlider, 1, 1);
          ui-> gridLayoutGradient->addWidget(btn_colorBot, 2, 0);
          ui->gridLayoutGradient->addWidget( ui->widgetBack, 2, 1);
          ui->gridLayoutGradient->addWidget(btn_colorTop, 2, 2);
          ui->gridLayoutGradient->addWidget( ui->verticalSlider2, 3, 1);
          ui-> gridLayoutGradient->addWidget(btn_colorRight, 4, 1,Qt::AlignHCenter);
          QSpacerItem *verticalSpacerGr = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

          ui-> gridLayoutGradient->addItem(verticalSpacerGr, 2, 3);


     }
}

//____________________________________________________________________________________
void DockSetting::saveSettings()
{
    saveStyleQss();

    QSettings setting("elokab","settings");

    setting.beginGroup("Dock-Style");

    QString arrangement=EDGE::arrangementToString(ui->comboBoxArrangement->currentIndex());
    setting.setValue("Arrangement",arrangement);

    QString position=EDGE::posToString(ui->comboBoxPosition->currentIndex());
    setting.setValue("Position",position);

    setting.setValue("iconSize",ui->spinBoxIconSize->value());

    setting.setValue("animmationStyle",ui->comboBoxAnim->currentIndex());

    setting.setValue("Expanding",ui->checkBoxExpanding->isChecked());



    // setting.setValue("styleSheet",getStyle());
    setting.setValue("color1",colorToList(btn_colorTop->color()));

    setting.setValue("color2",colorToList(btn_colorRight->color()));

    setting.setValue("color3",colorToList(btn_colorBot->color()));

    setting.setValue("color4",colorToList(btn_colorLeft->color()));

    setting.setValue("colorBorder",colorToList(btn_colorBorder->color()));

    setting.setValue("colorSelect",colorToList(btn_colorSelect->color()));

    //     setting.setValue("InfoBgrColor",btn_colorInfoBgr->color());

    //     setting.setValue("InfoBorderColor",btn_colorInfoBorder->color());

    //     setting.setValue("InfoFontColor",btn_colorInfoFont->color());

    setting.setValue("CustomToolTip",ui->cBoxCustumToolTip->isChecked());

    setting.setValue("stop",m_stop);

    setting.setValue("stop2",m_stop2);

    setting.setValue("pixFileName", ui->lineEditImageFile->text());

    setting.setValue("imageRepeat", ui->checkBoxRepeat->isChecked());


    setting.setValue("borderLeft", ui->spinBoxBleft->value());

    setting.setValue("borderRight", ui->spinBoxBright->value());

    setting.setValue("borderTop", ui->spinBoxBtop->value());

    setting.setValue("borderBot", ui->spinBoxBbottom->value());

    setting.setValue("backgroundStyle", ui->comboBoxBackStyle->currentIndex());

    setting.setValue("image3D", ui->comboBox3D->currentIndex());

    setting.setValue("showSelection",ui->checkBoxSelectionColor->isChecked());

    //setting.setValue("showPreview",ui->checkBoxPreview->isChecked());

    //setting.setValue("CssPath",QDir::homePath()+"/.config/elokab/dockTheme.css");

    setting.endGroup();

    QSettings s("elokab","elokabsettings");

    s.beginGroup("AppChangeSetting");
    QString name=s.value("DockTheme").toString();
    QString newName="changed_0";
    int i=0;
    while (name==newName) {
        i++;
        newName="changed_"+QString::number(i);
    }
    s.setValue("DockTheme",newName);
    s.endGroup();

    // iconsSetting->saveIconThemes();
}



//____________________________________________________________________________________
void DockSetting::on_verticalSlider_sliderMoved(int position)
{
     qreal q=position;
     QVariant d=q/100;
     m_stop=d.toString();
     getStyleGradien();
}

//____________________________________________________________________________________
void DockSetting::on_verticalSlider2_sliderMoved(int position)
{
     qreal q=position;
     QVariant d=q/100;
     m_stop2=d.toString();
     getStyleGradien();
}

//____________________________________________________________________________________
QString DockSetting::getStyleGradien()
{

     QString grad1=QString("stop:0 rgba(%1, %2, %3, %4), ")
               .arg(btn_colorTop->color().red()).arg(btn_colorTop->color().green())
               .arg(btn_colorTop->color().blue()).arg(btn_colorTop->color().alpha());
     QString grad2=QString("stop:%1 rgba(%2, %3, %4, %5),")
               .arg(m_stop).arg(btn_colorRight->color().red()).arg(btn_colorRight->color().green())
               .arg(btn_colorRight->color().blue()).arg(btn_colorRight->color().alpha());
     QString grad4=QString("stop:%1 rgba(%2, %3, %4, %5),")
               .arg(m_stop2).arg(btn_colorLeft->color().red()).arg(btn_colorLeft->color().green())
               .arg(btn_colorLeft->color().blue()).arg(btn_colorLeft->color().alpha());
     QString grad3=QString( "stop:1 rgba(%1, %2, %3, %4));")
               .arg(btn_colorBot->color().red()).arg(btn_colorBot->color().green())
               .arg(btn_colorBot->color().blue()).arg(btn_colorBot->color().alpha());
     QString borderRadius;



     if(ui->comboBoxPosition->currentIndex()==EDGE::POS_BOT){ //bottum edge
          borderRadius=QString("border-top-left-radius :%1px;"
                               "border-top-right-radius : %2px;") .arg(QString::number(ui->spinBoxBleft->value()))
                    .arg(QString::number(ui->spinBoxBright->value()));
     }else if(ui->comboBoxPosition->currentIndex()==EDGE::POS_TOP){ //top edge
          borderRadius=QString("border-bottom-left-radius :%1px;"
                               "border-bottom-right-radius :%2px;") .arg(QString::number(ui->spinBoxBleft->value()))
                    .arg(QString::number(ui->spinBoxBright->value()));
     }else if(ui->comboBoxPosition->currentIndex()==EDGE::POS_RIGHt){  //right edge
          borderRadius=QString("border-top-left-radius :%1px;"
                               "border-bottom-left-radius :%2px;") .arg(QString::number(ui->spinBoxBtop->value()))
                    .arg(QString::number(ui->spinBoxBbottom->value()));
     }else if(ui->comboBoxPosition->currentIndex()==EDGE::POS_LEFT){    //left edge
          borderRadius=QString("border-top-right-radius :%1px;"
                               "border-bottom-right-radius :%2px;") .arg(QString::number(ui->spinBoxBtop->value()))
                    .arg(QString::number(ui->spinBoxBbottom->value()));
     }

     QString styleBorder= QString("\n  border:	1px solid rgba(%1, %2, %3,%4);"
                                  " %5")
               .arg(btn_colorBorder->color().red()).arg(btn_colorBorder->color().green())
               .arg(btn_colorBorder->color().blue()).arg(btn_colorBorder->color().alpha()).arg(borderRadius);

     QString styleGradient;
     if(ui->comboBoxPosition->currentIndex()<2){    //horizontal gradient
          styleGradient= QString("QWidget #widgetBgr{background-color: qlineargradient(spread:pad,"
                                 "x1:0, y1:0, x2:0, y2:1, %1 %2 %3 %4")
                    .arg(grad1).arg(grad2).arg(grad4).arg(grad3);
     }else{                                            //verticale gradient
          styleGradient= QString("QWidget #widgetBgr{background-color: qlineargradient(spread:pad,"
                                 "x1:0, y1:0, x2:1, y2:0, %1 %2 %3 %4")
                    .arg(grad1).arg(grad2).arg(grad4).arg(grad3);
     }

     // background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0,
     //stop:0 rgba(0, 0, 0, 255), stop:0.321429 rgba(0, 0, 255, 255), stop:0.668367 rgba(255, 0, 0, 255), stop:1 rgba(255, 255, 255, 255));
     ui->widgetBgr->setStyleSheet(styleGradient+styleBorder+"}");
     return styleGradient+styleBorder+"}";
}

//____________________________________________________________________________________
void DockSetting::on_toolButtonImage_clicked()
{
     QString  fileName = QFileDialog::getOpenFileName(this,
                                                      tr("Open Image"), QDir::homePath(), tr("Image Files (*.png *.jpg *.bmp *.svg)"));

     if(!fileName.isEmpty()){
          ui->lineEditImageFile->setText(fileName);

     }
}

//____________________________________________________________________________________
void DockSetting::saveStyleQss()
{


    QString myStyle,mystyleButton;

    if(ui->comboBoxBackStyle->currentIndex()==STYLE_GRAD){
        myStyle=getStyleGradien();
    }else if(ui->comboBoxBackStyle->currentIndex()==STYLE_IMAGE){
        myStyle=getStyleImage();
    }else if(ui->comboBoxBackStyle->currentIndex()==STYLE_3D){
        myStyle=getStyle3D();
    }

    mystyleButton=QString( "\n"
                           "QToolButton {\n"
                           " background-color:transparent;\n"
                           "  border: 1px solid  transparent;\n"
                           "  border-radius: 4px;\n"
                           " }"
                           "\n"
                           "QAction#DockButtonTask {\n"
                           "background-color: qconicalgradient(cx:0.5, cy:0.9, angle:90,"
                           "stop:0.324022 rgba(255, 0, 0, 0), stop:0.357542 rgba(0, 0, 0, 0),"
                           "stop:0.385475 rgba(%1, %2, %3, 255), stop:0.597765 rgba(%1, %2, %3, 255),"
                           "stop:0.625698 rgba(0, 0, 0, 0), stop:0.670391 rgba(255, 0, 0, 0));"
                           "  border-radius: 4px;\n"
                           " }"
                           "\n"
                           "  QToolButton:checked {\n"
                           "  border: 1px solid  rgba(%1, %2, %3, %4);\n"
                           "  border-radius: 4px;}\n"
                           "\n"
                           "  QToolButton:checked:hover {\n"
                           "  border: 1px solid  rgba(%1, %2, %3, %4);\n"
                           "  border-radius: 4px;}"
                           ).arg(btn_colorSelect->color().red()).arg(btn_colorSelect->color() .green()).arg(btn_colorSelect->color().blue())
            .arg(btn_colorSelect->color().alpha()) ;
    QString mystyleButtonHover;
    if(ui->checkBoxSelectionColor->isChecked()){
        mystyleButtonHover=QString("\n  QToolButton:hover {\n"
                                   "  background-color:  rgba(%1, %2, %3, %4);\n"
                                   "  border: 1px solid  rgba(%5, %6, %7, %8);\n"
                                   "  border-radius: 4px;\n"
                                   "      }\n"
                                   "  QToolButton:hover {\n"
                                   "  background-color: rgba(%1, %2, %3, %4);\n"
                                   "  border: 1px solid  rgba(%5, %6, %7, %8)\n;"
                                   "  border-radius: 4px;\n"
                                   "      }")
                .arg(btn_colorSelect->color().red()).arg(btn_colorSelect->color() .green()).arg(btn_colorSelect->color().blue())
                .arg(btn_colorSelect->color().alpha()) .arg(btn_colorBorder->color().red()).arg(btn_colorBorder->color() .green())
                .arg(btn_colorBorder->color().blue()).arg(btn_colorBorder->color().alpha());


    }else{
        mystyleButtonHover=QString(" \n QToolButton:hover {\n"
                                   "  background-color: transparent;\n"
                                   "  border: 1px solid  transparent;}\n"
                                   "  QToolButton#btnTask:hover {"
                                   "  background-color: transparent;\n"
                                   "  border: 1px solid  transparent;\n}");
    }
    QFile fileCsv(Edir::configDir()+"/dockTheme.css");
    if (!fileCsv.open(QFile::WriteOnly)){
        //    return ;
     //   qDebug()<<Edir::configDir()+"/dockTheme.css";
    }

    QTextStream out(&fileCsv);
    out.setCodec(QTextCodec::codecForLocale());
    out<<myStyle+mystyleButton+mystyleButtonHover;
    //   return myStyle+mystyleButton+mystyleButtonHover;
}

//____________________________________________________________________________________
QString DockSetting::getStyleImage()
{
     QString myStyle;
     QString borderRadius;
     if(ui->checkBoxRepeat->isChecked()){
          if(ui->comboBoxPosition->currentIndex()==EDGE::POS_BOT){
               borderRadius=QString("border-top-left-radius :%1px;"
                                    "border-top-right-radius : %2px;") .arg(QString::number(ui->spinBoxBleft->value()))
                         .arg(QString::number(ui->spinBoxBright->value()));
          }else if(ui->comboBoxPosition->currentIndex()==EDGE::POS_TOP){
               borderRadius=QString("border-bottom-left-radius :%1px;"
                                    "border-bottom-right-radius :%2px;") .arg(QString::number(ui->spinBoxBleft->value()))
                         .arg(QString::number(ui->spinBoxBright->value()));
          }else if(ui->comboBoxPosition->currentIndex()==EDGE::POS_RIGHt){
               borderRadius=QString("border-top-left-radius :%1px;"
                                    "border-bottom-left-radius :%2px;") .arg(QString::number(ui->spinBoxBleft->value()))
                         .arg(QString::number(ui->spinBoxBright->value()));
          }else if(ui->comboBoxPosition->currentIndex()==EDGE::POS_LEFT){
               borderRadius=QString("border-top-right-radius :%1px;"
                                    "border-bottom-right-radius :%2px;") .arg(QString::number(ui->spinBoxBleft->value()))
                         .arg(QString::number(ui->spinBoxBright->value()));
          }

          QString styleBorder= QString("\n  border:	1px solid rgba(%1, %2, %3,%4);"
                                       " %5")
                    .arg(btn_colorBorder->color().red()).arg(btn_colorBorder->color().green())
                    .arg(btn_colorBorder->color().blue()).arg(btn_colorBorder->color().alpha()).arg(borderRadius);


          myStyle=  QString("QWidget #widgetBgr{"
                            " background-image: url(%1) %2 %3 %4 %5;"
                            "background-repeat: repeat-xy;"
                            " border-top:%2 px transparent;"
                            " border-left: %3px transparent;"
                            " border-bottom:%4 px transparent;"
                            " border-right: %5px transparent;"


                            ).arg(ui->lineEditImageFile->text())
                    .arg(QString::number(ui->spinBoxBtop->value()))
                    .arg(QString::number(ui->spinBoxBleft->value()))
                    .arg(QString::number(ui->spinBoxBbottom->value()))
                    .arg(QString::number(ui->spinBoxBright->value()));

          myStyle=myStyle+styleBorder+ " }";
          qDebug()<<myStyle+styleBorder;
     }else{
          myStyle= QString("QWidget #widgetBgr{"
                           "border-image: url(%1) %2 %3 %4 %5;"
                           " border-top:%2 px transparent;"
                           " border-left: %3px transparent;"
                           " border-bottom:%4 px transparent;"
                           " border-right: %5px transparent;"
                           " }"

                           ).arg(ui->lineEditImageFile->text())
                    .arg(QString::number(ui->spinBoxBtop->value()))
                    .arg(QString::number(ui->spinBoxBleft->value()))
                    .arg(QString::number(ui->spinBoxBbottom->value()))
                    .arg(QString::number(ui->spinBoxBright->value()));
     }

     return myStyle;
}

//____________________________________________________________________________________
QString DockSetting::getStyle3D()
{

     QString imageName=ui->comboBox3D->itemData(ui->comboBox3D->currentIndex()).toString();
     QString style3d;
     if(ui->comboBoxPosition->currentIndex()==EDGE::POS_BOT){
          ui->spinBoxBtop->setValue(0);
          ui->spinBoxBleft->setValue(30);
          ui->spinBoxBbottom->setValue(7);
          ui->spinBoxBright->setValue(30);
          style3d=QString("QWidget #widgetBgr{"
                          "border-image: url(%1) 0 30 0 30;"
                          " border-top:%2 px transparent;"
                          " border-bottom:0px transparent;"
                          " border-right: 30px transparent;"
                          "  border-left: 30px transparent;"
                          " }"

                          ).arg(imageName).arg(ui->spinBoxIconSize->value()/2+8);

     }else if(ui->comboBoxPosition->currentIndex()==EDGE::POS_TOP){
          imageName+="Top";
          ui->spinBoxBtop->setValue(7);
          ui->spinBoxBleft->setValue(30);
          ui->spinBoxBbottom->setValue(0);
          ui->spinBoxBright->setValue(30);
          style3d=QString("QWidget #widgetBgr{"
                          "border-image: url(%1) 0 30 0 30;"
                          " border-top:0 px transparent;"
                          " border-bottom:%2 px transparent;"
                          " border-right: 30px transparent;"
                          "  border-left: 30px transparent;"
                          " }"

                          ).arg(imageName).arg(ui->spinBoxIconSize->value()/2+8);
     }else if(ui->comboBoxPosition->currentIndex()==EDGE::POS_RIGHt){
          ui->spinBoxBtop->setValue(30);
          ui->spinBoxBleft->setValue(0);
          ui->spinBoxBbottom->setValue(30);
          ui->spinBoxBright->setValue(7);
          imageName+="Right";
          style3d=QString("QWidget #widgetBgr{"
                          "border-image: url(%1) 30 0 30 0;"
                          " border-top:30 px transparent;"
                          " border-bottom:30px transparent;"
                          " border-right: 0px transparent;"
                          "  border-left: %2px transparent;"
                          " }"

                          ).arg(imageName).arg(ui->spinBoxIconSize->value()/2+8);
     }else if(ui->comboBoxPosition->currentIndex()==EDGE::POS_LEFT){
          ui->spinBoxBtop->setValue(30);
          ui->spinBoxBleft->setValue(7);
          ui->spinBoxBbottom->setValue(30);
          ui->spinBoxBright->setValue(0);
          imageName+="Left";
          style3d=QString("QWidget #widgetBgr{"
                          "border-image: url(%1) 30 0 30 0;"
                          " border-top:30 px transparent;"
                          " border-bottom:30px transparent;"
                          " border-right: %2px transparent;"
                          "  border-left: 0px transparent;"
                          " }"

                          ).arg(imageName).arg(ui->spinBoxIconSize->value()/2+8);
     }

     //  QString imageName=ui->comboBox3D->itemData(ui->comboBox3D->currentIndex()).toString();


     return style3d;
}


//____________________________________________________________________________________
void DockSetting::on_comboBoxPosition_activated(int index)
{
     if(index<2)
          setupGradientTools(true);
     else
          setupGradientTools(false);

     getStyleGradien();
}
void DockSetting::on_comboBoxTheme_currentIndexChanged(int index)
{
    QString path=ui->comboBoxTheme->itemData(index).toString();
    loadSetting(path+"/settings.conf");
      getStyleGradien();
}
Q_EXPORT_PLUGIN2(setting_DockSetting, DockSetting)


