#ifndef DOCKSETTING_H
#define DOCKSETTING_H
#include "../spluginsinterface.h"
#include <QWidget>

#include <ButtonColor>
#include <QDialog>
#include <QAbstractButton>

#define GRAD_TOP   1
#define GRAD_RIGHT   2
#define GRAD_BOT   3
#define GRAD_LEFT  4
#define COLOR_BORDER  5
#define COLOR_SELECT  6
class EDGE
{
public:
    EDGE() {}

    enum ArrangementType {TYPE_AUTO ,     /*type auto hide*/
                          TYPE_SPACE ,    /*type reserve space*/
                          TYPE_TOP ,     /*type on top*/
                          TYPE_BOT     /*type on bottom*/ };

    enum PositionEdge { POS_BOT ,    /*bottom edge*/
                        POS_TOP ,    /*top edge*/
                        POS_RIGHt ,     /*right edge*/
                        POS_LEFT      /*left edge*/ };

  static  PositionEdge posToInt(QString pos){
        if(pos=="bottom")return POS_BOT;
        else if(pos=="top")return POS_TOP;
        else if(pos=="right")return POS_RIGHt;
        else if(pos=="left")return POS_LEFT;
        else return POS_BOT;
    }

 static   ArrangementType arrangementToInt(QString type){
        if(type=="auto-hide")return TYPE_AUTO;
        else if(type=="reserve-space")return TYPE_SPACE;
        else if(type=="on-top")return TYPE_TOP;
        else if(type=="on-bottom")return TYPE_BOT;
        else return TYPE_AUTO;
    }

  static  QString posToString(int pos){
        switch (pos) {
        case POS_BOT:return "bottom";
        case POS_TOP:return "top";
        case POS_RIGHt:return "right";
        case POS_LEFT:return "left";
        }
        return "bottom";
    }

  static  QString arrangementToString(int type){
        switch (type) {
        case TYPE_AUTO:return "auto-hide";
        case TYPE_SPACE:return "reserve-space";
        case TYPE_TOP:return "on-top";
        case TYPE_BOT:return "on-bottom";
        }
        return "auto-hide";
    }

};
namespace Ui {
class DockSetting;
}

class DockSetting : public QWidget,PluginsInterface
{
    Q_OBJECT
    Q_INTERFACES(PluginsInterface)
    
public:
    explicit DockSetting(QWidget *parent = 0);
    ~DockSetting();
signals :
    void accepteChange();
private:
    Ui::DockSetting *ui;

    QSettings *msetting;
    QString m_stop,m_stop2;
    QString m_iconsTheme;


    ButtonColor *btn_colorTop,*btn_colorRight,*btn_colorBot,*btn_colorLeft;
    ButtonColor  *btn_colorBorder,*btn_colorSelect;

public slots:

    void saveSettings();
    QWidget *widget(){return this;}

private slots:

    void setupThemes();

    void loadSetting(const QString &path=QString());

    void on_verticalSlider_sliderMoved(int position);

    void on_verticalSlider2_sliderMoved(int position);

    void setupGradientTools(bool horizontal);

    void on_toolButtonImage_clicked();

    QString getStyleGradien();

    void saveStyleQss();

    QString getStyleImage();

    QString getStyle3D();

    void on_comboBoxPosition_activated(int index);

    void on_comboBoxTheme_currentIndexChanged(int index);
};

#endif // DOCKSETTING_H
