/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "widgetautostart.h"
#include "ui_widgetautostart.h"
#include "dialogedit.h"
#include <QSettings>
#include <QDebug>
#include <EDir>
#include <QTextCodec>
#include <unistd.h>

#define TYPE_EXEC "Application"
#define TYPE_SCRIPT "Script"

widgetAutoStart::widgetAutoStart(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::widgetAutoStart)
{
    ui->setupUi(this);



    //! load settings-------------------------------------------------
    QSettings setting("elokab","autostart") ;
    int count = setting.beginReadArray("AutoStart");
   QStringList listUser;
    for (int i = 0; i < count; ++i){

        setting.setArrayIndex(i);
        QString   DesktopName = setting.value("FileName").toString();
       // qDebug()<<"name  "<<DesktopName;
        listUser.append(DesktopName);

    } // for
    setting.endArray();


    QStringList list=ElokabAutoStart::filesList();

    foreach (QString s, list) {
            QStringList lisDescktop=ElokabAutoStart::elokabDescktopEntry(s);
            //   return list<<name 0<<icon 1<<coment  2<<exec 3<<QString::number(needtry) 4<<QString::number(autoStart) 5;
            QTreeWidgetItem *item=new QTreeWidgetItem(ui->treeWidget);
            item->setText(0,lisDescktop.at(0));//name
            item->setIcon(0,QIcon::fromTheme(lisDescktop.at(1),QIcon::fromTheme("application-x-desktop")));//icon
            item->setText(1,lisDescktop.at(2));//coment
            item->setText(2,lisDescktop.at(3));//exec

            item->setData(3,Qt::UserRole,lisDescktop.at(4).toInt());//needsystray
          qDebug()<<"item need systray"<<lisDescktop.at(0)<<lisDescktop.at(4);
            item->setData(5,Qt::UserRole,s);//filename

            if(s.startsWith(QDir::homePath())){
                    item->setData(4,Qt::UserRole,true);//writ

            }else{
                    item->setData(4,Qt::UserRole,false);//reid

            }

            if(listUser.contains(lisDescktop.at(3)))
                    item->setCheckState(0,Qt::Unchecked);
            else
                    item->setCheckState(0,Qt::Checked);

    }
    ui->treeWidget->resizeColumnToContents(0);
    ui->treeWidget->resizeColumnToContents(1);
     m_autoChanged=false;
}

widgetAutoStart::~widgetAutoStart()
{
    delete ui;
}
/***************************************
 *save the setting
 ****************************************/
void widgetAutoStart::saveSettings()
{
    if(  m_autoChanged==false)
        return;
    QSettings settings("elokab","autostart") ;

    settings.remove("AutoStart");
    settings.beginWriteArray("AutoStart");

    int count=ui->treeWidget->topLevelItemCount();
 int xA=0;
    for(int i=0;i<count;i++){
      QTreeWidgetItem *item=ui->treeWidget->topLevelItem(i);


              if(item->checkState(0)==Qt::Unchecked){
              settings.setArrayIndex(xA);
              settings.setValue("FileName", item->text(2));
              xA++;
              }
}


    settings.endArray();
      m_autoChanged=false;
}
/***************************************
 *add new programme
 ****************************************/
void widgetAutoStart::on_pushButtonAddPro_clicked()
{
    DialogEdit *dlg=new DialogEdit;
    if(dlg->exec()==QDialog::Accepted){
        if(dlg->programmName.isEmpty()||dlg->name.isEmpty())
            return;
        QTreeWidgetItem *item=new QTreeWidgetItem(ui->treeWidget);
        item->setText(0,dlg->name);
        item->setIcon(0,QIcon::fromTheme("application-x-desktop"));//icon
item->setCheckState(0,Qt::Checked);
        item->setText(1,dlg->comment);
        item->setText(2,dlg->programmName);

       item->setData(3,Qt::UserRole,dlg->needTray);
       item->setData(4,Qt::UserRole,true);//writ

QString fileName=Edir::dataHomeDir()+"/autostart/"+dlg->name.replace(" ","-")+".desktop";
  item->setData(5,Qt::UserRole,fileName);//writ
       QSettings settings(fileName,QSettings::IniFormat);
  //  settings.setIniCodec(QTextCodec::codecForTr());
  //  settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
  //QString(QChar::Separator_Space)
    settings.beginGroup("Desktop Entry");

    settings.setValue("Type","Application");
    settings.setValue("Name",dlg->name);
     settings.setValue("Comment",dlg->comment);
     settings.setValue("Exec",dlg->programmName);
     settings.setValue("Icon",dlg->iconName);
     settings.setValue("OnlyShowIn","X-Elokab");
     settings.setValue("X-Elokab-Need-SysTray",dlg->needTray);
     settings.setValue("X-Elokab-Autostart",true);


    settings.endGroup();
   m_autoChanged=true;
removeUnicodeSpace(fileName);
    }
}

/***************************************
 *remove curent program
 ****************************************/

void widgetAutoStart::on_pushButtonRemove_clicked()
{
     QTreeWidgetItem *item=ui->treeWidget->currentItem();
     if(item){
             QString fileName=item->data(5,Qt::UserRole).toString();
             QFile::remove(fileName);
         ui->treeWidget->takeTopLevelItem(ui->treeWidget->indexOfTopLevelItem(item));
           m_autoChanged=true;
     }
}
/***************************************
 *edit curent program
 ****************************************/
void widgetAutoStart::on_pushButtonEdit_clicked()
{


        QTreeWidgetItem *item=ui->treeWidget->currentItem();
        if(!item)
                return;

        DialogEdit *dlg=new DialogEdit;
        dlg->name=item->text(0);
        dlg->programmName=item->text(2);
        dlg->comment=item->text(1);
        dlg->needTray=item->data(3,Qt::UserRole).toBool();
        dlg->iconName=item->icon(0).name();
        qDebug()<<item->data(3,Qt::UserRole).toBool();
        dlg->setProgram();
        if(dlg->exec()==QDialog::Accepted){
                if(dlg->programmName.isEmpty())
                        return;

                item->setText(0,dlg->name);
                item->setText(2,dlg->programmName);
                item->setText(1,dlg->comment);
                item->setData(3,Qt::UserRole,dlg->needTray);


                QString fileName=item->data(5,Qt::UserRole).toString();

                QSettings settings(fileName,QSettings::IniFormat);

           settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
                //QString(QChar::Separator_Space)
                settings.beginGroup("Desktop Entry");

                settings.setValue("Name",dlg->name);
                settings.setValue("Comment",dlg->comment);
                settings.setValue("Exec",dlg->programmName);

                settings.setValue("X-Elokab-Need-SysTray",dlg->needTray);
                settings.setValue("Icon",dlg->iconName);

                settings.endGroup();
                m_autoChanged=true;
              removeUnicodeSpace(fileName);
        }

}
void widgetAutoStart::removeUnicodeSpace(const QString &fileName)
{
    qApp->processEvents();
    qDebug()<<fileName;
    //read--------------------
    QFile file(fileName);
    if (!file.open(QIODevice::ReadOnly ))
        return;
    qDebug()<<fileName;
    QTextStream in(&file);
    in.setCodec(QTextCodec::codecForName("UTF-8"));

    QString txt = in.readAll();
    file.close();
    //---------------------------------
    QTextCodec *codec = QTextCodec::codecForName("UTF-8");
    QByteArray encodedString =QByteArray::fromPercentEncoding(txt.toUtf8());

    QString string = codec->toUnicode(encodedString);
    // Write----------------------
    qApp->processEvents();
    QFile fileS(fileName);

    if (! fileS.open(QIODevice::WriteOnly ))
        return;
    qDebug()<<txt;
    QTextStream out(&fileS);
    out.setCodec(QTextCodec::codecForName("UTF-8"));
    out<<string;
    fileS.close();
}

void widgetAutoStart::on_treeWidget_itemActivated(QTreeWidgetItem *item, int /*column*/)
{
     // qDebug()<<item->data(3,Qt::UserRole).toBool();
       qDebug()<<item->data(4,Qt::UserRole).toBool();
}

void widgetAutoStart::on_treeWidget_itemSelectionChanged()
{
        QTreeWidgetItem *item=ui->treeWidget->currentItem();
        if(!item)
            return;
        ui->pushButtonEdit->setEnabled(item->data(4,Qt::UserRole).toBool());
        ui->pushButtonRemove->setEnabled(item->data(4,Qt::UserRole).toBool());
}

void widgetAutoStart::on_treeWidget_itemChanged(QTreeWidgetItem */*item*/, int /*column*/)
{
       // qDebug()<<"itemChanged"<<item->text(0);
     m_autoChanged=true;
}
Q_EXPORT_PLUGIN2(setting_AutoStart, widgetAutoStart)
