/* coded by Ketmar // Vampire Avalon (psyc://ketmar.no-ip.org/~Ketmar)
 * (c)DWTFYW
 *
 * This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details.
 */
#include "cfgfile.h"

#include <QtCore/QDir>
#include <QtCore/QFile>
#include <QtCore/QStringList>
#include <QtCore/QTextStream>
#include <QDebug>

QMultiMap<QString, QString> loadCfgFile(const QString &fname, bool forceLoCase)
{
    QMultiMap<QString, QString> res;
    QFile fl(fname);
    if (fl.open(QIODevice::ReadOnly))
    {
        QTextStream stream;
        stream.setDevice(&fl);
        stream.setCodec("UTF-8");
        QString curPath = "/";
        while (1)
        {
            QString s = stream.readLine();
            if (s.isNull()) break;
            s = s.trimmed();
            if (s.isEmpty() || s[0] == '#' || s[0] == ';') continue;
            if (s[0] == '[')
            {
                // new path
                int len = s.length()-1;
                if (s[len] == ']') len--;
                s = s.mid(1, len).simplified();
                s += '/';
                curPath = s;
                continue;
            }
            int eqp = s.indexOf('=');
            if (eqp < 0) continue; // invalid entry
            QString name = s.left(eqp).simplified();
            QString value = s.mid(eqp+1).simplified();
            if (name.isEmpty()) continue; // invalid entry
            name.prepend(curPath);
            if (forceLoCase) name = name.toLower();
            res.insert(name, value);
        }
        fl.close();
    }
    return res;
}

#define XD_CODEC  "koi8-r"
void fixXDefaults(const QString &themeName)
{
    QStringList lst;
    {
        QFile fl(QDir::home().path()+"/.Xdefaults");
        if (fl.open(QIODevice::ReadOnly))
        {
            QTextStream stream;
            stream.setDevice(&fl);
            stream.setCodec(XD_CODEC);
            while (1)
            {
                QString s = stream.readLine();
                if (s.isNull()) break;
                if (!s.startsWith("Xcursor*theme:")) lst << s;
            }
            fl.close();
        }
    }
    while (lst.size() > 0)
    {
        QString s(lst[lst.size()-1]);
        if (!s.trimmed().isEmpty()) break;
        lst.removeAt(lst.size()-1);
    }
    {
        //QByteArray ba(themeName.toUtf8());
        QFile fl(QDir::home().path()+"/.Xdefaults");
        if (fl.open(QIODevice::WriteOnly))
        {
            QTextStream stream;
            stream.setDevice(&fl);
            stream.setCodec(XD_CODEC);
            foreach (const QString &s, lst)
            {
                stream << s << "\n";
            }
            stream << "\nXcursor*theme: " << themeName << "\n";
            fl.close();
        }
    }
}

const QString findDefaultTheme()
{
    QString res = "default";
    QFile fl(QDir::home().path()+"/.Xdefaults");
    if (fl.open(QIODevice::ReadOnly))
    {
        QTextStream stream;
        stream.setDevice(&fl);
        stream.setCodec(XD_CODEC);
        while (1)
        {
            QString s = stream.readLine();
            if (s.isNull()) break;
            if (!s.startsWith("Xcursor*theme:")) continue;
            s.remove(0, 14);
            s = s.trimmed();
            if (s.isEmpty()) s = "default";
            res = s;
        }
        fl.close();
    }
    return res;
}
/*! ـــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــ
                               بداية تحميل السمات
ــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــ*/
/*//! [1]
QList<XCursorThemeData *> loadListThemesCursor()
{
    // البحث في المجلدات السمات بحث عن سمة


        foreach (const QString &dirPath, m_dirsPaths()){

                QDir dir(dirPath);

                if (!dir.exists()) continue;

                foreach (const QString &name, dir.entryList(QDir::AllDirs |
                                                            QDir::NoDotAndDotDot | QDir::Readable | QDir::Executable))
                {
                        // اذا كان موجودا من قبل
                        if (themeExist(name)){

                                qDebug() << "duplicate theme:" << dir.path()+name;
                        }
                        if (!dir.cd(name)){
                                qDebug() << "can't cd:" << dir.path()+name;
                                continue;
                        }

                        //-----------------------------------------------
                        bool dirCursorExist = dir.exists("cursors");

                        bool fileIndexExist=QFile::exists(dir.path()+"/index.theme");

                        if(!dirCursorExist&& !fileIndexExist){
                                dir.cdUp();
                                continue;
                        }

                        // انشاء سمة
                        XCursorThemeData *theme = new XCursorThemeData(dir);
                        // التجاوز اذا كانت السمة مخفية
                        if (theme->isHidden())
                        {

                                delete theme;
                                dir.cdUp();
                                continue;
                        }
                        //في حالة وجود ملف السمة وغدم وجود مجلد المؤشر سيتم البحث عن اسم
                        //مجلد المؤشر ثم البحث عنه
                        if (!dirCursorExist)
                        {
                                bool foundCursorTheme = false;
                                foreach (const QString &name, theme->inherits())
                                {
                                     foundCursorTheme = isCursorTheme(name);
                                }
                                //  اذا كان الرد بالسلب سيتم حذف السمة المؤقتة
                                if (!foundCursorTheme)
                                {
                                        delete theme;
                                        dir.cdUp();
                                        continue;
                                }
                        }
                        m_listThemes.append(theme);



                        //----------------------------------------------
                        dir.cdUp(); // العودة الى المجلد الاعلى
                }
        }

}
//! [2] // البحث اذا كان موجودا في القائمة من قبل
bool themeExist(const QString &name)
{
        const uint hash = qHash(name);
        foreach (const XCursorThemeData *theme, m_listThemes)
                if (theme->hash() == hash)
                        return true;
        return false;
}
//! [3] البحث عن مجلد المؤشرات في الملف
bool isCursorTheme(const QString &theme, const int depth)
{
      //اذا كان المجلد يحتوي على ملف index.theme
        //ولا يحتوي على مجلد المؤشرات cursor
        //سيتم البحث في مجلدات المؤشرات عن مجلد المؤشرات
        //يواسطة الوراثة inherits
    if (depth > 10) return false;

    foreach (const QString &dirPath, m_dirsPaths())
    {
        QDir dir(dirPath);
        if (!dir.exists() || !dir.cd(theme)) continue;

        if (dir.exists("cursors")) return true;

        if (!dir.exists("index.theme")) continue;
        // فتح الملف للبحث هن الوراثة
        QMultiMap<QString, QString> cfg = loadCfgFile(dir.path()+"/index.theme", true);
        QStringList inherits = cfg.values("icon theme/inherits");

        for (int f = inherits.size()-1; f >= 0; --f)
        {
            QString inh = inherits.at(f);

            if (inh == theme) continue;
            if (isCursorTheme(inh, depth+1)) return true;
        }
    }
    return false;
}
QStringList m_dirsPaths()
{
        QStringList dirsPaths;
QString path = XcursorLibraryPath();


   // "~/.icons:/usr/share/icons:/usr/share/pixmaps" تقسيم النص
   m_dirsPdirsPathsaths = path.split(':', QString::SkipEmptyParts);

   // حذف المتشابه
   QMutableStringListIterator i(dirsPaths);
   while (i.hasNext())
   {
       const QString path = i.next();
       QMutableStringListIterator j(i);
       while (j.hasNext()) if (j.next() == path) j.remove();
   }
   if(dirsPaths.count()==0)
           dirsPaths()<<QDir::homePath()+"/icons"<<"/usr/share/icons";
   return dirsPaths;

}*/
/*! ـــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــ
                          نهاية تحميل السمات
ــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــ*/
