/***************************************************************************
 *   elokab Copyright (C) 2014 AbouZakaria <yahiaui@gmail.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "wiggetcursor.h"
#include "ui_wiggetcursor.h"

#include "cfgfile.h"
#include "crtheme.h"


#include <QtCore/QDir>

#include <X11/Xlib.h>
#include <X11/Xcursor/Xcursor.h>

#include <QSettings>
#include <QDebug>
#include <QX11Info>
#include <QStyle>
#include <QProcess>
WiggetCursor::WiggetCursor(QWidget *parent) :
        QWidget(parent),
        ui(new Ui::WiggetCursor)
{
    ui->setupUi(this);
    //temp
    ui->buttonBox->setVisible(false);

    qDebug()<<QIcon::themeName();
    setWindowIcon(QIcon::fromTheme("preferences-desktop-mouse",QIcon::fromTheme("mouse")));

    ui->labelInformation->setPixmap( QIcon::fromTheme("dialog-warning",QIcon(":/images/dialog-warning")) .pixmap(24).scaled(24,24));


    //تحميل السمات
    loadListThemesCursor();


    ui->comboBoxCursor->setIconSize(QSize(24, 24));
    //تحميل السمات على الادات combo
    foreach (XCursorThemeData *theme , m_listThemes) {
        QPixmap pix=(previewPix(*theme, "left_ptr"));
        ui->comboBoxCursor->addItem(pix,theme->title());
    }

    connect(ui->comboBoxCursor, SIGNAL(activated(int)),
            SLOT(currentComboChanged(int)));

    //تحميل السمة المحفوظة ان وجدت او سمة النظام
    setCurrentCursor();
    /**/


    //QString  i="XCURSOR_THEME";
    //   qDebug()<< "XCURSOR_THEME = "<<  qgetenv(i.toUtf8().constData());
    //  qDebug()<< "envirenments = "<< QProcess::systemEnvironment ();
    //createlistPixmapThemes();
}

WiggetCursor::~WiggetCursor()
{
        delete ui;
}
/***************************************************
                جلب المؤشر المحفوظ او الافتراضي
 ***************************************************/
void WiggetCursor::setCurrentCursor()
{
        //السمة المحفوظة يدويا
        QSettings sT("elokab","elokabsettings");
        sT.beginGroup("Cursor");
        QString curentTheme=  sT.value("XCURSOR_THEME").toString();
        sT.endGroup();

        if(curentTheme.isEmpty())
                curentTheme=findDefaultTheme();

        if(curentTheme.isEmpty())
                curentTheme = getCurrentTheme();

        if (curentTheme.isEmpty())
                return;

        qDebug()<<"Saved Theme = "<<curentTheme;

        if(curentTheme=="default"){
           ui->comboBoxCursor->setCurrentIndex(-1);
           m_oldIndex=-1;
        }else{
             //التأشير على اname()لسمة في الادات ان كانت موجودة
             uint hash = qHash(curentTheme);
             for (int i = 0; i < m_listThemes.count(); ++i)
             {
                  const XCursorThemeData *theme = m_listThemes.at(i);
                  if (theme->hash() == hash) {

                       ui->comboBoxCursor->setCurrentIndex(i);
                       m_oldIndex=i;

                       ui->labelDescreption->setText(tr("Description : ")+theme->description());
                       updatePreview(*theme);

                       break;
                  }
             }
        }


    //qDebug()<<"cursor========="<<ui->comboBoxCursor->currentIndex();
}


/***************************************************
                   تطبيق التغييرات
 ***************************************************/
//! [1]
void WiggetCursor::on_buttonBox_clicked(QAbstractButton *button)
{
        if(ui->buttonBox->standardButton(button)==QDialogButtonBox::Apply){
                saveSettings();
        }
}
//! [2]
void WiggetCursor::saveSettings()
{
//     qDebug()<<"oldIndex :"+m_oldIndex<<"newIndex:"+index;
        int index=ui->comboBoxCursor->currentIndex();
        qDebug()<<"oldIndex :"<<m_oldIndex<<"newIndex:"<<index;


        if (index < 0 || index >=m_listThemes.count()) return ;
        if(m_oldIndex==index)return;

       const XCursorThemeData *theme = m_listThemes.at(index);

        if (!theme) return;
         m_oldIndex=index;
        applyTheme(*theme);
        fixXDefaults(theme->name());
        QSettings Setting("elokab","elokabsettings");

        Setting.beginGroup("Cursor");
        Setting.setValue("XCURSOR_THEME", theme->name());
        Setting.endGroup();
qDebug()<<"oldIndex :"<<m_oldIndex<<"newIndex:"<<index<<" applique cursor theme :"+theme->name();
}

/***************************************************
                تحديث المعاينة على الادوات
                    عند تغير السمة
 ***************************************************/
//! [1] tr("Description : ")عند تغير السمة
void WiggetCursor::currentComboChanged(int index)
{

        if (index < 0 || index >=m_listThemes.count()) return ;


        const XCursorThemeData *theme =m_listThemes.at(index);
          if (theme){
                ui->labelDescreption->setText(tr("Description : ")+theme->description());
                updatePreview(*theme);

        }

}
//! [2] تحديث أدوات المعاينة
void WiggetCursor::updatePreview(const XCursorThemeData &theme)
{
        ui->label_cross->setPixmap(previewPix(theme, "cross"));
        ui->label_cross->setCursor(theme.loadCursor("cross"));

        ui->label_ibeam->setPixmap(previewPix(theme, "ibeam"));
        ui->label_ibeam->setCursor(theme.loadCursor("ibeam"));

        ui->label_left_ptr->setPixmap(previewPix(theme, "left_ptr"));
        ui->label_left_ptr->setCursor(theme.loadCursor("left_ptr"));

        ui->label_left_ptr_watch->setPixmap(previewPix(theme, "left_ptr_watch"));
        ui->label_left_ptr_watch->setCursor(theme.loadCursor("left_ptr_watch"));

        ui->label_wait->setPixmap(previewPix(theme, "wait"));
        ui->label_wait->setCursor(theme.loadCursor("wait"));

        ui->label_pointing_hand->setPixmap(previewPix(theme, "pointing_hand"));
        ui->label_pointing_hand->setCursor(theme.loadCursor("pointing_hand"));

        ui->label_whats_this->setPixmap(previewPix(theme, "whats_this"));
        ui->label_whats_this->setCursor(theme.loadCursor("whats_this"));

        ui->label_size_all->setPixmap(previewPix(theme, "size_all"));
        ui->label_size_all->setCursor(theme.loadCursor("size_all"));

        ui->label_size_fdiag->setPixmap(previewPix(theme, "size_fdiag"));
        ui->label_size_fdiag->setCursor(theme.loadCursor("size_fdiag"));

        ui->label_split_h->setPixmap(previewPix(theme, "split_h"));
        ui->label_split_h->setCursor(theme.loadCursor("split_h"));


    }


//! [3] انشاء صورة للمعاينة
QPixmap WiggetCursor::previewPix(const XCursorThemeData &theme,const QString &name)
{
        int size = style()->pixelMetric(QStyle::PM_LargeIconSize);

        QImage image = theme.loadImage(name, size);
        if (image.isNull()) return QPixmap();
        int maxSize = size*2;
        if (image.height() > maxSize || image.width() > maxSize)
                image = image.scaled(maxSize, maxSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
        QPixmap  mPixmap = QPixmap::fromImage(image);

        return mPixmap;
}

/*! ـــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــ
                               بداية تحميل السمات
ــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــ*/
//! [1]
void WiggetCursor::loadListThemesCursor()
{
    // البحث في المجلدات السمات بحث عن سمة


     QString path = XcursorLibraryPath();


        // "~/.icons:/usr/share/icons:/usr/share/pixmaps" تقسيم النص
        m_dirsPaths = path.split(':', QString::SkipEmptyParts);

        // حذف المتشابه
        QMutableStringListIterator i(m_dirsPaths);
        while (i.hasNext())
        {
            const QString path = i.next();
            QMutableStringListIterator j(i);
            while (j.hasNext()) if (j.next() == path) j.remove();
        }
        // تبدسل ~/ الى مجلد البيت
        m_dirsPaths.replaceInStrings(QRegExp("^~\\/"), QDir::home().path() + '/');
      //  qDebug()<<dirsPaths;
        if(m_dirsPaths.count()==0)
                m_dirsPaths<<QDir::homePath()+"/.icons"<<"/usr/share/icons";

        foreach (const QString &dirPath, m_dirsPaths){

                QDir dir(dirPath);

                if (!dir.exists()) continue;

                foreach (const QString &name, dir.entryList(QDir::AllDirs |
                                                            QDir::NoDotAndDotDot | QDir::Readable | QDir::Executable))
                {
                        // اذا كان موجودا من قبل
                        if (themeExist(name)){

                                qDebug() << "duplicate theme:" << dir.path()+name;
                        }
                        if (!dir.cd(name)){
                                qDebug() << "can't cd:" << dir.path()+name;
                                continue;
                        }

                        //-----------------------------------------------
                        bool dirCursorExist = dir.exists("cursors");

                        bool fileIndexExist=QFile::exists(dir.path()+"/index.theme");

                        if(!dirCursorExist&& !fileIndexExist){
                                dir.cdUp();
                                continue;
                        }

                        // انشاء سمة
                        XCursorThemeData *theme = new XCursorThemeData(dir);
                        // التجاوز اذا كانت السمة مخفية
                        if (theme->isHidden())
                        {

                                delete theme;
                                dir.cdUp();
                                continue;
                        }
                        //في حالة وجود ملف السمة وغدم وجود مجلد المؤشر سيتم البحث عن اسم
                        //مجلد المؤشر ثم البحث عنه
                        if (!dirCursorExist)
                        {
                                bool foundCursorTheme = false;
                                foreach (const QString &name, theme->inherits())
                                {
                                     foundCursorTheme = isCursorTheme(name);
                                }
                                //  اذا كان الرد بالسلب سيتم حذف السمة المؤقتة
                                if (!foundCursorTheme)
                                {
                                        delete theme;
                                        dir.cdUp();
                                        continue;
                                }
                        }
                        m_listThemes.append(theme);



                        //----------------------------------------------
                        dir.cdUp(); // العودة الى المجلد الاعلى
                }
        }

}
//! [2] // البحث اذا كان موجودا في القائمة من قبل
bool WiggetCursor::themeExist(const QString &name) const
{
        const uint hash = qHash(name);
        foreach (const XCursorThemeData *theme, m_listThemes)
                if (theme->hash() == hash)
                        return true;
        return false;
}
//! [3] البحث عن مجلد المؤشرات في الملف
bool WiggetCursor::isCursorTheme(const QString &theme, const int depth)
{
      //اذا كان المجلد يحتوي على ملف index.theme
        //ولا يحتوي على مجلد المؤشرات cursor
        //سيتم البحث في مجلدات المؤشرات عن مجلد المؤشرات
        //يواسطة الوراثة inherits
    if (depth > 10) return false;

    foreach (const QString &dirPath, m_dirsPaths)
    {
        QDir dir(dirPath);
        if (!dir.exists() || !dir.cd(theme)) continue;

        if (dir.exists("cursors")) return true;

        if (!dir.exists("index.theme")) continue;
        // فتح الملف للبحث هن الوراثة
        QMultiMap<QString, QString> cfg = loadCfgFile(dir.path()+"/index.theme", true);
        QStringList inherits = cfg.values("icon theme/inherits");

        for (int f = inherits.size()-1; f >= 0; --f)
        {
            QString inh = inherits.at(f);

            if (inh == theme) continue;
            if (isCursorTheme(inh, depth+1)) return true;
        }
    }
    return false;
}
/*! ـــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــ
                          نهاية تحميل السمات
ــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــــ*/

//////////////creat temp dir for all pixmap/////////////////////////////
void WiggetCursor::createlistPixmapThemes()
{
        QString localDir=QDir::homePath()+"/.local/share/elokab-mouse/";
        QDir dir;
      dir.mkpath(localDir);
  //remove all themes
      QDir dirS(localDir);
  QProcess   process ;
  foreach ( QString name, dirS.entryList(QDir::AllEntries|
                                              QDir::NoDotAndDotDot| QDir::Readable | QDir::Executable ))
        {

               process.startDetached("rm -r \""+localDir+name+"\"");

        }
//create new
  foreach ( XCursorThemeData *theme, m_listThemes){

        QString fileName=theme->title();
          dirS.mkpath(localDir+fileName);

          QSettings settings(localDir+fileName+"/theme.desktop",QSettings::IniFormat);
         // settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
          settings.beginGroup("Cursor-Theme");

          settings.setValue("Name",theme->name());
          settings.setValue("Comment",theme->description());
          settings.setValue("TiTle",fileName);

          settings.endGroup();
          QPixmap pix=theme->icon();
          pix.save(localDir+fileName+"/left_ptr.png");

          QPixmap pixhand(previewPix(*theme, "pointing_hand"));
          pixhand.save(localDir+fileName+"/pointing_hand.png");

          QPixmap pixwait=previewPix(*theme, "wait");
          pixwait.save(localDir+fileName+"/wait.png");

          QPixmap pixwhats_this=previewPix(*theme, "whats_this");
          pixwhats_this.save(localDir+fileName+"/whats_this.png");

  }


}
Q_EXPORT_PLUGIN2(setting_CursorTheme, WiggetCursor)
