/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtGui>

#include "worldtimeclock.h"

WorldTimeClock::WorldTimeClock(QWidget *parent)
    : QWidget(parent)
{
    timeZoneOffset = 0;

    timer = new QTimer(this);
  //  connect(timer, SIGNAL(timeout()), this, SLOT(update()));
    connect(timer, SIGNAL(timeout()), this, SLOT(uptatTime()));

    m_dateTime = QTime::currentTime();

    setWindowTitle(tr("World Time Clock"));
    resize(200, 200);
    timer->start(1000);
}
void WorldTimeClock::uptatTime()
{
        m_dateTime = QTime::currentTime();
        emit timeChanged(m_dateTime.currentTime())  ;
        update();
}

void WorldTimeClock::paintEvent(QPaintEvent *)
{

    static const QPoint hourHand[3] = {
        QPoint(3, 4),
        QPoint(-3, 4),
        QPoint(0, -40)
    };
    static const QPoint minuteHand[3] = {
        QPoint(3, 4),
        QPoint(-3, 4),
        QPoint(0, -70)
    };
    static const QPoint scondHand[3] = {
        QPoint(3, 4),
        QPoint(-3, 4),
        QPoint(0, -90)
    };
    QColor hourColor(0, 0, 0);
    QColor minuteColor(0, 0, 0, 191);
    QColor scondColor(255, 0, 0);

    int side = qMin(width(), height());
   // QTime time = m_dateTime.currentTime();
  //  time = time.addSecs(timeZoneOffset);

    QPainter painter(this);
painter.setRenderHint(QPainter::Antialiasing);

QPixmap pix(":/images/clock.png");
painter.drawPixmap(rect(),pix,pix.rect());

    painter.translate(width() / 2, height() / 2);
    painter.scale(side / 200.0, side / 200.0);
//-----------hour
    painter.setPen(Qt::NoPen);
    painter.setBrush(hourColor);

    painter.save();
    painter.rotate(30.0 * ((m_dateTime.hour() + m_dateTime.minute() / 60.0)));
    painter.drawConvexPolygon(hourHand, 3);
    painter.restore();

    painter.setPen(hourColor);

//    for (int i = 0; i < 12; ++i) {
//        painter.drawLine(88, 0, 96, 0);

//     //   painter.drawText((width()/2)-20, (height()/2)-20,QString::number(i+1));
// painter.rotate(30.0);
//    }
//----------minute
    painter.setPen(Qt::NoPen);
    painter.setBrush(minuteColor);

    painter.save();
    painter.rotate(6.0 * (m_dateTime.minute() + m_dateTime.second() / 60.0));
    painter.drawConvexPolygon(minuteHand, 3);
    painter.restore();

    painter.setPen(minuteColor);

//    for (int j = 0; j < 60; ++j) {
//        if ((j % 5) != 0)
//            painter.drawLine(92, 0, 96, 0);
//        painter.rotate(6.0);
//    }

    //----------second
        painter.setPen(Qt::NoPen);
        painter.setBrush(scondColor);

        painter.save();
        painter.rotate(360.0 * (m_dateTime.minute() + m_dateTime.second() / 60.0));
        painter.drawConvexPolygon(scondHand, 3);
        painter.restore();



       // painter.setPen(scondColor);

//        for (int r = 0; r < 360; ++r) {
//            if ((r % 5) != 0)
//                painter.drawLine(92, 0, 96, 0);
//            painter.rotate(360.0);
//        }





   // emit updated(m_dateTime);
}
void WorldTimeClock::setTime ( const QTime & t)
{
       timer->stop();
  m_dateTime=t;
qDebug()<<m_dateTime;
 //QDateTime::setTime(t);
  update();
}
void WorldTimeClock::revert()
{

 timer->start();
}
void WorldTimeClock::setTimeZone(int hourOffset)
{
    timeZoneOffset = qMin(qMax(-12, hourOffset), 12) * 3600;
    update();
}
