#include "selokabautostart.h"
#include <EDir>

//_______________________________________________________________________
SelokabAutostart::SelokabAutostart(QWidget *parent) :
    QWidget(parent)
{

    setupMe();
    QSettings setting("elokab","elokabsettings");
    setting.beginGroup("Session");
    mList=setting.value("DefaultAutostart").toStringList();

    QStringList list;
    list<<Edir::dataHomeAppDir()+"/servicesautostart";
    list<<Edir::dataAppDir()+"/servicesautostart";
    foreach (QString dir, list) {
        QDir dirS(dir);

        foreach (QString subFile, dirS.entryList(QStringList()<<"*.desktop"
                  ,QDir::Files| QDir::NoSymLinks| QDir::NoDotAndDotDot )){
            QString file=(dir+"/"+subFile);
            qDebug()<<"service"<<file;
            addEntry(file);
        }
    }
    mItemChanged=false;
}
//________________________________________________________________________
void SelokabAutostart::setupMe()
{
    gridLayout = new QGridLayout(this);

    label = new QLabel(this);
    label->setText(tr("A configuration tool for managing Default Elokab programs start up automatically with elokab"));

    treeWidget = new QTreeWidget(this);
    connect(treeWidget,SIGNAL(itemChanged(QTreeWidgetItem*,int)),this,SLOT(itemChanged(QTreeWidgetItem*,int)));
    QTreeWidgetItem *qtreewidgetitem = treeWidget->headerItem();
    qtreewidgetitem->setText(0,tr("Name"));

    gridLayout->addWidget(label, 0, 0);

    gridLayout->addWidget(treeWidget, 1, 0);


}

//_________________________________________________________________________
void SelokabAutostart::addEntry(const QString &file)
{

    QSettings setting(file,QSettings::IniFormat);
    setting.setIniCodec(QTextCodec::codecForName("UTF-8"));
    setting.beginGroup("Desktop Entry");
    if(setting.value("Type").toString()!="service")
        return;
    QString lc=this->locale().name().section("_",0,0);
    QString nameOld=setting.value("Name",QFileInfo(file).baseName()).toString();
    QString name=setting.value("Name["+lc+"]",nameOld).toString();
    QString iconS=setting.value("Icon").toString();
    QString exec=setting.value("Exec").toString();
    QIcon icon=QIcon::fromTheme(iconS,QIcon::fromTheme("application-x-executable"));
    QTreeWidgetItem *item=new QTreeWidgetItem(treeWidget);
    item->setText(0,name);
    item->setIcon(0,icon);
    item->setData(0,Qt::ToolTipRole,exec);

    if(mList.isEmpty()||mList.contains(exec))
        item->setCheckState(0,Qt::Checked);
    else
        item->setCheckState(0,Qt::Unchecked);

}

//_________________________________________________________________________
void SelokabAutostart::saveSettings()
{
    if(!mItemChanged)
        return;

    QStringList list;
    for (int i = 0; i < treeWidget->topLevelItemCount(); ++i) {
        QTreeWidgetItem *item=treeWidget->topLevelItem(i);
        if(item->checkState(0)==Qt::Checked){
            QString prog=item->data(0,Qt::ToolTipRole).toString();
            list.append(prog);
        }
    }
    QSettings setting("elokab","elokabsettings");
    setting.beginGroup("Session");
    setting.setValue("DefaultAutostart",list);

    mItemChanged=false;
}

//________________________________________________________________________
void SelokabAutostart::itemChanged(QTreeWidgetItem*,int)
{
    mItemChanged=true;
}

Q_EXPORT_PLUGIN2(setting_SelokabDefaultStart, SelokabAutostart)
