/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "widgetfonts.h"
#include "ui_widgetfonts.h"
#include <QStyleFactory>
#include <QSettings>
#include <QDebug>
#include <QDir>
#include <QLocale>
#include <QTextCodec>
widgetfonts::widgetfonts(QWidget *parent) :
        QWidget(parent),oldStyleChanged(false),
        ui(new Ui::widgetfonts)
{
        ui->setupUi(this);



        QSettings globalSetting("elokab","elokabsettings");

        ///widget fonts
        globalSetting.beginGroup("Font");
        QFont fontStyle=globalSetting.value("font").value<QFont>();
        globalSetting.endGroup();



        ///fonts
        QFontDatabase db;
        foreach(int size, db.standardSizes()){
                ui->comboBoxFontSize->addItem(QString::number(size));
        }

        int ind= ui->comboBoxFontSize->findText(QString::number(fontStyle.pointSize()));
        ui->comboBoxFontSize->setCurrentIndex(ind);

        int indf= ui->fontComboBox->findText(fontStyle.family());
        ui->fontComboBox->setCurrentIndex(indf);

        foreach(QString s,db.styles(fontStyle.family())){
                ui->comboBoxFontStyle->addItem(s);
        }


        QFontInfo ft(fontStyle);

        int inds= ui->comboBoxFontStyle->findText((ft.family()));

        if(inds==-1)
                ui->comboBoxFontStyle->setCurrentIndex(0);
        else
                ui->comboBoxFontStyle->setCurrentIndex(inds);




}

widgetfonts::~widgetfonts()
{
        delete ui;
}


void widgetfonts::on_comboBoxFontStyle_activated(const QString &/*arg1*/)
{

        updatPreview();


}


void widgetfonts::on_comboBoxFontSize_activated(int /*index*/)
{
        updatPreview();

}

void widgetfonts::on_fontComboBox_activated(const QString &arg1)
{
        QString curstyle=ui->comboBoxFontStyle->currentText();

        QFontDatabase db;
        //  qDebug()<<f.family();

        ui->comboBoxFontStyle->clear();
        foreach(QString s,db.styles(arg1)){
                ui->comboBoxFontStyle->addItem(s);
        }

        int inds= ui->comboBoxFontStyle->findText(curstyle);
        if(inds==-1)
                ui->comboBoxFontStyle->setCurrentIndex(0);
        else
                ui->comboBoxFontStyle->setCurrentIndex(inds);

        updatPreview();

}
//! update the text preview
void widgetfonts::updatPreview()
{
        QFont displayFont;
        displayFont.setFamily(ui->fontComboBox->currentText());
        displayFont.setPointSize(ui->comboBoxFontSize->currentText().toInt());
        QString fontStyle=(ui->comboBoxFontStyle->currentText());

        QFontDatabase fontDatabase;
        const QFont::StyleStrategy oldStrategy = displayFont.styleStrategy();
        displayFont = fontDatabase.font(displayFont.family(), fontStyle, displayFont.pointSize());
        displayFont.setStyleStrategy(oldStrategy);
        ui->lineEditPreview->setFont(displayFont);
        oldStyleChanged=true;

}

//! save if style or font canged
void widgetfonts::saveSettings()
{
    // widColors->saveColors();
    qDebug()<<"StyleChanged   : "<<oldStyleChanged;
    if (oldStyleChanged==false)
        return;
    //for elokab widget style
    QSettings globalSetting("elokab","elokabsettings");


    //for elokab font style
    QFont displayFont;
    displayFont.setFamily(ui->fontComboBox->currentText());
    displayFont.setPointSize(ui->comboBoxFontSize->currentText().toInt());
    QString fontStyle=(ui->comboBoxFontStyle->currentText());


    QFontDatabase fontDatabase;
    displayFont = fontDatabase.font(displayFont.family(), fontStyle, displayFont.pointSize());

    globalSetting.beginGroup("Font");
    globalSetting.setValue("font",displayFont.toString());
    globalSetting.endGroup();


    //for all app qt
    QSettings globalQt("Trolltech");

    globalQt.beginGroup("qt");
    globalQt.setValue("font",displayFont.toString());
    globalQt.endGroup();
    applyChanges();

    oldStyleChanged=false ;
/*
    //for GTK2------------------------------------------------
    if(!ui->checkBoxGtk->isChecked())
        return;

    QFile files(QDir::homePath()+"/.gtkrc-2.0");
    if(!files.open( QFile::ReadOnly))
        return;
    QString fontGtk="gtk-font-name=\""+ui->fontComboBox->currentText()+" "
            +ui->comboBoxFontStyle->currentText()+" "
           +ui->comboBoxFontSize->currentText()+"\"";
    QTextStream textStream(&files);
    textStream.setCodec(QTextCodec::codecForName("UTF-8"));
    QString line ;//premier line;
    QString alltext;
    while   (!textStream.atEnd()) {

        line = textStream.readLine().trimmed();

        if(line.startsWith("gtk-font-name"))
            line=fontGtk;

        alltext+=line+"\n";
    }

    files.close();
    if(!files.open( QFile::WriteOnly))
        return;
textStream.setDevice(&files);
    textStream<<alltext;
      files.close();
      */
}
#ifdef Q_WS_X11
extern void qt_x11_apply_settings_in_all_apps();
#endif // Q_WS_X11
void widgetfonts::applyChanges()
{
#if defined(Q_WS_X11)
    qt_x11_apply_settings_in_all_apps();
#endif // Q_WS_X11
}


Q_EXPORT_PLUGIN2(setting_fonts, widgetfonts)
