/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "widgeticonsthemes.h"
#include "ui_widgeticonsthemes.h"

#include <QSettings>
#include <QDir>
#include <QDebug>
#include <QTextCodec>
widgetIconsThemes::widgetIconsThemes(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::widgetIconsThemes)
{
    ui->setupUi(this);
   // createIconThemeLink();
    QStringList listThemePath=QIcon::themeSearchPaths();
  //  listThemePath.append(QIcon::themeSearchPaths());
// listThemePath.append(QDir::homePath()+"/.local/share/icons");
//        listThemePath.append(QDir::homePath()+"/.kde/share/icons");
//        listThemePath.append(QDir::homePath()+"/.kde4/share/icons");
        listThemePath.append(QDir::homePath()+"/.icons");
        listThemePath.append("/usr/share/icons");

        listThemePath.removeDuplicates();

 // QIcon::setThemeSearchPaths(listThemePath);

    QStringList pathList;
    int count=listThemePath.count();
    //QIcon::setThemeSearchPaths(listThemePath);
    for (int i=0;i<count;i++){
        QString dirPath= listThemePath.at(i);
        QDir dirS(dirPath);
        foreach(QString dir,dirS.entryList(QDir::AllDirs| QDir::NoDotAndDotDot )) {
                QString path=getDirIconTeme(dirPath+"/"+dir);
                if(!path.isEmpty())
                pathList<< path;
        }

    }
    pathList.removeDuplicates();
    pathList.removeAll("");
    ui->comboBoxIcons->addItems(pathList);

    QSettings globalSetting("elokab","elokabsettings");

    globalSetting.beginGroup("Themes");
    m_iconThemes=globalSetting.value("iconsTheme",QIcon::themeName()).toString();
    globalSetting.endGroup();
    int indx=ui->comboBoxIcons->findText(m_iconThemes);
    ui->comboBoxIcons->setCurrentIndex(indx);

    on_comboBoxIcons_activated(indx);
    if(!QFile::exists(QDir::homePath()+"/.icons/hicolor"))
            createHicolor();
}

widgetIconsThemes::~widgetIconsThemes()
{
    delete ui;
}


QString widgetIconsThemes::getDirIconTeme(QString dir)
{

          QDir dirPath(dir);
          if(!QFile::exists(dir+"/index.theme"))return QString();
          QSettings settings(dir+"/index.theme",QSettings::IniFormat);
           settings.beginGroup("Icon Theme");
          if(!settings.contains("Directories"))return QString() ;
          if(settings.value("Hidden",false).toBool())return QString() ;
          settings.endGroup();

  return dirPath.dirName();

}
void widgetIconsThemes::on_comboBoxIcons_activated(int /*index*/)
{
    QIcon::setThemeName(ui->comboBoxIcons->currentText());

    ui->toolButtonFile->setIcon(QIcon::fromTheme("unknown").pixmap(48,48));
    ui->toolButtonFolder->setIcon(QIcon::fromTheme("folder").pixmap(48,48));
    ui->toolButtonExec->setIcon(QIcon::fromTheme("application-x-executable").pixmap(48,48));
    emit themeChanged();

}
void widgetIconsThemes::saveSettings()
{

        if(m_iconThemes==ui->comboBoxIcons->currentText()){return;}

                qDebug()<<"iconThemes has changed";
                m_iconThemes=ui->comboBoxIcons->currentText();

                QString iconsTheme=ui->comboBoxIcons->currentText();
                QSettings globalSetting("elokab","elokabsettings");

                globalSetting.beginGroup("Themes");

                globalSetting.setValue("iconsTheme",iconsTheme);

                globalSetting.endGroup();

        if(!QFile::exists(QDir::homePath()+"/.icons/hicolor/index.theme"))
                createHicolor();

     ///-----------------------------------------------------
//        QSettings settings(QDir::homePath()+"/.icons/hicolor/index.theme",QSettings::IniFormat);

//         settings.setIniCodec(QTextCodec::codecForName("UTF-8"));

//        settings.beginGroup("Icon Theme");

//        settings.setValue("Inherits",iconsTheme);

//        settings.endGroup();

   //     removeUnicodeSpace(QDir::homePath()+"/.icons/hicolor/index.theme");

      ///-------------------------------------------------
   //   qDebug()<<QIcon::themeSearchPaths();
      QIcon::setThemeName(iconsTheme);

}

void widgetIconsThemes::createHicolor()
{
/*
        QString localDir=QDir::homePath()+"/.icons/hicolor";
        QDir dir;
      dir.mkpath(localDir);
      qApp->processEvents();
      //-----------
 QString userDir="/usr/share/icons/hicolor";
        QDir dirS(userDir);

    foreach ( QString name, dirS.entryList(QDir::AllEntries|
                                                QDir::NoDotAndDotDot| QDir::Readable | QDir::Executable ))
          {

                QFile::link(userDir+"/"+name,localDir+"/"+name);

          }
    QFile::copy(userDir+"/index.theme",localDir+"/index.theme");
*/
}
void widgetIconsThemes::removeUnicodeSpace(const QString &fileName)
{

        qApp->processEvents();

        //read--------------------
        QFile file(fileName);
        if (!file.open(QIODevice::ReadOnly ))
                return;

        QTextStream in(&file);
        in.setCodec(QTextCodec::codecForName("UTF-8"));

        QString txt = in.readAll();
        file.close();
        //---------------------------------
        //---------------------------------
      QTextCodec *codec = QTextCodec::codecForName("UTF-8");
        QByteArray encodedString =QByteArray::fromPercentEncoding(txt.toUtf8());

        QString string = codec->toUnicode(encodedString);
        // Write----------------------
        qApp->processEvents();
        QFile fileS(fileName);

        if (! fileS.open(QIODevice::WriteOnly ))
                return;

        QTextStream out(&fileS);
        out.setCodec(QTextCodec::codecForName("UTF-8"));
        out<<string;
        fileS.close();
}
void widgetIconsThemes::createIconThemeLink()
{
          QString localDir=(QDir::homePath()+"/.icons");


     QStringList listThemePath;

            listThemePath.append(QDir::homePath()+"/.kde/share/icons");
            listThemePath.append(QDir::homePath()+"/.kde4/share/icons");

        int count=listThemePath.count();

        for (int i=0;i<count;i++){
            QString dirPath= listThemePath.at(i);
            QDir dirS(dirPath);

            foreach(QString dir,dirS.entryList(QDir::AllDirs| QDir::NoSymLinks| QDir::NoDotAndDotDot )) {
               QString s=getDirIconTeme(dirPath+"/"+dir);

               bool exist=false;
               foreach (QString path, QIcon::themeSearchPaths()) {

                          if (QFile::exists(path+"/"+s)){

                              exist=true;
                            break;
                          }

               }

               if(exist==false){

                       QFile::link(dirPath+"/"+s,localDir+"/"+s);

               }
            }

        }

}

Q_EXPORT_PLUGIN2(setting_IconsTheme, widgetIconsThemes)
