/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "dialogedit.h"
#include "ui_dialogedit.h"
#include <QFileDialog>
#include <QSettings>
#include <QTextCodec>
#include <QDebug>
#define TYPE_EXEC "Application"
#define TYPE_SCRIPT "Script"
DialogEdit::DialogEdit(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DialogEdit)
{
    ui->setupUi(this);
}

DialogEdit::~DialogEdit()
{
    delete ui;
}

void DialogEdit::on_buttonBox_accepted()
{

    name=ui->lineEditName->text();
    programmName=ui->lineEditProgramName->text();
    comment=ui->lineEditComment->text();
     needTray=ui->checkBoxSystemTray->isChecked();
     iconName=ui->lineEditIcon->text();
}
void DialogEdit::setProgram()
{
    ui->lineEditName->setText(name);
    ui->lineEditProgramName->setText(programmName);
    ui->lineEditComment->setText(comment);
ui->lineEditIcon->setText(iconName);

    ui->checkBoxSystemTray->setChecked(needTray);
}

void DialogEdit::on_pushButtonShoose_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(
                this,
                "Select programm",
                "",
                "All  (*)");
    if(!fileName.isEmpty()){
        QFileInfo fi(fileName);
        QString sufix=fi.suffix();
        if(sufix=="desktop"){
            QString line= getInfoFromDesktopFile( fileName );
            ui->lineEditName->setText(line.section("|",0,0));
            ui->lineEditProgramName->setText(line.section("|",1,1));
      ui->lineEditComment->setText(line.section("|",2,2));
        }else{
            ui->lineEditName->setText(fi.baseName());
            ui->lineEditProgramName->setText(fileName);

        }

    }
}
QString DialogEdit::getInfoFromDesktopFile(QString fileName )
{
    QString lC=  QLocale::system().name().section("_",0,0);

    QFileInfo fi(fileName);
    QString baseName=fi.baseName();
    QSettings settings(fileName,QSettings::IniFormat);
    settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
    settings.beginGroup("Desktop Entry");
    QString nameold=settings.value("Name",baseName).toString();
    QString name=settings.value("Name["+lC+"]",nameold).toString();
    QString exec=settings.value("Exec",baseName).toString();
    QString coment=settings.value("Comment").toString();
    settings.endGroup();
    exec.remove("%U");
    exec.remove("%u");
    exec.remove("-caption");
    exec.remove("%c");
    exec.remove("%i");
    exec.remove("%f");
    exec.remove("%F");


    return name+"|"+exec+"|"+coment;
}

void DialogEdit::on_lineEditIcon_textChanged(const QString &arg1)
{
        if(arg1.length()>2){
                ui->toolButtonIcon->setIcon(QIcon::fromTheme(arg1));
        }
}
