/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "widgetshortcuts.h"
#include "ui_widgetshortcuts.h"
#include "dialogeditshortcut.h"
#include <EIcon>
#include <QSettings>
#include <QDebug>
widgetShortcuts::widgetShortcuts(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::widgetShortcuts)
{
    ui->setupUi(this);
    ui->treeWidgetCustomShort->hideColumn(3);
    ui->treeWidgetCustomShort->hideColumn(4);
    ui->treeWidgeGlobaltShort->hideColumn(3);
    ui->treeWidgeGlobaltShort->hideColumn(4);
    loadSettings("shortcut",ui->treeWidgeGlobaltShort);
    loadSettings("customShortcut",ui->treeWidgetCustomShort);
    // QStringList list=   s.childKeys();
    ui->treeWidgeGlobaltShort->expandAll();
    ui->treeWidgeGlobaltShort->resizeColumnToContents(0);
    ui->toolButtonAdd->setIcon(EIcon::fromTheme("list-add","add"));
ui->toolButtonCustomEdit->setIcon(EIcon::fromTheme("list-edit","edit"));
ui->toolButtonDelete->setIcon(EIcon::fromTheme("list-remove","remove"));
ui->toolButtonGlobalEdit->setIcon(EIcon::fromTheme("list-edit","edit"));

    // ui->treeWidgetShoert->resizeColumnToContents(1);
    keysChanged=false;
}

widgetShortcuts::~widgetShortcuts()
{
    delete ui;
}
void widgetShortcuts::loadSettings(const QString filePath,QTreeWidget *treeWidget)
{
    //Destination Command
    QSettings s("elokab",filePath);
    s.sync();
    foreach(const QString & group , s.childGroups())
    {
        s.beginGroup(group);
        qDebug()<<group;
        QTreeWidgetItem *item=new QTreeWidgetItem(treeWidget);
        item->setText(0,group);
        item->setText(1,s.value("Key","").toString());
        item->setText(2,s.value("Comment","").toString());

        QString data;
        if(s.contains("Destination")){

            data=s.value("Destination").toString();
            item->setText(3,"Destination");
            item->setText(4,s.value("Destination").toString());

        }else if(s.contains("Command")){
            item->setText(3,"Command");
            item->setText(4,s.value("Command").toString());
        }

        qDebug()<<"contains"<<item->text(0)<<item->text(3)<<item->text(4);
        foreach(const QString & childgroup , s.childGroups())
        {
            s.beginGroup(childgroup);

            QTreeWidgetItem *childItem=new QTreeWidgetItem(item);
            childItem->setText(0,childgroup);
            childItem->setText(1,s.value("Key","").toString());
            childItem->setText(2,s.value("Comment","").toString());

            if(s.contains("Destination")){
                childItem->setText(3,"Destination");
                childItem->setText(4,s.value("Destination").toString());
            }else if(s.contains("Command")){
                childItem->setText(3,"Command");
                childItem->setText(4,s.value("Command").toString());
            }
            qDebug()<<"contains"<<childItem->text(0)<<childItem->text(3)<<childItem->text(4);


            s.endGroup();
        }
        s.endGroup();
        if(item->childCount()>0)
            item->setData(0,Qt::UserRole,"GROUP");
    }
}

void widgetShortcuts::saveSettings()
{
    if (keysChanged==false)
        return;

    outSettings("shortcut",ui->treeWidgeGlobaltShort);
    outSettings("customShortcut",ui->treeWidgetCustomShort);


    QSettings s("elokab","elokabsettings");
    s.beginGroup("Shortcut");
    QString name=s.value("ShortcutChanged").toString();
    QString newName="changed_0";
    int i=0;
    while (name==newName) {
        i++;
        newName="changed_"+QString::number(i);
    }
    s.setValue("ShortcutChanged",newName);
    s.endGroup();
    keysChanged=true;
}
void widgetShortcuts::outSettings(const QString filePath,QTreeWidget *treeWidget)
{
    QSettings s("elokab",filePath);
    foreach (QString g, s.childGroups()) {
        qDebug()<<"group"<<g;
        s.beginGroup(g);
        foreach (QString k, s.childKeys()) {
            qDebug()<<"key"<<k;
            s.remove(k);
        }
        s.endGroup();
    }
    s.sync();
    int count=treeWidget->topLevelItemCount();
    for (int i = 0; i < count; ++i) {
        QTreeWidgetItem *item=treeWidget->topLevelItem(i);

        QString group=item->text(0);
        QString key=item->text(1);
        QString comment=item->text(2);
        QString dataKey=item->text(3);
        QString dataValue=item->text(4);
        qDebug()<<"dataValue"<<group<<dataKey<<dataValue;
        s.beginGroup(group);
        if(item->childCount()>0){
            int countChlids=item->childCount();
            for (int r = 0; r < countChlids; ++r) {
                QTreeWidgetItem *childItem=item->child(r);
                QString childgroup=childItem->text(0);
                QString childkey=childItem->text(1);
                QString childcomment=childItem->text(2);
                QString dataKey=childItem->text(3);
                QString dataValue=childItem->text(4);
                qDebug()<<"dataValue"<<childItem<<dataKey<<dataValue;

                s.beginGroup(childgroup);

                s.setValue("Key",childkey);
                s.setValue("Comment",childcomment);
                if(!dataKey.isNull()){
                    s.setValue(dataKey,dataValue);
                }

                s.endGroup();
            }
        }else{

            s.setValue("Key",key);
            s.setValue("Comment",comment);
            if(!dataKey.isNull()){
                s.setValue(dataKey,dataValue);
            }

        }
        s.endGroup();

    }

}

void widgetShortcuts::on_toolButtonGlobalEdit_clicked()
{
    QTreeWidgetItem *item=ui->treeWidgeGlobaltShort->currentItem();
    if(!item)return;
    if(item->data(0,Qt::UserRole).toString()=="GROUP")
        return;

    editGlobalItem(item);

}
void widgetShortcuts::on_treeWidgeGlobaltShort_itemActivated(QTreeWidgetItem *item, int /*column*/)
{
    if(item->data(0,Qt::UserRole).toString()=="GROUP")
        return;
    editGlobalItem(item);


}
void widgetShortcuts::editGlobalItem(QTreeWidgetItem *item)
{
    DialogEditShortcut *dlg=new DialogEditShortcut;
    dlg->setKey(item->text(1));
    dlg->setEditable(false);
    if(dlg->exec()==QDialog::Accepted){
        QString txt=dlg->getKey();

        item->setText(1,txt);
        keysChanged=true;

    }
    delete dlg;
}
void widgetShortcuts::on_treeWidgetCustomShort_itemActivated(QTreeWidgetItem *item, int /*column*/)
{
    if(item->data(0,Qt::UserRole).toString()=="GROUP")
        return;

    editCustomItem(item);
}

void widgetShortcuts::on_toolButtonCustomEdit_clicked()
{
    QTreeWidgetItem *item=ui->treeWidgetCustomShort->currentItem();
    if(!item)return;
    if(item->data(0,Qt::UserRole).toString()=="GROUP")
        return;

    editCustomItem(item);
}

void widgetShortcuts::editCustomItem(QTreeWidgetItem *item)
{
    DialogEditShortcut *dlg=new DialogEditShortcut;
    dlg->setProg(item->text(0));
    dlg->setKey(item->text(1));
    dlg->setComment(item->text(2));
    dlg->setEditable(true);
    if(dlg->exec()==QDialog::Accepted){
        QString key=dlg->getKey();
        QString prog=dlg->getProg();
        if(!prog.isEmpty()){
            item->setText(0,prog);
            item->setText(1,key);
            item->setText(2,dlg->getComment());
            keysChanged=true;
        }

    }
    delete dlg;
}


void widgetShortcuts::on_toolButtonAdd_clicked()
{
    DialogEditShortcut *dlg=new DialogEditShortcut;

    dlg->setEditable(true);
    if(dlg->exec()==QDialog::Accepted){
        QString key=dlg->getKey();
        QString prog=dlg->getProg();
        if(!prog.isEmpty()){
            QTreeWidgetItem *item=new QTreeWidgetItem(ui->treeWidgetCustomShort);
            item->setText(0,prog);
            item->setText(1,key);
            item->setText(2,dlg->getComment());
            item->setText(3,"Command");
            item->setText(4,prog);
            keysChanged=true;
        }
    }
    delete dlg;
}

Q_EXPORT_PLUGIN2(setting_KeyShortcuts, widgetShortcuts)

void widgetShortcuts::on_toolButtonDelete_clicked()
{
    QTreeWidgetItem *item=ui->treeWidgetCustomShort->currentItem();
    if(!item)return;
    int indx=ui->treeWidgetCustomShort->indexOfTopLevelItem(item);
    ui->treeWidgetCustomShort->takeTopLevelItem(indx);
    keysChanged=true;
}
