/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#include "widgetkeysettings.h"
#include "ui_widgetkeysettings.h"
#include <EIcon>
#include "dialogkeyadd.h"
#include <QDebug>
#include <QFile>
#include <QDir>
#include <QSettings>
#include <QProcess>
WidgetKeySettings::WidgetKeySettings(QWidget *parent) :
        QWidget(parent),
        ui(new Ui::WidgetKeySettings)
{
        ui->setupUi(this);
        setObjectName("WidgetKeySettings");
ui->treeWidgetKeyboard->setColumnHidden(1,true);
ui->treeWidgetKeyboard->expandAll();
ui->toolButtonUp->setIcon(EIcon::fromTheme("go-up"));
ui->toolButtonDown->setIcon(EIcon::fromTheme("go-down"));
ui->toolButtonAdd->setIcon(EIcon::fromTheme("list-add","document-add"));
ui->toolButtonRemove->setIcon(EIcon::fromTheme("list-remove","edit-delete"));
         loadSettings();
settingsChanged=false;
}

WidgetKeySettings::~WidgetKeySettings()
{
        delete ui;
}


void WidgetKeySettings::on_toolButtonAdd_clicked()
{
    DialogKeyAdd *dlg = this->findChild<DialogKeyAdd*>("addKey");

    if (!dlg)
    {
            dlg = new DialogKeyAdd(this);
              connect(dlg, SIGNAL(accepteChange(QString)), this, SLOT(addNewKey(QString)));
    }

    dlg->show();
    dlg->raise();
    dlg->activateWindow();
}

void WidgetKeySettings::addNewKey(QString arg)
{
        qDebug()<<arg;
        if(!arg.isEmpty()){
                QTreeWidgetItem *item=new QTreeWidgetItem;
                item->setText(0,arg.section("|",0,0));
                item->setText(1,arg.section("|",1,1));
                QString Variant=arg.section("|",2,2);
                if(Variant=="basic"){
                        item->setText(2,"Default");
                }else{
                        item->setText(2,Variant);
                }

                ui->treeWidget->addTopLevelItem(item);
                settingsChanged=true;
        }

}


void WidgetKeySettings::on_toolButtonRemove_clicked()
{
    QTreeWidgetItem *curItem=ui->treeWidget->currentItem();
    if(!curItem)return;



    int index= ui->treeWidget->indexOfTopLevelItem(curItem);
    ui->treeWidget->takeTopLevelItem(index);
settingsChanged=true;
}

void WidgetKeySettings::on_toolButtonUp_clicked()
{
        QTreeWidgetItem *curItem=ui->treeWidget->currentItem();
        if(!curItem)return;

        int index= ui->treeWidget->indexOfTopLevelItem(curItem);
        if(index==0)return;
        QTreeWidgetItem *item=new QTreeWidgetItem;
        item=curItem->clone();

        ui->treeWidget->takeTopLevelItem(index);
        ui->treeWidget->insertTopLevelItem(index-1,item);
        ui->treeWidget->setCurrentItem(item);
         settingsChanged=true;
}

void WidgetKeySettings::on_toolButtonDown_clicked()
{
        QTreeWidgetItem *curItem=ui->treeWidget->currentItem();
        if(!curItem)return;

        int index= ui->treeWidget->indexOfTopLevelItem(curItem);
        if(index== ui->treeWidget->topLevelItemCount()-1)return;
        QTreeWidgetItem *item=new QTreeWidgetItem;
        item=curItem->clone();

        ui->treeWidget->takeTopLevelItem(index);
        ui->treeWidget->insertTopLevelItem(index+1,item);
        ui->treeWidget->setCurrentItem(item);
         settingsChanged=true;
}

void WidgetKeySettings::saveSettings()

{

    //        if(settingsChanged==false)
    //                return;
    QString strlayou="setxkbmap -layout " ;
    QString strVariant=" -variant " ;
    QString strOption=" -option " ;



    QStringList listLayou;
    QStringList listLayouName;
    QStringList listVariant;
    QStringList listOption;
    QStringList listInfo;
    for(int i=0; i<ui->treeWidget->topLevelItemCount();i++){

        QTreeWidgetItem *item=ui->treeWidget->topLevelItem(i);
        listLayouName<<item->text(0)+"|"+item->text(1)+"|"+item->text(2);
        listLayou<<item->text(1);
        listVariant<<item->text(2);
    }

    if(ui->treeWidgetKeyboard->topLevelItem(0)->child(0)->checkState(0)==Qt::Checked)
        listOption<<ui->treeWidgetKeyboard->topLevelItem(0)->child(0)->text(1);

    for(int i=0; i<ui->treeWidgetKeyboard->topLevelItem(1)->childCount();i++){

        QTreeWidgetItem *item=ui->treeWidgetKeyboard->topLevelItem(1)->child(i);
        if(item->checkState(0)==Qt::Checked){
            listOption<<item->text(1);
            listInfo<<item->text(0);
        }


    }



    strlayou+= "\""+listLayou.join(",")+"\"";
    strVariant+="\""+listVariant.join(",")+"\"";
    strOption+="\""+listOption.join(",")+"\"";

    strVariant.remove("Default");
    QProcess pr;
    pr.start("setxkbmap -option \"\"");
    pr.waitForFinished();

    pr.start(strlayou+" "+strOption+" "+strVariant);

    pr.waitForFinished();

    pr.start("setxkbmap -query");
    qDebug()<<"setxkbmap -query :"  << pr.readAll();
    pr.waitForFinished();
    qDebug()<< pr.readAll();
    //        qDebug()<<strlayou;
    //        qDebug()<<strVariant;
    //        qDebug()<<   strOption;
    QSettings settings("elokab","elokabsettings");
    settings.beginGroup("KeyMap");
    //   m_ks=( settings.value("Key",QKeySequence(Qt::AltModifier+Qt::META)).toString());
    settings.setValue("Names",listLayouName);
    //    settings.setValue("Layouts",listLayou);
    settings.setValue("Options",listOption);
    settings.setValue("Info",listInfo);

    settings.setValue("SwitshByAppName",ui->checkBoxSwitchByApp->isChecked());
    QString name=settings.value("KeyChanged").toString();
    QString newName="changed_0";
    int i=0;
    while (name==newName) {
        i++;
        newName="changed_"+QString::number(i);
    }
    settings.setValue("KeyChanged",newName);

    settings.endGroup();

    settingsChanged=false;
}
void WidgetKeySettings::loadSettings()
{



        QStringList listLayouName;
        QStringList listOption;

        QSettings settings("elokab","elokabsettings");
        settings.beginGroup("KeyMap");

        listLayouName= settings.value("Names","English|us|Default").toStringList();
        listOption= settings.value("Options","").toStringList();
ui->checkBoxSwitchByApp->setChecked(settings.value("SwitshByAppName",false).toBool());
        settings.endGroup();

        foreach (QString s, listLayouName) {
                QTreeWidgetItem *item=new QTreeWidgetItem;
                 item->setText(0,s.section("|",0,0));
                 item->setText(1,s.section("|",1,1));
                 item->setText(2,s.section("|",2,2));

                ui->treeWidget->addTopLevelItem(item);
        }
        foreach (QString s, listOption) {
                if(s=="terminate:ctrl_alt_bksp")
                    ui->treeWidgetKeyboard->topLevelItem(0)->child(0)->setCheckState(0,Qt::Checked);

                QTreeWidgetItem *itemP=ui->treeWidgetKeyboard->topLevelItem(1);
                for(int i=0; i<itemP->childCount();i++){
                        QString txt=itemP->child(i)->text(1);
                        if (txt==s)
                                itemP->child(i)->setCheckState(0,Qt::Checked);
                }
        }



}

void WidgetKeySettings::on_treeWidgetKeyboard_itemChanged(QTreeWidgetItem */*item*/, int /*column*/)
{
  //  qDebug()<<item->text(0);
    settingsChanged=true;
}
Q_EXPORT_PLUGIN2(setting_KeySettings, WidgetKeySettings)


