#include "screenlockerwidget.h"
#include "ui_screenlockerwidget.h"
#include <QSettings>
#include <QFileDialog>

ScreenLockerWidget::ScreenLockerWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ScreenLockerWidget)
{
    ui->setupUi(this);
    btnBgColor=new ButtonColor;
    btnTimerColor=new ButtonColor;
    ui->horizontalLayoutBgColor->addWidget(btnBgColor);
    ui->horizontalLayoutTimerColor->addWidget(btnTimerColor);

    QSettings settings("elokab","screensaver");
    int screenType=settings.value("ScreenType",0).toInt();

    QColor  bgrColor=settings.value("BgrColor",QColor(255, 170, 0)).value<QColor>();
    QColor timerColor=settings.value("TimerColor",QColor(255, 255, 255)).value<QColor>();

    QString imagePath=settings.value("ImagePath","home").toString();


    QString xScreenPath=settings.value("xScreenPath","/usr/lib/xscreensaver/glmatrix").toString();

    settings.endGroup();
    btnBgColor->setColor(bgrColor);
    btnTimerColor->setColor(timerColor);
    ui->lineEditImagePath->setText(imagePath);
    ui->lineEditXscreenPath->setText(xScreenPath);
    switch (screenType) {
    case 0:
        ui->radioButtonBgColor->setChecked(true);
        break;
    case 1:
        ui->radioButtonBgImage->setChecked(true);
        break;
    case 2:
        ui->radioButtonScreenSaver->setChecked(true);
        break;
    default:
        break;
    }
}

ScreenLockerWidget::~ScreenLockerWidget()
{
    delete ui;
}

void ScreenLockerWidget::saveSettings()
{
        int screenType=0;
        if( ui->radioButtonBgColor->isChecked())
            screenType=0;
        if( ui->radioButtonBgImage->isChecked())
            screenType=1;
        if( ui->radioButtonScreenSaver->isChecked())
            screenType=2;


        QString bgrStyle="";

        switch (screenType) {
        case 0:
            bgrStyle=QString("background-color: rgb(%1, %2, %3);")
                    .arg(QString::number(btnBgColor->color().red()))
                    .arg(QString::number(btnBgColor->color().green()))
                    .arg(QString::number(btnBgColor->color().blue()));
            break;
        case 1:

            bgrStyle=QString("border-image: url(%1) ;")
                    .arg(ui->lineEditImagePath->text());

            break;
        default:
            break;
        }
       QString timerStyle=QString("color:rgb(%1, %2, %3);")
               .arg(QString::number(btnTimerColor->color().red()))
               .arg(QString::number(btnTimerColor->color().green()))
               .arg(QString::number(btnTimerColor->color().blue()));

    QSettings settings("elokab","screensaver");

    settings.setValue("ScreenType",screenType);
    settings.setValue("BgrColor",btnBgColor->color());
    settings.setValue("TimerColor",btnTimerColor->color());
    settings.setValue("ImagePath",ui->lineEditImagePath->text());
    settings.setValue("xScreenPath",ui->lineEditXscreenPath->text());

  settings.setValue("BgrStyle",bgrStyle);
  settings.setValue("TimerStyle",timerStyle);




    settings.endGroup();
}

void ScreenLockerWidget::on_toolButtonBgImage_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                     ui->lineEditImagePath->text(),
                                                     tr("Images (*.png *.xpm *.jpg)"));

    if(!fileName.isEmpty())
         ui->lineEditImagePath->setText(fileName);
}

void ScreenLockerWidget::on_toolButtonXscreenSaver_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                     ui->lineEditXscreenPath->text(),
                                                     tr("All files (*)"));

    if(!fileName.isEmpty())
         ui->lineEditXscreenPath->setText(fileName);
}
Q_EXPORT_PLUGIN2(setting_ScreenLocker, ScreenLockerWidget)
