﻿/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "widgetstyle.h"
#include "ui_widgetstyle.h"
#include "dialoggtk.h"
#include <QStyleFactory>
#include <QSettings>
#include <QDebug>
#include <EDir>

#include <QLocale>
#include <QTextCodec>
WidgetStyle::WidgetStyle(QWidget *parent) :
        QWidget(parent),oldStyleChanged(false),
        ui(new Ui::WidgetStyle)
{
        ui->setupUi(this);



//        QDir styleDir(qApp->applicationDirPath());

//        styleDir.cdUp();
//        styleDir.cd( styleDir.absolutePath()+"/share/elokab/elokab-settings/qstyle") ;
       QDir styleDir( Edir::dataAppDir()+"/qstyle");
        QString curentStyle;

        QSettings globalSetting("elokab","elokabsettings");
        ///style settings
        globalSetting.beginGroup("Style");
        curentStyle=globalSetting.value("Name","default").toString();
        globalSetting.endGroup();
        ///widget fonts
        globalSetting.beginGroup("Font");
        QFont fontStyle=globalSetting.value("font").value<QFont>();
        globalSetting.endGroup();

        ///widget style

        QString lc=this->locale().name().section("_",0,0);
        ui->comboBoxStyle->addItem(trUtf8("default"),"default");
        listComment.append(trUtf8("Style Default System :the change will take effect the next time the applications is started"));
        foreach (QString s, QStyleFactory::keys()) {
            QSettings gs(styleDir.absoluteFilePath(s+".themerc"),QSettings::IniFormat);
            gs.setIniCodec(QTextCodec::codecForName("UTF-8"));
            gs.beginGroup("Misc");
             QString name=gs.value("Name["+lc+"]",s).toString();
             QString commentold=gs.value("Comment",s).toString();
             QString comment=gs.value("Comment["+lc+"]",commentold).toString();
            gs.endGroup();

             ui->comboBoxStyle->addItem(name,s);
             listComment.append(comment);
        }

        //ui->comboBoxStyle->addItems(QStyleFactory::keys());
        ui->comboBoxStyle->setCurrentIndex(ui->comboBoxStyle->findData(curentStyle));
        int indx=ui->comboBoxStyle->currentIndex();
        if(indx>-1)
        ui->labelStyleDefault->setText(trUtf8("Description:")+listComment.at(indx));

    ui->pushButtonConfigGtk->setEnabled(curentStyle=="GTK+");





        QApplication::setStyle(QStyleFactory::create(curentStyle));
//widColors=new WidgetColors ;
//ui->verticalLayoutColors->addWidget(widColors);

}

WidgetStyle::~WidgetStyle()
{
        delete ui;
}


void WidgetStyle::on_comboBoxFontStyle_activated(const QString &/*arg1*/)
{

        updatPreview();


}


void WidgetStyle::on_comboBoxFontSize_activated(int /*index*/)
{
        updatPreview();

}

//! update the text preview
void WidgetStyle::updatPreview()
{
//        QFont displayFont;
//        displayFont.setFamily(ui->fontComboBox->currentText());
//        displayFont.setPointSize(ui->comboBoxFontSize->currentText().toInt());
//        QString fontStyle=(ui->comboBoxFontStyle->currentText());

//        QFontDatabase fontDatabase;
//        const QFont::StyleStrategy oldStrategy = displayFont.styleStrategy();
//        displayFont = fontDatabase.font(displayFont.family(), fontStyle, displayFont.pointSize());
//        displayFont.setStyleStrategy(oldStrategy);
//        ui->lineEditPreview->setFont(displayFont);
//        oldStyleChanged=true;

}

//! save if style or font canged
void WidgetStyle::saveSettings()
{
       // widColors->saveColors();
        qDebug()<<"StyleChanged   : "<<oldStyleChanged;
        if (oldStyleChanged==false)
                return;
        //for elokab widget style
        int index=ui->comboBoxStyle->currentIndex();

        QString wStyle=  ui->comboBoxStyle->itemData(index,Qt::UserRole).toString();
        QSettings globalSetting("elokab","elokabsettings");
        globalSetting.beginGroup("Style");
        globalSetting.setValue("Name",wStyle);
        globalSetting.endGroup();



        //for all app qt
        QSettings globalQt("Trolltech");

        globalQt.beginGroup("qt");
        globalQt.setValue("style",wStyle);
    //    globalQt.setValue("font",displayFont.toString());
        globalQt.endGroup();
        applyChanges();
        oldStyleChanged=false;
}
#ifdef Q_WS_X11
extern void qt_x11_apply_settings_in_all_apps();
#endif // Q_WS_X11
void WidgetStyle::applyChanges()
{
#if defined(Q_WS_X11)
    qt_x11_apply_settings_in_all_apps();
#endif // Q_WS_X11
}

void WidgetStyle::on_comboBoxStyle_activated(int index)
{

QString wName= ui->comboBoxStyle->itemData(index,Qt::UserRole).toString();

qDebug()<<wName;
    ui->pushButtonConfigGtk->setEnabled(wName=="GTK+");

        ui->labelStyleDefault->setText(trUtf8("Description:")+listComment.at(index));
            QApplication::setStyle(QStyleFactory::create(wName));

    oldStyleChanged=true;
}
Q_EXPORT_PLUGIN2(setting_WidgetStyle, WidgetStyle)

void WidgetStyle::on_pushButtonConfigGtk_clicked()
{
    DialogGtk *dlg=new DialogGtk;
    if(dlg->exec()==QDialog::Accepted){

        QSettings s("elokab","elokabsettings");
        s.beginGroup("Themes");
        QString iconThemes=s.value("iconsTheme",QIcon::themeName()).toString();
        s.endGroup();

        QString themName=dlg->themName();
         QString pathName=dlg->pathName();
        QFont font=dlg->fontName();

        //----------------------------------------------------------------
        QFile fileS(QDir::homePath()+"/.gtkrc-2.0");

        if (! fileS.open(QIODevice::WriteOnly ))
            return ;

        QTextStream out(&fileS);
        out.setCodec(QTextCodec::codecForName("UTF-8"));
        out<<"# File created by elokab Gtk Config"<<endl;;
        out<<"#Configs for GTK2 programs"<<endl;

        out<<"include \""<<pathName+"/gtk-2.0/gtkrc\""<<endl;
        out<<"style \"user-font\""<<endl;
        out<<"{"<<endl;
        out<<  "font_name=\""<<font.family()<<" "<<font.styleName()<<"\""<<endl;
        out<<"}"<<endl;
        out<<"gtk-theme-name=\""<<themName<<"\""<<endl;
        out<<"gtk-font-name=\""<<font.family()<<" "<<font.styleName()<<" "<<QString::number(font.pointSize())<<"\""<<endl;
        out<<"gtk-icon-theme-name=\""+iconThemes+"\""<<endl;
        fileS.close();
        // ---------------------------------------------------------------------
        s.beginGroup("Style");
        s.setValue("GtkThemeName",themName);
        s.endGroup();
        /*
# File created by KDE Gtk ConfigMicrosoft Sans Serif Regular
# Configs for GTK2 programs

include "/  home/yousef/.themes/Clearlooks/gtk-2.0/gtkrc"
style "user-font"
{
    font_name="Microsoft Sans Serif Regular"
}
widget_class "*" style "user-font"
gtk-font-name="Microsoft Sans Serif Regular 10"
gtk-theme-name="Clearlooks"
gtk-icon-theme-name="Elokab"
gtk-fallback-icon-theme="Elokab"
gtk-toolbar-style=GTK_TOOLBAR_ICONS
gtk-menu-images=1
gtk-button-images=0
*/

            QApplication::setStyle(QStyleFactory::create("GTK+"));
    }

}
