#include "globalshortcuts.h"
#include <QDBusInterface>

/************************************************************
 *               CLASS ShortcutItem
 ***********************************************************/
ShortcutItem:: ShortcutItem(const QKeySequence& sequence,SType type) :
   m_type(type)
{
       if (!setShortcut(sequence))
          qWarning() << "QxtGlobalShortcut failed to register:" << sequence.toString();

       qDebug() << "Add Shortcut"<<sequence.toString();
       connect(this,SIGNAL(activated()),this,SLOT(exec()));
}

//_____________________________________________________________
ShortcutItem::~ShortcutItem()
{

}

//_____________________________________________________________
void ShortcutItem::exec()
{
    qDebug()<<"activated"<<this->shortcut().toString()<<m_type;

    if(m_type==Type_Exec){
      QProcess::startDetached(m_command);
    }else{
        qDebug()<<"org.elokab."+m_dbusMap<<"/org/elokab/"+m_dbusMap<<
                "org.elokab."+m_dbusMap+".Interface"<<this->shortcut().toString();

        QDBusInterface dbus("org.elokab."+m_dbusMap,
                "/org/elokab/"+m_dbusMap,
                "org.elokab."+m_dbusMap+".Interface");

        if (!dbus.isValid()) {
            qDebug() << "QDBusInterface is not valid!";
            return ;
        }
        dbus.call("setShortcut",shortcut().toString());
    }

}

/************************************************************
 *               CLASS GlobalShortcuts
 ***********************************************************/
GlobalShortcuts::GlobalShortcuts(QObject *parent) :
    QObject(parent)
{
    m_shortSettings=new ElokabSettings(this);
    connect(m_shortSettings,SIGNAL(shortcutsKeysChanged()) ,this,SLOT(rechargeAllSettings()));

    QTimer::singleShot(1000,this,SLOT(rechargeAllSettings()));
}

//_____________________________________________________________
GlobalShortcuts::~GlobalShortcuts()
{
    qDeleteAll(listShortcuts);
}

//_____________________________________________________________
void GlobalShortcuts::rechargeAllSettings()
{
       foreach (ShortcutItem *item, listShortcuts) {
            delete item;
        }
        listShortcuts.clear();


    rechargeSettings("shortcut");
    rechargeSettings("customShortcut");

}

//_____________________________________________________________
void GlobalShortcuts::rechargeSettings(const QString &filePath)
{


    QSettings s("elokab",filePath);

    s.sync();
    foreach(const QString & group , s.childGroups())
    {
        s.beginGroup(group);

        QString key=s.value("Key").toString();
        QString destination=s.value("Destination").toString();
        QString command=s.value("Command").toString();
        /*
         *if shortcut contains key else ignore the shortcut
         *if shortcut contains Destination type=Type_Dbus
         *or shortcut contains Command type=Type_Exec
         *else ignore the shortcut
         */
        if(!key.isEmpty())
        {

            if(!destination.isEmpty()){
                ShortcutItem *item=new ShortcutItem(QKeySequence(key),Type_Dbus);
                item-> setDebusMap(destination);
                listShortcuts.append(item);
            }else if(!command.isEmpty()){
                ShortcutItem *item= new ShortcutItem(QKeySequence(key),Type_Exec);
                item->setExecCommand(command);
                listShortcuts.append(item);
            }

        }

        foreach(const QString & childgroup , s.childGroups())
        {
            s.beginGroup(childgroup);

            QString key=s.value("Key").toString();
            QString destination=s.value("Destination").toString();
            QString command=s.value("Command").toString();

            if(!key.isEmpty())
            {

                if(!destination.isEmpty()){
                    ShortcutItem *item=new ShortcutItem(QKeySequence(key),Type_Dbus);
                    item-> setDebusMap(destination);
                    listShortcuts.append(item);
                }else if(!command.isEmpty()){
                    ShortcutItem *item= new ShortcutItem(QKeySequence(key),Type_Exec);
                    item->setExecCommand(command);
                    listShortcuts.append(item);
                }

            }

            s.endGroup(); //...childgroup
        }

        s.endGroup(); //...group
    }
}
