//////////////////////////////////////////
//  File      : passworddlg.cpp			//
//  Written by: ludmiloff@gmail.com		//
//  Copyright : GPL2
// modified by : abou zakaria
//////////////////////////////////////////


#include "passworddlg.h"

#include "ui_password.h"

class AmePasswordDlg::PasswordDlgPrivate
{
public:
	PasswordDlgPrivate(AmePasswordDlg *qd)
		: qd(qd)
	{}

	void init();
	
	AmePasswordDlg *qd;
	Ui_passwordFrm ui;

};

void AmePasswordDlg::PasswordDlgPrivate::init()
{

    ui.setupUi(qd);


}


AmePasswordDlg::AmePasswordDlg(const QString &name,const QString &icon,QWidget *parent)
	: QDialog(parent), d (new PasswordDlgPrivate(this))
{

	d->init();
    d->ui.labelWarning->setText("");
    QString  i="USER";
    d->ui.usernameEdit->setText(qgetenv(i.toUtf8().constData()));
    d->ui.passwordEdit->setFocus();
    d->ui.labelPixmap->setPixmap(QIcon::fromTheme(icon,QIcon::fromTheme(name,QIcon::fromTheme("application-x-executable"))).pixmap(48));
 d->ui.labelProgName->setText(name);
 connect(d->ui.usernameEdit, SIGNAL(returnPressed()), this, SLOT(accept()));
 connect(d->ui.passwordEdit, SIGNAL(returnPressed()), this, SLOT(accept()));
}



AmePasswordDlg::~AmePasswordDlg()
{

	delete d;
	d = NULL;
}



QString AmePasswordDlg::username() const
{
	return d->ui.usernameEdit->text();
}

QString AmePasswordDlg::password() const
{
    return d->ui.passwordEdit->text();
}

bool AmePasswordDlg::userHook()
{
	return true;
}

bool AmePasswordDlg::selectFirstEmptyField()
{
	if (d->ui.usernameEdit->text().isEmpty()) {
		d->ui.usernameEdit->setFocus();
		d->ui.usernameEdit->selectAll();
		return true;
	}
	if (d->ui.passwordEdit->text().isEmpty()) {
		d->ui.passwordEdit->setFocus();
		d->ui.passwordEdit->selectAll();
		return true;
	}
	return false;
}

bool AmePasswordDlg::check()
{
	return 
	   (!d->ui.usernameEdit->text().isEmpty() && 
		!d->ui.passwordEdit->text().isEmpty() &&
		userHook());
}

void AmePasswordDlg::accept()
{
	if (check()) 
	{
		QDialog::accept();
	} else {
		// TODO: password is wrong, sound error
		// assume passoword field is wrong
		selectFirstEmptyField();
	}
}

AmeSudoDlg::AmeSudoDlg(const QString &name,const QString &icon,const QString &cmd, QWidget *parent)
    : AmePasswordDlg(name,icon,parent)
{

	command = cmd;
	proc = new AmeUserProcess();

}
bool AmeSudoDlg::findProgram(const QString &program)
{

    QFileInfo fi(program);
    if (fi.isExecutable())
        return true;

    QString path = qgetenv("PATH");
    foreach(QString dir, path.split(":"))
    {
        QFileInfo fi= QFileInfo(dir + QDir::separator() + program);
         qDebug()<<fi.filePath();
        if (fi.isExecutable() )
            return true;
    }
    return false;
}
AmeSudoDlg::~AmeSudoDlg()
{
}

void AmeSudoDlg::accept()
{
	if (!selectFirstEmptyField()) {
		//qDebug() << "BEFORE ACCEPT";
		proc->setUser(username(), password());
		proc->exec(command, AmeUserProcess::WaitForExit);
		int status = proc->status();
		if (status == AmeUserProcess::Finished) {
			//qDebug() << "STATUS " << status; 
			QDialog::accept(); 
		} 
		if (status == AmeUserProcess::WrongPassword || status == AmeUserProcess::WrongCommand ||
			status == AmeUserProcess::Crashed) {
			qDebug() << "WRONG PASSWORD";

            d->ui.labelWarning->setText("Warning: Incorerect password ,please try again");
			d->ui.passwordEdit->selectAll();
			d->ui.passwordEdit->setFocus();
		}
	}
}
